/* segment chanca1 in program nc */

/* sets up channel parameters */

#include <stdio.h>
#include "nc.h"
#include "y.tab.h"
#include "ncsub.h"
#include "ncomp.h"
#include "control.h"

chantype *makchantype(int ctype, int cnum, int nstates, int nparm, double bt);
void chanrate(chan *cpnt, chanparm *chp);
double calcchaninf(double v, chanparm *chp);
double calcchantau(double v, chanparm *chp);
double calcchantaua(double v, chanparm *chp);
double calcchantaub(double v, chanparm *chp);
chantype *getchantype(int ctype, int cnum);
void mak2state(chanstate *spnt, double (*frate)(chan *cpnt),
                                double (*rrate)(chan *cpnt), double rate);
double rchanf(chan *cpnt);
double rchanr(chan *cpnt);
void varcopyu(void);

double calcna2m(double v, int func);
double calcna2h(double v, int func);


double al0 (chan *cpnt);
double bet0(chan *cpnt);
double al1 (chan *cpnt);
double bet1(chan *cpnt);

static double (*alc) (chan*) = al0;
static double (*betc)(chan*) = bet0;
static double (*alch) (chan*) = al1;
static double (*betch)(chan*) = bet1;


#ifdef __cplusplus
extern "C" {
#endif

double exp(double);
double log(double);

#ifdef __cplusplus
}
#endif

double ncabs(double x);

/*----------------------------------------*/

double calcca0m(double v, int func)

/* Calculate alphac, betac per second as a function of mV.
   See "catype.n" for calibration script.
*/

{
   double val,x,y;

  switch (func) {
  case 1:					/* alpha c */
    y = -0.1 * (v+15.);
    x = exp (y) - 1.;
    if (ncabs(x) > 1e-5)                        /* singularity at v = -15 mv */
       val = 0.6 * y / x;
    else
       val = 0.6;
    break;

  case 2:					/* beta c */
    val = (2 * exp ((v+38) / -18.)); 
    break;
  }
  return val*MSSEC;
}

/*----------------------------------------*/

double calcca2m(double v, int func)

/* Calculate alphac, betac per second as a function of mV.

   Ca-T type channel from Kawai et al (1996) J. Gen. Physiol. 108:525-535.
   Corrected for voffset due to their use of 10 mM Ca++.
   See "catype.n" for calibration script.

*/

{
   double val,x,y;

  switch (func) {
  case 1:					/* alpha c */
    y = -0.1 * (v+49.);
    x = exp (y) - 1.;
    if (ncabs(x) > 1e-5)                        /* check for singularity */
       val = y / x;
    else
       val = 1.0;
    val *= 40.0;
    break;

  case 2:					/* beta c */
    val = (60.0 * exp ((v+69) / -25.)); 
    break;
  }
  return val;
}

/*----------------------------------------*/

double calcca2h(double v, int func)

/* Calculate alphach, betach as a function of mv.

   Ca-T type channel from Kawai et al (1996) J. Gen. Physiol.  108:525-535.
   Corrected for voffset due to their use of 10 mM Ca++.
   See "catype.n" for calibration script.
*/

{
   double val,x,y;

  switch (func) {
  case 1:					/* alpha c */
    val = (12 * exp ((v+73.) / -9.)); 
    break;

  case 2:					/* beta c */
    y = -0.05 * (v+70.);
    x = exp (y) - 1.;
    if (ncabs(x) > 1e-5)                        /* check for singularity */
       val = y / x;
    else
       val = 1.0;
    val *= 24.; 
    val *= 1- 1 / (1 + ( exp ((v+50.) / -25.)));

    break;
  }
  return val;
}

/*--------------------------------------------*/

datum cinf (datum &v, datum &typ)

/* Return the equilibrium Ca activation at a given voltage v. */
/* Note that calcchaninf only works for HH type channel */

{
   chantype *ch;

  varcopyu();
  ch = getchantype(CA,(int)typ.val);
  v.val = calcchaninf(v.val,&ch->parm[0]);
  return v;
}

/* - - - - - - - - - - - - - - - - - - - - - -*/

datum ctau (datum &v, datum &typ)

/* Return the absolute tau (in sec) for Na activation at a given
   voltage v and the "relative" taum and tauh.
*/

{
   chantype *ch;

  varcopyu();
  ch = getchantype(CA,(int)typ.val);
  v.val = calcchantau(v.val,&ch->parm[0]);
  return v;
}

/* - - - - - - - - - - - - - - - - - - - - - -*/

datum ctaua (datum &v, datum &typ)

/* Return the absolute tau (in sec) for Na activation at a given
   voltage v and the "relative" taum and tauh.
*/

{
   chantype *ch;

  varcopyu();
  ch = getchantype(CA,(int)typ.val);
  v.val = calcchantaua(v.val,&ch->parm[0]);
  return v;
}

/* - - - - - - - - - - - - - - - - - - - - - -*/

datum ctaub (datum &v, datum &typ)

/* Return the absolute tau (in sec) for Na activation at a given
   voltage v and the "relative" taum and tauh.
*/

{
   chantype *ch;

  varcopyu();
  ch = getchantype(CA,(int)typ.val);
  v.val = calcchantaub(v.val,&ch->parm[0]);
  return v;
}

/*--------------------------------------------*/

datum chinf (datum &v, datum &typ)

/* Return the equilibrium Ca activation at a given voltage v. */
/* Note that calcchaninf only works for HH type channel */

{
   chantype *ch;

  varcopyu();
  ch = getchantype(CA,(int)typ.val);
  if (ch->numparm > 1)
   v.val = calcchaninf(v.val,&ch->parm[1]);
  else v.val = 0;
  return v;
}

/* - - - - - - - - - - - - - - - - - - - - - -*/

datum chtau (datum &v, datum &typ)

/* Return the absolute tau (in sec) for Na activation at a given
   voltage v and the "relative" taum and tauh.
*/

{
   chantype *ch;

  varcopyu();
  ch = getchantype(CA,(int)typ.val);
  if (ch->numparm > 1)
   v.val = calcchantau(v.val,&ch->parm[1]);
  else v.val = 0;
  return v;
}

/* - - - - - - - - - - - - - - - - - - - - - -*/

datum chtaua (datum &v, datum &typ)

/* Return the absolute tau (in sec) for Na activation at a given
   voltage v and the "relative" taum and tauh.
*/

{
   chantype *ch;

  varcopyu();
  ch = getchantype(CA,(int)typ.val);
  if (ch->numparm > 1)
   v.val = calcchantaua(v.val,&ch->parm[1]);
  else v.val = 0;
  return v;
}

/* - - - - - - - - - - - - - - - - - - - - - -*/

datum chtaub (datum &v, datum &typ)

/* Return the absolute tau (in sec) for Na activation at a given
   voltage v and the "relative" taum and tauh.
*/

{
   chantype *ch;

  varcopyu();
  ch = getchantype(CA,(int)typ.val);
  if (ch->numparm > 1)
   v.val = calcchantaub(v.val,&ch->parm[1]);
  else v.val = 0;
  return v;
}

/*--------------------------------------------*/

chantype *makca0(void)

/* Make L-type Ca channel with kinetics of c3 like HH.  */

/* Unitary current 20 pS @ 35 deg C from:

     Karschin and Lipton, (1989) J. Physiol.  418: 379-396.
*/
{
     chantype *ch;
     chanstate *spnt;
     chanparm *parm;
     double m,h;
     int nstate, nparm;

   nstate = 2;				/* 2 Markov states */
   nparm = 1;				/* make 1 set of params, c */
   ch = makchantype(CA, 0, nstate, nparm, BASETC);/* make chan const,parm */
   ch->hh = 1;				/* HH chan = uses "m" */
   ch->unitary = dcalu;
   ch->vrev = R2F * ktemp * log(dcao/(dcai+dpki));   /* GHK equation */
   ch->perm[PCA] = 1.0;			/* rel permeability to Ca ions */
   ch->perm[PNA] = dpnaca;		/* rel permeability to Na ions */
   ch->perm[PK] = dpkca;		/* permeability to K ions / perm Ca */
   spnt = ch->state;
   parm = ch->parm;

   parm[0].nfval    = 2; 		/* number of func vals (am, bm, etc.) */
   parm[0].nival    = 2; 		/* number of implicit vals (m1, m2) */
   parm[0].chancalc =  calcca0m; 	/* default rate function */
   parm[0].funcname = "calcca0m"; 	/* user rate function */
   parm[0].dq = dqca; 			/* Q10 for c rate function */
   parm[0].voff = dcaoffs; 		/* voltage offset from user */

   mak2state(spnt,rchanf,rchanr,CHANRATE);

   return ch;
}

/*--------------------------------------------*/

chantype *makca1(void)

/* Sequential-state description of L-type Ca++ channel, c3 like m3h.

   See Armstrong and Matteson, Curr. Topics Membranes Transport 22:331-352.

   Number states from 0 to n; 
   Set numstate = n;
   Set numtrans = number of transitions.
   Set cond     = conductance of state;

   For each state, set transitions:
    Set trans =   State to go to on this transition. 
    Set trate =   Function that returns basic rate for transition. 
    Set ratemul = Multiplier for rate function.
    Set rateo  =  Set to 0 for m, set to 1 for h:  sets voffset and tau.


                  0 <-> 1 <-> 2 <-> 3 
*/

  {
     chantype *ch;
     chanstate *spnt;
     chanparm *parm;
     double m,h;
     int nstate, nparm;

   nstate = 4;				/* 4 Markov states */
   nparm = 1;				/* make 1 set of params, m, h */
   ch = makchantype(CA, 1, nstate, nparm, BASETC);/* make chan const,parm */
   ch->unitary = dcalu;
   ch->vrev = R2F * ktemp * log(dcao/(dcai+dpki));   /* GHK equation */
   ch->perm[PCA] = 1.0;			/* rel permeability to Ca ions */
   ch->perm[PNA] = dpnaca;		/* rel permeability to Na ions */
   ch->perm[PK] = dpkca;		/* permeability to K ions / perm Ca */
   spnt = ch->state;
   parm = ch->parm;

   parm[0].nfval    = 2; 		/* number of func vals (am, bm, etc.) */
   parm[0].nival    = 0; 		/* number of implicit vals (m1, m2) */
   parm[0].chancalc =  calcca0m; 	/* default rate function */
   parm[0].funcname = "calcca1m"; 	/* user rate function */
   parm[0].dq = dqca; 			/* Q10 for c rate function */

   m = 1.0;
   h = 1.0;
   spnt[0].numtrans   = 1;
   spnt[0].cond       = 0;
   spnt[0].trans  [0] = 1;
   spnt[0].trate  [0] = alc;
   spnt[0].ratemul[0] = 3.0 * m;
   spnt[0].rateo  [0] = 0;	/* offsm, taum */

   spnt[1].numtrans   = 2;
   spnt[1].cond       = 0;
   spnt[1].trans  [0] = 2;	  	
   spnt[1].trate  [0] = alc;
   spnt[1].ratemul[0] = 2.0 * m;
   spnt[1].rateo  [0] = 0;
   spnt[1].trans  [1] = 0;	
   spnt[1].trate  [1] = betc;
   spnt[1].ratemul[1] = 1.0 * m;
   spnt[1].rateo  [1] = 0;

   spnt[2].numtrans   = 2;
   spnt[2].cond       = 0;
   spnt[2].trans  [0] = 3;
   spnt[2].trate  [0] = alc;
   spnt[2].ratemul[0] = 1.0 * m;
   spnt[2].rateo  [0] = 0;	/* offsm */
   spnt[2].trans  [1] = 1;
   spnt[2].trate  [1] = betc;
   spnt[2].ratemul[1] = 2.0 * m;
   spnt[2].rateo  [1] = 0;	/* offsm */

   spnt[3].numtrans   = 1;
   spnt[3].cond       = 1.0;
   spnt[3].trans  [0] = 2;
   spnt[3].trate  [0] = betc;
   spnt[3].ratemul[0] = 3.0 * m;
   spnt[3].rateo  [0] = 0;	/* offsm */

   return ch;
}

/*--------------------------------------------*/

chantype *makca2(void)

/* Make T-type channel with kinetics of c2h like HH.  */

/* Unitary current 8 pS @ 35 deg C from:

     Karschin and Lipton, (1989) J. Physiol.  418: 379-396.

     qfactor = exp (log(Q10=1.4) * (35-6.3)/10) = 2.626

     conductance @ 6.3 deg C = 8 pS / 2.626 = 3 pS @ 6.3 deg
*/

{
     chantype *ch;
     chanstate *spnt;
     chanparm *parm;
     double m,h;
     int nstate, nparm;

   nstate = 2;				/* 2 Markov states */
   nparm = 2;				/* make 2 set of params, c */
   ch = makchantype(CA, 2, nstate, nparm, BASETC);/* make chan const,parm */
   ch->hh = 1;				/* HH chan = uses "m", "h" */
   ch->unitary = dcatu;
   ch->vrev = R2F * ktemp * log(dcao/(dcai+dpki));   /* GHK equation */
   ch->perm[PCA] = 1.0;			/* rel permeability to Ca ions */
   ch->perm[PNA] = dpnaca;		/* rel permeability to Na ions */
   ch->perm[PK] = dpkca;		/* permeability to K ions / perm Ca */
   spnt = ch->state;
   parm = ch->parm;

   parm[0].nfval    = 2; 		/* number of func vals (am, bm, etc.) */
   parm[0].nival    = 2; 		/* number of implicit vals (m1, m2) */
   parm[0].chancalc =  calcca2m; 	/* default rate function */
   parm[0].funcname = "calcca2m"; 	/* user rate function */
   parm[0].dq = dqca; 			/* Q10 for c rate function */

   parm[1].nfval    = 2; 		/* number of func vals (am, bm, etc.) */
   parm[1].nival    = 2; 		/* number of implicit vals (m1, m2) */
   parm[1].chancalc =  calcca2h; 	/* default rate function */
   parm[1].funcname = "calcca2h"; 	/* user rate function */
   parm[1].dq = dqca; 			/* Q10 for c rate function */

   mak2state(spnt,rchanf,rchanr,CHANRATE);

   return ch;
}

/*--------------------------------------------*/

chantype *makca3(void)

/* Sequential-state description of T-type Ca++ channel, c2 like m2h.

   See Armstrong and Matteson, Curr. Topics Membranes Transport 22:331-352.

   Number states from 0 to n; 
   Set numstate = n;
   Set numtrans = number of transitions.
   Set cond     = conductance of state;

   For each state, set transitions:
    Set trans =   State to go to on this transition. 
    Set trate =   Function that returns basic rate for transition. 
    Set ratemul = Multiplier for rate function.
    Set rateo  =  Set to 0 for m, set to 1 for h:  sets voffset and tau.


                  0 <-> 1 <-> 2 

                  |     |     |

                  3 <-> 4 <-> 5 
*/

  {
     chantype *ch;
     chanstate *spnt;
     chanparm *parm;
     double m,h;
     int nstate, nparm;

   nstate = 6;				/* 6 Markov states */
   nparm = 2;				/* make 2 sets of params, m, h */
   ch = makchantype(CA, 3, nstate, nparm, BASETC);/* make chan const,parm */
   ch->unitary = dcatu;
   ch->vrev = R2F * ktemp * log(dcao/(dcai+dpki));   /* GHK equation */
   ch->perm[PCA] = 1.0;			/* rel permeability to Ca ions */
   ch->perm[PNA] = dpnaca;		/* rel permeability to Na ions */
   ch->perm[PK] = dpkca;		/* rel permeability to K ions */
   spnt = ch->state;
   parm = ch->parm;

   parm[0].nfval    = 2; 		/* number of func vals (am, bm, etc.) */
   parm[0].nival    = 0; 		/* number of implicit vals (m1, m2) */
   parm[0].chancalc =  calcca2m; 	/* default rate function */
   parm[0].funcname = "calcca3m"; 	/* user rate function */
   parm[0].dq = dqca; 			/* Q10 for c rate function */

   parm[1].nfval    = 2; 		/* number of func vals (am, bm, etc.) */
   parm[1].nival    = 0; 		/* number of implicit vals (m1, m2) */
   parm[1].chancalc =  calcca2h; 	/* default rate function */
   parm[1].funcname = "calcca3h"; 	/* user rate function */
   parm[1].dq = dqca; 			/* Q10 for c rate function */

   m = 1.0;
   h = 1.0;
   spnt[0].numtrans   = 2;
   spnt[0].cond       = 0;
   spnt[0].trans  [0] = 1;
   spnt[0].trate  [0] = alc;
   spnt[0].ratemul[0] = 2.0 * m;
   spnt[0].rateo  [0] = 0;
   spnt[0].trans  [1] = 3;
   spnt[0].trate  [1] = betch;
   spnt[0].ratemul[1] = 1.0 * h;
   spnt[0].rateo  [1] = 1;	/* offsh, tauh */

   spnt[1].numtrans   = 3;
   spnt[1].cond       = 0;
   spnt[1].trans  [0] = 2;	  	
   spnt[1].trate  [0] = alc;
   spnt[1].ratemul[0] = 1.0 * m;
   spnt[1].rateo  [0] = 0;
   spnt[1].trans  [1] = 0;	
   spnt[1].trate  [1] = betc;
   spnt[1].ratemul[1] = 1.0 * m;
   spnt[1].rateo  [1] = 0;
   spnt[1].trans  [2] = 4;	
   spnt[1].trate  [2] = betch;
   spnt[1].ratemul[2] = 1.0 * h;
   spnt[1].rateo  [2] = 1;

   spnt[2].numtrans   = 2;
   spnt[2].cond       = 1.0;
   spnt[2].trans  [0] = 1;
   spnt[2].trate  [0] = betc;
   spnt[2].ratemul[0] = 2.0 * m;
   spnt[2].rateo  [0] = 0;	/* offsm */
   spnt[2].trans  [1] = 5;
   spnt[2].trate  [1] = betch;
   spnt[2].ratemul[1] = 1.0 * h;
   spnt[2].rateo  [1] = 1;	/* offsh */

   spnt[3].numtrans   = 2;
   spnt[3].cond       = 0;
   spnt[3].trans  [0] = 4;
   spnt[3].trate  [0] = alc;
   spnt[3].ratemul[0] = 2.0 * m;
   spnt[3].rateo  [0] = 0;	/* offsm */
   spnt[3].trans  [1] = 0;
   spnt[3].trate  [1] = alch;
   spnt[3].ratemul[1] = 1.0 * h;
   spnt[3].rateo  [1] = 0;	/* offsm */

   spnt[4].numtrans   = 3;
   spnt[4].cond       = 0;
   spnt[4].trans  [0] = 5;	  	
   spnt[4].trate  [0] = alc;
   spnt[4].ratemul[0] = 1.0 * m;
   spnt[4].rateo  [0] = 0;
   spnt[4].trans  [1] = 3;	
   spnt[4].trate  [1] = betc;
   spnt[4].ratemul[1] = 1.0 * m;
   spnt[4].rateo  [1] = 0;
   spnt[4].trans  [2] = 1;	
   spnt[4].trate  [2] = alch;
   spnt[4].ratemul[2] = 1.0 * h;
   spnt[4].rateo  [2] = 1;

   spnt[5].numtrans   = 2;
   spnt[5].cond       = 0;
   spnt[5].trans  [0] = 4;
   spnt[5].trate  [0] = betc;
   spnt[5].ratemul[0] = 2.0 * m;
   spnt[5].rateo  [0] = 0;	/* offsm */
   spnt[5].trans  [1] = 2;
   spnt[5].trate  [1] = alch;
   spnt[5].ratemul[1] = 1.0 * h;
   spnt[5].rateo  [1] = 1;	/* offsh */


   return ch;
}

/*--------------------------------------------*/

/* check:

   Ueda Y, Kaneko, A., and Kaneda, M. (1992) Voltage -dependent
ionic currents in solitary horizontal cells isolated from cat
retina.  J. Neurophysiol. 68: 1143-1150.

   Usui, S., Kamiyama, Y, Ishii, H, and Ikeno, H (1996a)
Reconstruction of retinal horizontal cell responses by the ionic
current model.  Vis. Research 36: 1711-1719.

   Usui, S., Kamiyama, Y, Ishii, H, and Ikeno, H (1996b) Ionic
current model of bipolar cells in the lower vertebrate retina.
Vis. Research 36: 4069-4076.


   Sullivan, JM, and Lasater, EM (1990) Sustained and transient
potassium currents of cultured horizontal cells isolated from
white bass retina.  J. Neurophysiol. 64: 1758-1766.

Satoh H. Aoki K. Watanabe SI. Kaneko A. L-type calcium
channels in the axon terminal of mouse bipolar cells.
Neuroreport. 9:2161-5, 1998.

de la Villa P. Vaquero CF. Kaneko A. Two types of calcium
currents of the mouse bipolar cells recorded in the retinal slice
preparation. European Journal of Neuroscience. 10(1):317-23, 1998

Kaneda M. Mochizuki M. Aoki K. Kaneko A. Modulation of GABAC
response by Ca2+ and other divalent cations in horizontal cells
of the catfish retina.  Journal of General Physiology.
110(6):741-7, 1997. 

0. Sasaki T. Kaneko A. L-Glutamate-induced responses in OFF-type
bipolar cells of the cat retina. Vision Research. 36(6):787-95,
1996 

Downing JE. Kaneko A. Cat retinal ganglion cells show transient
responses to acetylcholine and sustained responses to
L-glutamate. Neuroscience Letters.  137(1):114-8, 1992 

Kaneda M. Kaneko A. Voltage-gated calcium currents in isolated
retinal ganglion cells of the cat. Japanese Journal of
Physiology. 41(1):35-48, 1991.

Kaneko A. Tachibana M. Pinto LH. Transient calcium current of
retinal bipolar cells of the mouse. [Review] [12 refs]
Neuroscience Research - Supplement.  10:S67-76, 1989

Kaneko A. Pinto LH. Tachibana M. Transient calcium current of
retinal bipolar cells of the mouse. Journal of Physiology.
410:613-29, 1989 Mar.

Tachibana M. Kaneko A. Retinal bipolar cells receive negative
feedback input from GABAergic amacrine cells. [Review] [37 refs]
Visual Neuroscience.  1(3):297-305, 1988.  

Kaneko A. The functional role of retinal horizontal cells.
[Review] [86 refs] Japanese Journal of Physiology. 37(3):341-58,
1987.   Complete Reference 

Lohrke S. Hofmann HD. Voltage-gated currents of rabbit A- and
B-type horizontal cells in retinal monolayer cultures. Visual
Neuroscience. 11(2):369-78, 1994 

Fenwick, E.M, Marty, A., and Neher, E. 1982b Sodium and calcium
channels in bovine chromaffin cells.  J. Physiol. 331::599-635.

Nowycky M.C., Fox, A.P. and Tsien, R.W. 1985b Long opening mode
of neuronal calcium channels and its promotion by the
dihydropyridine calcium agonist Bay K 8644. PNAS 82:2178-2182.


*/

