/* segment chancgmp in program nc */

/* sets up channel parameters */

#include <stdio.h>
#include "nc.h"
#include "y.tab.h"
#include "ncsub.h"
#include "ncomp.h"
#include "control.h"

#ifdef __cplusplus
extern "C" {
#endif

double exp(double);
double log(double);

#ifdef __cplusplus
}
#endif

double al0 (chan *cpnt);
				/* Opening rate constant for CNG channel */
				/*  Dependent only on Vm.  */
static double (*betcgmp)(chan*) = al0; 
double rnt(chan *cp);     /* rate functions */
double rt(chan *cp);     

chantype *makchantype(int ctype,int cnum,int nstates,int nparm,double bt);
double getntresp (chan *ch, comp *cpnt);
int     isntresp (chan *ch, comp *cpnt);

double qrate(chanparm *chp);

/*--------------------------------------------*/

double calccgmp (double v, int func)

/* Calculate "beta" rate function for cGMP-gated channel,
   given voltage in mv.  Return rate calib in 1/sec;

   The "func" parameter is not used here because there is only
   one function for the cGMP channel.
*/

{
    double rate;

  rate = exp (0.23 * 0.001 * v * FR / ktemp);
  return rate;

}

/*--------------------------------------------*/

chantype *makcgmp1(void)

/* number states from 0 to n; 
   set numstate = n;
   set numtrans = number of transitions.
   set cond     = conductance of state;

   for each state, set transitions:
    set trans = state to go to on this transition. 
    set trate = function that returns basic rate for transition. 
    set ratemul = multiplier for rate function.
    set rateo  =  Set to 0 for m, set to 1 for h:  sets voffset and tau.
*/

/* Taken from:
                Taylor, W.R., and Baylor, D.A. (1995) 
                J. Physiol. 483: 567-582.

   in which a 7-state cGMP-gated markov diagram is provided.
Only alpha and beta for the transition between the open and
closed state were provided.  Rates for cGMP binding were inferred
(RGS) from probability of the activated state (includes both open
and closed state) at different ligand concentrations and from
kinetics (Baylor, 1988).  Inactivated states were arranged
according to frequency and duration reported (RGS 10/98).

For info on conductance, see: 

  Ruiz, M. and Karpen, J.W. (1997), Nature 389: 389-92.

  Karpen has recent manuscript (now in review) on cGMP binding rates.
       (RGS 11/98)
*/  

  {
     double rt(chan *cp);     /* rate functions */
     chantype *ch;
     chanstate *spnt;
     chanparm *parm;
     float *respamp;
     double a,b,c,d;
     int nstate, nparm;

   nstate = 8;                          /* 8 Markov states */
   nparm = 1;                           /* make 2 sets of params, m, h */
   ch=makchantype(CGMP,1,nstate,nparm,BASETSYN);/* make chan const, parm */
   ch->unitary = 25e-12;
   ch->trconc =  50e-6;			/* max conc of nt */
   ch->perm[PNA] = 1.0;			/* permeability to Na+ ions */
   ch->perm[PK]  = 1.0;			/* permeability to K+ ions */
   ch->perm[PCA] = 0.15;		/* permeability to Ca++ ions */
					/*  See Yau & Baylor, 1989 */
   ch->vrev = 0;			/* default reversal potential */
   spnt = ch->state;
   parm = ch->parm;
   respamp = ch->respamp;

   parm[0].nfval    = 1;                /* num of func vals (am, bm, etc.) */
   parm[0].nival    = 0;                /* don't use implicit vals (yet) */
   parm[0].chancalc =  calccgmp;        /* default rate function */
   parm[0].funcname = "calccgmp";       /* user rate function */
   parm[0].dq = dqm;                    /* Q10 for m rate function */

   respamp[CGMP-GLU] = 1.0;              /* response to cGMP */
   respamp[CAMP-GLU] = 1e-2;             /* response to cAMP */

   a = 250e6;
   b = 8.0e2;
   c = 10;
   d = 0.2;

   spnt[0].numtrans   = 1;
   spnt[0].cond       = 0;
   spnt[0].trans  [0] = 1;
   spnt[0].trate  [0] = rnt;
   spnt[0].ratemul[0] = 1 * a;
   spnt[0].rateo  [0] = 0;
                                     /*                        7            */
   spnt[1].numtrans   = 2;           /*                                     */
   spnt[1].cond       = 0;           /*                        |            */ 
   spnt[1].trans  [0] = 2;           /*                                     */
   spnt[1].trate  [0] = rnt;         /* 0 <->  1  <->  2  <->  3  <->  6    */
   spnt[1].ratemul[0] = 1 * a;       /*                                     */
   spnt[1].rateo  [0] = 0;	     /*                                     */
   spnt[1].trans  [1] = 0;	     /*                |       |            */
   spnt[1].trate  [1] = rt;          /*                                     */
   spnt[1].ratemul[1] = 5 * b;       /*                                     */
   spnt[1].rateo  [1] = 0;	     /*                4  <->  5            */

   spnt[2].numtrans   = 3;
   spnt[2].cond       = 0;
   spnt[2].trans  [0] = 3;
   spnt[2].trate  [0] = rnt;
   spnt[2].ratemul[0] = 4 * a;
   spnt[2].rateo  [0] = 0;
   spnt[2].trans  [1] = 1;
   spnt[2].trate  [1] = rt;
   spnt[2].ratemul[1] = 5 * b;
   spnt[2].rateo  [1] = 0;
   spnt[2].trans  [2] = 4;
   spnt[2].trate  [2] = betcgmp;	/* opening rate constant */
   spnt[2].ratemul[2] = 1.51e4;
   spnt[2].rateo  [2] = 1;

   spnt[3].numtrans   = 4;
   spnt[3].cond       = 0;
   spnt[3].trans  [0] = 6;		/* state 3 = the open state */
   spnt[3].trate  [0] = rt;
   spnt[3].ratemul[0] = 4 * c;		/* transition to short I */
   spnt[3].rateo  [0] = 1;
   spnt[3].trans  [1] = 2;
   spnt[3].trate  [1] = rt;
   spnt[3].ratemul[1] = 1 * b;
   spnt[3].rateo  [1] = 0;
   spnt[3].trans  [2] = 5;
   spnt[3].trate  [2] = betcgmp;	/* opening rate constant */
   spnt[3].ratemul[2] = 1.51e4;
   spnt[3].rateo  [2] = 1;
   spnt[3].trans  [3] = 7;		/* state 3 = the open state */
   spnt[3].trate  [3] = rt;
   spnt[3].ratemul[3] = 4 * d;		/* transition to long I */
   spnt[3].rateo  [3] = 1;

   spnt[4].numtrans   = 2;
   spnt[4].cond       = 0.25;
   spnt[4].trans  [0] = 5;
   spnt[4].trate  [0] = rnt;
   spnt[4].ratemul[0] = 4 * a;	
   spnt[4].rateo  [0] = 0;
   spnt[4].trans  [1] = 2;
   spnt[4].trate  [1] = rt;
   spnt[4].ratemul[1] = 2.1e4;		/* closing rate constant */
   spnt[4].rateo  [1] = 1;

   spnt[5].numtrans   = 2;
   spnt[5].cond       = 1.0;		/* the open state */
   spnt[5].trans  [0] = 3;
   spnt[5].trate  [0] = rt;
   spnt[5].ratemul[0] = 2.1e4;		/* closing rate constant */
   spnt[5].rateo  [0] = 1;
   spnt[5].trans  [1] = 4;
   spnt[5].trate  [1] = rt;
   spnt[5].ratemul[1] = 1 * b;
   spnt[5].rateo  [1] = 0;

   spnt[6].numtrans   = 1;
   spnt[6].cond       = 0;
   spnt[6].trans  [0] = 3;		/* state 6 = short Inactivated state */
   spnt[6].trate  [0] = rt;
   spnt[6].ratemul[0] = 30 * c;
   spnt[6].rateo  [0] = 1;

   spnt[7].numtrans   = 1;
   spnt[7].cond       = 0;
   spnt[7].trans  [0] = 3;		/* state 7 = long Inactivated state */
   spnt[7].trate  [0] = rt;
   spnt[7].ratemul[0] = 20 * d;
   spnt[7].rateo  [0] = 1;

   return ch;
}

