/* segment changaba in program nc */

/* sets up channel parameters */

#include <stdio.h>
#include "nc.h"
#include "y.tab.h"
#include "ncsub.h"
#include "ncomp.h"
#include "control.h"

#ifdef __cplusplus
extern "C" {
#endif

double exp(double);
double log(double);

#ifdef __cplusplus
}
#endif

chantype *makchantype(int ctype, int cnum, int nstates, int nparm, double bt);

double rnt(chan *spnt);
double rt(chan *spnt);

/*--------------------------------------------*/

double calcgaba1(double v, int func) {}

/*--------------------------------------------*/

chantype *makgaba1(void)

/* number states from 0 to n; 
   set numstate = n;
   set numtrans = number of transitions.
   set cond     = conductance of state;

   for each state, set transitions:
    set trans = state to go to on this transition. 
    set trate = function that returns basic rate for transition. 
    set ratemul = multiplier for rate function.
    set rateo  =  Set to 0 for m, set to 1 for h:  sets voffset and tau.
*/

/* Taken from:

		Busch and Sakmann (1990) 
			[cited by Destexhe, Mainen and Sejnowski (1998) ]

   in which a 5-state GABA-A Markov diagram is provided:

                  gaba      gaba
                0  <->  1   <->   2
                        |         |
                        |         |
                        3         4     Open states

*/

  {
     double rnt(chan *cp),rt(chan *cp);     /* rate functions */
     chantype *ch;
     chanstate *spnt;
     chanparm *parm;
     float *respamp;
     double a,b,c,d;
     int nstate, nparm;


   nstate = 5;                          /* 5 Markov states */
   nparm = 1;                           /* make 2 sets of params, m, h */
   ch=makchantype(GABA,1,nstate,nparm,BASETSYN);/* make chan const, parm */
   ch->perm[PCL] = 1.0;			/* permeability to Cl- ions */
   ch->perm[PK]  = 0.0;			/* permeability to K+ ions */
   ch->unitary = 30e-12;
   ch->trconc = 100e-6;			/* default max trans conc */
   ch->vrev = vcl;			/* default reversal potential */
   spnt = ch->state;
   parm = ch->parm;
   respamp = ch->respamp;

   parm[0].nfval    = 0;                /* num of func vals (am, bm, etc.) */
   parm[0].nival    = 0;                /* don't use implicit vals (yet) */
   parm[0].chancalc =  calcgaba1;       /* default rate function */
   parm[0].funcname = "calcgaba1";      /* user rate function */
   parm[0].dq = dqsyn;                  /* Q10 for m rate function */

   respamp[GABA-GLU] = 1.0;		/* response to GABA */
   respamp[BIC-GLU] = -1.0;		/* response to bicucculine */
   respamp[PTX-GLU] = -1.0;		/* response to picrotoxin */

   spnt[0].numtrans   = 1;
   spnt[0].cond       = 0;
   spnt[0].trans  [0] = 1;
   spnt[0].trate  [0] = rnt;
   spnt[0].ratemul[0] = 20e6;
   spnt[0].rateo  [0] = 0;

   spnt[1].numtrans   = 3;
   spnt[1].cond       = 0;
   spnt[1].trans  [0] = 2;
   spnt[1].trate  [0] = rnt;
   spnt[1].ratemul[0] = 10e6;
   spnt[1].rateo  [0] = 0;
   spnt[1].trans  [1] = 0;
   spnt[1].trate  [1] = rt;
   spnt[1].ratemul[1] = 4.6e3;
   spnt[1].rateo  [1] = 0;
   spnt[1].trans  [2] = 3;
   spnt[1].trate  [2] = rt;
   spnt[1].ratemul[2] = 3.3e3;
   spnt[1].rateo  [2] = 3;

   spnt[2].numtrans   = 2;
   spnt[2].cond       = 0;
   spnt[2].trans  [0] = 4;
   spnt[2].trate  [0] = rt;
   spnt[2].ratemul[0] = 10.6e3;
   spnt[2].rateo  [0] = 3;
   spnt[2].trans  [1] = 1;
   spnt[2].trate  [1] = rt;
   spnt[2].ratemul[1] = 9.2e3;
   spnt[2].rateo  [1] = 0;	

   spnt[3].numtrans   = 1;
   spnt[3].cond       = 1.0;
   spnt[3].trans  [0] = 1;			/* state 3 = the open state */
   spnt[3].trate  [0] = rt;
   spnt[3].ratemul[0] = 9.8e3;
   spnt[3].rateo  [0] = 3;

   spnt[4].numtrans   = 1;
   spnt[4].cond       = 1.0;
   spnt[4].trans  [0] = 2;
   spnt[4].trate  [0] = rt;
   spnt[4].ratemul[0] = 410;	
   spnt[4].rateo  [0] = 3;

   return ch;

}
/*--------------------------------------------*/

chantype *makgaba2(void)

/* number states from 0 to n; 
   set numstate = n;
   set numtrans = number of transitions.
   set cond     = conductance of state;

   for each state, set transitions:
    set trans = state to go to on this transition. 
    set trate = function that returns basic rate for transition. 
    set ratemul = multiplier for rate function.
    set rateo  =  Set to 0 for m, set to 1 for h:  sets voffset and tau.
*/

/* Taken from:
                Jonas, P., Major, G., and Sakmann, B. (1993) 
                J. Physiol. 472:615-5663.

   in which a 7-state AMPA markov diagram is provided.  Two sets
of rate constants were derived, 1) which gave a good fit, and 2)
which was derived assuming independence of binding between
binding of glutamate molecules.  The one defined here is the 
"assuming independence".

*/

  {
     double rnt(chan *cp),rt(chan *cp);     /* rate functions */
     chantype *ch;
     chanstate *spnt;
     chanparm *parm;
     float *respamp;
     double a,b,c,d;
     int nstate, nparm;


   nstate = 7;                          /* 7 Markov states */
   nparm = 1;                           /* make 2 sets of params, m, h */
   ch=makchantype(GABA,2,nstate,nparm,BASETSYN);/* make chan const, parm */
   ch->perm[PCL] = 1.0;			/* permeability to Cl- ions */
   ch->perm[PK]  = 0.0;			/* permeability to K+ ions */
   ch->unitary = dampau;
   ch->trconc = 100e-6;			/* default max trans conc */
   ch->vrev = vcl;			/* default reversal potential */
   spnt = ch->state;
   parm = ch->parm;
   respamp = ch->respamp;

   parm[0].nfval    = 0;                /* num of func vals (am, bm, etc.) */
   parm[0].nival    = 0;                /* don't use implicit vals (yet) */
   parm[0].chancalc =  calcgaba1;       /* default rate function */
   parm[0].funcname = "calcgaba2";      /* user rate function */
   parm[0].dq = dqsyn;                  /* Q10 for m rate function */

   respamp[GABA-GLU] = 1.0;		/* response to GABA */
   respamp[BIC-GLU] = -1.0;		/* response to bicucculine */
   respamp[PTX-GLU] = -1.0;		/* response to picrotoxin */

   a = 1.0;	/* forward: alpha */
   b = 1.0;     /* reverse: beta */
   c = 1.0;     /* deactivation */
   d = 1.0;     /* re-activation */
 
   spnt[0].numtrans   = 1;
   spnt[0].cond       = 0;
   spnt[0].trans  [0] = 1;
   spnt[0].trate  [0] = rnt;
   spnt[0].ratemul[0] = 1.0 * a;
   spnt[0].rateo  [0] = 0;	/* offsm */

   spnt[1].numtrans   = 3;
   spnt[1].cond       = 0;
   spnt[1].trans  [0] = 2;
   spnt[1].trate  [0] = rnt;
   spnt[1].ratemul[0] = 1.0 * a;
   spnt[1].rateo  [0] = 0;
   spnt[1].trans  [1] = 0;
   spnt[1].trate  [1] = rt;
   spnt[1].ratemul[1] = 1.0 * b;
   spnt[1].rateo  [1] = 0;
   spnt[1].trans  [2] = 4;
   spnt[1].trate  [2] = rt;
   spnt[1].ratemul[2] = 1.0 * c;
   spnt[1].rateo  [2] = 1;

   spnt[2].numtrans   = 3;
   spnt[2].cond       = 0;
   spnt[2].trans  [0] = 3;
   spnt[2].trate  [0] = rt;		/* function of time */
   spnt[2].ratemul[0] = 1.0 * a;
   spnt[2].rateo  [0] = 0;	/* offsm */
   spnt[2].trans  [1] = 1;
   spnt[2].trate  [1] = rt;		/* function of nt conc, time */
   spnt[2].ratemul[1] = 1.0 * b;
   spnt[2].rateo  [1] = 0;	/* offsm */
   spnt[2].trans  [2] = 5;
   spnt[2].trate  [2] = rt;
   spnt[2].ratemul[2] = 1.0 * c;
   spnt[2].rateo  [2] = 1;	/* offsh */

   spnt[3].numtrans   = 2;
   spnt[3].cond       = 1.0;
   spnt[3].trans  [0] = 6;			/* state 3 = the open state */
   spnt[3].trate  [0] = rt;
   spnt[3].ratemul[0] = 1.0 * c;
   spnt[3].rateo  [0] = 1;	/* offsh */
   spnt[3].trans  [1] = 2;
   spnt[3].trate  [1] = rt;
   spnt[3].ratemul[1] = 1.0 * b;
   spnt[3].rateo  [1] = 0;	/* offsm */

   spnt[4].numtrans   = 2;
   spnt[4].cond       = 0;
   spnt[4].trans  [0] = 5;
   spnt[4].trate  [0] = rnt;
   spnt[4].ratemul[0] = 1.0 * a;	
   spnt[4].rateo  [0] = 0;
   spnt[4].trans  [1] = 1;
   spnt[4].trate  [1] = rt;
   spnt[4].ratemul[1] = 1.0 * d;
   spnt[4].rateo  [1] = 1;

   spnt[5].numtrans   = 3;
   spnt[5].cond       = 0;
   spnt[5].trans  [0] = 6;			/* state 5 = inactivated state*/
   spnt[5].trate  [0] = rt;
   spnt[5].ratemul[0] = 1.0 * a;
   spnt[5].rateo  [0] = 1;	/* offsh */
   spnt[5].trans  [1] = 2;
   spnt[5].trate  [1] = rt;
   spnt[5].ratemul[1] = 1.0 * d;
   spnt[5].rateo  [1] = 1;	/* offsh */
   spnt[5].trans  [2] = 4;
   spnt[5].trate  [2] = rt;
   spnt[5].ratemul[2] = 1.0 * b;
   spnt[5].rateo  [2] = 0;	/* offsm */

   spnt[6].numtrans   = 2;
   spnt[6].cond       = 0;
   spnt[6].trans  [0] = 3;			/* state 6 = inactivated state*/
   spnt[6].trate  [0] = rt;
   spnt[6].ratemul[0] = 1.0 * d;
   spnt[6].rateo  [0] = 1;	/* offsh */
   spnt[6].trans  [1] = 5;
   spnt[6].trate  [1] = rt;
   spnt[6].ratemul[1] = 1.0 * b;
   spnt[6].rateo  [1] = 0;	/* offsm */

   return ch;

}

