/* segment chank4 in program nc */
/* Ih, the hyperpolarization-activated inward rectifier */

/* sets up channel parameters */

#include <stdio.h>
#include "nc.h"
#include "y.tab.h"
#include "ncsub.h"
#include "ncomp.h"
#include "control.h"

chantype *makchantype(int ctype, int cnum, int nstates, int nparm, double bt);
double calcchaninf(double v, chanparm *chp);
double calcchantau(double v, double tau, chanparm *chp);

double al0 (chan *cpnt);
double bet0(chan *cpnt);

static double (*aln) (chan*) = al0;
static double (*betn)(chan*) = bet0;

#ifdef __cplusplus
extern "C" {
#endif

double exp(double);

#ifdef __cplusplus
}
#endif

double ncabs(double x);

/*----------------------------------------*/

double calck4n (double v, int func)

/* Calculate K4 rate functions given voltage in mv.
/* for Ih, the hyperpolarization-activated inward rectifier */

/* Approximately matches behavior of channels in 

Hestrin, S. (1987)  J.  Physiol.  390, 319-333.
Maricq AV, and Korenbrot JI, (1990) J. Neurophysiol. 64:1917-28

*/

/* See tcomp31d for a test of this channel */

/* Calc rate in terms of 1/sec. 

   The "func" parameter defines:

    1	alpha n 
    2	beta  n 
*/

#define NUMKFUNC 8

{
   double val,x,y;

  if (func > NUMKFUNC) func = NUMKFUNC;
  switch (func) {
						/* alpha functions */

  case 1:					/* alpha n */
    y = -0.3 * (-v-70.); 
    x = exp (y) - 1.;
    if (ncabs(x) > 1e-5)                        /* singularity at v = -55 mv */
          val =  y / x;
    else  val = 1.0;
    val *= 0.3;
    break;
						/* beta n */
  case 2:               
    val = 0.15 * exp ((-v-50) / -30.);

    break;
  }
  return val; 
}

/*----------------------------------------*/

chantype *makk4(void)

/* The inward rectifier in rod and cone inner segments: Ih */

{
     chantype *ch;
     chanstate *spnt;
     chanparm *parm;
     double n;
     int nstate, nparm;

   nstate = 3;                          /* 5 Markov states */
   nparm = 1;                           /* make 1 set of params, "n" */
   ch=makchantype(K,4,nstate,nparm,BASETC); /* make chan state const, parm */
   ch->unitary = dkihu;
   ch->perm[PK]  = 1.0;			/* permeability to K+ ions */
   ch->perm[PNA] = 0.48;		/* permeability to Na+ ions */
   ch->perm[PCA] = dpcak;		/* permeability to Ca++ ions */
   ch->vrev = (vk+vna)/2;		/* default reversal potential */
   spnt = ch->state;
   parm = ch->parm;

   parm[0].nfval    = 2;                /* number of func vals (am, bm, etc.) */
   parm[0].nival    = 0;                /* number of implicit vals (m1, m2) */
   parm[0].chancalc =  calck4n;         /* default rate function */
   parm[0].funcname = "calck4n";        /* user rate function */
   parm[0].dq = dqn;                    /* Q10 for n rate function */

   n = 1.0;
   spnt[0].numtrans   = 1;
   spnt[0].cond       = 0;
   spnt[0].trans  [0] = 1;
   spnt[0].trate  [0] = aln;
   spnt[0].ratemul[0] = 1.0*n;
   spnt[0].rateo  [0] = 0;
                                     /*   0 <-> 1 <-> 2   */
   spnt[1].numtrans   = 2;
   spnt[1].cond       = 0;
   spnt[1].trans  [0] = 2;	  	
   spnt[1].trate  [0] = aln;
   spnt[1].ratemul[0] = 0.5*n;
   spnt[1].rateo  [0] = 0;
   spnt[1].trans  [1] = 0;
   spnt[1].trate  [1] = betn;
   spnt[1].ratemul[1] = 2.5*n;
   spnt[1].rateo  [1] = 1;

   spnt[2].numtrans   = 1;            /* state 2 = the open state */
   spnt[2].cond       = 1.0;
   spnt[2].trans  [0] = 1;
   spnt[2].trate  [0] = betn;
   spnt[2].ratemul[0] = 2.5*n;
   spnt[2].rateo  [0] = 1;

   return ch;
}

/*----------------------------------------*/
