/* segment chankca in program nc */

/* sets up KCa channel parameters */

#include <stdio.h>
#include "nc.h"
#include "y.tab.h"
#include "ncsub.h"
#include "ncomp.h"
#include "control.h"

chantype *makchantype(int ctype, int cnum, int nstates, int nparm, double bt);
void makchanimpl (chanparm *chp);
void mak2state(chanstate *spnt, double (*frate)(chan *cpnt),
                                double (*rrate)(chan *cpnt), double rate);
double rchanf(chan *cpnt);
double rchanr(chan *cpnt);

#ifdef __cplusplus
extern "C" {
#endif

double exp(double);
double log(double);

#ifdef __cplusplus
}
#endif

double ncabs(double x);
double qrate(chanparm *chp);
double rca(chan *cpnt);
double rt(chan *cpnt);

/*----------------------------------------*/

double calckca1 (double v, int func)

{
  return 1.0;
}

/*----------------------------------------*/

double calckca2 (double v, int func)

{
  return 1.0;
}

/*----------------------------------------*/

double akcacalc(double v, double ca, double rate, double d1, double k1)

/* Alpha for calcium-activated potassium channel,
      from Hines, 1989 and Moczydlowski and Latorre (1983).

   Given Vm in volts, return rate.
   (Original given units of mV.)

   Does not include effect of timestep and temperature.
*/

{
   double xalpha,r,vm;

    vm = v * MVOLT;
    if (ca==0.0) ca = 1e-9;
    r = rate;
    xalpha = r / (1.0 + k1/ca * exp (d1* -vm/10.));
//fprintf (stderr,"%g %g %g %g %g %g\n",v,ca,rate,d1,k1,xalpha);

    return xalpha;
}

/*----------------------------------------*/

double bkcacalc(double v, double ca, double rate, double d2, double k2)

/* Beta for calcium-activated potassium channel,
      from Hines, 1989 and Moczydlowski and Latorre (1983).

   Given Vm in volts, return rate.
   (Original given units of mV.)

   Does not include effect of timestep and temperature.
*/

{
   double xbeta,r,vm;

    vm = v * MVOLT;
    if (ca==0.0) ca = 1e-8;
    r = rate;
    xbeta = r / (1.0 + ca / (k2 * exp (d2* -vm/10.)));
    return xbeta;
}

/*----------------------------------------*/

void kcatab(double v, double ca, int typ, double nrate, 
		double d1, double d2, double k1, double k2, chanparm *chp)
{
     double a,b,ab,ab2,tval;
     double kca1, kca2, qkca;
     static int parmmade=0;

    if (! parmmade) {
	makchanimpl(chp);			/* make dummy tables */
	parmmade = 1;				/* to store kca1,kca2 */
    }
    if (nrate==0) nrate = 1e-4;
    qkca = qrate(chp);				/* calculate rate factor */
    a = qkca * akcacalc(v,ca,nrate,d1,k1);
    b = qkca * bkcacalc(v,ca,nrate,d2,k2);
    ab  = a+b;
    ab2 = ab*0.5;

    tval = (1.0 - ab2) /
    	   (1.0 + ab2); 

    if (tval>0) {
      kca1 = tval;				/* C-N implicit */
      kca2 = a / (1.0+ab2); 
    }
    else {					/* purely implicit */
      kca1 = 1.0 / (1.0+ab);
      kca2 = a   / (1.0+ab); 
    }
    chp->ival[0] = kca1;                        /* save values for impl calc */
    chp->ival[1] = kca2;
}

/*----------------------------------------*/

double alnca(chan *cpnt)
{
     double v;
     kcachan *kcapnts;
     cacomp *capnt;
     double ca,qkca;

  if (!cpnt) return 0.0;
  v = cpnt->comp1->v - cpnt->voffsm;
  kcapnts = (kcachan*)cpnt;
  if (!(capnt=cpnt->comp1->capnt)) {
        ca = dcai;
  }
  else ca = capnt->cais[0];
  qkca = qrate(cpnt->chtyp->parm);		/* calculate rate factor */
  return (qkca*akcacalc(v,ca,kcapnts->arate,kcapnts->d1,kcapnts->k1)); 
}

/*----------------------------------------*/

double betnca(chan *cpnt)
{
     double v;
     kcachan *kcapnts;
     cacomp *capnt;
     double ca,qkca;

  if (!cpnt) return 0.0;
  v = cpnt->comp1->v - cpnt->voffsm;
  kcapnts = (kcachan*)cpnt;
  if (!(capnt=cpnt->comp1->capnt)) {
         ca = dcai;
  }
  else ca = capnt->cais[0];
  qkca = qrate(cpnt->chtyp->parm);		/* calculate rate factor */
  return (qkca*bkcacalc(v,ca,kcapnts->arate,kcapnts->d2,kcapnts->k2));
}

/*----------------------------------------*/

double alnkca6(chan *cpnt)

/* Like alnca, but no Ca sensitivity */

/*  Use for rate func for g (to open state). */

/* According to Horrigan et al (1999), alpha and beta are same,
     but d and g differ by factor of 2.
*/

{
     double v;
     kcachan *kcapnts;
     cacomp *capnt;
     double qkca;

#define GF 0.7

  if (!cpnt) return 0.0;
  v = cpnt->comp1->v - cpnt->voffsm;
  kcapnts = (kcachan*)cpnt;
  qkca = qrate(cpnt->chtyp->parm);		/* calculate rate factor */
  return (qkca*akcacalc(v,1.0,kcapnts->arate,kcapnts->d1*GF,1.0)); 
}

/*----------------------------------------*/

double betnkca6(chan *cpnt)

/* Like betnca, but no Ca sensitivity */

/*  Use for rate func for d (to closed state). */

{
     double v;
     kcachan *kcapnts;
     cacomp *capnt;
     double qkca;

#define DF 0.4

  if (!cpnt) return 0.0;
  v = cpnt->comp1->v - cpnt->voffsm;
  kcapnts = (kcachan*)cpnt;
  qkca = qrate(cpnt->chtyp->parm);		/* calculate rate factor */
  return (qkca*bkcacalc(v,1.0,kcapnts->arate,kcapnts->d2*DF,1.0));
}

/*----------------------------------------*/

chantype *makkca0(void)

/* Kca channel, type 0 */
/*  SK type channel */

{
     chantype *ch;
     chanstate *spnt;
     chanparm *parm;
     double n;
     int nstate, nparm;

   nstate = 2;                          /* 2 Markov states */
   nparm = 1;                           /* make 1 set of params, "n" */
   ch=makchantype(KCa,0,nstate,nparm,BASETC);  /* make chan state const, parm */
   ch->hh = 1;				/* HH chan - uses "m" */
   ch->perm[PK]  = 1.0;			/* permeability to K+ ions */
   ch->perm[PNA] = dpnak*.5;		/* permeability to Na+ ions / perm K */
   ch->perm[PCA] = dpcak*.5;		/* permeability to Ca++ ions /perm K */
   ch->unitary = dkcasu;		/* 8.4 pS @6.3 makes 22 ps @ 35 deg */
   ch->vrev = vk;			/* default reversal potential */
   spnt = ch->state;
   parm = ch->parm;

   parm[0].chancalc =  calckca1;	/* user function */
   parm[0].funcname = "calckca1";	/* default rate func */
   parm[0].nfval    = 0; 
   parm[0].nival    = 2;                /* number of implicit vals (m1, m2) */
   parm[0].dq = dqkca;                  /* Q10 for n rate function */
   parm[0].voff = dkoffsn;              /* voltage offset from user */

   mak2state(spnt,rchanf,rchanr,100);	
					/* Rate set to approximate openings */
					/*  of sKCa channel in Wang, Robinson */
					/*  and Chalupa, (1998). */
   return ch;
}

/*----------------------------------------*/

chantype *makkca1(void)

/* Sequential-state sKCa channel, like K type 0 */
/*  SK type channel */

{
     chantype *ch;
     chanstate *spnt;
     chanparm *parm;
     double n;
     int nstate, nparm;

   nstate = 2;                          /* 2 Markov states */
   nparm = 1;                           /* make 1 set of params, "n" */
   ch=makchantype(KCa,1,nstate,nparm,BASETC);  /* make chan state const, parm */
   ch->perm[PK]  = 1.0;			/* permeability to K+ ions */
   ch->perm[PNA] = dpnak*.5;		/* permeability to Na+ ions / perm K */
   ch->perm[PCA] = dpcak*.5;		/* permeability to Ca++ ions /perm K */
   ch->unitary = dkcasu;		/* 8.4 pS @6.3 makes 22 ps @ 35 deg */
   ch->vrev = vk;			/* default reversal potential */
   spnt = ch->state;
   parm = ch->parm;

   parm[0].chancalc =  calckca1;        /* dummy rate func */
   parm[0].funcname = "calckca1";       /* dummy user func */
   parm[0].nfval    = 0;                /* number of func vals (am, bm, etc.) */
   parm[0].nival    = 0;                /* number of implicit vals (m1, m2) */
   parm[0].dq = dqkca;                  /* Q10 for n rate function */
   parm[0].voff = dkoffsn;              /* voltage offset from user */

   mak2state(spnt,alnca,betnca,100);      /* 2 state model, Ca and V dependent */

   return ch;
}

/*--------------------------------------------*/

chantype *makkca2(void)

/* bKCa channel, type 2 */ 
/*   Like type 0, except BK type channel */

/* Q10 for BKCa cond =~ 1.4, Barret, Magleby and Pallotta, 1982,
                             J. Physiol. 331: 211-230
*/

{
     chantype *ch;
     chanstate *spnt;
     chanparm *parm;
     double n;
     int nstate, nparm;

   nstate = 2;                          /* 2 Markov states */
   nparm = 1;                           /* make 1 set of params, "n" */
   ch=makchantype(KCa,2,nstate,nparm,BASETC);  /* make chan state const, parm */
   ch->hh = 1;				/* HH chan - uses "m" */
   ch->perm[PK]  = 1.0;			/* permeability to K+ ions */
   ch->perm[PNA] = dpnak*.5;		/* permeability to Na+ ions / perm K */
   ch->perm[PCA] = dpcak*.5;		/* permeability to Ca++ ions /perm K */
   ch->unitary = dkcabu;		/* 44 pS @6.3 makes 115 ps @ 22 deg */
   ch->vrev = vk;			/* default reversal potential */
   spnt = ch->state;
   parm = ch->parm;

   parm[0].chancalc =  calckca2;	/* user function */
   parm[0].funcname = "calckca2";	/* default rate func */
   parm[0].nfval    = 0; 
   parm[0].nival    = 2;                /* number of implicit vals (m1, m2) */
   parm[0].dq = dqkca;                  /* Q10 for n rate function */
   parm[0].voff = dkoffsn;              /* voltage offset from user */

   mak2state(spnt,rchanf,rchanr,10);

   return ch;
}

/*----------------------------------------*/

chantype *makkca3(void)

/* Sequential-state bKCa channel, like KCa type 2 */
/*  BK type channel */

{
     chantype *ch;
     chanstate *spnt;
     chanparm *parm;
     double n;
     int nstate, nparm;

   nstate = 2;                          /* 2 Markov states */
   nparm = 1;                           /* make 1 set of params, "n" */
   ch=makchantype(KCa,3,nstate,nparm,BASETC);  /* make chan state const, parm */
   ch->unitary = dkcabu;		/* 44 pS @6.3 makes 115 ps @ 35 deg */
   					/* 68 pS @6.3 makes 115 ps @ 22 deg */
   ch->perm[PK]  = 1.0;			/* permeability to K+ ions */
   ch->perm[PNA] = dpnak*.5;		/* permeability to Na+ ions / perm K */
   ch->perm[PCA] = dpcak*.5;		/* permeability to Ca++ ions /perm K */
   ch->vrev = vk;			/* default reversal potential */
   spnt = ch->state;
   parm = ch->parm;

   parm[0].chancalc =  calckca2;        /* dummy rate func */
   parm[0].funcname = "calckca2";       /* dummy user func */
   parm[0].nfval    = 0;                /* number of func vals (am, bm, etc.) */
   parm[0].nival    = 0;                /* number of implicit vals (m1, m2) */
   parm[0].dq = dqkca;                  /* Q10 for n rate function */
   parm[0].voff = dkoffsn;              /* voltage offset from user */

   mak2state(spnt,alnca,betnca,10);     /* 2 state model, Ca and V dependent */

   return ch;
}

/*--------------------------------------------*/

chantype *makkca4(void)

/* Sequential-state sKCa channel, 

   from:  Hirshberg et al., (1998) J. Gen. Physiol. 111: 565-581

  in which a 6-state markov diagram is provided to describe the
cloned rSK2 channel. Two sets of rate constants were provided, one for
normal high-probability gating, and another for simulating
"low-probability" gating behavior (not used here).  This channel
has no gating voltage sensitivity.

        f1      f2        f3
    0  <->  1  <->   2   <->    3
        r1      r2        r3

                       ^           ^
                 f4  | r4   f5  |  r5
                 \/         \/           

                     4          5

State 0 is unbound
      1 is bound with 1 Ca++
      2 is bound with 2 Ca++
      3 is bound with 3 Ca++
      4 is open, short duration
      5 is open, long duration

From Hirshberg et al (1998): Single channel currents were
recorded from Xenopus oocytes expressing the apamin-sensitive
clone rSK2. Channel activity was detectable in 0.2 uM [Ca]i and
was maximal above 2 uM [Ca]i.  Analysis of stationary currents
revealed two open times and three closed times, with only the
longest closed time being ca dependent, decreasing with
increasing [Ca]i concentrations.  In addition, elevated [Ca]i
concentrations resulted in a larger percentage of long openings
and short closures.  Membrane voltage did not have significant
effects on either open or closed times.  The open probability was
~0.6 in 1 uM free [Ca].

*/

{
     chantype *ch;
     chanstate *spnt;
     chanparm *parm;
     double n;
     int nstate, nparm;

#define ca1f1 200e6
#define ca1r1 80.
#define ca1f2 160e6
#define ca1r2 80.0
#define ca1f3 80e6
#define ca1r3 200.0
#define ca1f4 160.0
#define ca1r4 1000.0
#define ca1f5 1200.0
#define ca1r5 100.0

   nstate = 6;                          /* 2 Markov states */
   nparm = 1;                           /* make 1 set of params */
   ch=makchantype(KCa,4,nstate,nparm,BASETCA);  /* make chan state const, parm*/
   ch->unitary = dkcasu;		/* 8.4 pS @6.3 makes 22 ps @ 35 deg */
   ch->perm[PK]  = 1.0;			/* permeability to K+ ions */
   ch->perm[PNA] = dpnak*.5;		/* permeability to Na+ ions / perm K */
   ch->perm[PCA] = dpcak*.5;		/* permeability to Ca++ ions /perm K */
   ch->vrev = vk;			/* default reversal potential */
   spnt = ch->state;
   parm = ch->parm;

   parm[0].chancalc =  calckca2;        /* dummy rate func */
   parm[0].funcname = "calckca2";       /* dummy user func */
   parm[0].nfval    = 0;                /* number of func vals (am, bm, etc.) */
   parm[0].nival    = 0;                /* number of implicit vals (m1, m2) */
   parm[0].dq = dqkca;                  /* Q10 for n rate function */
   parm[0].voff = dkoffsn;              /* voltage offset from user */

   spnt[0].numtrans   = 1;
   spnt[0].cond       = 0;
   spnt[0].trans  [0] = 1;
   spnt[0].trate  [0] = rca;
   spnt[0].ratemul[0] = ca1f1;
   spnt[0].rateo  [0] = 0;      /* arate */

   spnt[1].numtrans   = 2;
   spnt[1].cond       = 0;
   spnt[1].trans  [0] = 2;
   spnt[1].trate  [0] = rca;
   spnt[1].ratemul[0] = ca1f2;
   spnt[1].rateo  [0] = 0;	/* arate */
   spnt[1].trans  [1] = 0;
   spnt[1].trate  [1] = rt;
   spnt[1].ratemul[1] = ca1r1;
   spnt[1].rateo  [1] = 1;	/* brate */

   spnt[2].numtrans   = 3;
   spnt[2].cond       = 0;
   spnt[2].trans  [0] = 3;
   spnt[2].trate  [0] = rca;             /* function of ca conc, time */
   spnt[2].ratemul[0] = ca1f3;
   spnt[2].rateo  [0] = 0;      /* arate */
   spnt[2].trans  [1] = 1;
   spnt[2].trate  [1] = rt;             /* function of time */
   spnt[2].ratemul[1] = ca1r2;
   spnt[2].rateo  [1] = 1;      /* brate */
   spnt[2].trans  [2] = 4;
   spnt[2].trate  [2] = rt;
   spnt[2].ratemul[2] = ca1f4;
   spnt[2].rateo  [2] = 2;      /* crate */
 
   spnt[3].numtrans   = 2;
   spnt[3].cond       = 0;
   spnt[3].trans  [0] = 5;
   spnt[3].trate  [0] = rt;             /* function of time */
   spnt[3].ratemul[0] = ca1f5;
   spnt[3].rateo  [0] = 2;      /* crate */
   spnt[3].trans  [1] = 2;
   spnt[3].trate  [1] = rt;             /* function of time */
   spnt[3].ratemul[1] = ca1r3;
   spnt[3].rateo  [1] = 1;      /* brate */
 
   spnt[4].numtrans   = 1;
   spnt[4].cond       = 1.0;
   spnt[4].trans  [0] = 2;
   spnt[4].trate  [0] = rt;             /* function of time */
   spnt[4].ratemul[0] = ca1r4;
   spnt[4].rateo  [0] = 3;      /* drate */
 
   spnt[5].numtrans   = 1;
   spnt[5].cond       = 1.0;
   spnt[5].trans  [0] = 3;
   spnt[5].trate  [0] = rt;             /* function of time */
   spnt[5].ratemul[0] = ca1r5;
   spnt[5].rateo  [0] = 3;      /* drate */

   return ch;
}

/*--------------------------------------------*/

chantype *makkca5(void)

/* Sequential-state sKCa channel, 

   from:  Sah and Clements (1999) J. Neurosci. 19: 3657-3664 

  in which a 6-state markov diagram is provided.  This is the
channel that underlies the "slow afterhyperpolarization" in
hippocampal pyramidal neurons. Its identity is unknown at
present, but it is apamin-insensitive and has no gating voltage
sensitivity.

        4rb     3rb       2rb       rb      ro 
    0  <->  1  <->   2   <->    3  <->  4  <->  5
        ru      2ru       3ru       4ru     rc

State 0 is unbound
      1 is bound with 1 Ca++
      2 is bound with 2 Ca++
      3 is bound with 3 Ca++
      4 is bound with 4 Ca++
      5 is open, flickering

From Sah and Clements, (1999): "The rates were chosen based on
the following three assumptions concerning the Ca-activated K
channels: 1) the steady-state dose-response curve has a steep
activation above the resting [Ca]i of 50 nM and has an EC50 of
150 nM, so that it is efficiently activated by small increases in
[Ca]i;  2) when [Ca]i falls rapidly, the decay of sIAHP is
limited by the channel closing and Ca dissociation rates to give
a time constant of ~1.5 sec; and 3) the peak open probability of
the channel is ~0.6, and its mean open time is 2.5 msec based on
estimates from noise analysis of sIAHP (Sah and Isaacson, 1995).

*/

{
     chantype *ch;
     chanstate *spnt;
     chanparm *parm;
     double n;
     int nstate, nparm;

#define ca5rb 10e6 
#define ca5ru 0.5
#define ca5ro 600.0
#define ca5rc 400.0

   nstate = 6;                          /* 2 Markov states */
   nparm = 1;                           /* make 1 set of params */
   ch=makchantype(KCa,5,nstate,nparm,BASETCA);  /* make chan state const, parm*/
   ch->unitary = dkcasu;		/* 8.4 pS @6.3 makes 22 ps @ 35 deg */
   ch->perm[PK]  = 1.0;			/* permeability to K+ ions */
   ch->perm[PNA] = dpnak*.5;		/* permeability to Na+ ions / perm K */
   ch->perm[PCA] = dpcak*.5;		/* permeability to Ca++ ions /perm K */
   ch->vrev = vk;			/* default reversal potential */
   spnt = ch->state;
   parm = ch->parm;

   parm[0].chancalc =  calckca2;	/* dummy rate func */
   parm[0].funcname = "calckca2";	/* dummy user func */
   parm[0].nfval    = 0;		/* number of func vals (am, bm, etc.) */
   parm[0].nival    = 0;		/* number of implicit vals (m1, m2) */
   parm[0].dq       = dqkca;		/* Q10 for n rate function */
   parm[0].voff     = dkoffsn;		/* voltage offset from user */

   spnt[0].numtrans   = 1;
   spnt[0].cond       = 0;
   spnt[0].trans  [0] = 1;
   spnt[0].trate  [0] = rca;
   spnt[0].ratemul[0] = ca5rb * 4.0;
   spnt[0].rateo  [0] = 0;      /* arate */

   spnt[1].numtrans   = 2;
   spnt[1].cond       = 0;
   spnt[1].trans  [0] = 2;
   spnt[1].trate  [0] = rca;
   spnt[1].ratemul[0] = ca5rb * 3.0;
   spnt[1].rateo  [0] = 0;	/* arate */
   spnt[1].trans  [1] = 0;
   spnt[1].trate  [1] = rt;
   spnt[1].ratemul[1] = ca5ru;
   spnt[1].rateo  [1] = 1;	/* brate */

   spnt[2].numtrans   = 2;
   spnt[2].cond       = 0;
   spnt[2].trans  [0] = 3;
   spnt[2].trate  [0] = rca;
   spnt[2].ratemul[0] = ca5rb * 2.0;
   spnt[2].rateo  [0] = 0;	/* arate */
   spnt[2].trans  [1] = 1;
   spnt[2].trate  [1] = rt;
   spnt[2].ratemul[1] = ca5ru * 2.0;
   spnt[2].rateo  [1] = 1;	/* brate */

   spnt[3].numtrans   = 2;
   spnt[3].cond       = 0;
   spnt[3].trans  [0] = 4;
   spnt[3].trate  [0] = rca;
   spnt[3].ratemul[0] = ca5rb;
   spnt[3].rateo  [0] = 0;	/* arate */
   spnt[3].trans  [1] = 2;
   spnt[3].trate  [1] = rt;
   spnt[3].ratemul[1] = ca5ru * 3.0;
   spnt[3].rateo  [1] = 1;	/* brate */

   spnt[4].numtrans   = 2;
   spnt[4].cond       = 0;
   spnt[4].trans  [0] = 5;
   spnt[4].trate  [0] = rt;
   spnt[4].ratemul[0] = ca5ro;
   spnt[4].rateo  [0] = 3;	/* arate */
   spnt[4].trans  [1] = 3;
   spnt[4].trate  [1] = rt;
   spnt[4].ratemul[1] = ca5ru * 4.0;
   spnt[4].rateo  [1] = 1;	/* drate */

   spnt[5].numtrans   = 1;
   spnt[5].cond       = 1.0;	/* the open state */
   spnt[5].trans  [0] = 4;
   spnt[5].trate  [0] = rt;             /* function of time */
   spnt[5].ratemul[0] = ca5rc;
   spnt[5].rateo  [0] = 3;      /* drate */

   return ch;
}

/*--------------------------------------------*/

chantype *makkca6(void)

/* Sequential-state bKCa channel, 

   from:  Horrigan, FT, Cui, J, and Aldrich, RW (1999)
   J. Gen. Physiol. 114: 277-304. 

  in which a 10-state markov diagram is provided.

        4a      3a        2a        a
    0  <->   1  <->   2   <->  3   <->  4 
        b       2b        3b        4b

      ^        ^        ^        ^        ^
 go | do  g1 | d1  g2 | d2  g3 | d3  g4 | d4
 \/       \/       \/       \/       \/  

       4af      3af      2af       af 
    5  <->   6  <->   7  <->   8  <->   9
       b/f      2b/f     3b/f     4b/f


State 0 is unbound
      1 is bound with 1 Ca++
      2 is bound with 2 Ca++
      3 is bound with 3 Ca++
      4 is bound with 4 Ca++
      5 is open, short duration
      6 is open, short duration
      7 is open, short duration
      8 is open, long duration
      9 is open, long duration

Original Horrigan rate constants (20 deg C):

     sec-1

a  = 1276  / [Ca] (uM)
b  = 35370 / [Ca] (um)
d0 =  0.0074
d1 =  0.126
d2 =  2.14
d3 = 25.7
d4 = 49.3
g0 = 3700
g1 = 3700
g2 = 3700
g3 = 2612
g4 =  295

za =  0.275
zb = -0.275
zd =  0.262
zg = -0.138
f  = sqrt(17) = 4.123
d0/g0 = 2e-6

We modify the Horrigan model by adding Ca sensitivity
from states 0 -> 4, taken from the 8-state model of McManus and
Magleby (1991).

The rate functions for voltage sensitivity are slightly modified
from the "alnca()" and "betnca()" functions above. The idea is
that the voltage sensitivity of the vertical transitions is less
than for the [Ca]-dependent transitions. 

This channel is untested. The states are set up, and the rate
functions might work OK, but the rate multipliers are probably
not correct.  

The reason is that the rate constants from Horrigan et al. (1999)
are for voltage sensitivity, and the rate constants from McManus
and Magleby are for [Ca] sensitivity.  The Markov states from
both studies look similar but they are not the same and cannot
easily be incorporated into one model. The system is very complex
and the kinetics are not well understood (as of 4/2000).  

For example, Cui et al (1997) tried (as an exercise) to add
voltage sensitivity to the vertical transitions of McManus and
Magleby's model, and found that they could get it to match at
high and low [Ca] but at medium [Ca] it didn't match.

It appears that more states will be necessary to fully represent
the effect of both [Ca] and voltage.  It is already known that
[Ca] has a bigger effect on kinetics, and but to incorporate the
voltage sensitivity, one needs more than the 8 - 10 state model
that works for [Ca] alone.

*/

{
     chantype *ch;
     chanstate *spnt;
     chanparm *parm;
     double n;
     int nstate, nparm;

#define ca6a  1276.0
#define ca6b  35370.0
#define ca6d0 0.0074
#define ca6d1 0.126
#define ca6d2 2.14
#define ca6d3 25.7
#define ca6d4 49.3
#define ca6g0 3700.0
#define ca6g1 3700.0
#define ca6g2 3700.0
#define ca6g3 2612.0
#define ca6g4 295.0
#define ca6f  4.123
#define kf    1.0
#define kr    1.0

   nstate = 10;                          /* 2 Markov states */
   nparm = 1;                           /* make 1 set of params */
   ch=makchantype(KCa,6,nstate,nparm,BASETCA);  /* make chan state const, parm*/
   ch->unitary = dkcabu;		/* 44 pS @6.3 makes 115 ps @ 35 deg */
   					/* 68 pS @6.3 makes 115 ps @ 22 deg */
   ch->perm[PK]  = 1.0;			/* permeability to K+ ions */
   ch->perm[PNA] = dpnak*.5;		/* permeability to Na+ ions / perm K */
   ch->perm[PCA] = dpcak*.5;		/* permeability to Ca++ ions /perm K */
   ch->vrev = vk;			/* default reversal potential */
   spnt = ch->state;
   parm = ch->parm;

   parm[0].chancalc =  calckca2;        /* dummy rate func */
   parm[0].funcname = "calckca2";       /* dummy user func */
   parm[0].nfval    = 0;                /* number of func vals (am, bm, etc.) */
   parm[0].nival    = 0;                /* number of implicit vals (m1, m2) */
   parm[0].dq = dqkca;                  /* Q10 for n rate function */
   parm[0].voff = dkoffsn;              /* voltage offset from user */

   spnt[0].numtrans   = 2;
   spnt[0].cond       = 0;
   spnt[0].trans  [0] = 1;
   spnt[0].trate  [0] = alnca;
   spnt[0].ratemul[0] = ca6a*4.0*kf;	/* rate func multiplier */
   spnt[0].rateo  [0] = 0;
   spnt[0].trans  [1] = 5;
   spnt[0].trate  [1] = alnkca6;
   spnt[0].ratemul[1] = ca6g0;
   spnt[0].rateo  [1] = 2;      /* crate */

   spnt[1].numtrans   = 3;
   spnt[1].cond       = 0;
   spnt[1].trans  [0] = 2;
   spnt[1].trate  [0] = alnca;
   spnt[1].ratemul[0] = ca6a*3.0*kf;
   spnt[1].rateo  [0] = 0;
   spnt[1].trans  [1] = 0;
   spnt[1].trate  [1] = betnca;
   spnt[1].ratemul[1] = ca6b;
   spnt[1].rateo  [1] = 1;
   spnt[1].trans  [2] = 6;
   spnt[1].trate  [2] = alnkca6;
   spnt[1].ratemul[2] = ca6g1;
   spnt[1].rateo  [2] = 2;

   spnt[2].numtrans   = 3;
   spnt[2].cond       = 0;
   spnt[2].trans  [0] = 3;
   spnt[2].trate  [0] = alnca;             /* function of ca, v */
   spnt[2].ratemul[0] = ca6a*2.0*kf;
   spnt[2].rateo  [0] = 0;      /* arate */
   spnt[2].trans  [1] = 1;
   spnt[2].trate  [1] = betnca;             /* function of v */
   spnt[2].ratemul[1] = ca6b*2.0;
   spnt[2].rateo  [1] = 1;      /* brate */
   spnt[2].trans  [2] = 7;
   spnt[2].trate  [2] = alnkca6;
   spnt[2].ratemul[2] = ca6g2;
   spnt[2].rateo  [2] = 2;      /* drate */
 
   spnt[3].numtrans   = 3;
   spnt[3].cond       = 0;
   spnt[3].trans  [0] = 4;
   spnt[3].trate  [0] = alnca;             /* function of ca, v */
   spnt[3].ratemul[0] = ca6a*kf;
   spnt[3].rateo  [0] = 0;      /* arate */
   spnt[3].trans  [1] = 2;
   spnt[3].trate  [1] = betnca;             /* function of time */
   spnt[3].ratemul[1] = ca6b*3.0;
   spnt[3].rateo  [1] = 1;      /* brate */
   spnt[3].trans  [2] = 8;
   spnt[3].trate  [2] = alnkca6;             /* function of v */
   spnt[3].ratemul[2] = ca6g3;
   spnt[3].rateo  [2] = 2;      /* drate */
 
   spnt[4].numtrans   = 2;
   spnt[4].cond       = 0;
   spnt[4].trans  [0] = 3;
   spnt[4].trate  [0] = betnca;             /* function of ca, v */
   spnt[4].ratemul[0] = ca6b*4.0;
   spnt[4].rateo  [0] = 1;      /* brate */
   spnt[4].trans  [1] = 9;
   spnt[4].trate  [1] = alnkca6;             /* function of v */
   spnt[4].ratemul[1] = ca6g4;
   spnt[4].rateo  [1] = 2;      /* drate */
 
   spnt[5].numtrans   = 2;
   spnt[5].cond       = 1.0;
   spnt[5].trans  [0] = 6;
   spnt[5].trate  [0] = alnca;
   spnt[5].ratemul[0] = ca6a*ca6f*4.0*kf;
   spnt[5].rateo  [0] = 0;      /* offsm */
   spnt[5].trans  [1] = 0;
   spnt[5].trate  [1] = betnkca6;
   spnt[5].ratemul[1] = ca6d0;
   spnt[5].rateo  [1] = 3;      /* drate */

   spnt[6].numtrans   = 3;
   spnt[6].cond       = 1.0;
   spnt[6].trans  [0] = 7;
   spnt[6].trate  [0] = alnca;
   spnt[6].ratemul[0] = ca6a*ca6f*3.0*kf;
   spnt[6].rateo  [0] = 0;
   spnt[6].trans  [1] = 5;
   spnt[6].trate  [1] = betnca;
   spnt[6].ratemul[1] = ca6b/ca6f;
   spnt[6].rateo  [1] = 1;
   spnt[6].trans  [2] = 1;
   spnt[6].trate  [2] = betnkca6;
   spnt[6].ratemul[2] = ca6d1;
   spnt[6].rateo  [2] = 3;

   spnt[7].numtrans   = 3;
   spnt[7].cond       = 1.0;
   spnt[7].trans  [0] = 8;
   spnt[7].trate  [0] = alnca;             /* function of ca, v */
   spnt[7].ratemul[0] = ca6a*ca6f*2.0*kf;
   spnt[7].rateo  [0] = 0;      /* offsm */
   spnt[7].trans  [1] = 6;
   spnt[7].trate  [1] = betnca;             /* function of v */
   spnt[7].ratemul[1] = ca6b/ca6f/2.0;
   spnt[7].rateo  [1] = 1;      /* offsm */
   spnt[7].trans  [2] = 2;
   spnt[7].trate  [2] = betnkca6;
   spnt[7].ratemul[2] = ca6d2;
   spnt[7].rateo  [2] = 3;      /* drate */
 
   spnt[8].numtrans   = 3;
   spnt[8].cond       = 1.0;
   spnt[8].trans  [0] = 9;
   spnt[8].trate  [0] = alnca;             /* function of ca, v */
   spnt[8].ratemul[0] = ca6a*ca6f;
   spnt[8].rateo  [0] = 0;      /* offsm */
   spnt[8].trans  [1] = 7;
   spnt[8].trate  [1] = betnca;             /* function of v */
   spnt[8].ratemul[1] = ca6b/ca6f*3.0;
   spnt[8].rateo  [1] = 1;      /* offsm */
   spnt[8].trans  [2] = 3;
   spnt[8].trate  [2] = betnkca6;             /* function of v */
   spnt[8].ratemul[2] = ca6d3;
   spnt[8].rateo  [2] = 3;      /* drate */
 
   spnt[9].numtrans   = 2;
   spnt[9].cond       = 1.0;
   spnt[9].trans  [0] = 8;
   spnt[9].trate  [0] = betnca;             /* function of v */
   spnt[9].ratemul[0] = ca6b/ca6f*4.0;
   spnt[9].rateo  [0] = 1;      /* offsm */
   spnt[9].trans  [1] = 4;
   spnt[9].trate  [1] = betnkca6;             /* function of v */
   spnt[9].ratemul[1] = ca6d4;
   spnt[9].rateo  [1] = 3;      /* drate */
 
   return ch;
}

/*--------------------------------------------*/
