/* code.c */

#include "nc.h"
#include "y.tab.h"
#include "adef.h"

#ifdef __cplusplus
extern "C" {
#endif

#include <stdio.h>
/* #include <ctype.h> */
/* #ifdef CPML
#include <cpml.h>
#else
#include <math.h>
#endif */
#include <errno.h>
#include "stdplt.h"
void exit(int code);

#include <string.h>

#ifdef __cplusplus
}
#endif

#ifdef XSTIMM		/* define XSTIMM when stim should ignore neur elems */
  #define XSTIM		/* define XSTIM for stim */
#endif 

#define pushm(d) *stackp++ = (d)	/*  */
#define popm()  *--stackp		/* function pop still needed */

/* #define CODEBUG			/*  */
/*#define popm() pop()			/*  */

#ifndef MAXSIZE
# define MAXSIZE 8191				/* max size of double array */
# define NSTACK 400
# define NPROG  2000
#else						/* larger machine */
# define NSTACK 1000
# if MAXSIZE < 65536
#  define NPROG  4096
# else
#  define NPROG 65536
# endif
#endif


extern char *progname;
extern char *runfile;

static datum stack[NSTACK];	/* the stack */
datum *stackp=0;		/* next free spot on stack */

Inst	prog[NPROG];	/* the machine */
Inst	*progp;		/* next free spot for code generation */
Inst	*pc;		/* program counter during execution */
Fpa	*fpa;		/* pointer to function with arguments */
Inst	funcs[NPROG];	/* procedures and functions */
Inst	*funcbase = funcs; /* start of curent subprogram */
int	returning;	/* 1 if return stmt seen */
int	continuing;	/* 1 if continue stmt seen */
int	breaking;	/* 1 if break stmt seen */
int	stopping;	/* 1 if return, continue, or break stmt seen */

typedef struct Frame {	/* proc/func call stack frame */
	Symbol	*sp;	/* symbol table entry */
	Inst	*retpc;	/* where to resume after return */
	datum	*argn;	/* n-th argument on stack */
	int	nargs;	/* number of arguments */
	int	nlocal;	/* number of local variables */
} Frame;

#define NFRAME 200
Frame	pframe[NFRAME];
Frame	*fp;		/* frame pointer */

static char evnon[] = {"attempt to evaluate non-variable"};
static char uninit[] = {"variable not initialized"};
static char diftyp[] = {"variables are different type"};
static char divzero[] = {"divide by zero"};

extern char *rootframe;		/* root graphics frame */

#ifdef __cplusplus
extern "C" {
#endif

double pow(double a, double b);
double atof(const char *);
char *strcat (char *s1, const char *s2);
char *strtok(char *s, const char *delim);
char *strstr(const char *haystack, const char *needle);
FILE *fopen (const char *path, const char *mode);
#include "gr.h"

#ifdef __cplusplus
}
#endif

#include "gprim.h"

array *garray(Symbol *sp);
double *garrayval(Symbol *sp);
char *emalloc(unsigned int n);
double *darr2(Symbol *sp, int narg);
void execerror (char *s, char *t);
void constpush(void);
void execute (Inst *pc);
void gtext(char *s);
void fft (double *real, double *imag, int size, int type);
int yyparse();
int checknum(datum d);
int checkstr(datum d);
void onintr(void);
char *fgets(char *ptr, int n, FILE *fp);
void erarr (array *arrp);
void efree(void *ptr);
void plotpen (int val, int i);

/*-----------------------------------------------------------*/

void xdebugf(void)
{
  /* fprintf ("anything you want...\n"); */
}

/*-----------------------------------------------------------*/

void initcode(void) {
	int i;

	progp = prog;
	funcbase = funcs;
	stackp = stack;
	fp = pframe;
	returning = 0;
	continuing = 0;
	breaking = 0;
	stopping = 0;
        for (i=0; i<NFRAME; i++) {
             pframe[i].sp    = (Symbol *)NULL;
             pframe[i].retpc = (Inst *)NULL;
             pframe[i].argn  = (datum *)NULL;
             pframe[i].nargs = 0;
             pframe[i].nlocal = 0;
	}
}

void resetcode(void) {
	progp = prog;
	stackp = stack;
	fp = pframe;
	returning = 0;
	continuing = 0;
	breaking = 0;
	stopping = 0;
}

void push(datum d)
{
	if (stackp >= &stack[NSTACK])
		execerror ("stack too deep", (char *)0);
	*stackp++ = d;
}


void expop(void)
{
	if (stackp == stack)
		execerror ("stack underflow", (char *)0);
	--stackp;
	return;
}

datum pop(void)
{
	if (stackp == stack)
		execerror ("stack underflow", (char *)0);
	return *(--stackp);
}

/*------------------------------------------*/

static datum dsave;

void popsav(void)

/* temporarily save a value from the stack */

{
	if (stackp == stack)
		execerror ("stack underflow", (char *)0);
	dsave = *(--stackp);
}

void pushsav()
/* restore the stack from previously saved value */
{
	if (stackp >= &stack[NSTACK])
		execerror ("stack too deep", (char *)0);
	*stackp++ = dsave;
}

/*------------------------------------------*/

void stackmove(void)

/* move the top value on the stack down n levels */

{
     datum dsave, *stp;
     int i,n;

   n = (long int) (*pc++);
   stp = stackp; 
   if ((stp - n) < stack) {
	execerror ("stackmove: stack underflow", (char *)0);
	return;
   }
   dsave = *(--stp);		/* pop value off stack */
   for (i=0; i<n; i++,stp--) {
     *stp = *(stp-1);
   }
   *stp = dsave;		/* push value on bottom */
}

/*------------------------------------------*/

int checknum(datum d)
{
  int val;

  switch(d.vtype) {
	case NUMBER:
		val = 1;
		break;
	case LITCHAR:
		val = 2;
		break;
	default:
	case STRING:
		execerror ("wrong type: must have number",0);
		val = 0;
		break;
  }
  return val;
}

/*------------------------------------------*/

int checkstr(datum d)
{
  switch(d.vtype) {
	case NUMBER:
	case LITCHAR:
	default:
		execerror ("wrong type: must have string",0);
		return 0;
		break;
	case STRING:
		return 1;
		break;
  }
}

/*------------------------------------------*/

void constpush(void)
{
	datum d;

#ifdef CODEBUG
  fprintf (stderr,"constpush\n");
#endif

	d.type = CONST; 
	d.vtype = ((Symbol *)*pc)->type;	/* since literal, vtype=type */
	switch (d.vtype) {
	  case NUMBER:
	  case LITCHAR:
		d.val = ((Symbol *)*pc)->val;
		break;
	  case STRING:
		d.str = ((Symbol *)*pc)->name;
		break;
	}

	pc++;
	pushm (d);
}

/*------------------------------------------*/

void varpush(void)
{
	datum d1={0},d2={0};
        int type;

#ifdef CODEBUG
  fprintf (stderr,"varpush\n");
#endif
	type = (long int) (*pc++);
	d1.type = type;
	switch (type) {
	  case ARG:
	  case LOCALVAR:
		d1.argnum = (long int)(*pc++);
		pushm(d1);
	        break;
	  case UNDEF:
	  case CONST:
	  case VAR:
	     d1.sym = (Symbol *)(*pc++);
	     d1.vtype = d1.sym->vtype;    /* set NUMBER, STRING, etc.*/
	     pushm(d1);
	     break;
	  case LOCALARR:
	      d1.argnum = (long int)(*pc++);
	      d2.sym = (Symbol *)(*pc++); /* don't convert to double */
	      d2.type = VAR;
	      pushm(d2);		  /* number of dimensions */
	      d1.vtype = 0;	 	  /* set NUMBER, STRING, etc.*/
	      pushm(d1); 
	      break;
	  case ARRAYVAL:
	  case ARRAY:
	      d1.sym = (Symbol *)(*pc++);
	      d2.sym = (Symbol *)(*pc++); /* don't convert to double */
	      d2.type = VAR;
	      pushm(d2);		    /* number of dimensions */
	      d1.vtype = d1.sym->vtype;   /* set NUMBER, STRING, etc.*/
	      pushm(d1); 
	      break;
	default: 
	      fprintf (stderr,"type %d\n",d1.sym->type);
	      execerror ("Assignment to non-variable", d1.sym->name);
	      break;
	  }
}

/*------------------------------------------*/

void breakcode(void)
{
#ifdef CODEBUG
  fprintf (stderr,"breakcode\n");
#endif
	breaking = stopping = 1;
}

/*------------------------------------------*/

void contcode(void)
{
#ifdef CODEBUG
  fprintf (stderr,"contcode\n");
#endif
	continuing = stopping  = 1;
}

/*------------------------------------------*/

int forval(datum d)
{
   int val;

   switch (d.vtype) {
     case NUMBER:
     case LITCHAR:
	val = (int)(d.val);
	break;
     case STRING:
	val = strlen(d.str);
	break;
   }
   return val;
}

/*------------------------------------------*/

void forcode(void)
{
	datum d;
	Inst *savepc = pc;
	int val;

#ifdef CODEBUG
  fprintf (stderr,"forcode\n");
#endif
	execute (savepc+4);	/* init */
	d = popm();
	execute(savepc+(int)*savepc); /* condition */
	d = popm();
	while (val=forval(d)) {
		execute(savepc+(int)*(savepc+2)); /* body */
		if (stopping) {
			if (returning)
				break;
			stopping = 0;
			if (breaking) {
				breaking = 0;
				break;
			}
			continuing = 0;
		}
		execute(savepc+(int)*(savepc+1)); /* end loop expr*/
		d = popm();
		execute(savepc+(int)*savepc); /* condition */
		d = popm();
	}
	if (!returning)
		pc = savepc+(int)*(savepc+3);	/* next stmt */
}

/*------------------------------------------*/

void whilecode(void)
{
	datum d;
	Inst *savepc = pc;
	int val;

#ifdef CODEBUG
  fprintf (stderr,"whilecode\n");
#endif
	execute (savepc+2);	/* condition */
	d = popm();
	while (val=forval(d)) {
		execute(savepc+(int)*savepc);	/* body */
		if (stopping) {
			if (returning)
				break;
			stopping = 0;
			if (breaking) {
				breaking = 0;
				break;
			}
			continuing = 0;
		}
		execute(savepc+2);	/* condition */
		d = popm();
	}
	if (!returning)
		pc = savepc+(int)*(savepc+1);	/* next stmt */
}

/*------------------------------------------*/

void ifcode(void)
{
	datum d;
	Inst *savepc = pc;	/* then part */
	int val;

#ifdef CODEBUG
  fprintf (stderr,"ifcode\n");
#endif
	execute(savepc+3);	/* condition */
	d = popm();
	if (val=forval(d))
		execute(savepc+(int)*savepc);
	else if (*(savepc+1)) /* else part? */
		execute(savepc+(int)*(savepc+1));
	if (!returning)
		pc = savepc+(int)*(savepc+2); /* next stmt */
}

/*------------------------------------------*/

void copycode (Inst *src, Inst *dest, int n)

{
  int i;

  for (i=0; i<n; i++) {
    *dest++ = *src++;
  }
}

/*------------------------------------------*/

void define(Symbol *sp, int narg, Inst *p)/* put func/proc in symbol table */
         
{
     int n;

#ifdef CODEBUG
  fprintf (stderr,"define\n");
#endif
	sp->defn = funcbase;	/* start of code */
	sp->vtype = narg;	/* number of arguments declared */
	n = progp - p;
	copycode(p,funcbase,n);	/* copy func to separate func space */
	funcbase += n;		/* next code starts here */
	progp = p;
}

/*------------------------------------------*/

void local(void)	/* add # of local variables to arg num in stack frame */

{
    int i,d,s,nargs,narr,targn;
    datum d1;

#ifdef CODEBUG
  fprintf (stderr,"local\n");
#endif
	d1.val = LARGENUM;		/* set to uninitialized value */
	d1.type = 0;			/* reset type */
	d1.vtype = 0;			/* reset value type */
	nargs = (long int)*pc++;
	narr  = (long int)*pc++;		/* number of local arrays */
	fp->nargs += nargs;
	fp->nlocal += nargs;
	for (i=0; i<(nargs-narr); i++) 	/* increment stack for local vars */
	    pushm(d1);
	fp->argn = stackp - 1;		/* last local var */
	for (i=0; i<narr; i++) {	/* recover local array pointers */
	  s = i-(nargs-1);
          targn = fp->argn[s].type;
	  d = targn - fp->nargs;
	  if (s!=d) {			/* don't move if no extra args */
             fp->argn[d].arrp  = fp->argn[s].arrp;  /* move to correct place */
             fp->argn[s].arrp  = (array *)NULL; 
             fp->argn[s].type  = 0; 
             fp->argn[d].vtype = fp->argn[s].vtype; 
             fp->argn[s].vtype = 0;
	  }
          fp->argn[d].type  = LOCALARR; 
	}	
}

/*------------------------------------------*/

void locend(void)

{
    int i,nargs;
    datum d={0};

#ifdef CODEBUG
  fprintf (stderr,"local\n");
#endif

	nargs = (long int)*pc++;
	fp->nargs -= nargs;
	fp->nlocal -= nargs;
	for (i=0; i<nargs; i++) { 	/* decrement stack for local vars */
	    d = popm();
	    if (d.type==LOCALARR) erarr(d.arrp);
	}
	fp->argn = stackp - 1;		/* last local var */
}

/*------------------------------------------*/

void call(void)		/* call a function */
{
	Symbol *sp;
					/* for function */
#ifdef CODEBUG
  fprintf (stderr,"call\n");
#endif
 	sp = (Symbol *)pc[0]; /* symbol table entry */
	if (fp++ >= &pframe[NFRAME-1])
		execerror(sp->name, ": call nested too deeply");
	fp->sp = sp;
	fp->nargs = (long int)pc[1];	/* # of args passed from call */
	fp->nlocal = 0;			/* reset # of local variables */
	fp->retpc = pc + 2;
	fp->argn = stackp - 1;	/* last argument (including local variables) */
	if (sp->vtype != fp->nargs) 
		execerror(sp->name, ": wrong number of arguments");
	if (sp->type == UNDEF) {
		execerror(sp->name, ": undefined function");
		returning = stopping = 1;
	}
	else {
		execute(sp->defn);
		returning = stopping = 0;
	}
}

/*------------------------------------------*/

void callproc(Symbol *procp, int npar, double par1, double par2)
{
    datum d1,d2;

#ifdef CODEBUG
  fprintf (stderr,"callproc\n");
#endif
   d1.val = par1; 		/* simulate a call from interpreter */
   d1.type = VAR; 
   d1.vtype = NUMBER; 
   d2.val = par2; 
   d2.type = VAR; 
   d2.vtype = NUMBER; 
   if (npar>0) pushm (d1);
   if (npar>1) pushm (d2);
   if (fp++ >= &pframe[NFRAME-1])
		execerror(procp->name, ": call nested too deeply");
   fp->sp = procp;
   fp->nargs = npar; 
   fp->retpc = pc;
   fp->argn = stackp - 1;	/* last argument (including local variables) */
   if (procp->vtype != fp->nargs) 
		execerror(procp->name, ": wrong number of arguments");
   execute(procp->defn);
   returning = stopping = 0;
}

/*------------------------------------------*/

double callfunc(Symbol *funcp, int npar, double par1, double par2, 
					 double par3, double par4)
{
    datum d1,d2,d3,d4;

#ifdef CODEBUG
  fprintf (stderr,"callfunc\n");
#endif
   d1.val = par1; 		/* simulate a call from interpreter */
   d1.type = VAR; 
   d1.vtype = NUMBER; 
   d2.val = par2; 
   d2.type = VAR; 
   d2.vtype = NUMBER; 
   d3.val = par3; 
   d3.type = VAR; 
   d3.vtype = NUMBER; 
   d4.val = par4; 
   d4.type = VAR; 
   d4.vtype = NUMBER; 
   if (npar>0) pushm (d1);
   if (npar>1) pushm (d2);
   if (npar>2) pushm (d3);
   if (npar>3) pushm (d4);
   if (fp++ >= &pframe[NFRAME-1])
		execerror(funcp->name, ": call nested too deeply");
   fp->sp = funcp;
   fp->nargs = npar; 
   fp->retpc = pc;
   fp->argn = stackp - 1;	/* last argument (including local variables) */
   if (funcp->vtype != fp->nargs) 
		execerror(funcp->name, ": wrong number of arguments");
   execute(funcp->defn);
   returning = stopping = 0;
   d1 = popm(); 
   return (d1.val);
}

/*------------------------------------------*/

void callfunc8 (Symbol *funcp, int npar, double par1, double par2,
			double par3, double par4, double par5,
			double par6, double par7, double par8)

/* simulate a call with 8 arguments and no return */

{
    datum d[8];
    int i;

#ifdef CODEBUG
  fprintf (stderr,"callfunc\n");
#endif
   d[0].val = par1; 		/* simulate a call from interpreter */
   d[1].val = par2;
   d[2].val = par3;
   d[3].val = par4;
   d[4].val = par5;
   d[5].val = par6;
   d[6].val = par7;
   d[7].val = par8;
   npar = limit(npar,0,8);
   for (i=0; i<npar; i++) {
     d[i].vtype = NUMBER; 
     d[i].type = VAR;
     pushm(d[i]);
   }
   if (fp++ >= &pframe[NFRAME-1])
		execerror(funcp->name, ": call nested too deeply");
   fp->sp = funcp;
   fp->nargs = npar; 
   fp->retpc = pc;
   fp->argn = stackp - 1;	/* last argument (including local variables) */
   if (funcp->vtype != fp->nargs) 
		execerror(funcp->name, ": wrong number of arguments");
   execute(funcp->defn);
   returning = stopping = 0;
}

/*------------------------------------------*/

void ret(void)		/* common return from func or proc */
{
	int i;
	datum d={0};
#ifdef CODEBUG
  fprintf (stderr,"ret\n");
#endif
	for (i=0; i<fp->nargs; i++) {
	    d = popm();  /* pop arguments */
            if (d.type==LOCALARR) erarr(d.arrp);	/* erase array */
	}
	pc = (Inst *)fp->retpc;
	--fp;
	returning = stopping = 1;
}

/*------------------------------------------*/

void funcret(void)	/* return from a function */
{
	datum d;
#ifdef CODEBUG
  fprintf (stderr,"funcret\n");
#endif
	if (fp->sp->type == PROCEDURE)
		execerror(fp->sp->name, "(proc) returns value");
	d = popm();	/* preserve function return value */
	ret();
	pushm(d);
}

/*------------------------------------------*/

void procret(void)	/* return from a procedure */
{
#ifdef CODEBUG
  fprintf (stderr,"procret\n");
#endif
	if (fp->sp->type == FUNCTION)
		execerror(fp->sp->name,"func returns no value");
	ret();
}

/*------------------------------------------*/

void xexit(void)

/* exit program completely from any stack location. */

{
  fprintf (stderr,"Exiting...\n");
  exit(1);
}

/*------------------------------------------*/

void getarg(int narg, double **val, short int **vtyp)	
			/* return pointer to argument */
	          
{
	char tbuf[40];

	if (narg > fp->nargs)
	    if (fp->sp) 
	       execerror(fp->sp->name, ": not passed enough arguments");
	    else
	       execerror((char*)NULL,": not passed enough arguments");

	if (fp->argn[narg - fp->nargs].type==LOCALARR) {
	      static Symbol sp;
	      static short num;

	  sp.arrp = fp->argn[narg - fp->nargs].arrp;
	  sprintf(tbuf,"local array var %d\n",narg);
	  sp.name = tbuf;
	  *val = garrayval (&sp);
	}
	else {
	  *val  = &fp->argn[narg - fp->nargs].val;
	}
	*vtyp = &fp->argn[narg - fp->nargs].vtype;
}

/*------------------------------------------*/

void bltin(void)

/* call a built-in function, with as many as five arguments */
/*  Arguments may be either NUMBER or STRING. */

{
	int i,narg;
	datum d[5];

#ifdef CODEBUG
  fprintf (stderr,"bltin\n");
#endif
	errno=0;
	narg = (long int)*pc++;			/* get number of arguments */
	for (i=0; i<narg; i++)
	   d[narg-i-1] = popm();
	fpa = (Fpa *)pc++;

	switch (narg) {
	case 0: d[0] = (*fpa)();
		break;
	case 1: d[0] = (*fpa)(&d[0]);
		break;
	case 2: d[0] = (*fpa)(&d[0],&d[1]);
		break;
	default:if (narg>5) {
		    narg = 5;
		    execerror ("more than 5 arguments.",0);
		}
		d[0] = (*fpa)(&d[0],&d[1],&d[2],&d[3],&d[4]);
		break;
	}
	pushm(d[0]);
}

/*------------------------------------------*/

Symbol *getvar(double **val, short int **typ, short int **vtyp)

/* Get pointer to variable */
                      
{
	datum d;
	int narg;
	static short int atyp;
	static Symbol locvar;

	d = popm();			/* get pointer to variable */
	if (d.type==0) {
	   execerror(evnon, 0);
           return ((Symbol *)0); 
	}
	narg = d.argnum;
	if (narg > 0 && narg < 1000) {	/* if var is arg to a function */
		getarg(narg,val,vtyp);	
		atyp = ARG;
		*typ = &atyp;
		switch (**vtyp) {
		 default:
		 case NUMBER:
	           if (**val==LARGENUM) { /* return error if uninitialized */
		      static char ebuf[100];
		      int argn;

		   if ((argn=narg-(fp->nargs-fp->nlocal)) <=0)
		     sprintf (ebuf,"proc %s: #%d argument",fp->sp->name,narg);
		   else if (fp->sp)
			sprintf (ebuf,"proc %s: #%d local",fp->sp->name,argn);
		   else
			sprintf (ebuf,"proc: #%d local",argn);
           	   locvar.name=ebuf;	/* fake a proper symbol and name */
           	   locvar.val=0;
		   return &locvar;
		  }
		  break;
		 case STRING: 
		  break;
	      }
	}
	else {
	  *typ  = &d.sym->type; 
	  *vtyp = &d.sym->vtype; 
	  switch (d.type) {
	  case UNDEF:
	  case CONST:
	  case VAR:   *val = &d.sym->val;
		      break;
	  case ARRAY:  
		       *val = (double *)garray(d.sym); 
			break;
	  case ARRAYVAL:  
		       *val = garrayval(d.sym); 
			atyp = d.type;
			*typ = &atyp;
		      break;
	  default: execerror(evnon, 0);
		      break;
	  }
	}
	if (*val) {
	 if (**typ != ARRAY)
	   switch (**vtyp) {	/* return var of variable if uninitialized */
	   default:
	   case NUMBER: 
	     if (**val==LARGENUM) return (d.sym);
	     break;
	   case STRING: 	
	     if (*val==NULL) return (d.sym);
	     break;
	   }
	}
	else {
	  *val = &d.val;
	  return ((Symbol *)d.sym); 
	}
      return ((Symbol *)0); 
}

/*------------------------------------------*/

void evalvar(void)		/* evaluate variable on stack */
{
	datum d1;
	double *val;
	short *typ,*vtyp;
	Symbol *s;

#ifdef CODEBUG
  fprintf (stderr,"evalvar\n");
#endif
	if (s=getvar(&val,&typ,&vtyp)) {
           execerror (s->name, uninit);
	   d1.val=0;
	   stopping = returning = 1;
	}
	d1.type  = *typ;
	d1.vtype = *vtyp;	     /* set value type: NUMBER, STRING, etc. */

 /*      if (d1.type==CONST) d1.vtype = NUMBER;	/* convert CONST to NUMBER */
	if (d1.type == ARRAY) {
		d1.arrp = (array *)val;
	} 
	else
 	 switch (d1.vtype) {
	   case NUMBER:
	   case LITCHAR:
		d1.val  = *val;
		break;
	   case STRING:
		d1.str  = *(char **)val;
	 	break;
	 }
	pushm(d1);
}

/*------------------------------------------*/

void notinit(void)	/* evaluate whether variable on stack is initialized */
{
	datum d1;
	double *val;
	short *typ,*vtyp;
	Symbol *s;

#ifdef CODEBUG
  fprintf (stderr,"notinit\n");
#endif
	if (s=getvar(&val,&typ,&vtyp)) d1.val = 1;
	else                           d1.val = 0;  
	d1.type  = CONST;
	d1.vtype = NUMBER;
	pushm(d1);
}

/*------------------------------------------*/

void popone(void)
{
	datum d1,d2;

#ifdef CODEBUG
  fprintf (stderr,"popone\n");
#endif
	d1 = popm();
	d2 = popm();
	pushm(d1);
}

/*------------------------------------------*/

int getarr(datum *d, double **parr, int *siz)

/* Find the beginning of an array and its size. */

{
     int ndim,arraysiz;
     int i;
     array *apnt;

if (d->type == ARRAY) {
   apnt = d->arrp;
  *parr = apnt->arr;
  ndim = apnt->ndim;                           /* get # of dimensions */
  arraysiz = 1;
  for (i=0; i<ndim; i++) {
    arraysiz *= apnt->dim[i];                  /* mult by next lower dim */
  }
  *siz = arraysiz;
  return (1);
 }
 else return (0);
}

/*------------------------------------------*/

void copyarr(datum *src, double **p, int *siz)

/* Copy array for result of arithmetic array operation onto
   the top of the stack. Return pointer to array and its size. */

{
   datum d;
   Symbol sp;
   double **parr;
     array *apnt;
   int i, arraysiz,ndim;
   static char tnam[] = "temparr";

   parr = (double **)src->arrp;
   apnt = src->arrp;
   *parr = apnt->arr;
   ndim = apnt->ndim;                           /* get # of dimensions */
   arraysiz = 1;
   for (i=0; i<ndim; i++) {
     d.type = VAR;
     d.val = apnt->dim[i];
     arraysiz *= int(d.val);
     pushm (d);
   }
   *siz = arraysiz;
   sp.name = tnam; 
   sp.type = ARRAY; 
   darr2(&sp,ndim);
   *p = *(double **)sp.arrp;
   d.arrp = sp.arrp; 
   d.type = sp.type;
   d.vtype = src->vtype;
   pushm (d);
}

/*------------------------------------------*/

void add(void)
{
	datum d1,d2;
	int len1,len2;
	char *strp;
	double *val1, *val2;

#ifdef CODEBUG
  fprintf (stderr,"add\n");
#endif
	d2 = popm();
	d1 = popm();
	if ((d1.vtype==STRING || d2.vtype==STRING) && (d1.vtype != d2.vtype)) {
		execerror (diftyp,0);
	} 
	if (d1.type == ARRAY && d2.type==ARRAY) {
		double *p1,*p2,*p3;
		int i,siz1,siz2,siz3;
	
          getarr(&d1, &p1, &siz1);
          getarr(&d2, &p2, &siz2);
	  if (siz1 != siz2) {
 	    execerror("arrays not same size", (char *)0);
	    pushm(d1);
	    return;
	  }
          copyarr(&d1, &p3, &siz3);
	  
	  for (i=0; i<siz1; i++) {
	    *(p3+i) = *(p1+i) + *(p2+i);
	  }
	} 
	else if (d1.type == ARRAY && d2.type!=ARRAY) {
		double *p1,*p3;
		int i,siz1,siz3;
	
          getarr(&d1, &p1, &siz1);
          copyarr(&d1, &p3, &siz3);

	  for (i=0; i<siz1; i++) {
	    *(p3+i) = *(p1+i) + d2.val;
	  }

	} 
	else if (d1.type != ARRAY && d2.type==ARRAY) {
		double *p2,*p3;
		int i,siz2,siz3;
	
          getarr(&d2, &p2, &siz2);
          copyarr(&d2, &p3, &siz3);

	  for (i=0; i<siz2; i++) {
	    *(p3+i) =  d1.val + *(p2+i);
	  }

	} else {
	switch (d1.vtype) {
	  case NUMBER:
	  case LITCHAR:
		d1.val += d2.val;
		break;
	  case STRING:
		len1 = strlen(d1.str);
		len2 = strlen(d2.str);
		strp = emalloc(len1+len2+1);
		strcpy (strp,d1.str);
		strcat (strp,d2.str);
		d1.str = strp;
		break;
	}
	d1.type = VAR;
	pushm(d1);
     }
}

void postinc(void)
{
	datum d1;
	double *p;
	short *typ,*vtyp;
	Symbol *s;

#ifdef CODEBUG
  fprintf (stderr,"postinc\n");
#endif
	if (s=getvar(&p,&typ,&vtyp))
           execerror (s->name, uninit);
	d1.val = *p;
	d1.vtype = NUMBER;
	d1.type = VAR;
	pushm(d1);
/*	(*p)++;   */
	*p += 1; 
}

void postdec(void)
{
	datum d1;
	double *p;
	short *typ,*vtyp;
	Symbol *s;

#ifdef CODEBUG
  fprintf (stderr,"postdec\n");
#endif
	if (s=getvar(&p,&typ,&vtyp))
           execerror (s->name, uninit);
	d1.val = *p;
	d1.vtype = NUMBER;
	d1.type = VAR;
	pushm(d1);
/*	(*p)--; */
	*p -= 1;
}

void preinc(void)
{
	datum d1;
	double *p;
	short *typ,*vtyp;
	Symbol *s;

#ifdef CODEBUG
  fprintf (stderr,"preinc\n");
#endif
	if (s=getvar(&p,&typ,&vtyp))
           execerror (s->name, uninit);
/*	(*p)++; */
	*p += 1;
	d1.val = *p;
	d1.vtype = NUMBER;
	d1.type = VAR;
	pushm(d1);
}

void predec(void)
{
	datum d1;
	double *p;
	short *typ,*vtyp;
	Symbol *s;

#ifdef CODEBUG
  fprintf (stderr,"predec\n");
#endif
	if (s=getvar(&p,&typ,&vtyp))
           execerror (s->name, uninit);
/*	(*p)--; */
	*p -= 1;
	d1.val = *p;
	d1.vtype = NUMBER;
	d1.type = VAR;
	pushm(d1);
}

void sub(void)
{
	datum d1,d2;
#ifdef CODEBUG
  fprintf (stderr,"sub\n");
#endif
	d2 = popm();
	d1 = popm();
	if ((d1.vtype==STRING || d2.vtype==STRING) && (d1.vtype != d2.vtype)) {
		execerror (diftyp,0);
	} 
	if (d1.type == ARRAY && d2.type==ARRAY) {
		double *p1,*p2,*p3;
		int i,siz1,siz2,siz3;
	
          getarr(&d1, &p1, &siz1);
          getarr(&d2, &p2, &siz2);
	  if (siz1 != siz2) {
 	    execerror("arrays not same size", (char *)0);
	    pushm(d1);
	    return;
	  }
          copyarr(&d1, &p3, &siz3);
	  
	  for (i=0; i<siz1; i++) {
	    *(p3+i) = *(p1+i) - *(p2+i);
	  }
	} 
	else if (d1.type == ARRAY && d2.type!=ARRAY) {
		double *p1,*p3;
		int i,siz1,siz3;
	
          getarr(&d1, &p1, &siz1);
          copyarr(&d1, &p3, &siz3);

	  for (i=0; i<siz1; i++) {
	    *(p3+i) = *(p1+i) - d2.val;
	  }

	} 
	else if (d1.type != ARRAY && d2.type==ARRAY) {
		double *p2,*p3;
		int i,siz2,siz3;
	
          getarr(&d2, &p2, &siz2);
          copyarr(&d2, &p3, &siz3);

	  for (i=0; i<siz2; i++) {
	    *(p3+i) =  d1.val - *(p2+i);
	  }

	} else {
	 d1.val -= d2.val;
	 d1.type = VAR;
	 pushm(d1);
	}
}

void mul(void)
{
	datum d1,d2;
#ifdef CODEBUG
  fprintf (stderr,"mul\n");
#endif
	d2 = popm();
	d1 = popm();
	if (d1.type == ARRAY && d2.type==ARRAY) {
		double *p1,*p2,*p3;
		int i,siz1,siz2,siz3;
	
          getarr(&d1, &p1, &siz1);
          getarr(&d2, &p2, &siz2);
	  if (siz1 != siz2) {
 	    execerror("arrays not same size", (char *)0);
	    pushm(d1);
	    return;
	  }
          copyarr(&d1, &p3, &siz3);
	  
	  for (i=0; i<siz1; i++) {
	    *(p3+i) = *(p1+i) * *(p2+i);
	  }
	} 
	else if (d1.type == ARRAY && d2.type!=ARRAY) {
		double *p1,*p3;
		int i,siz1,siz3;
	
          getarr(&d1, &p1, &siz1);
          copyarr(&d1, &p3, &siz3);

	  for (i=0; i<siz1; i++) {
	    *(p3+i) = *(p1+i) * d2.val;
	  }

	} 
	else if (d1.type != ARRAY && d2.type==ARRAY) {
		double *p2,*p3;
		int i,siz2,siz3;
	
          getarr(&d2, &p2, &siz2);
          copyarr(&d2, &p3, &siz3);

	  for (i=0; i<siz2; i++) {
	    *(p3+i) = *(p2+i) * d1.val;
	  }

	} else {
	 d1.val *= d2.val;
	 d1.type = VAR;
	 pushm(d1);
	}
}

void mod(void)
{
	datum d1,d2;
	int denom;
#ifdef CODEBUG
  fprintf (stderr,"mod\n");
#endif
	d2 = popm();
	d1 = popm();
	if (d2.val == 0.0)
	  	execerror("stopping in mod, ",divzero);
	if (d1.type == ARRAY && d2.type==ARRAY) {
		double *p1,*p2,*p3;
		int i,siz1,siz2,siz3;
	
          getarr(&d1, &p1, &siz1);
          getarr(&d2, &p2, &siz2);
	  if (siz1 != siz2) {
 	    execerror("arrays not same size", (char *)0);
	    pushm(d1);
	    return;
	  }
          copyarr(&d1, &p3, &siz3);
	  
	  for (i=0; i<siz1; i++) {
	    if ((denom=int(*(p2+i)))==0) execerror("stopping in mod, ",divzero);
	    *(p3+i) = int(*(p1+i)) % denom;
	  }
	} 
	else if (d1.type == ARRAY && d2.type!=ARRAY) {
		double *p1,*p3;
		int i,siz1,siz3;
	
          getarr(&d1, &p1, &siz1);
          copyarr(&d1, &p3, &siz3);

	  denom = int(d2.val);
	  for (i=0; i<siz1; i++) {
	    *(p3+i) = int(*(p1+i)) % denom; 
	  }

	} 
	else if (d1.type != ARRAY && d2.type==ARRAY) {
		double *p2,*p3;
		int i,siz2,siz3;
	
          getarr(&d2, &p2, &siz2);
          copyarr(&d2, &p3, &siz3);

	  for (i=0; i<siz2; i++) {
	    if ((denom=int(*(p2+1)))==0) execerror("stopping in mod, ",divzero);
	    *(p3+i) = int(d1.val) % denom; 
	  }

	} else {

	 d1.val = int(d1.val) % int(d2.val);
	 d1.type = VAR;
	 pushm(d1);
	}
}

void xdiv(void)
{
	datum d1,d2;
	double denom;

#ifdef CODEBUG
  fprintf (stderr,"xdiv\n");
#endif
	d2 = popm();
	if (d2.val == 0.0)
		execerror("division by zero", (char *)0);
	d1 = popm();
	if (d1.type == ARRAY && d2.type==ARRAY) {
		double *p1,*p2,*p3;
		int i,siz1,siz2,siz3;
	
          getarr(&d1, &p1, &siz1);
          getarr(&d2, &p2, &siz2);
	  if (siz1 != siz2) {
 	    execerror("arrays not same size", (char *)0);
	    pushm(d1);
	    return;
	  }
          copyarr(&d1, &p3, &siz3);
	 
	  for (i=0; i<siz1; i++) {
	    if ((denom= *(p2+i))==0.0) execerror("stopping in div, ",divzero);
	    *(p3+i) = *(p1+i) / denom;
	  }
	} 
	else if (d1.type == ARRAY && d2.type!=ARRAY) {
		double *p1,*p3;
		int i,siz1,siz3;
	
          getarr(&d1, &p1, &siz1);
          copyarr(&d1, &p3, &siz3);

	  denom = d2.val;
	  for (i=0; i<siz1; i++) {
	    *(p3+i) = *(p1+i) / denom;
	  }

	} 
	else if (d1.type != ARRAY && d2.type==ARRAY) {
		double *p2,*p3;
		int i,siz2,siz3;
	
          getarr(&d2, &p2, &siz2);
          copyarr(&d2, &p3, &siz3);

	  for (i=0; i<siz2; i++) {
	    if ((denom= *(p2+i))==0.0) execerror("stopping in div, ",divzero);
	    *(p3+i) = d1.val / denom;
	  }

	} else {
	 d1.val /= d2.val;
	 d1.type = VAR;
	 pushm(d1);
	}
}

void varexp(double **p1, double *p2, short int *p3)

/* Check for valid variable and
return variable pointer and expression value */

{
	datum d1,*d;
	double *p;
	short *typ,*vtyp;
	Symbol *s;

        d1 = popm();                    /* get expression to be assigned */
        d  = stackp-1;                  /* get address of variable   */
	if (s=getvar(&p,&typ,&vtyp))	/* get variable to set value */
           execerror (s->name, uninit);

        if (*vtyp == 0 ||               /* check to make sure same type */
            *vtyp == d1.vtype) {
          *vtyp = d1.vtype;             /* assign value's type */
	  *p3 = *vtyp;
          *p1 = p;			/* assign var address */
          switch (d1.vtype) {
            case NUMBER:
            case LITCHAR:
		*p2 = d1.val;		/* assign numerical value */
                break;
            case STRING:		/* don't assign string value here. */
                break;
          }             
	}
	else {
/*              fprintf (stderr,"d1 %d  d2 %d\n",d1.vtype,d2.vtype); */
/*                execerror (diftyp,d->sym->name); */
                execerror (diftyp,0);

	}
}

void addeq(void)
{
	double *var,expr;
	datum d1;
	short int vtyp;
	int len1,len2;
	char *strp;

#ifdef CODEBUG
  fprintf (stderr,"addeq\n");
#endif
	d1 = *(stackp-1);	/* get expression to be added */
	varexp (&var,&expr,&vtyp);
	switch (d1.vtype) {
	  case NUMBER:
	  case LITCHAR:
        	*var += expr;
		d1.val = *var;
		break;
	  case STRING:
		len1 = strlen(*(char **)var);
		len2 = strlen(d1.str);
		strp = emalloc(len1+len2+1);
		strcpy (strp,*(char **)var);
		strcat (strp,d1.str);
		*(char **)var = strp;
		d1.str = strp;
		break;
	}
	d1.vtype = vtyp;
	d1.type = VAR;
	pushm(d1);
}

void subeq(void)
{
	double *var,expr;
	short int vtyp;
	datum d;

#ifdef CODEBUG
  fprintf (stderr,"subeq\n");
#endif
	varexp (&var,&expr,&vtyp);
        *var -= expr;
	d.val = *var;
	d.vtype = NUMBER;
	d.type = VAR;
	pushm(d);
}

void muleq(void)
{
	double *var,expr;
	short int vtyp;
	datum d;

#ifdef CODEBUG
  fprintf (stderr,"muleq\n");
#endif
	varexp (&var,&expr,&vtyp);
        *var *= expr;
	d.val = *var;
	d.vtype = NUMBER;
	d.type = VAR;
	pushm(d);
}

void diveq(void)
{
	double *var,denom;
	short int vtyp;
	datum d;

#ifdef CODEBUG
  fprintf (stderr,"diveq\n");
#endif
	varexp (&var,&denom,&vtyp);
	if (denom==0.0) execerror("stopping in diveq, ",divzero);
        *var /= denom;
	d.val = *var;
	d.vtype = NUMBER;
	d.type = VAR;
	pushm(d);
}

void negate(void) {
	datum d;
#ifdef CODEBUG
  fprintf (stderr,"negate\n");
#endif
	d = popm();
	d.val = -d.val;
	pushm(d);
}

void gt(void)
{
	datum d1,d2;
#ifdef CODEBUG
  fprintf (stderr,"gt\n");
#endif
	d2 = popm();
	d1 = popm();
	d1.val = (double)(d1.val > d2.val);
	pushm(d1);
}

void lt(void) 
{
	datum d1,d2;
#ifdef CODEBUG
  fprintf (stderr,"lt\n");
#endif
	d2 = popm();
	d1 = popm();
	d1.val = (double)(d1.val < d2.val);
	pushm(d1);
}

void ge(void) 
{
	datum d1,d2;
#ifdef CODEBUG
  fprintf (stderr,"ge\n");
#endif
	d2 = popm();
	d1 = popm();
	d1.val = (double)(d1.val >= d2.val);
	pushm(d1);
}
void le(void) 
{
	datum d1,d2;
#ifdef CODEBUG
  fprintf (stderr,"le\n");
#endif
	d2 = popm();
	d1 = popm();
	d1.val = (double)(d1.val <= d2.val);
	pushm(d1);
}
/*---------------------------------------------------------*/
void eq(void)
{
	datum d1,d2;
	int cmp;

#ifdef CODEBUG
  fprintf (stderr,"eq\n");
#endif
 	d2 = popm();
	d1 = popm();
	if (d1.vtype != d2.vtype) {
		execerror (diftyp,0);
	}
	switch (d1.vtype) {
	  case NUMBER:
	  case LITCHAR:
		d1.val = (double)(d1.val == d2.val);
		break;
	  case STRING:
		cmp = strcmp(d1.str,d2.str);
		d1.val = (double)(cmp==0);
		break;
	}
	d1.vtype = NUMBER;
	pushm(d1);
}
/*---------------------------------------------------------*/
void ne(void)
{
	datum d1,d2;
	int cmp;

#ifdef CODEBUG
  fprintf (stderr,"ne\n");
#endif
	d2 = popm();
	d1 = popm();
	if (d1.vtype != d2.vtype) {
		execerror (diftyp,0);
	}
	switch (d1.vtype) {
	  case NUMBER:
	  case LITCHAR:
		d1.val = (double)(d1.val != d2.val);
		break;
	  case STRING:
		cmp = strcmp(d1.str,d2.str);
		d1.val = (double)(cmp!=0);
		break;
	}
	d1.vtype = NUMBER;
	pushm(d1);
}

/*---------------------------------------------------------*/
void xand(void)
{
	datum d1,d2;
	int val1,val2;

#ifdef CODEBUG
  fprintf (stderr,"xand\n");
#endif
	d2 = popm();
	d1 = popm();
	switch (d1.vtype) {
	  case NUMBER:
	  case LITCHAR:
		val1 = (int)(d1.val != 0.0);
		break;
	  case STRING:
		val1 = (strlen(d1.str) != 0);
		break;
	}
	switch (d2.vtype) {
	  case NUMBER:
	  case LITCHAR:
		val2 = (int)(d2.val != 0.0);
		break;
	  case STRING:
		val2 = (strlen(d2.str) != 0);
		break;
	}
	d1.val = (double)(val1 && val2);
	d1.vtype = NUMBER;
	pushm(d1);
}

/*---------------------------------------------------------*/

void bitand(void)
{
	datum d1,d2;
#ifdef CODEBUG
  fprintf (stderr,"bitand\n");
#endif
	d2 = popm();
	d1 = popm();
	d1.val = (double)((int)d1.val &  (int)d2.val);
	pushm(d1);
}

/*---------------------------------------------------------*/

void bitor(void)
{
	datum d1,d2;
#ifdef CODEBUG
  fprintf (stderr,"bitor\n");
#endif
	d2 = popm();
	d1 = popm();
	d1.val = (double)((int)d1.val |  (int)d2.val);
	pushm(d1);
}

/*---------------------------------------------------------*/

void orx(void)
{
	datum d1,d2;
	int val1,val2;

#ifdef CODEBUG
  fprintf (stderr,"orx\n");
#endif
	d2 = popm();
	d1 = popm();
	switch (d1.vtype) {
	  case NUMBER:
	  case LITCHAR:
		val1 = (int)(d1.val != 0.0);
		break;
	  case STRING:
		val1 = (strlen(d1.str) != 0);
		break;
	}
	switch (d2.vtype) {
	  case NUMBER:
	  case LITCHAR:
		val2 = (int)(d2.val != 0.0);
		break;
	  case STRING:
		val2 = (strlen(d2.str) != 0);
		break;
	}
	d1.val = (double)(val1 || val2);
	d1.vtype = NUMBER;
	pushm(d1);
}

/*---------------------------------------------------------*/

void xnot(void)
{
	datum d;
	int cmp;

#ifdef CODEBUG
  fprintf (stderr,"xnot\n");
#endif
	d = popm();
	switch (d.vtype) {
	  case NUMBER:
	  case LITCHAR:
		d.val = (double)(d.val == 0.0);
		break;
	  case STRING:
		cmp = strlen(d.str);
		d.val = (double)(cmp==0);
		break;
	}
	d.vtype = NUMBER;
	pushm(d);
}

void power(void)
{
	datum d1,d2;
#ifdef CODEBUG
  fprintf (stderr,"power\n");
#endif
	d2 = popm();
	d1 = popm();
	d1.val = pow(d1.val, d2.val);
	pushm(d1);
}

/*---------------------------------------------------------*/

void assign(void)
{
	datum d1,d2,*d;
	double *p;
	short *typ,*vtyp;

#ifdef CODEBUG
  fprintf (stderr,"assign\n");
#endif
	d1 = popm();			/* get expression to be assigned */
	d2 = popm();
	pushm (d2);
	d = stackp-1;			/* get address of variable   */
	if (d->type==0) {
	      execerror("assigment to nonvariable","");
	}
	getvar(&p,&typ,&vtyp);		/* get variable to set value */

	if (*vtyp == 0 ||		/* check to make sure same type */
	    *vtyp == d1.vtype) {
	if (*typ == ARRAY) {
		double *p1,*p2;
		int i,siz1,siz2;

	   if (d1.type != ARRAY) {
	      execerror("Need array to initialize with","");
	   } 
          getarr(&d1, &p1, &siz1);
	  d2.type  = *typ;
	  d2.vtype = *vtyp;
	  d2.arrp = (array *)p;
          getarr(&d2, &p2, &siz2);
	  if (siz1 != siz2) {
 	    execerror("assign: arrays not same size", (char *)0);
	    pushm(d1);
	    return;
	  }
	  for (i=0; i<siz1; i++) {
	    *(p2+i) = *(p1+i);
	  } 
	}
	else {
	  *vtyp = d1.vtype;		/* assign value's type */
	  switch (d1.vtype) {
	    case NUMBER:
	    case LITCHAR:
		*p = d1.val;			/* assign value */
		break;
	    case STRING:
		*(char **)p = d1.str;		/* assign string value */
		break;
	  }		
	  if (*typ==UNDEF) *typ=VAR;		/* make sure it's a variable*/
	 }

	}  /* if (typ==) */

	else {
         if (d->sym) {
	    if (d->argnum >0 && d->argnum < 100) {
			char tbuf[40];
		switch (d->type) {
		 case ARG:
		 case LOCALVAR:
		  sprintf (tbuf,"local var %d",d->argnum);
		  break;
		 case LOCALARR:
		  sprintf (tbuf,"local array var %d",d->argnum);
		  break;
	         }
	         execerror("Assignment of incorrect type for",tbuf);
	    } 
	    else
	     execerror("Assignment of incorrect type for",d->sym->name);
	  }
	}
	pushm(d1);
}

/*---------------------------------------------------------*/

void print(void)	/* pop top value from stack, print it */
{
	datum d;

#ifdef CODEBUG
  fprintf (stderr,"print\n");
#endif
	d = popm();
#ifndef XSTIMM
	if (d.type == ARRAY) {
	     int i,arraysiz;
	     double *pnt;

          getarr(&d, &pnt, &arraysiz);
	  for (i=0; i<arraysiz; i++) {
	    if (d.vtype!=STRING && *(pnt+i)==LARGENUM) 
	     printf ("\tuninit\n");
	    else
	    switch (d.vtype) {
	      case 0:
	      case NUMBER:
		    printf ("\t%.8g\n",*(pnt+i));
		    break;
	      case LITCHAR:
		    printf ("\t%c\n",(char)*(pnt+i));
		    break;
	      case STRING:
		 /* printf("\t%s\n",*(char **)(pnt+i));*//* don't print strings*/
		    break;
	    }
	  }
	}
	else
	 switch (d.vtype) {
	   case 0:
	   case NUMBER:
		 printf ("\t%.8g\n",d.val);
		 break;
	   case LITCHAR:
		 printf ("\t%c\n",(char)d.val);
		 break;
	   case STRING:
		 printf ("\t%s\n",d.str);
		 break;
	 }
#endif
}

/*---------------------------------------------------------*/

void prexpr(void)	/* print value of expression: no tab */
{
	datum d;
#ifdef CODEBUG
  fprintf (stderr,"prexpr\n");
#endif
	d = popm();
#ifndef XSTIMM
	if (d.type == ARRAY) {
	     int i, arraysiz;
	     double *pnt;

         getarr(&d, &pnt, &arraysiz);
	 for (i=0; i<arraysiz; i++) {
	   if (d.vtype!=STRING && *(pnt+i)==LARGENUM) 
	    printf ("\tuninit\n");
	   else
	    switch (d.vtype) {
	     case 0:
	     case NUMBER:
		   printf("%.8g ", *(pnt+i));
		   break;
	     case LITCHAR:
		   printf("%c ", (char)*(pnt+i));
		   break;
	     case STRING:
		   /* printf("%s ", *(char**)(pnt+i));*//* don't print strings */
		   break;
	   }
	 }
	}
	else
	 switch (d.vtype) {
	  case 0:
	  case NUMBER:
		printf("%.8g ", d.val);
		break;
	  case LITCHAR:
		printf("%c ", (char)d.val);
		break;
	  case STRING:
		printf("%s ", d.str);
		break;
	}
#endif
}

void crlf(void)
{
#ifdef CODEBUG
  fprintf (stderr,"crlf\n");
#endif
#ifndef XSTIMM
	printf("\n");
#endif
}

/*---------------------------------------------------------*/

void pfopen (void)		/* fopen (filename, mode) statement */

{

#define XBUFSIZ 256
	static datum  d1;
	int i,mode;
	char *filnam,*filmode;
        FILE *ftemp;
        char xbuf[XBUFSIZ];

#ifdef CODEBUG
  fprintf (stderr,"pfopen\n");
#endif
        if (!checkstr(d1=popm())) {		/* get mode off stack */
	   return;
	}
	filmode = d1.str;
        if (!checkstr(d1=popm())) {		/* get filename off stack */
	   return; 
        }
        filnam = d1.str;			/* file to be read */
        if      (strcmp(filnam,"stdin") ==0) ftemp = stdin;
        else if (strcmp(filnam,"stdout")==0) ftemp = stdout;
        else if (strcmp(filnam,"stderr")==0) ftemp = stderr;
        else if ((ftemp=fopen(filnam,filmode)) == NULL) {
	   sprintf (xbuf,"fgets: can't open file '%.50s'.\n",filnam);
	   execerror ("warning,",xbuf);
	   return;
        }
	d1.val = (long int)ftemp;
	d1.vtype=NUMBER;
	push(d1);
}

/*---------------------------------------------------------*/

void pfclose (void)		/* fclose (filedesc) statement */

{

#define XBUFSIZ 256
	static datum  d1;
	int i;
        FILE *fildesc;
        char xbuf[XBUFSIZ];

#ifdef CODEBUG
  fprintf (stderr,"pfclose\n");
#endif
        if (!checknum(d1=popm())) {		/* get filedesc off stack */
	   return;
	}
	fildesc = (FILE *)(long int)d1.val;
        fclose(fildesc);
}

/*---------------------------------------------------------*/

#define MAXARGS 10

void pscanf(void) 			/* scanf statement */

{
#define STRLENGTH 100
	static datum  d1;
	static double *v[MAXARGS];
	static short int *vvtyp[MAXARGS];
	static char s[MAXARGS][STRLENGTH],*fmt;
	int i,scase,indx,narg,rval;
        short int *typ;
	char *strp;

#ifdef CODEBUG
  fprintf (stderr,"pscanf\n");
#endif
	narg = (long int) *pc++;
	if (narg>MAXARGS) 
	   execerror("scanf: more than 10 arguments","");
	for (i=0; i<narg; i++) {
	   indx = MAXARGS-1-i;
	   getvar(&v[indx],&typ,&vvtyp[indx]);	/* pop var off stack */ 
	}					/*  save pointer to value */
	for (i=0; i<narg; i++) {
            v[i] = v[MAXARGS-narg+i];
            vvtyp[i] = vvtyp[MAXARGS-narg+i];
	}
	for (; i<MAXARGS; i++) {		/* zero the remaining ptrs */
            v[i] = (double*)NULL;
            vvtyp[i] = (short int*)NULL;
	}
	d1 = popm();				/* get format string */
	if (d1.vtype!=STRING)
	   execerror("scanf: string required for format","");

	fmt = d1.str;
	for (strp=fmt,i=0; i<narg; i++) {	/* check format string */
	   if (strp = strstr(strp,"%")) {
	     if (*(++strp)=='s') *vvtyp[i] = STRING;
	     else {
		  char ch;
		while (ch=*++strp,isnum(ch)) ;	/* check for "%20s" */
	        if (*(strp)=='s') *vvtyp[i] = STRING;
	     }
	   }
	}
	for (scase=i=0; i<narg; i++) {
	   switch (*vvtyp[i]) {
	    case 0:  *vvtyp[i] = NUMBER;
		     break;
	    case NUMBER:
	    case LITCHAR:
		break;
	    case STRING:
	        scase += 1 << i;
		break;
	   }
	}
	rval = 0;
	/* fprintf (stderr,"scase %d\n",scase);   /* */
	switch (scase) {
	case 0:
	  rval=scanf(fmt,v[0],v[1],v[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	case 1:
	  rval=scanf(fmt,s[0],v[1],v[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	case 2:
	  rval=scanf(fmt,v[0],s[1],v[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	case 3:
	  rval=scanf(fmt,s[0],s[1],v[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	case 4:
	  rval=scanf(fmt,v[0],v[1],s[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	case 5:
	  rval=scanf(fmt,s[0],v[1],s[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	case 6:
	  rval=scanf(fmt,v[0],s[1],s[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	case 7:
	  rval=scanf(fmt,s[0],s[1],s[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	default:
          execerror("scanf: string variables must be in first 3\n","");
	  break;
	}
					/* assign the new values */
	for (i=0; i<narg; i++) {
	   int len;

	   switch (*vvtyp[i]) {
	    case 0:
	    case NUMBER:
	    case LITCHAR:
		break;
	    case STRING:
		if ((len=strlen(s[i]))>0) {
		  *((char **)v[i]) = emalloc (len+1);
		  strcpy(*((char **)v[i]),s[i]);
	          *vvtyp[i] = STRING;
		}
		break;
	   }
	}
   d1.val = rval;
   d1.vtype=NUMBER;
   push(d1);
}

/*---------------------------------------------------------*/

void pfscanf(void) 		/* fscanf (filedesc,"",args) statement */

{
#define STRLENGTH 100
	static datum  d1;
	static double *v[MAXARGS];
	static short int *vvtyp[MAXARGS];
	static char s[MAXARGS][STRLENGTH],*fmt;
	int i,scase,indx,narg,rval,oflg;
        short int *typ;
	char *strp;
        FILE *ftemp;

#ifdef CODEBUG
  fprintf (stderr,"pfscanf\n");
#endif
	narg = (long int) *pc++;
	if (narg>MAXARGS) 
	   execerror("fscanf: more than 10 arguments","");
	for (i=0; i<narg; i++) {
	   indx = MAXARGS-1-i;
	   getvar(&v[indx],&typ,&vvtyp[indx]);	/* pop var off stack */ 
	}					/*  save pointer to value */
	for (i=0; i<narg; i++) {
            v[i] = v[MAXARGS-narg+i];
            vvtyp[i] = vvtyp[MAXARGS-narg+i];
	}
	for (; i<MAXARGS; i++) {		/* zero the remaining ptrs */
            v[i] = (double*)NULL;
            vvtyp[i] = (short int*)NULL;
	}
	d1 = popm();				/* get format string */
	if (d1.vtype!=STRING)
	   execerror("scanf: string required for format","");
	fmt = d1.str;

	oflg = 0;
        d1 = popm();			/* get filedesc off stack */
	if (d1.vtype==STRING) {		/* must be file name, not descr */
	    char *filnam;
	  filnam = d1.str;
          if      (strcmp(filnam,"stdin")==0) ftemp = stdin;
          else if ((oflg=1,ftemp=fopen(filnam,"r")) == NULL) {
		char xbuf[100];
	    sprintf (xbuf,"fprintf: can't open file '%.50s'.\n",filnam);
	    execerror ("warning,",xbuf);
	    return;
          }
	}
        else ftemp = (FILE *)((long int)d1.val);  /* existing file descriptor */
	if (!ftemp) {
           execerror ("fprintf","can't open file");
           return;
        }

	for (strp=fmt,i=0; i<narg; i++) {	/* check format string */
	   if (strp = strstr(strp,"%")) {
	     if (*(++strp)=='s') *vvtyp[i] = STRING;
	     else {
		  char ch;
		while (ch=*++strp,isnum(ch)) ;	/* check for "%20s" */
	        if (*(strp)=='s') *vvtyp[i] = STRING;
	     }
	   }
	}
	for (scase=i=0; i<narg; i++) {
	   switch (*vvtyp[i]) {
	    case 0:  *vvtyp[i] = NUMBER;
		     break;
	    case NUMBER:
	    case LITCHAR:
		break;
	    case STRING:
	        scase += 1 << i;
		break;
	   }
	}
	rval = 0;
	/* fprintf (stderr,"scase %d\n",scase);   /* */
	switch (scase) {
	case 0:
	rval=fscanf(ftemp,fmt,v[0],v[1],v[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	case 1:
	rval=fscanf(ftemp,fmt,s[0],v[1],v[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	case 2:
	rval=fscanf(ftemp,fmt,v[0],s[1],v[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	case 3:
	rval=fscanf(ftemp,fmt,s[0],s[1],v[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	case 4:
	rval=fscanf(ftemp,fmt,v[0],v[1],s[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	case 5:
	rval=fscanf(ftemp,fmt,s[0],v[1],s[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	case 6:
	rval=fscanf(ftemp,fmt,v[0],s[1],s[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	case 7:
	rval=fscanf(ftemp,fmt,s[0],s[1],s[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	default:
          execerror("fscanf: string variables must be in first 3\n","");
	  break;
	}
	if (oflg) fclose (ftemp);
					/* assign the new values */
	for (i=0; i<narg; i++) {
	   int len;

	   switch (*vvtyp[i]) {
	    case 0:
	    case NUMBER:
	    case LITCHAR:
		break;
	    case STRING:
		if ((len=strlen(s[i]))>0) {
		  *((char **)v[i]) = emalloc (len+1);
		  strcpy(*((char **)v[i]),s[i]);
	          *vvtyp[i] = STRING;
		}
		break;
	   }
	}
   d1.val = rval;
   d1.vtype=NUMBER;
   push(d1);
}

/*---------------------------------------------------------*/

void pfgets(void) 		/* fgets (var,filedesc) statement */

{
#define XBUFSIZ 256
	static datum  d1;
	static char **v;
	static short int *vtyp, *typ;
	int i,n,len,rval,oflg;
	char *strp, *fildesc;
        FILE *ftemp;
        char xbuf[XBUFSIZ];

#ifdef CODEBUG
  fprintf (stderr,"pfgets\n");
#endif
	oflg = 0;
        d1 = popm();			/* get filedesc off stack */
	if (d1.vtype==STRING) {		/* must be file name, not descr */
	    char *filnam;
	  filnam = d1.str;
          if      (strcmp(filnam,"stdin")==0) ftemp = stdin;
          else if ((oflg=1,ftemp=fopen(filnam,"r")) == NULL) {
		char ebuf[100];
	    sprintf (ebuf,"fprintf: can't open file '%.50s'.\n",filnam);
	    execerror ("warning,",ebuf);
	    return;
          }
	}
        else ftemp = (FILE *)((long int)d1.val);  /* existing file descriptor */
	if (!ftemp) {
           execerror ("fprintf","can't open file");
           return;
        }
        if (!checknum(d1=popm())) {		/* get N off stack */
	   return; 
        }
	n = int(d1.val);
	if (n > XBUFSIZ) n = XBUFSIZ;
	getvar((double**)&v,&typ,&vtyp);	/* pop var off stack */ 
	if (fgets (xbuf,n,ftemp)) {
	  rval = 1;
	  len = strlen(xbuf);
	  *v = emalloc (len+1);
	  strcpy(*v,xbuf);
	  *vtyp = STRING;
	}
	else rval = 0;
	if (oflg) fclose(ftemp);
	d1.val = rval;
	d1.vtype=NUMBER;
	push(d1);
}

/*---------------------------------------------------------*/

void psscanf(void) 		/* fscanf (filname,"") statement */

{
#define STRLENGTH 100
	static datum  d1;
	static double *v[MAXARGS];
	static short int *vvtyp[MAXARGS];
	static char s[MAXARGS][STRLENGTH],*fmt;
	int i,scase,indx,narg,rval;
        short int *typ,*vtyp;
	char *strp, *filnam, *sval;
        FILE *ftemp;
	Symbol *es;

#ifdef CODEBUG
  fprintf (stderr,"psscanf\n");
#endif
	narg = (long int) *pc++;
	if (narg>MAXARGS) 
	   execerror("sscanf: more than 10 arguments","");
	for (i=0; i<narg; i++) {
	   indx = MAXARGS-1-i;
	   getvar(&v[indx],&typ,&vvtyp[indx]);	/* pop var off stack */ 
	}					/*  save pointer to value */
	for (i=0; i<narg; i++) {
            v[i] = v[MAXARGS-narg+i];
            vvtyp[i] = vvtyp[MAXARGS-narg+i];
	}
	for (; i<MAXARGS; i++) {		/* zero the remaining ptrs */
            v[i] = (double*)NULL;
            vvtyp[i] = (short int*)NULL;
	}
	checkstr(d1=popm());				/* get format string */
	fmt = d1.str;
	checkstr(d1=popm());				/* get buffer string */
	sval = d1.str;

	for (strp=fmt,i=0; i<narg; i++) {	/* check format string */
	   if (strp = strstr(strp,"%")) {
	     if (*(++strp)=='s') *vvtyp[i] = STRING;
	     else {
		  char ch;
		while (ch=*++strp,isnum(ch)) ;	/* check for "%20s" */
	        if (*(strp)=='s') *vvtyp[i] = STRING;
	     }
	   }
	}
	for (scase=i=0; i<narg; i++) {
	   switch (*vvtyp[i]) {
	    case 0:  *vvtyp[i] = NUMBER;
		     break;
	    case NUMBER:
	    case LITCHAR:
		break;
	    case STRING:
	        scase += 1 << i;
		break;
	   }
	}
	rval = 0;
	/* fprintf (stderr,"scase %d\n",scase);   /* */
	switch (scase) {
	case 0:
	rval=sscanf(sval,fmt,v[0],v[1],v[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	case 1:
	rval=sscanf(sval,fmt,s[0],v[1],v[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	case 2:
	rval=sscanf(sval,fmt,v[0],s[1],v[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	case 3:
	rval=sscanf(sval,fmt,s[0],s[1],v[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	case 4:
	rval=sscanf(sval,fmt,v[0],v[1],s[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	case 5:
	rval=sscanf(sval,fmt,s[0],v[1],s[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	case 6:
	rval=sscanf(sval,fmt,v[0],s[1],s[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	case 7:
	rval=sscanf(sval,fmt,s[0],s[1],s[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	default:
          execerror("sscanf: string variables must be in first 3\n","");
	  break;
	}
					/* assign the new values */
	for (i=0; i<narg; i++) {
	   int len;

	   switch (*vvtyp[i]) {
	    case 0:
	    case NUMBER:
	    case LITCHAR:
		break;
	    case STRING:
		if ((len=strlen(s[i]))>0) {
		  *((char **)v[i]) = emalloc (len+1);
		  strcpy(*((char **)v[i]),s[i]);
	          *vvtyp[i] = STRING;
		}
		break;
	   }
	}
   d1.val = rval;
   d1.vtype=NUMBER;
   push(d1);
}

/*---------------------------------------------------------*/

void pprintf(void) 			/* printf statement */
{
	datum  d1,d[MAXARGS];
	double v[MAXARGS];
	char  *s[MAXARGS],*fmt;
	int i,scase,indx,narg;

#ifdef CODEBUG
  fprintf (stderr,"pprintf\n");
#endif
	narg = (long int) *pc++;
	if (narg>MAXARGS) 
	   execerror("printf: more than 10 arguments","");
	for (i=0; i<narg; i++)
	   d[MAXARGS-1-i] = popm();
	d1 = popm();
	if (d1.vtype!=STRING)
	   execerror("printf: string required for format","");
	for (scase=i=0; i<narg; i++) {
	   indx = MAXARGS-narg+i;
	   switch (d[indx].vtype) {
	    case 0:
	    case NUMBER:
	    case LITCHAR:
	   	v[i] = d[indx].val;	
		break;
	    case STRING:
	   	s[i] = d[indx].str;	
	        scase += 1 << i;
		break;
	   }
	}
	fmt = d1.str;
	/* fprintf (stderr,"scase %d\n",scase);   /* */
	switch (scase) {
	case 0:
	  printf(fmt,v[0],v[1],v[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	case 1:
	  printf(fmt,s[0],v[1],v[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	case 2:
	  printf(fmt,v[0],s[1],v[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	case 3:
	  printf(fmt,s[0],s[1],v[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	case 4:
	  printf(fmt,v[0],v[1],s[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	case 5:
	  printf(fmt,s[0],v[1],s[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	case 6:
	  printf(fmt,v[0],s[1],s[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	case 7:
	  printf(fmt,s[0],s[1],s[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	default:
	  execerror("printf: string variables must be in first 3\n","");
	  break;
	}
	fflush(stdout);
}

/*---------------------------------------------------------*/

void psprintf(void) 			/* sprintf statement */
{
	datum  d1,d[MAXARGS];
	double v[MAXARGS];
	char  *s[MAXARGS],*fmt;
	int i,scase,indx,narg,len;
	short int *typ,*vtyp1;
        double *dstr;
        static char cbuf[120];
 
#ifdef CODEBUG
  fprintf (stderr,"psprintf\n");
#endif
	narg = (long int) *pc++;
	if (narg>MAXARGS) 
	   execerror("printf: more than 10 arguments","");
	for (i=0; i<narg; i++)
	   d[MAXARGS-1-i] = popm();
	d1 = popm();
	if (d1.vtype!=STRING)
	   execerror("printf: string required for format","");
        getvar(&dstr,&typ,&vtyp1);		/* get pointer to output str */
        if (*typ == UNDEF) *typ = VAR;		/* set output type to var */
        *vtyp1 = STRING;			/* set output vtype to string */
	for (scase=i=0; i<narg; i++) {
	   indx = MAXARGS-narg+i;
	   switch (d[indx].vtype) {
	    case 0:
	    case NUMBER:
	    case LITCHAR:
	   	v[i] = d[indx].val;	
		break;
	    case STRING:
	   	s[i] = d[indx].str;	
	        scase += 1 << i;
		break;
	   }
	}
	fmt = d1.str;
        cbuf[0]=0;
	/* fprintf (stderr,"scase %d\n",scase);   /* */
	switch (scase) {
	case 0:
	  sprintf(cbuf,fmt,v[0],v[1],v[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	case 1:
	  sprintf(cbuf,fmt,s[0],v[1],v[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	case 2:
	  sprintf(cbuf,fmt,v[0],s[1],v[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	case 3:
	  sprintf(cbuf,fmt,s[0],s[1],v[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	case 4:
	  sprintf(cbuf,fmt,v[0],v[1],s[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	case 5:
	  sprintf(cbuf,fmt,s[0],v[1],s[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	case 6:
	  sprintf(cbuf,fmt,v[0],s[1],s[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	case 7:
	  sprintf(cbuf,fmt,s[0],s[1],s[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	default:
	  execerror("sprintf: string variables must be in first 3\n","");
	  break;
	}
	if ((len=strlen(cbuf))>0) {		/* copy the new string to var */
	  *((char **)dstr) = emalloc (len+1);
	  strcpy(*((char **)dstr),cbuf);
	}
}

/*---------------------------------------------------------*/

void pfprintf(void) 			/* fprintf (stderr,"") statement */
{
	datum  d1,d[MAXARGS];
	double v[MAXARGS];
	char  *s[MAXARGS],*fmt;
	int i,scase,indx,narg,oflg;
	FILE *ftemp;

#ifdef CODEBUG
  fprintf (stderr,"dprintf\n");
#endif
	narg = (long int) *pc++;
	if (narg>MAXARGS) 
	   execerror("fprintf: more than 10 arguments","");
	for (i=0; i<narg; i++)
	   d[MAXARGS-1-i] = popm();
	if (!checkstr(d1=popm())) {	/* get format string off stack */
	    return;
	}
	fmt = d1.str;
	if (d1.vtype!=STRING)
	   execerror("fprintf: string required for format","");
	for (scase=i=0; i<narg; i++) {
	   indx = MAXARGS-narg+i;
	   switch (d[indx].vtype) {
	    case 0:
	    case NUMBER:
	    case LITCHAR:
	   	v[i] = d[indx].val;	
		break;
	    case STRING:
	   	s[i] = d[indx].str;	
	        scase += 1 << i;
		break;
	   }
	}
	oflg = 0;			/* file open flag */
        d1 = popm();			/* get filedesc off stack */
	if (d1.vtype==STRING) {		/* must be file name, not descr */
	    char *filnam;
	  filnam = d1.str;
          if      (strcmp(filnam,"stdout")==0)  ftemp = stdout;
          else if (strcmp(filnam,"stderr")==0)  ftemp = stderr;
          else if ((oflg=1,ftemp=fopen(filnam,"w")) == NULL) {
		char ebuf[100];
	    sprintf (ebuf,"fprintf: can't open file '%.50s'.\n",filnam);
	    execerror ("warning,",ebuf);
	    return;
          }
	}
        else ftemp = (FILE *)((long int)d1.val);  /* existing file descriptor */
	if (!ftemp) {
           execerror ("fprintf","can't open file");
           return;
        }

	/* fprintf (stderr,"scase %d\n",scase);   /* */
	switch (scase) {
	case 0:
	fprintf(ftemp,fmt,v[0],v[1],v[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	case 1:
	fprintf(ftemp,fmt,s[0],v[1],v[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	case 2:
	fprintf(ftemp,fmt,v[0],s[1],v[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	case 3:
	fprintf(ftemp,fmt,s[0],s[1],v[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	case 4:
	  fprintf(ftemp,fmt,v[0],v[1],s[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	case 5:
	  fprintf(ftemp,fmt,s[0],v[1],s[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	case 6:
	  fprintf(ftemp,fmt,v[0],s[1],s[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	case 7:
	  fprintf(ftemp,fmt,s[0],s[1],s[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
        default:
	  execerror("fprintf: string variables must be in first 3\n","");
          break;
	}
	fflush(ftemp);
	if (oflg) fclose(ftemp);
}

/*---------------------------------------------------------*/

void txtf(void) 			/* textf statement */
{
	datum  d1,d[MAXARGS];
	double v[MAXARGS];
	char  *s[MAXARGS];
	char  *fmt;
	char cbuf[120];
	int i,scase,indx,narg;

#ifdef CODEBUG
  fprintf (stderr,"txtf\n");
#endif
	narg = (long int) *pc++;
	if (narg>MAXARGS) 
	   execerror("textf: more than 10 arguments","");
	for (i=0; i<narg; i++)
	   d[MAXARGS-1-i] = popm();
	d1 = popm();
	if (d1.vtype!=STRING)
	   execerror("textf: string required for format","");
	fmt = d1.str;
	for (scase=i=0; i<narg; i++) {
	   indx = MAXARGS-narg+i;
	   switch (d[indx].vtype) {
	    case 0:
	    case NUMBER:
	    case LITCHAR:
	   	v[i] = d[indx].val;	
		break;
	    case STRING:
	   	s[i] = d[indx].str;	
	        scase += 1 << i;
		break;
	   }
	}
	/* fprintf (stderr,"scase %d\n",scase);   /* */
	switch (scase) {
	case 0:
	sprintf(cbuf,fmt,v[0],v[1],v[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	case 1:
	sprintf(cbuf,fmt,s[0],v[1],v[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	case 2:
	sprintf(cbuf,fmt,v[0],s[1],v[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	case 3:
	sprintf(cbuf,fmt,s[0],s[1],v[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	case 4:
	  sprintf(cbuf,fmt,v[0],v[1],s[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	case 5:
	  sprintf(cbuf,fmt,s[0],v[1],s[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	case 6:
	  sprintf(cbuf,fmt,v[0],s[1],s[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	case 7:
	  sprintf(cbuf,fmt,s[0],s[1],s[2],v[3],v[4],v[5],v[6],v[7],v[8],v[9]);
	  break;
	default:
	  execerror("fprintf: string variables must be in first 3\n","");
	  break;
	}
 gtext(cbuf);
}

/*---------------------------------------------------------*/

int garrsiz(Symbol *apnt)
                
/*  find absolute size of array */

{
   int i,ndim,size;
   array *ap;

   ap = apnt->arrp;			/* pointer to array */
   ndim = ap->ndim;
   for (size=1,i=0; i<ndim; i++) {
	size *= ap->dim[i];		/* multiply dims for size */
   }
   return size;
}

/*---------------------------------------------------------*/

void dlocarray(void)	/* allocate space for local array */
			/* stack var points to array */
{
	int argn,ndim;
	double *dpnt;
	Symbol sp;
	char tnam[40];
	datum d;

#ifdef CODEBUG
  fprintf (stderr,"dlocarray\n");
#endif
	argn = (long int) *pc++;
	ndim = (long int) *pc++;                     /* number of args */
	sprintf (tnam,"local array arg %d\n",argn);
	sp.name = tnam; 
	sp.type = ARRAY; 
	dpnt = darr2(&sp,ndim);
	d.arrp  = sp.arrp;
	d.type = argn;			/* save arg number temporarily */
	d.vtype  = 0;
	pushm (d);
}

/*---------------------------------------------------------*/

void darray(void)	/* allocate space for array */

/* Allocate memory space for array and
 its descriptor block, and set descriptor block.
	
	Desciptor block:

	0	pointer to start of array
	1	number of dimensions in array
	2	dimension 1
	3	dimension 2
		etc.
*/

#define MAXDIM  20				/* maximum # of dimensions */

{
	int ndim;
	Symbol *sp;
	double *dpnt;

#ifdef CODEBUG
  fprintf (stderr,"darray\n");
#endif
	sp = (Symbol *)(*pc++);			/* pointer to array symbol */
	ndim = (long int) *pc++;			/* number of args */
	dpnt = darr2(sp,ndim);
}

/*---------------------------------------------------------*/

double *darr2(Symbol *sp, int ndim)		/* define array */
{
	int i,dim1,**tdpnt,darr[MAXDIM],x;
	datum d;
	unsigned size, dsize;
	array *apnt;

	if (sp->type!=ARRAY) {
		execerror(sp->name,"array already defined");
		return((double*)NULL);
	}
	if (ndim==0) {		/* if array size was defined by initialization */
	    ndim = 1;
	    dim1 = (long int) *pc++;
	    d.val = dim1;	
	    pushm(d);
	}
	size = (ndim + 1) * sizeof(int) + sizeof (char *);
	apnt = sp->arrp = (array *)emalloc(size);/* allocate space for limits */
	if (!apnt) {
		execerror("no space for array ",sp->name);
		return((double*)NULL);
	}
	apnt->ndim = ndim;
	size = 1;
	for (i=0; i<ndim; i++) {
	  d = popm();				/* get dimensions from stack */
	  x = (int)(d.val);			/* this line fixes bug */
	  darr[(MAXDIM-1)-i] = x;		/* save backwards in temp arr */
	  size *= x;				/* */
	}
	for (i=0; i<ndim; i++) {
	  apnt->dim[i] = darr[MAXDIM-ndim+i];	/* put first dim first */
	}
	if (size == 0) {
		execerror("array dim is zero",sp->name);
		return((double*)NULL);
	}
	if (size < 0) {
		execerror("array dim is negative",sp->name);
		return((double*)NULL);
	}
	else if (size > MAXSIZE) {
		execerror("array too large",sp->name);
		return((double*)NULL);
	}
	dsize = size * sizeof(double);		
/*	fprintf(stderr,"size %u\n",dsize); 	/* */
	apnt->arr = (double *)emalloc(dsize);	/* allocate space for array */
	if (! apnt->arr) {
		execerror("run out of space for array ",sp->name);
		return ((double*)NULL);
	}
	for (i=0; i<size; i++) 
	  apnt->arr[i] = LARGENUM;	/* make array uninitialized */
	sp->type = ARRAY;
	return (apnt->arr);
}

/*------------------------------------------------------*/

array *garray(Symbol *sp)	/* return a pointer to whole array */
               
{
	datum d;
	int i,narg,darr[MAXDIM],ndim;
	int *dpnt,*tdpnt,*index,offset,x;
	double *parr;

	d = popm();				/* get number of indexes */
	narg = d.argnum;
	if (narg > 0) {
	   execerror("array shouldn't have index","");
	   return ((array*)NULL);
	}
	if (sp->arrp==NULL || *(double **)sp->arrp==NULL) {
	   execerror("array space not allocated for",sp->name);
	   return ((array*)NULL);
	}
	return (sp->arrp);	/* return pointer to array */
}

/*------------------------------------------------------*/

double
*garrayval(Symbol *sp)	/* return a pointer to an array value */
               
{
	datum d;
	int i,narg,darr[MAXDIM],ndim;
	int *tdpnt,*index,offset,x;
	array *apnt;
	double *parr;

	d = popm();				/* get number of indexes */
	narg = d.argnum;
	index = darr+MAXDIM;
	for (i=0; i<narg; i++) {
	  d = popm();				/* get indexes from stack */
	  x = (int)(d.val);
	  *(--index) = x;			/* save backwards in temp arr*/
	}
	if (sp->arrp==NULL || *(double **)sp->arrp==NULL) {
	   execerror("array space not allocated for",sp->name);
	   return ((double *)NULL);
	}
	parr = *(double **)sp->arrp;	/* get pointer to array */

	apnt = sp->arrp;
	ndim = apnt->ndim;			/* get # of dimensions */
	if (narg != ndim) {
	   execerror("array indices different than dimensions of",
					sp->name);
	   return ((double *)NULL);
	}
	index = darr+MAXDIM-narg;		/* pointer to first index */
	for (i=0; i<ndim; i++) {
	       int indx;
	  indx = *index++;
	  if (indx >= apnt->dim[i]) {
        fprintf 
	(stderr,"illegal index size for array '%s', dim %d, index=%d, max=%d\n",
		sp->name,i+1,indx,apnt->dim[i]-1);
            execerror("illegal index size for",sp->name);
	  }
	}
	offset = 0;
	index = darr+MAXDIM-narg;		/* pointer to first index */
	for (i=1; i<ndim; i++) {
	  offset += *index++;			/* add the index */
	  offset *= apnt->dim[i];		/* mult by next lower dim */
	}
	offset += *index;			/* add the final index */
	parr += offset;				/* calculate array pointer */
	return (parr);
}

/*------------------------------------------------------*/

void initarr(void)	/* initialize array */

{
	Symbol *sp;
	datum d1,*d;
	int arg,i,ndim,narg,ninit,size;
	double *dpnt;
        array *apnt;

#ifdef CODEBUG
  fprintf (stderr,"initarr\n");
#endif
	sp = (Symbol *)(*pc++);			/* pointer to array symbol */
	ninit = (long int) *pc++;		/* number of initialize vals */
	narg = (long int) sp;
	if (narg==0) {
	   d = stackp-1-ninit;		/* get local arr from stack */
	   apnt = d->arrp;
	   arg = 1;
	}
        else if (narg > 0 && narg < 1000) {  /* if var is arg to a function */
             d = &fp->argn[narg - fp->nargs];
	     apnt = d->arrp;
	     arg = 1;
	}
        else {
	     apnt = sp->arrp;
	     arg = 0;
	}
        ndim = apnt->ndim;                      /* get # of dimensions */

        for (size=1,i=0; i<ndim; i++) {		/* calculate array size */
          size *= apnt->dim[i];
	}
	for (i=ninit-1; i>=0; i--) {	/* initialize array */
	  d1=popm();
	  if (arg) d->vtype = d1.vtype;	/* set array type (number, etc.) */
	  else    sp->vtype = d1.vtype; /* set array type (number, etc.) */
	  if (i<size) 
	   switch (d1.vtype) {
	    case NUMBER:
	    case LITCHAR:
	      apnt->arr[i] = d1.val; 	/* set array val */
	      break;
	    case STRING:
	      *((char**)(&(apnt->arr[i]))) = d1.str; 	/* set array val */
	      break;
	   }
	}
	if (d1.vtype==STRING)
	  for (i=size-1; i>=ninit; i--) {	/* initialize string array */
              *((char**)(&(apnt->arr[i]))) = (char*)NULL;  /* init array val */
	  }
}

/*------------------------------------------------------*/
void dofft(void) /* fourier transform on an array */
		 /* array must be:   arr[length][2]
		    where arr is the array name;
		    and length is the length of the array.
		    arr[0] is transformed into the amplitude fft, and
	 	    arr[1] is transformed into the phase fft.
		*/
{
	int size1;
	double *realpt, *imagpt;
	Symbol *sp,*param;
	array *arrp;

#ifdef CODEBUG
  fprintf (stderr,"dofft\n");
#endif
	param = (Symbol *)(*pc++);		/* pointer to fft type symbol */
	sp = (Symbol *)(*pc++);			/* pointer to array symbol */
	arrp = sp->arrp;		/* get pointer to address of array */
	if (!arrp) {
	   execerror("fft: array space not allocated for",sp->name);
	   return;
	}
	realpt = *(double **)arrp;
	size1 = arrp->dim[1];				/* get second dim */
	imagpt = realpt + size1;
	fft (realpt,imagpt,size1,param->type);
}

/*------------------------------------------------------*/

void erarr (array *arrp)

{
  if (arrp) {
    efree((char *)*((double **)arrp));	/* erase array */
    efree((char *)arrp);		/* erase array descriptor */
  }
}

/*------------------------------------------------------*/

void erasarr(void)			/* erase array */

{
	Symbol *sp;
	
#ifdef CODEBUG
  fprintf (stderr,"erarr\n");
#endif
	sp = (Symbol *)(*pc++);			/* pointer to array symbol */
	switch (sp->type) {
	case  ARRAY:
		sp->type = UNDEF;
		erarr (sp->arrp);
		break;
	case VAR:    
		sp->val = LARGENUM;
		sp->type = UNDEF;
		break;
	default:
		break;
	}
}

/*------------------------------------------------------*/

double *retvar (char *arrname, int arg1, int arg2, int arg3, int narg)

/* Allow access to variable or array from outside simulator.
    Given name and indices of array, return pointer to element in array.
    If number of indices is 0, assume it is a normal variable.
*/

{
   Symbol *arrsym; 
   double *arrpnt;
   datum d0,d1,d2,d3;

  if (arrname) arrsym = lookup(arrname);
  if (!arrname || !arrsym) {
         fprintf (stderr,"retarr: invalid name for array\n");
         return ((double*)NULL);
  }
  switch (narg) {

    case 0:
          if (arrsym->type==VAR)
	      arrpnt = &arrsym->val;
	  else arrpnt = (double*)NULL;
	  break;
    case 1:
	  d1.val = arg1;
	  d0.sym = (Symbol *)narg;
	  pushm (d1);
	  pushm (d0);
	  arrpnt = garrayval(arrsym);
	  break;
    case 2:
	  d1.val = arg1;
	  d2.val = arg2;
	  d0.sym = (Symbol *)narg;
	  pushm (d1);
	  pushm (d2);
	  pushm (d0);
	  arrpnt = garrayval(arrsym);
	  break;
    case 3:
	  d1.val = arg1;
	  d2.val = arg2;
	  d3.val = arg3;
	  d0.sym = (Symbol *)narg;
	  pushm (d1);
	  pushm (d2);
	  pushm (d3);
	  pushm (d0);
	  arrpnt = garrayval(arrsym);
	  break;
    default:
	  arrpnt = (double*)NULL;
	  break;
  }
 return (arrpnt);
}

/*------------------------------------------------------*/

void varread(void)	/* read into variable */
{
	datum d1,d2;
	double *val;
	short *typ,*vtyp;
	char vbuf[120];
	int len;

#ifdef CODEBUG
  fprintf (stderr,"varread\n");
#endif
	d2 = *(stackp-1);
	getvar(&val,&typ,&vtyp);
	switch (*vtyp) {
	  case 0:
		switch (fscanf(stdin, "%lf", val)) {
		case EOF:
			d1.val = *val = 0.0;
			break;
		case 0:
			vbuf[0]=0;
			switch (fscanf(stdin, "%s",vbuf)) {
			case EOF:
				d1.val = *val = 0;
				break;
			case 0:
				execerror("non-string read into", d2.sym->name);
				break;
			default:
				d1.val = 1.0;
				if ((len=strlen(vbuf))>0) {
				  *((char **)val) = emalloc (len+1);
				  strcpy(*((char **)val),vbuf);
				}
				break;
			}
        		*vtyp = STRING;
			break;
		default:
			d1.val = 1.0;
        		*vtyp = NUMBER;
			break;
		}
		break;
	  case NUMBER:
	  default:
		switch (fscanf(stdin, "%lf", val)) {
		case EOF:
			d1.val = *val = 0.0;
			break;
		case 0:
			execerror("non-number read into", d2.sym->name);
			break;
		default:
			d1.val = 1.0;
			break;
		}
        	*vtyp = NUMBER;
		break;

	  case STRING:
		vbuf[0]=0;
		switch (fscanf(stdin, "%s",vbuf)) {
		case EOF:
			d1.val = *val = 0;
			break;
		case 0:
			execerror("non-string read into", d2.sym->name);
			break;
		default:
			d1.val = 1.0;
			if ((len=strlen(vbuf))>0) {
			  *((char **)val) = emalloc (len+1);
			  strcpy(*((char **)val),vbuf);
			}
			break;
		}
        	*vtyp = STRING;
		break;
	}
	/* *typ = VAR; */
	d1.vtype = NUMBER;
	d1.type = VAR;
	pushm(d1);
}


/*------------------------------------------------------*/

void xfread(void)

/* define and read array from file */
/* Can be 1 or 2 dimensional; 
   Number of lines defines first dimension;
   Number of elements on first line (first row) defines second dimension;
   Variables as well as numbers are allowed;
   but these variables must already exist in symbol table;
   Comment lines start with "#".
*/

#define STRSIZ 480

{
    char *filnam;
    FILE *ftemp;
    Symbol *var,*s;
    int i,j,narg,nwid,nlong;
    datum d1,d2,d3;
    char *cp,*np,*strp;
    char str[STRSIZ];
    double *dim1,*dim2,*dpnt;
    short *typ,*vtyp1,*vtyp2;
    char xbuf[150];

#ifdef CODEBUG
  fprintf (stderr,"xfread\n");
#endif
     var = (Symbol *)(*pc++);		/* array to be created */
     narg = (long int) *pc++;		/* number of var's (dimensions) */
     if (narg==2) getvar(&dim2,&typ,&vtyp2); /* get pointer to second dim */
     getvar(&dim1,&typ,&vtyp1);		/* get pointer to first dim */

     if (!checkstr(d1=popm())) {
	return; 
     }
     filnam = d1.str;		/* file to be read */
     if (strcmp(filnam,"stdin")==0) ftemp = stdin;
     else if ((ftemp=fopen(filnam,"r")) == NULL) {
           char ebuf[100];
	 sprintf (ebuf,"fread: can't open file '%.50s'.\n",filnam);
	 execerror ("warning,",ebuf);
	return;
     }
    while ((strp=fgets(str,STRSIZ,ftemp)) && *str== '#'); /* get first line */
    if (!strp) {
	 sprintf (xbuf,"fread: file '%.50s' is zero length.\n",filnam);
	 execerror ("warning,",xbuf);
	return;
    }

     for (cp=np=str,nwid=0; cp; nwid++) {	/* how many numbers on line */
        cp = strtok(np," ,\t\n");		/* get pointer to next token */
        np = (char *)NULL;
     }
     nwid--;
						/* how many lines in file */
    for (nlong=1; ; nlong++) {
      while ((strp=fgets(str,STRSIZ,ftemp)) && *str=='#');/* get next line */
      if (!strp) break;
    }

     rewind(ftemp);				/* go back to beginning */

     d1.val = nlong;
     d2.val = nwid;
     d1.type = d2.type = CONST;
     d1.vtype = d2.vtype = NUMBER;

     if (nwid > 1 && nlong > 1) {
	pushm(d1);
	pushm(d2);
	dpnt=darr2(var,2);
     }
     else {
	if (nlong > 1) pushm(d1);
	else 	       pushm(d2);
	dpnt=darr2(var,1);
     } 

   for (i=0; i<nlong; i++) {
     while ((strp=fgets (str,STRSIZ,ftemp)) && *str=='#');   /* get next line */
     if (!strp) return;
     np = str;
     for (j=0; j<nwid; j++) {
       if (nwid > 1 && nlong > 1) {
	  d3.sym = (Symbol *)2;		/* set up array indices */
	  d1.val = i;
	  d2.val = j;
	  pushm(d1);
	  pushm(d2);
	  pushm(d3);
       }
       else {
	  d3.sym = (Symbol *)1;
	  if (nlong>1) d1.val = i;
	  else         d1.val = j;
	  pushm(d1);
	  pushm(d3);
       } 
        cp = strtok(np," ,\t\n");
	if (cp==NULL) {
	    sprintf (xbuf,"Error in format of file while reading array.\n");
	    execerror ("warning,",xbuf);
	}
	np = (char *)NULL;	
	if (isalpha(*cp)) {
	    s = lookup(cp); 
	    if (s) *garrayval(var) = s->val;
	    else {
	    sprintf (xbuf,
		"# Missing def for var '%s' while reading array, set to 0.\n",
			cp);
	     execerror ("warning,",xbuf);
	     *garrayval(var) = 0;
	    }
	} 
       else   {  double tval;
            tval = atof(cp);
            *garrayval(var) = tval;
       }
     }   /* for j */
   }     /* for i */
  *dim1 = nlong;
  *vtyp1 = var->vtype = NUMBER;
  if(narg==2) {
	 *dim2 = nwid;
         *vtyp2 = NUMBER;
  }
}

/*------------------------------------------------------*/

void xfwrite(void)

/* write array to file.
 
   Array can be 1 or 2 dimensional; 
   Size of first dimension defines number of lines;
   Size of second dimension defines number of columns;
*/

{
    char *filnam;
    FILE *ftemp;
    Symbol *var;
    double *vpnt;
    int i,j;
    int dims,dim1,dim2,**apnt,*dpnt;
    datum d;

#ifdef CODEBUG
  fprintf (stderr,"xfwrite\n");
#endif
  if (!checkstr(d=popm())) {
	 pc+=2;
         return;
  }
  filnam = d.str;			/* file to be written */
  var = (Symbol *)(*pc++);		/* array */
  apnt = (int **)var->arrp;		/* pointer to pointer to array */
  dpnt = (int *)(apnt + 1);		/* pointer to number of dims */
  dims = *dpnt;				/* number of dimensions */
  dim1 = *(dpnt+1);			/* size of dimensions */
  dim2 = *(dpnt+2);
  vpnt = (double *)*apnt;

  if (strcmp(filnam,"stdout")==0) 
	ftemp = stdout;
  else if ((ftemp=fopen(filnam,"w")) == NULL) {
    fprintf (stderr,"fwrite: can't open file %s\n",filnam);
    return;
  }

  switch (dims) {

  case 2:
   for (i=0; i<dim1; i++) {
     for (j=0; j<dim2; j++,vpnt++) {
       if (j==(dim2-1))
          fprintf (ftemp,"%g",*vpnt);
       else
          fprintf (ftemp,"%g ",*vpnt);
     }
     fprintf (ftemp,"\n");
   }
   break;

   case 1:
     for (i=0; i<dim1; i++) {
          fprintf (ftemp,"%g\n",*vpnt++);
     }
    break;

    default:
    break;

  }  /* switch dims */

  fclose(ftemp);
}

/*------------------------------------------------------*/

void gplot(void)

/* low-level plotting commands */

{
   int narg,fill;
   Symbol *param;
   datum d1,d2;
   double x,y;
   double x1,y1,x2,y2,x3,y3,x4,y4;
   char *str=0;

#ifdef CODEBUG
  fprintf (stderr,"gplot\n");
#endif
 param  = (Symbol *)*pc++;
 narg = (long int) *pc++;
 if (param->type==GRECT) {
     d1 = popm(); fill = (int)(d1.val);
     d1 = popm(); y4 = d1.val;
     d1 = popm(); x4 = d1.val;
     d1 = popm(); y3 = d1.val;
     d1 = popm(); x3 = d1.val;
     d1 = popm(); y2 = d1.val;
     d1 = popm(); x2 = d1.val;
     d1 = popm(); y1 = d1.val;
     d1 = popm(); x1 = d1.val;
  }
 else if (param->type==GWINDOW) {
     d1 = popm(); y2 = d1.val;
     d1 = popm(); x2 = d1.val;
     d1 = popm(); y1 = d1.val;
     d1 = popm(); x1 = d1.val;
  }
 else
  if (narg > 1) {
   d2 = popm();
   y = d2.val;
 }	
 else y = 0;
 d1 = popm();
 switch (d1.vtype) {
   default:
   case 0:
   case NUMBER:
    x   = d1.val; break;
   case STRING:
    str = d1.str; break;
 }
#ifndef XSTIMM
 switch (param->type) {


   case GMOVE:  gmove (x,y);
		break;

   case GDRAW:  gdraw (x,y);
		break;

   case GRMOVE: grmove (x,y);
		break;

   case GRDRAW: grdraw (x,y);
		break;

   case GPEN:   gpen ((int) x);
		break;

#ifndef XSTIM
   case GVPEN:  plotpen (int(x), int(y-1));
		break;
#endif

   case GROT:   grotate (x);
		break;

   case GCROT:  gcrotate (x);
		break;

   case GORIG:  gorigin (x,y);
		break;

   case GFRAME:	if (str) gframe (str);
		break;

   case GCWID:	gcwidth (x);
		break;

   case GSIZ:	gsize (x);
		break;

   case GDASH:	gdash ((int)x);
		break;

   case GCIRC:	gcirc (x,(int)y);		/* radius, fill */
		break;

   case GWINDOW:gwindow (x1,x2,y1,y2);
		break;

   case GRECT:	grect (x1,y1,x2,y2,x3,y3,x4,y4,fill);
		break;
	}
#endif
}

/*------------------------------------------------------*/

#define GRPH

#ifdef GRPH

#include "ncplot.h"

extern plotfr plotnod[PLOTNODSIZ];  /* holds nodes to be recorded */
extern int numplots;            /* number of plots to be displayed */
extern int vidmode;		/* =1 -> graphics, =0 -> text */

void mplot (double y, double x, int n, int i, int pflag);
void varcopy();
void varcopyu();
void plotinit(int numplots);
void plotrst(int numplots);
void plotpen (int val, int i);
void plotchar (int val,int lines, int i);
void plotcsiz (double val, int numplots);
void plotfilt (int nfilt, float *timec, int numplots);
void plotvpen  (Symbol *vis, int numplots);

void grph(void)

/* make a graph */
{
  datum d1,d2;
  Symbol *param;
  static int narg,i,n,npops;
  static double xval,yval[PLOTNODSIZ];
  double setvar(char *str);
#define TTIMEC 10
  float ttimec[TTIMEC];

 param  = (Symbol *)*pc++;
 narg = (long int) *pc++;			/* (x,y1,y2,...,yn) */
 if (param == 0) {				/* get arguments for graph */
	n = narg - 1;
	if (n >= PLOTNODSIZ) n=PLOTNODSIZ-1;
/*	if (n > numplots) n = numplots; /* */
	for (i=n-1; i>=0; i-- ) {
	   d2 = popm();
	   yval[i] = d2.val;
	}
	checknum(d1 = popm());
	xval = d1.val;
#ifndef XSTIMM
	for (i=0; i<n; i++) {
	   mplot (yval[i],xval,n,i,1);
	}
#endif
 }
 else {
 vidmode = (int)setvar("vidmode");
 switch (param->type) {
   case X:			/* bug: can't use for indiv separate graphs */
   case Y:    npops = 2;
	      break;
   case SIZE: npops = 1;
	      break;
   case INIT:
   case FILT:
   case RESTART:
	      npops=0;
	      break;
   case PEN:
   case VPEN:
   case CHAR:
   case CCHAR:
	      npops = 0;
	      break;
 }

 if (npops==2) checknum(d2 = popm());
 if (npops>=1) checknum(d1 = popm());

#ifndef XSTIMM
 switch (param->type) {
   case X:			 /* bug: can't use for indiv separate graphs */
	if (!strcmp(gfrname(),rootframe))	 /* if root fr */
	      numplots = 0;		   /* number of plots for this graph */
	if (d1.val < d2.val) { double t; t = d1.val; d1.val=d2.val; d2.val=t; }
	plotnod[numplots].pxmax  = d1.val;
	plotnod[numplots].pxmin  = d2.val;
	plotnod[numplots].xrange = d1.val - d2.val;
	break;

   case Y:
	if (strcmp(gfrname(),rootframe))	 /* if diff than root frame */
	strcpy(plotnod[numplots].plframe,gfrname()); /* copy frame name */
	if (d1.val < d2.val) { double t; t = d1.val; d1.val=d2.val; d2.val=t; }
	plotnod[numplots].pymax  = d1.val;
	plotnod[numplots].pymin  = d2.val;
	plotnod[numplots].yrange = d1.val - d2.val;
	plotnod[numplots++].pmod  = GREC;
	break;

  case INIT:
	varcopy();
	plotinit(numplots);
	break;

  case RESTART:
	varcopy();
	plotrst(numplots);
	break;

  case PEN:
	if (narg>0) {			/* If on line by itself, */
	  for (i=narg; i>0; i-- ) {	/* set plot colors before graph init */
	   checknum(d1 = popm());	/* Can also be used w/graph*/
	   plotpen ((int)d1.val,i-1);
	  }
	}
	else {				/* If after "max min" then set color */
	   checknum(d1 = popm());	/*  for this plot only */
	   plotpen ((int)d1.val,numplots);
	}
	break;

  case CHAR:
	if (narg>0) {			/* If on line by itself, */
	  for (i=narg; i>0; i-- ) {	/*  set plot chars before graph init */
	   checknum(d1 = popm());	/*(Can also be used w/graphing)*/
	   plotchar ((int)d1.val,LINES,i-1);
	  }
	}
	else {				/* If after "max min" then set char */
	   checknum(d1 = popm());	/*  for this plot only */
	   plotchar ((int)d1.val,LINES,numplots);
	}
	break;

  case CCHAR:
	if (narg>0) {			/* If on line by itself, */
	  for (i=narg; i>0; i-- ) {	/*  set plot chars before graph init */
	   checknum(d1 = popm());	/*(Can also be used w/graphing)*/
	   plotchar ((int)d1.val,NOLINES,i-1);
	  }
	}
	else {				/* If after "max min" then set char */
	   checknum(d1 = popm());	/*  for this plot only */
	   plotchar ((int)d1.val,NOLINES,numplots);
	}
	break;
  case SIZE:
	plotcsiz (d1.val,numplots);
	break;

  case FILT:
	varcopyu();
        if (narg >= TTIMEC) narg = TTIMEC - 1;
        for (i=0; i<TTIMEC; i++) {
            ttimec[i] = 0;
        }
        for (i=narg-1; i>=0; i--) {
            checknum(d1 = popm());
            ttimec[i] = d1.val;
        }
#ifndef XSTIM
	plotfilt (narg,ttimec,numplots);
#endif		/* ! XSTIM */
	break;

  case VPEN: {
	  Symbol *vpen;
	vpen = (Symbol *)*(pc-1);
	varcopyu();
	
#ifndef XSTIM
	plotvpen (vpen,numplots);
#endif		/* ! XSTIM */
	}
	break;

  }  /* switch */
#endif		/* ! XSTIMM */

 } /* else */
}

#endif /* GRPH */

/*------------------------------------------------------*/

Inst *code(Inst f)	/* install one instruction or operand */
	       
{
	Inst *oprogp = progp;

	if (progp >= &prog[NPROG])
		execerror("program too big", (char *)0);
	*progp++ = f;
	return oprogp;
}

void execute(Inst *p)
{
	Inst func;

	for (pc = p; *pc != STOPC && !stopping; ) {
/*	fprintf (stderr,"%d %d\n",(int)*pc,(int)expop);  /* debug pc */
                func = *pc++;   /* inside func, pc must point to "arg"  */
                ((Inst )(*func))();
/*              (*(*pc++))();   /* old way, doesn't always work */
				/*  because pc is incr after func call */
	}

}

void edit(void)
{
    char sysbuf[40];
    datum d1;

#ifdef CODEBUG
  fprintf (stderr,"edit\n");
#endif
        if (!checkstr(d1=popm())) return;
	sprintf(sysbuf,"vi %s", d1.str);
	system (sysbuf);
}

/*------------------------------------*/

