/* control.h */

extern int nrecrd;
extern int nstim;
extern int silent;
extern int vidmode;
extern int disp;
extern int prmap;
extern int stim_elem;

/* default values for some constants */

extern double vk;
extern double vna;
extern double vcl;
extern double dnai;		/* internal [Na] = 12 mM, used for Na vrev */
extern double dnao;		/* external [Na] = 145 mM, used for Na vrev */
extern double dpkna;		/* K perm / Na perm = .09, used for Na vrev */
extern double dpcana;		/* Ca perm / Na perm = .09, used for Na vrev */
extern double dko;		/* internal [K] = 5 mM, used for K vrev */
extern double dki;		/* internal [K] = 130 mM, used for K,Ca vrev */
extern double dpnak;		/* Na perm / K perm = .02, used for K vrev */
extern double dpcak;		/* Ca perm / K perm = .01, used for K vrev */
extern double dpnaca;		/* Na permeability rel to Ca, for Ca vrev */
extern double dpkca;		/* K permeability rel to Ca, for Ca vrev */
extern double dclo;		/* external [Cl] = 125 mM, used for Cl vrev */
extern double dcli;		/* internal [Cl] = 10 mM, used for Cl vrev */
extern double dcap;		/* membrane capacitance */
extern double ddia;
extern double dlen;
extern double dri;		/* axial resistance */
extern double drg;		/* gap junction resistance */
extern double drm;		/* membrane resistance */
extern double dsfa;		/* synaptic filters 1 */
extern double dfta;		/* synaptic time constant */
extern double dftah;		/* high pass synaptic time constant */
extern double dsfb;		/* synaptic filters 2 */
extern double dftb;		/* synaptic time constant */
extern double dscu;		/* unitary conductance of chans */
extern double dsvn;		/* number of vesicle release sites */
extern double dvsz;		/* size of vesicles released */
extern double dst;		/* synaptic threshold */
extern double dsc;		/* synaptic curve */
extern double dskd;		/* half-max saturation gain */
extern double dshc;		/* synaptic postsyn Hill coeff */
extern double dstr;		/* transmitter conc for markov st receptors */
extern double dgkd;		/* half-max cG saturation */
extern double dmaxsyn;		/* maximum cond for synapse      */
extern double dsn;		/* synaptic neurotransmitter gain */
extern double dsg;		/* synaptic cycG gain */
extern double dsei;		/* synaptic input gain for expon  */
extern double dmaxrod;		/* maximum cond for rod           */
extern double dmaxcon;		/* maximum cond for cone          */
extern double dnadens;		/* density for Na chan (S/cm2)    */
extern double dkdens;		/* density for K chan (S/cm2)     */
extern double dcadens;		/* density for Ca chan (S/cm2)    */
extern double dmaxna;		/* maximum cond for Na channel    */ 
extern double dnau;		/* unitary cond for Na channel    */ 
extern double dku;		/* unitry cond for K channel      */ 
extern double dkau;		/* unitry cond for KA channel     */ 
extern double dkcabu;		/* unitry cond for Kca BK channel */ 
extern double dkcasu;		/* unitry cond for Kca SK channel */ 
extern double dkihu;		/* unitry cond for Ih channel     */ 
extern double dcalu;		/* unitry cond for Ca L channel   */ 
extern double dcatu;		/* unitry cond for Ca T channel   */ 
extern double dampau;		/* unitry cond for AMPA channel   */ 
extern double dcgmpu;		/* unitry cond for cGMP channel   */ 
extern double dmaxampa;		/* maximum cond for AMPA channel  */ 
extern double dmaxnmda;		/* maximum cond for NMDA channel  */ 
extern double dmaxgaba;		/* maximum cond for GABA channel  */ 
extern double dnaoffsm;		/* offset activation   voltage for NA chan */
extern double dnaoffsh;		/* offset inactivation voltage for NA chan */
extern double dmaxk;		/* maximum cond for K channel     */ 
extern double dkoffsn;		/* offset activation   voltage for K chan  */
extern double dkoffsh;		/* offset inactivation voltage for K chan  */
extern float dnataud;		/* tau for Na noise in 2-state model */
extern float dktaud;		/* tau for K noise in 2-state model */
extern float dnataum;		/* act. tau for NA chan (must be float) */
extern float dnatauh;		/* inact. tau for NA chan (must be float) */
extern float dktaum;		/* act. tau for K chan (must be float) */
extern float dktauh;		/* inact. tau for K chan (must be float) */
extern float dcataum;		/* act. tau for CA chan (must be float) */
extern float dcatauh;		/* inact. tau for CA chan (must be float) */
extern float dkcataum;		/* act. tau for Kca chan (must be float) */
extern float dkcatauh;		/* inact. tau for Kca chan (must be float) */
extern float dsyntau;		/* tau for syn2 channel */
extern float dnmdamg;		/* mg conc for NMDA chan */
extern float dqm;		/* Q10 for am, bm */
extern float dqh;		/* Q10 for ah, bh, normally 3 */
extern float dqn;		/* Q10 for an, bn, normally 3 */
extern float dqca;		/* Q10 for ac, bc (Ca chan) */
extern float dqd;		/* Q10 for ad, bd (KA h) */
extern float dqsyn;		/* Q10 for synaptic chans */
extern float dqcab;		/* Q10 for Ca buffer */
extern float dqkca;		/* Q10 for Kca chan */
extern float dqc;		/* Q10 for channel conductance */
extern float dqcavmax;		/* Q10 for Ca pump */
extern float dqrec;		/* Q10 for photoreceptor transduction shape */
extern float dqcrec;		/* Q10 for photoreceptor transduction cond */
extern float dgjoff;		/* gap junction voltage offset */
extern float dgjtau;		/* activation time const for gap junction */
extern float dgjnv;		/* non-volt-sens fraction of gj conductance */
extern double dcavoff;		/* Q10 for voffset from cao */
extern double dcaspvrev;	/* Ca surface potential for vrev(mult dcavoff) */
extern double ddca;		/* Diffusion const for internal calcium cm2/s */
extern double dcoo;		/* external cobalt concentration (for cavoff) */
extern double dmgo;		/* external magnesium conc (for cavoff) */
extern double dcao;		/* external calcium concentration */
extern double dcai;		/* internal calcium concentration */
extern double dcapkm;		/* 1/2 sat Ca conc for pump */
extern double dcavmax;		/* Vmax for Ca pump (ma/cm2) */
extern double dcakex;		/* Rate for Na/Ca exchanger */
extern double dcabnd;		/* Ratio of bound to free calcium */
extern double dcabt;		/* Ca buffer total in shells */
extern double dcabti;		/* Ca buffer total in first shell */
extern double dcabf;		/* forward rate to  Ca.B */
extern double dcabr;		/* reverse rate to Ca.B */
extern double dcaoffs;		/* Voltage offset for ca chan */
extern double dmaxca;		/* maximum cond for Ca channel    */ 
extern double dbd1;		/* default d1 for KCa BK channel    */ 
extern double dbd2;		/* default d2 for KCa BK channel    */ 
extern double dbk1;		/* default k1 for KCa BK channel    */ 
extern double dbk2;		/* default k2 for KCa BK channel    */ 
extern double dsd1;		/* default d1 for KCa SK channel    */ 
extern double dsd2;		/* default d2 for KCa SK channel    */ 
extern double dsk1;		/* default k1 for KCa SK channel    */ 
extern double dsk2;		/* default k2 for KCa SK channel    */ 

extern double complam;
extern double lamcrit;
extern double xtime;
extern double timinc;
extern double stiminc;
extern double tempcel;
extern double ktemp;
extern double ktrb;
extern double dpki;
extern double frt;
extern double f2rt;
extern double r2ft;
extern double djnoise;
extern double endexp;
extern double psplen;
extern double recrd;
extern double plmax;
extern double plmin;
extern double ploti;
extern double rate;
extern double stloc;
extern double crit;
extern double relax;
extern double relincr;
extern double synaptau;
extern double calcnernst;	/* =1 -> calc Nernst pots., not internal ions */

extern int    dcashell;		/* Number of Ca diffusion shells */
extern int    debug;
extern int    debugz;
extern int    poisfl;
extern int    scatter;
extern int    plsep;
extern int    allowrev;
extern int    dashfl;
extern int    euler;
extern int    implicit;
extern int    stimelem;
extern int rseed;
extern int prseed;
extern int drseed;
extern int ncversion[3];
