/* module drand in program nc */

/* random number function */

#include "ncsub.h"
#include "ncomp.h"

#ifdef __cplusplus
extern "C" {
#endif

#include <stdio.h>
int getpid(void);

#ifdef __cplusplus
}
#endif

int r_random(void);
char *rinitstate(unsigned int seed, char *state, int n);
char *rsetstate(char *state);
void execerror(char *s, char *t);

/*-----------------------------------------------------------*/

/* GNU-style random number generator is used because state table
   is readily changed and can be set to several different sizes.
   Call setrand(seed) first to initialize. 
*/

#define MAXRAND 2147483647.0
#define INVMAXRAND 4.6566128752457969231e-10		/* 1 / MAXRAND */
#define RANDSTATSIZ 256

static char _rstate[RANDSTATSIZ] = {0};

double drand(void)
{
  int r_random(void);

  return (r_random() * INVMAXRAND);
}

/*-----------------------------------------------------------*/

/* Call this function to invoke a numbered RNG */
 
double rrand(int ngen)
{
  randstate *rpnt;
  int r_random(void);
  randstate *findrand(int ngen);
  void restorstate();
  double retval;

  rpnt = findrand(ngen);
  if (rpnt) {
    rsetstate (rpnt->rstate);
    retval = r_random() * INVMAXRAND;
    restorstate();
  }
  else { 
   retval = -10;
  }
  return retval;
}


/*-----------------------------------------------------------*/

void restorstate(void)

/* Call this procedure to restore default random state. */

{
  rsetstate (_rstate);
}

/*-----------------------------------------------------------*/

void setrand(int val)

/* Call this prodecure with seed to initialize entire RNG. */

{
  if (val<=0) val = getpid();
  rinitstate (val,_rstate,RANDSTATSIZ); 
}

/*-----------------------------------------------------------*/

#define RHASHSIZ 199             /* prime number to make better hash */

randstate *rhashtab[RHASHSIZ] = {0};  /* noise generator hash table */

#ifdef __cplusplus
extern "C" {
#endif

#include <stdio.h>

#ifdef __cplusplus
}
#endif

extern int cumrand;

char *emalloc(unsigned int n);

/*---------------------------------------------------*/

void rinithash(void)
{
   int i;

   for (i=0; i<RHASHSIZ; i++) 
     rhashtab[i] = 0;
}

/*---------------------------------------------------*/

int randhash(int n)

/* Make disordered index of rand noise generator number */

{
   if (n<0) n = -n;
   return (n % RHASHSIZ); 
}

/*---------------------------------------------------*/

void initrand (int ngen, int rseed)
              
/* Initialize and install a noise generator in hash table */
/*  Uses "direct chaining" with field "next". */

{
    int i,found;
    randstate *rpnt,*rlast,*stpnt;
    static int rinitfl=0;

  /* look first for existing random noise generator */

  i = randhash(ngen);
  for (found=0,rpnt = rhashtab[i]; rpnt; rpnt = rpnt->next) {
    if (rpnt->ngen==ngen) {
       found = 1;
       break;
    }
  }

  if (!found) {
     if ((stpnt=(randstate *)emalloc(sizeof(randstate))) == NULL) {
       fprintf (stderr,"No space left for rand noise gen %d\n",ngen);
     }
   }
   else stpnt = rpnt;

   if (stpnt) {
	if (rseed==0) rseed = ngen;
	if (rseed==0) rseed = 1;
	rinitstate (rseed,stpnt->rstate,RNDSIZ);
   }
   ++cumrand;

   if (!rinitfl) {			/* initialize table once at start */
        rinitfl = 1;
        rinithash();
   }
   stpnt->next = 0;
   stpnt->ngen = ngen;
   i=randhash(ngen); 			/* initial index into nhashtab*/ 
   if (!(rpnt=rhashtab[i])) {           /* install directly in table */
      rhashtab[i] = stpnt;
   }
   else {                               /* otherwise, go to end of list */
      for (; rpnt; rpnt=rpnt->next)     /* find last one */
          rlast = rpnt;    
      rlast->next = stpnt;              /* put new one at end of list */
   }
}

/*---------------------------------------------------*/

void delrand (randstate *rpnt)

/* Delete a random noise generator, given a pointer to it. */

{
   int i,found;
   randstate *rlast,*rpt;
 
  if (!rpnt) return;

	/* First, delete from hash table: */

  i = randhash(rpnt->ngen);
  rlast = (randstate*)NULL;
  for (found=0, rpt=rhashtab[i]; rpt; rlast=rpt, rpt=rpt->next) {
    if (rpt==rpnt) {
        found = 1;
        break;
    }
  }
  if (found) {
     if (rlast) rlast->next = rpt->next;     /* delete hash pointer */
     else rhashtab[i] = rpt->next;
  }
}

/*---------------------------------------------------*/

randstate *findrand(int ngen)
                      
/* Find random noise generator among list of them. 
   They are placed in hash table, which provides
   faster access than possible with one sequential list.
*/

{
   randstate *rpnt;
   int i,found;
        
  i = randhash(ngen);
  for (found=0,rpnt = rhashtab[i]; rpnt; rpnt = rpnt->next) {
    if (rpnt->ngen==ngen) {
       found = 1;
       break;
    }
  }

  if (found) return rpnt;
  else {
     fprintf (stderr,"findrand: can't find random noise gen %d\n",ngen);
  }
  return (randstate *)NULL; 
}

/*---------------------------------------------------*/

