/* init.c */

#include "nc.h"
#include "y.tab.h"
#include "ncval.h"
#include "ncsub.h"
#include "ncelem.h"
#include "controlx.h"
#include <string.h>

double iono[NPERM] = {0};	/* external ion concentrations */
double ioni[NPERM] = {0};	/* internal ion concentrations */
int    ionz[NPERM] = {0};	/* ion valences */

extern double varval[VVSIZE];
extern char *varpnt[VVSIZE];
extern int vartyp[VVSIZE];
extern int varset;
extern char *progname;
extern int setdisp;
extern int setprmap;
extern int setvid, unsetvid;
extern int pipefl;
extern int interp_only;
extern char *rootframe;

extern datum	ncasin(datum &x), ncatan(datum &x), nclog(datum &x);
extern datum	nclog10(datum &x), ncsqrt(datum &x), ncexp(datum &x);
extern datum	ncexp10(datum &x);
extern datum	ncsin(datum &x), nccos(datum &x), nctan(datum &x);
extern datum    ncfabs(datum &a), ncinteger(datum &x);
extern datum    ncmax(datum &a, datum &b), ncmin(datum&a, datum&b);
extern datum	ncrand(void), ncrrand(datum &ngen);
extern datum	ncpow(datum &x, datum &y), ncabs(double &x), elap_time(void);
extern datum	setvarval(void), printsym(void), xstrlen(datum &d);
extern datum	xstrtok(datum &d1,datum &d2), xstrcmp(datum &d1, datum &d2);
extern datum	xstrstr(datum &d1,datum &d2);
extern datum	ncgaus(datum &x, datum &r), ncpoisdev (datum &x);
extern datum	ncgasdev(void), ncrgasdev(datum &ngen);
extern datum	minf(datum &v, datum &typ);
extern datum	mtau(datum &v, datum &typ);
extern datum	hinf(datum &v, datum &typ);
extern datum	htau(datum &v, datum &typ);
extern datum	ninf(datum &v, datum &typ);
extern datum	ntau(datum &v, datum &typ);
extern datum	dinf(datum &v, datum &typ);
extern datum	dtau(datum &v, datum &typ);
extern datum	cinf(datum &v, datum &typ);
extern datum	ctau(datum &v, datum &typ);
extern datum	ctaua(datum &v, datum &typ);
extern datum	ctaub(datum &v, datum &typ);
extern datum	chinf(datum &v,datum &typ);
extern datum	chtau(datum &v,datum &typ);
extern datum	chtaua(datum &v,datum &typ);
extern datum	chtaub(datum &v,datum &typ);

Symbol *timeptr;
Symbol *ncompptr;
Symbol *plotiptr;

#ifdef __cplusplus
extern "C" {
#endif

#include "stdplt.h"
#include "gr.h" 
int getpid(void);
#ifdef CPML
#include <cpml.h>
#else
#include <math.h>
#endif

#ifdef __cplusplus
}
#endif

double lookpv(char *s);
Symbol *lookup(char *s);
void erasymb();
void setval(Symbol *sym, double val, int vtyp);
void dinit();
void varcopy();
void initrec();
void setrand (int rseed);
void iinit();

/*--------------------------------------------------------*/

static struct {		/* Keywords */
	char	*name;
	int	kval;
} keywords[] = {
	"proc",		PROC,
	"func",		FUNC,
	"return",	RETURN,
	"exit",		EXIT,
	"debugf",	DEBUGF,
	"if",		IF,
	"else",		ELSE,
	"while",	WHILE,
	"print",	PRINT,
	"printf",	PRINTF,
	"sprintf",	SPRINTF,
	"fprintf",	FPRINTF,
	"fopen",	FOPEN,
	"fclose",	FCLOSE,
	"scanf",	SCANF,
	"fscanf",	FSCANF,
	"sscanf",	SSCANF,
	"fgets",	FGETS,
	"read",		READ,
	"fread",	FREAD,
	"fwrite",	FWRITE,
	"notinit",	NOTINIT,
	"dim",		DIM,
	"fft",		FFT,
	"ifft",		IFFT,
	"pfft",		PFFT,
	"acov",		ACOV,
	"foreach",	FOREACH,
	"for",		FOR,
	"break",	BREAK,
	"continue",	CONTINUE,
	"edit",		EDIT,
	"system",	SYSTEM,
	"include",	INCLUDE,
	"local",	LOCAL,
	"graph",	GRAPH,
	"pen",		PEN,
	"restart",	RESTART,
	"init",		INIT,
	"X",		X,
	"Y",		Y,
	"Z",		Z,
	"max",		MAX,
	"min",		MIN,
	"char",		CHAR,
	"Char",		CCHAR,
	"vpen",		VPEN,
	"gmove",	GMOVE,
	"gdraw",	GDRAW,
	"grdraw",	GRDRAW,
	"grmove",	GRMOVE,
	"gpen",		GPEN,
	"gvpen",	GVPEN,
	"gcwidth",	GCWID,
	"gsize",	GSIZ,
	"gdash",	GDASH,
	"gwindow",	GWINDOW,
	"gframe",	GFRAME,
	"grot",		GROT,
	"gorigin",	GORIG,
	"gcrot",	GCROT,
	"gtext",	GTEXT,
	"gcirc",	GCIRC,
	"grect",	GRECT,
	0,		0
};

static struct {		/* Keywords */
	char	*name;
	int	kval;
} simwords[] = {
	"element",	ELEMENT,
	"matching",	MATCHING,
	"except",	EXCEPT,
	"comps",	COMPS,
	"range",	RANGE,
	"cable",	CABLE,
	"sphere",	SPHERE,
	"chan",		CHAN,
	"Na",		NA,
	"K",		K,
	"KCa",		KCa,
	"type",		TYPE,
	"density",	DENSITY,
	"ndensity",	NDENSITY,
	"tau",		TAU,
	"taum",		TAUM,
	"tauh",		TAUH,
	"taun",		TAUN,
	"taua",		TAUM,
	"taub",		TAUH,
	"tauc",		TAUC,
	"taud",		TAUD,
	"Ca",		CA,
	"cacomp",	CACOMP,
	"cao",		CAO,
	"cai",		CAI,
	"cbound",	CBOUND,
	"cabuf",	CABUF,
	"btot",		BTOT,
	"btoti",	BTOTI,
	"cshell",	CASHELL,
	"caperm",	CAPERM,
	"vmax",		VMAX,
	"km",		KM,
	"kex",		KEX,
	"capump",	CAPUMP,
	"caexch",	CAEXCH,
	"d1",		D1,
	"d2",		D2,
	"k1",		K1,
	"k2",		K2,
	"mg",		MG,
	"gj",		GJ,
	"puff",		PUFF,
	"GLU",		GLU,
	"glu",		GLU,
	"AMPA",		AMPA,
	"ampa",		AMPA,
	"NMDA",		NMDA,
	"nmda",		NMDA,
	"CNQX",		CNQX,
	"cnqx",		CNQX,
	"GABA",		GABA,
	"gaba",		GABA,
	"BIC",		BIC,
	"bic",		BIC,
	"PTX",		PTX,
	"ptx",		PTX,
	"GLY",		GLY,
	"gly",		GLY,
	"STRY",		STRY,
	"stry",		STRY,
	"cAMP",		CAMP,
	"camp",		CAMP,
	"cGMP",		CGMP,
	"cgmp",		CGMP,
	"resp",		RESP,
	"syn2",		SYN2,
	"mesgin",	MESGIN,
	"mesgout",	MESGOUT,
	"synapse",	SYNAPSE,
	"load",		LOAD,
	"resistor",	RESISTOR,
	"transducer",	TRANSDUCER,
	"rod",		ROD,
	"cone",		CONE,
	"dyad",		DYAD,
	"sdyad",	SDYAD,
	"scurve",	SCURVE,
	"numdyad",	NUMDYAD,
	"rsd",		RSD,
	"initrand",	INITRAND,
	"maxcond",	MAXCOND,
	"length",	LENGTH,
	"area",		AREA,
	"dia",		DIA,
	"dia1",		DIA,
	"dia2",		DIA2,
	"radius",	RADIUS,
	"width",	WIDTH,
	"stim",		STIM,
	"bar",		BAR,
	"spot",		SPOT,
	"rect",		RECT,
	"sine",		SINE,
	"gabor",	GABOR,
	"xenv",		XENV,
	"yenv",		YENV,
	"xenon",	XENON,
	"sun",		SUN,
	"tungsten",	TUNGSTEN,
	"wavel",	WAVEL,
	"pigm",		PIGM,
	"pathl",	PATHL,
	"attf",		ATTF,
	"filt",		FILT,
	"save",		SAVE,
	"restore",	RESTORE,
	"gndbatt",	GNDBATT,
	"batt",		BATT,
	"cap",		CAP,
	"gndcap",	GNDCAP,
	"node",		NODE,
	"node1a",	NODE1A,
	"node1b",	NODE1B,
	"node1c",	NODE1C,
	"node1d",	NODE1D,
	"node2a",	NODE2A,
	"node2b",	NODE2B,
	"node2c",	NODE2C,
	"node2d",	NODE2D,
	"vclamp",	VCLAMP,
	"cclamp",	CCLAMP,
	"loc",		LOC,
	"xloc",		XLOC,
	"yloc",		YLOC,
	"zloc",		ZLOC,
	"center",	CENTER,
	"sscale",	SSCALE,
	"dur",		DUR,
	"tfreq",	TFREQ,
	"drift",	DRIFT,
	"orient",	ORIENT,
	"sphase",	SPHASE,
	"contrast",	CONTRAST,
	"start",	START,
	"inten",	INTEN,
	"backgr",	BACKGR,
	"blur",		BLUR,
	"file",		SFILE,
	"plot",		PLOT,
	"V",		V,
	"I",		I,
	"L",		L,
	"S",		S,
	"FA0",		FA0,
	"FA1",		FA1,
	"FA2",		FA2,
	"FA3",		FA3,
	"FA4",		FA4,
	"FA9",		FA9,
	"FH0",		FH0,
	"FH1",		FH1,
	"FH2",		FH2,
	"FH3",		FH3,
	"FH4",		FH4,
	"FB0",		FB0,
	"FB1",		FB1,
	"FB2",		FB2,
	"FB3",		FB3,
	"FB4",		FB4,
	"FC0",		FC0,
	"FC1",		FC1,
	"FC2",		FC2,
	"FC3",		FC3,
	"FC4",		FC4,
	"G",		G,
	"M",		M,
	"H",		H,
	"G0",		G0,
	"G1",		G1,
	"G2",		G2,
	"IP",		IP,
	"IE",		IE,
	"IPE",		IPE,
	"display",	DISPLAY,
	"xrot",		XROT,
	"yrot",		YROT,
	"zrot",		ZROT,
	"size",		SIZE,
	"window",	WINDOW,
	"rmove",	RMOVE,
	"dscale",	DSCALE,
	"color",	COLOR,
	"hide",		HIDE,
	"calibline",	CALIBLIN,
	"rm",		RM,
	"ri",		RI,
	"rg",		RG,
	"cm",		CM,
	"jnoise",	JNOISE,
	"vbuf",		BUF,
	"opamp",	OPAMP,
	"chnoise",	CHNOISE,
	"photnoise",	PHOTNOISE,
	"darknoise",	DARKNOISE,
	"vesnoise",	VESNOISE,
	"N",		N,
	"unit",		UNIT,
	"vsize",	VSIZE,
	"refr",		REFR,
	"CoV",		COV,
	"open",		OPEN,
	"close",	CLOSE,
	"vrev",		VREV,
	"vrest",	VREST,
	"cplam",	CPLAM,
	"thresh",	THRESH,
	"delay",	DELAY,
	"nfilt1",	NFILT1,
	"nfilt1h",	NFILT1H,
	"nfilt2",	NFILT2,
	"nfilt3",	NFILT3,
	"timec1",	TIMEC1,
	"timec1h",	TIMEC1H,
	"timec2",	TIMEC2,
	"tfall2",	TFALL2,
	"timec3",	TIMEC3,
	"tfall3",	TFALL3,
	"hgain",	HGAIN,
	"cgain",	CGAIN,
	"rev",		REV,
	"vgain",	VGAIN,
	"gmax",		GMAX,
	"gnv",		GNV,
	"kd",		KD,
	"hcof",		HCOF,
	"linear",	LINEAR,
	"expon",	EXPON,
	"trconc",	TRCONC,
	"numconn",	NUMCONN,
	"connect",	CONNECT,
	"conn",		CONNECT,
	"only",		ONLY,
	"at",		AT,
	"to",		TO,
	"edist",	EEDIST,
	"e3dist",	E3DIST,
	"e2dist",	E2DIST,
	"efrac",	EFRAC,
	"ndist",	N3DIST,
	"n3dist",	N3DIST,
	"n2dist",	N2DIST,
	"ename",	ENAME,
	"modify",	MODIFY,
	"offset",	OFFSET,
	"offsetm",	OFFSETM,
	"offseth",	OFFSETH,
	"put",		PUT,
	"run",		RUN,	
	"step",		STEP,	
	"elimit",	ELIMIT,
	"erase",	ERASE,
	"all",		ALL,
	"model",	MODEL,
	"gausnn",	GAUSNN,
	"mean",		MEAN,
	"stdev",	STDEV,
	"reg",		REG,
	0,		0
};
	
static struct {		/* control vars */
	char	*name;
} controls[] = {
	"vidmode",
	"debug",
	"debugz",
	"rseed",
	"prseed",
	"stderr",
	"stdin",
	"stdout",
	"version",
	0,0
};

static struct {		/* control vars */
	char	*name;
} scontrols[] = {
	"vk",
	"vna",
	"vcl",
	"dnai",
	"dnao",
	"dko",
	"dki",
	"dpnaca",
	"dpkca",
	"dpkna",
	"dpcana",
	"dpnak",
	"dpcak",
	"dcli",
	"dclo",
	"dcap",
	"dnadens",
	"dkdens",
	"dcadens",
	"dcavoff",
	"dcaspvrev",
	"ddca",
	"dcoo",
	"dmgo",
	"dcao",
	"dcai",
	"dcdco",
	"dcabnd",
	"dcabt",
	"dcabti",
	"dcabf",
	"dcabr",
	"dcashell",
	"dcapkm",
	"dcavmax",
	"dcakex",
	"dmaxca", 
	"dcaoffs", 
	"dcataum",
	"dcatauh",
	"dkcataum",
	"dkcatauh",
	"dnataud",
	"dktaud",
	"dqm",
	"dqh",
	"dqn",
	"dqca",
	"dqd",
	"dqkca",
	"dqsyn",
	"dqcavmax",
	"dqcab",
	"dqc",
	"dqrec",
	"dqcrec",
	"drg",
	"dri",
	"drm",
	"dfta",
	"dsfa",
	"dftb",
	"dsfb",
	"dscu",
	"dsvn",
	"dvsz",
	"dst",
	"dsc",
	"dmaxsyn", 
	"dskd",
	"dshc",
	"dstr",
	"dgkd",
	"dsn", 
	"dsg", 
	"dmaxrod", 
	"dmaxcon", 
	"dmaxna", 
	"dmaxampa", 
	"dmaxnmda", 
	"dnmdamg", 
	"dmaxgaba", 
	"dnaoffsm", 
	"dnaoffsh", 
	"dnataum",
	"dnatauh",
	"dktaum",
	"dktauh",
	"dsyntau",
	"dgjoff", 
	"dgjtau", 
	"dgjnv", 
	"dbd1", 
	"dbd2", 
	"dbk1", 
	"dbk2", 
	"dsd1", 
	"dsd2", 
	"dsk1", 
	"dsk2", 
	"dmaxk", 
	"dnau",
	"dku", 
	"dkau", 
	"dkcabu", 
	"dkcasu", 
	"dkihu", 
	"dcatu", 
	"dcalu", 
	"dampau", 
	"dcgmpu", 
	"dkoffsn", 
	"dkoffsh", 
	"tempcel",
	"silent",
	"label",
	"disp",	
	"prmap",	
	"stim_elem",	
	"complam",
	"lamcrit",
	"relincr",
	"timinc",	
	"stiminc",	
	"time",	
	"ncomps",	
	"endexp",	
	"ploti",
	"crit",		
	"plmax",
	"plmin",
	"relax",		
	"plsep",		
	"djnoise",		
	"rightyax",		
	"allowrev",		
	"dash",		
	"euler",		
	"implicit",		
	"stimelem",		
	"synaptau",		
	"scatter",
	"calcnernst",
	0,0
};


static struct {		/* Constants */
	char *name;
	double cval;
} chtyp[] = {
	"PI", 		3.14159265358979323846,
	"E",		2.71828182845904523536,
	"GAMMA",	0.57721566490153286060,	/* Euler */
	"DEG",	       57.29577951308232087680,	/* deg/radian */
	"PHI",		1.61803398874989484820,	/* golden ratio */
	(char *)0,	0
};

struct builtin {		/* Built-ins */
	char *name;
	Fpa  func;
};

static builtin builtins[] = {
	"atan",	(Fpa )ncatan,
	"asin",	(Fpa )ncasin,
	"log",	(Fpa )nclog,	/* checks range */
	"log10", (Fpa )nclog10,/* checks range */
	"exp",	(Fpa )ncexp,	/* checks range */
	"exp10",(Fpa )ncexp10,	/* checks range */
	"pow",	(Fpa )ncpow,	/* checks range */
	"sin",	(Fpa )ncsin,	/* checks range */
	"cos",	(Fpa )nccos,	/* checks range */
	"tan",	(Fpa )nctan,	/* checks range */
	"sqrt",	(Fpa )ncsqrt,	/* checks range */
	"gauss",(Fpa )ncgaus,	/* checks range */
	"poisdev", (Fpa )ncpoisdev,/* checks range */
	"gasdev", (Fpa )ncgasdev,/* checks range */
	"rgasdev", (Fpa )ncrgasdev,/* checks range */
	"int",	(Fpa )ncinteger,
	"abs",	(Fpa )ncfabs,
	"amax",	(Fpa )ncmax,
	"amin",	(Fpa )ncmin,
	"rand",	(Fpa )ncrand,
	"rrand",(Fpa )ncrrand,
	"setvar", (Fpa )setvarval,
	"printsym", (Fpa )printsym,
	"elap_time", (Fpa )elap_time,
	"strlen", (Fpa )xstrlen,
	"strtok", (Fpa )xstrtok,
	"strcmp", (Fpa )xstrcmp,
	"strstr", (Fpa )xstrstr,
	(char *)0, (Fpa )0
};

static builtin sbuiltins[] = {
	"minf", (Fpa )minf,
	"mtau", (Fpa )mtau,
	"hinf", (Fpa )hinf,
	"htau", (Fpa )htau,
	"ninf", (Fpa )ninf,
	"ntau", (Fpa )ntau,
	"dinf", (Fpa )dinf,
	"dtau", (Fpa )dtau,
	"cinf", (Fpa )cinf,
	"ctau", (Fpa )ctau,
	"ctaua",(Fpa )ctaua,
	"ctaub",(Fpa )ctaub,
	"chinf",(Fpa )chinf,
	"chtau",(Fpa )chtau,
	"chtaua",(Fpa )chtaua,
	"chtaub",(Fpa )chtaub,
	(char *)0, (Fpa )0
};

void init(void)	/* install constants and built-ins in table */
{
	int i;
	Symbol *s;
	datum d, setvarval(void);

        if (strcmp(progname,"nci")==0) interp_only = 1;
	erasymb();			/* erase previous defs, in "symbol.c" */
	for (i=0; keywords[i].name; i++)
		s= install(keywords[i].name, keywords[i].kval, 0.0);
	for (i=0; controls[i].name; i++)
		s= install(controls[i].name, VAR, 0.0);
	for (i=0; chtyp[i].name; i++) {
		s = install(chtyp[i].name, CONST, chtyp[i].cval);
		setval (s,chtyp[i].cval,NUMBER);
	}
	for (i=0; builtins[i].name; i++) {
		s = install(builtins[i].name, BLTIN, 0.0);
		s->ptr = builtins[i].func;
	}
	if (!interp_only) {	/* simulator things */
	  for (i=0; simwords[i].name; i++)
		s= install(simwords[i].name, simwords[i].kval, 0.0);
	  for (i=0; scontrols[i].name; i++)
		s= install(scontrols[i].name, VAR, 0.0);
	  for (i=0; sbuiltins[i].name; i++) {
		s = install(sbuiltins[i].name, BLTIN, 0.0);
		s->ptr = sbuiltins[i].func;
	  }
	  dinit();		/* initialize default values */
	}
	iinit();		/* interpreter things */
	d = setvarval();	/* set variables from command-line */
	varcopy();		/* copy control variables from symbol table */
	if (!interp_only) {	/* simulator things */
	  initrec();		/* receptor constants */
	}
	frame (rootframe);
}

/****************************************/

double setvar (char *str)
{
  return (lookpv(str));
}

/****************************************/

datum setvarval(void)

/* set variables from command line.
    This overrides what values are set
    in a program.  You call this function
    "setvar()" your script file.

*/
{
   int i;
   Symbol *s;
   static datum d={0};
#define NR NUMBER

   for (i=0; i<VVSIZE; i++) {
    if (varpnt[i]) {
      if ((s=lookup(varpnt[i])) == 0)
           s=install(varpnt[i],VAR,varval[i]);
      if (s->type==VAR)  		/* set var from comnd line */
	   setval(lookup(varpnt[i]),varval[i],vartyp[i]);
    }
    else break;
   }
   setval (lookup("vidmode"), (double)vidmode,NR);	/* video display flag */
   if (setdisp>=0) disp = setdisp;	/* command line display flag */
   setval (lookup("disp"), (double)disp,NR); /* display flag */
   if (setprmap>=0) prmap = setprmap;	/* command line print flag */
   setval (lookup("prmap"),(double)prmap,NR); /* display flag */
   if (drseed) {
     if (drseed<0) drseed = getpid();	/* randomize if negative */
     setval (lookup("rseed"),(double)drseed,NR); /* random seed on comnd line*/
   }
  d.val = i;
  d.vtype = NUMBER;
  return d;
}

/****************************************/

double nernstv (double cratio)

/* calculate Nernst potential for a monovalent cation */

{
   return (RF * ktemp * log (cratio)); 
}

/****************************************/

double nernstc (double vrev, double co)

/* calculate internal concentration for a monovalent cation */

{
   return (co / exp(vrev * FR / ktemp)); 
}

/****************************************/

void setrseed(void)

/* run before any random sequence is generated */

{
    static int oldrseed = 0;

  rseed   = (int)setvar ("rseed");
  if (rseed != oldrseed) {
	oldrseed = rseed;
	setrand ((int) rseed);		/* set up random number seed */
  }
}

/****************************************/

void setion(void)

/* set concentrations of internal ions */

{
    int i;

 for (i=0; i<NPERM; i++) {
   iono[i] = ioni[i] = 0.0;  
   ionz[i] = 1;
 }

 iono[PNA] = dnao;
 ioni[PNA] = dnai;
 iono[PK]  = dko;
 ioni[PK]  = dki;
 iono[PCA] = dcao;
 ioni[PCA] = dcai;
 ionz[PCA] = 2;
 iono[PCL] = dclo;
 ioni[PCL] = dcli;
 ionz[PCL] = -1;
}

/****************************************/

void setvrevs(void)

/* If Nernst potential for ions is not set, then create it from */
/*  their internal and external concentrations. */

/* Initially, if Nernst potential is set, then set the internal ion concs.  */
/*  from the external value and Nernst potential. */

/* If "calcnernst" = 1, when temperature or ion concentrations */
/* change, calculate new Nernst potential (vna, vk, and vcl). */

/* If "calcnernst" = 0, when temperature or Nernst potential values */
/* change, calculate new internal ion concentrations. */

/* If "calcnernst" is between 0 and 1, when temperature or ion
 * concentrations change, change Nernst potential
 * by the amount specified in calcnernst, and then make up the rest
 * of the change from ion concentration change */

{
   int i;

 if (((vna=setvar("vna"))==LARGENUM) || calcnernst) {
     double ovna=vna;
   vna = nernstv(dnao/dnai);
   if (ovna==LARGENUM) ovna = vna;
   if ((calcnernst > 0) && (calcnernst < 1)) {
      vna = ovna + (vna-ovna)*calcnernst;
      dnai = nernstc(vna,dnao);
      setval (lookup("dnai"),dnai,NR);  /* sodium internal concentration */
   }
   setval (lookup("vna"),vna,NR);       /* sodium equilibrium potential    */
 }
 else {
   dnai = nernstc(vna,dnao);
   setval (lookup("dnai"),dnai,NR);	/* sodium internal concentration */
 }

 if (((vk=setvar("vk"))==LARGENUM) || calcnernst) {
     double ovk=vk;
   vk = nernstv(dko/dki);
   if (ovk==LARGENUM) ovk = vk;
   if ((calcnernst > 0) && (calcnernst < 1)) {
      vk = ovk + (vk-ovk)*calcnernst;
      dki = nernstc(vk,dko);
      setval (lookup("dki"),dki,NR);  /* sodium internal concentration */
   }
   setval (lookup("vk"),vk,NR);		/* potassium equilibrium potential */
 }
 else {
   dki = nernstc(vk,dko);
   setval (lookup("dki"),dki,NR);	/* potassium external concentration */
 }

 if (((vcl=setvar("vcl"))==LARGENUM) || calcnernst) {
     double ovcl=vcl;
   vcl = nernstv(dcli/dclo);
   if (ovcl==LARGENUM) ovcl = vcl;
   if ((calcnernst > 0) && (calcnernst < 1)) {
      vcl = ovcl + (vcl-ovcl)*calcnernst;
      dcli = nernstc(-vcl,dclo);
      setval (lookup("dcli"),dcli,NR);  /* sodium internal concentration */
   }
   setval (lookup("vcl"),vcl,NR);	/* chloride equilibrium potential */
 }
 else {
   dcli = nernstc(-vcl,dclo);
   setval (lookup("dcli"),dcli,NR);	/* chloride internal concentration */
 }
}

/****************************************/

void varcopy(void)
{
   double t;

  debug   = (int)setvar ("debug");
  debugz  = (int)setvar ("debugz");
  prseed   = (int)setvar ("prseed");
  setrseed();
  if (interp_only) return;

  calcnernst = setvar ("calcnernst");
  dnai    = setvar ("dnai");
  dnao    = setvar ("dnao");
  dko     = setvar ("dko");
  dki     = setvar ("dki");
  dpkna   = setvar ("dpkna");
  dpcana  = setvar ("dpcana");
  dpnak   = setvar ("dpnak");
  dpcak   = setvar ("dpcak");
  dpnaca  = setvar ("dpnaca");
  dpkca   = setvar ("dpkca");
  dcli    = setvar ("dcli");
  dclo    = setvar ("dclo");
  dcoo    = setvar ("dcoo");
  dmgo    = setvar ("dmgo");
  dcao    = setvar ("dcao");
  dcai    = setvar ("dcai");
  tempcel = setvar ("tempcel");
  ktemp = tempcel + KELVIN;
  frt   = FR / ktemp;
  f2rt  = F2R / ktemp;
  r2ft  = R2F * ktemp;

  setvrevs();
  setion();

  dpki  = dpkca * dki * CAZI;	     /* = perm K / perm Ca * [K]i / 4   */
				     /* see Hille 2nd ed, p 107 & p 344 */
  dcap    = setvar ("dcap");
  dnadens = setvar ("dnadens");
  dkdens  = setvar ("dkdens");
  dcadens = setvar ("dcadens");
  dcavoff = setvar ("dcavoff");
  dcaspvrev= setvar ("dcaspvrev");
  ddca    = setvar ("ddca");
  dcabnd  = setvar ("dcabnd");
  dcabt   = setvar ("dcabt");
  dcabti  = setvar ("dcabti");
  dcabf   = setvar ("dcabf");
  dcabr   = setvar ("dcabr");
  dcashell= (int)setvar ("dcashell");
  dcapkm  = setvar ("dcapkm");
  dcavmax = setvar ("dcavmax");
  dcakex  = setvar ("dcakex");
  dmaxca  = setvar ("dmaxca");
  dcaoffs  = setvar ("dcaoffs");
  dcataum  = setvar ("dcataum");
  dcatauh  = setvar ("dcatauh");
  dkcataum = setvar ("dkcataum");
  dkcatauh = setvar ("dkcatauh");
  dnataud  = setvar ("dnataud");
  dktaud   = setvar ("dktaud");
  dqm 	   = setvar ("dqm");
  dqh 	   = setvar ("dqh");
  dqn 	   = setvar ("dqn");
  dqca 	   = setvar ("dqca");
  dqd 	   = setvar ("dqd");
  dqkca    = setvar ("dqkca");
  dqsyn    = setvar ("dqsyn");
  dqcavmax = setvar ("dqcavmax");
  dqcab    = setvar ("dqcab");
  dqc 	   = setvar ("dqc");
  dqrec    = setvar ("dqrec");
  dqcrec   = setvar ("dqcrec");
  dri     = setvar ("dri");
  drm     = setvar ("drm");
  drg     = setvar ("drg");
  dsfa    = setvar ("dsfa");
  dfta    = setvar ("dfta");
  dsfb    = setvar ("dsfb");
  dftb    = setvar ("dftb");
  dscu    = setvar ("dscu");
  dsvn    = setvar ("dsvn");
  dvsz    = setvar ("dvsz");
  dst     = setvar ("dst");
  dsc     = setvar ("dsc");
  dmaxsyn = setvar ("dmaxsyn");
  dskd    = setvar ("dskd");
  dshc    = setvar ("dshc");
  dstr    = setvar ("dstr");
  dgkd    = setvar ("dgkd");
  dsg     = setvar ("dsg");
  dsn     = setvar ("dsn");
  dmaxrod = setvar ("dmaxrod");
  dmaxcon = setvar ("dmaxcon");
  dmaxna  = setvar ("dmaxna");
  dmaxampa = setvar ("dmaxampa");
  dmaxnmda = setvar ("dmaxnmda");
  dnmdamg = setvar ("dnmdamg");
  dnaoffsm= setvar ("dnaoffsm");
  dnaoffsh= setvar ("dnaoffsh");
  dnataum = setvar ("dnataum");
  dnatauh = setvar ("dnatauh");
  dktaum  = setvar ("dktaum");
  dktauh  = setvar ("dktauh");
  dsyntau = setvar ("dsyntau");
  dmaxk   = setvar ("dmaxk");
  dnau    = setvar ("dnau");
  dku     = setvar ("dku");
  dkau    = setvar ("dkau");
  dkcabu  = setvar ("dkcabu");
  dkcasu  = setvar ("dkcasu");
  dkihu   = setvar ("dkihu");
  dcalu   = setvar ("dcalu");
  dcatu   = setvar ("dcatu");
  dcgmpu  = setvar ("dcgmpu");
  dampau  = setvar ("dampau");
  dkoffsn = setvar ("dkoffsn");
  dkoffsh = setvar ("dkoffsh");
  dgjtau  = setvar ("dgjoff");
  dgjtau  = setvar ("dgjtau");
  dgjnv   = setvar ("dgjnv");
  dbd1 	  = setvar ("dbd1");
  dbd2 	  = setvar ("dbd2");
  dbk1 	  = setvar ("dbk1");
  dbk2 	  = setvar ("dbk2");
  dsd1 	  = setvar ("dsd1");
  dsd2 	  = setvar ("dsd2");
  dsk1 	  = setvar ("dsk1");
  dsk2 	  = setvar ("dsk2");
  complam = setvar ("complam");
  lamcrit = setvar ("lamcrit");
  relincr = setvar ("relincr");
  timinc  = setvar ("timinc");
  stiminc = setvar ("stiminc");
  xtime   = setvar ("time");
  endexp  = setvar ("endexp");
  ploti   = setvar ("ploti");
  plmax   = setvar ("plmax");
  plmin   = setvar ("plmin");
  crit    = setvar ("crit");
  relax   = setvar ("relax");
  djnoise = setvar ("djnoise");
  scatter = (int)setvar ("scatter");
  euler   = (int)setvar ("euler");
  stimelem= (int)setvar ("stimelem");
  implicit= (int)setvar ("implicit");
  synaptau= setvar ("synaptau");

  silent  = (int)setvar ("silent");
  vidmode = (int)setvar ("vidmode");
  plsep   = (int)setvar ("plsep");
  allowrev= (int)setvar ("allowrev");
  dashfl  = (int)setvar ("dash");
  disp    = (int)setvar ("disp");
  prmap   = (int)setvar ("prmap");
  stim_elem = (int)setvar ("stim_elem");
  ktrb  = sqrt(4*Bk*ktemp/timinc); 		/* For Johnson noise */
		/* See Hille 1992 p 325 on Johnson noise */

  if (stiminc < timinc) stiminc = timinc;
}

/****************************************/

void varcopyu(void)

/* Set params that might affect user-called functions. 
   Put into functions like "hinf", etc, which may be used without
   running a "step" or "run" which would initialize default
   variables. This is supposed to run faster than "varcopy" above. 
*/

{
  dcoo     = setvar ("dcoo");
  dmgo     = setvar ("dmgo");
  dcao     = setvar ("dcao");
  dcavoff  = setvar ("dcavoff");
  dnaoffsm = setvar ("dnaoffsm");
  dnaoffsh = setvar ("dnaoffsh");
  dkoffsn  = setvar ("dkoffsn");
  dkoffsh  = setvar ("dkoffsh");
  dcaoffs  = setvar ("dcaoffs");
  timinc   = setvar ("timinc");
  stiminc  = setvar ("stiminc");
  tempcel  = setvar ("tempcel");
  ktemp = tempcel + KELVIN;
}

/****************************************/

void dinit()
{

#define NR NUMBER

  int len;

 Symbol *lookup(char *str);

 setval (lookup("calcnernst"),0.6,NR);	/* =1 -> define vrevs from ion conc */
					/* =0 -> define ion conc from vrevs */
 setval (lookup("vna"),LARGENUM,NR);	/* sodium equilibrium potential    */
 setval (lookup("vk"),LARGENUM,NR);	/* potassium equilibrium potential */
 setval (lookup("vcl"),LARGENUM,NR);	/* chloride equilibrium potential  */

 tempcel = 37.0;
 setval (lookup("tempcel"),tempcel,NR);	/* temperature for chan rate chtyp */
 ktemp = tempcel + KELVIN;

 dnao = 143e-3;				/* Ames medium, Ames & Nesbett 1981 */
 dnai = nernstc(0.065,dnao);
 setval (lookup("dnao"),dnao,NR);	/* external [Na], used for Na vrev */
 setval (lookup("dnai"),dnai,NR);	/* internal [Na], used for Na vrev */
 setval (lookup("dpkna"),0.08,NR);	/* perm of K rel to Na, for Na vrev */
 setval (lookup("dpcana"),0.01,NR);	/* perm of Ca rel to Na, for Na vrev */

 dko = 5.0e-3;				/* Ames medium = 3.6 mM */
					/* Set dko a little higher  */
 dki = nernstc(-0.0892,dko);		/*  allows K vrev to be -.082 w/dpnak */
 setval (lookup("dko"),dko,NR);		/* external [K], used for K vrev */
 setval (lookup("dki"),dki,NR);		/* internal [K], used for K vrev */
 setval (lookup("dpnak"),0.01,NR);	/* perm of Na rel to K, for K vrev */
 setval (lookup("dpcak"),0.0001,NR);	/* perm of Ca rel to K, for K vrev */

					/* see Hille (1992) p 351-353 */
 setval (lookup("dpnaca"),1e-3,NR);	/* perm of Na rel to Ca, for Ca vrev */
 setval (lookup("dpkca"),3e-4,NR);	/* perm of K rel to Ca, for Ca vrev */

 dclo = 125.4e-3; 			/* Ames medium */
 dcli = nernstc(0.07,dclo);
 setval (lookup("dclo"),dclo,NR);	/* external [Cl], used for Cl vrev */
 setval (lookup("dcli"),dcli,NR);	/* internal [Cl], used for Cl vrev */

 setval (lookup("dcap"),1e-6,NR);	/* f/cm-cm     membrane capacitance */
 setval (lookup("dri"),200.,NR);	/* ohm-cm       axial resistance */
 setval (lookup("drm"),40000.,NR);	/* ohm-cm-cm    membrane resistance */
 setval (lookup("drg"),5e6,NR);		/* ohm-um-um    gap junc resistance */
 setval (lookup("dfta"),.2,NR);		/* synaptic time const (msec) */
 setval (lookup("dsfa"),2.0,NR);	/* synaptic number of filters */
 setval (lookup("dftb"),.2,NR);		/* synaptic time const (msec) */
 setval (lookup("dsfb"),0.0,NR);	/* synaptic number of filters */
 setval (lookup("dscu"),25e-12,NR);	/* unitary conductance of chans */
 setval (lookup("dsvn"),5.0,NR);	/* number of syn vesicle rel sites */
 setval (lookup("dvsz"),100.0,NR);	/* size of vesicles released */
 setval (lookup("dst"), -.04,NR);	/* synaptic threshold (volts) */
 setval (lookup("dsc"),(double)EXPON,NR);/* synaptic curve */
 setval (lookup("dmaxsyn"),200e-12,NR);	/* maximum cond for synapse (S) */
 setval (lookup("dskd"),1.0,NR);	/* half-max neurotr saturation (kd)*/
 setval (lookup("dshc"),1.0,NR);	/* neurotrans binding Hill coeff */
 setval (lookup("dstr"),100e-6,NR);	/* max neurotrans conc for synapse */
 setval (lookup("dgkd"),0.1,NR);	/* half-max cG saturation (kd) */
 setval (lookup("dsn"),2.0,NR);		/* synaptic gain, lin or exp */
 setval (lookup("dsg"),1.0,NR);		/* synaptic cG gain */
 setval (lookup("dmaxrod"),44e-12,NR);/* dark cond of rod gated channel (S) */
 setval (lookup("dmaxcon"),1.4e-9,NR);	/* dark cond of cone gated chan (S)*/
 setval (lookup("dnadens"),0.25,NR);	/* density of Na V-gated chan(S/cm2)*/
 setval (lookup("dkdens"),0.0707,NR);	/* density of K V-gated chan (S/cm2)*/
 setval (lookup("dcadens"),0.005,NR);	/* density of Ca V-gated chan(S/cm2)*/
 setval (lookup("dcavoff"),.018,NR);	/* Q10 for voffset due to cao (V) */
 setval (lookup("dcaspvrev"),.18,NR);	/* Ca surface potential (mult dcavoff) */
 					/* Add to vrev. */
 setval (lookup("ddca"),DCa,NR);	/* Diffusion const for Ca = 2e-6 */
                                        /*  slower in cytoplasm than in H2O*/
					/* see de Schutter&Smolen "Methods" */
					/*   (from Albritton et al, 1992) */
 setval (lookup("dcoo"),0.0,NR);	/* external Co++ (M) (for cavoff) */
 setval (lookup("dmgo"),DMGO,NR);	/* external Mg++ (M) (for cavoff) */
 setval (lookup("dcao"),DCAO,NR);	/* 1.15 mM = external Ca++ (M) */
 setval (lookup("dcai"),50e-9,NR);	/* conc. of internal Ca++ (M) */
 setval (lookup("dcabnd"),10.0,NR);	/* ratio of bound to free calcium */
 setval (lookup("dcabti"),30e-6,NR);	/* Ca buffer total in first shell */
 setval (lookup("dcabt"),3e-6,NR);	/* Ca buffer total in shells */
					/*  Yamada, Koch, Adams 1998) */
					/*  Simon, Llinas 1985 */
 setval (lookup("dcabf"),1e8,NR);	/* Ca buffer forw rate to Ca.B /M/sec */
 setval (lookup("dcabr"),100,NR);	/* Ca buffer reverse rate to Ca.B /sec */
 setval (lookup("dcashell"),10.0,NR);	/* Number of Ca diffusion shells */
 setval (lookup("dcapkm"),0.2e-6,NR);	/* 1/2 sat conc. of Ca++ pump (M) */
 setval (lookup("dcavmax"),2e-5,NR);	/* Vmax for Ca pump (ma/cm2)       */
 setval (lookup("dcakex"),5e-9,NR);	/* Rate for Na/Ca exchanger(A/mM4/cm2)*/
 setval (lookup("dmaxca"),1e-10,NR);	/* cond of Ca voltage gated channel */
 setval (lookup("dcaoffs"),0.0,NR);	/* activation offset for Ca chan  */
 setval (lookup("dcataum"),1.0,NR);	/* activation tau for Ca chan  */
 setval (lookup("dcatauh"),1.0,NR);	/* inactivation tau for Ca chan  */
 setval (lookup("dkcataum"),1.0,NR);	/* activation tau for Kca chan  */
 setval (lookup("dkcatauh"),1.0,NR);	/* inactivation tau for Kca chan  */
 setval (lookup("dnataud"),0.16,NR);	/* tau for Na noise in 2-state modl */
 setval (lookup("dktaud"),0.6,NR);	/* tau for K noise in 2-state model */

 setval (lookup("dqm"),2.0,NR);		/* Q10 for am, bm */
 setval (lookup("dqh"),3.0,NR);		/* Q10 for ah, bh */
 setval (lookup("dqn"),3.0,NR);		/* Q10 for an, bn */
 setval (lookup("dqca"),3.0,NR);	/* Q10 for ac, bc (Ca chan) */
 setval (lookup("dqd"),3.0,NR);		/* Q10 for ad, bd (KA h) */
 setval (lookup("dqkca"),3.0,NR);	/* Q10 for kca a, b */
 setval (lookup("dqsyn"),3.0,NR);	/* Q10 for synaptic chans (ampa,etc)*/
 setval (lookup("dqcavmax"),2.0,NR);	/* Q10 for Ca pump */
 setval (lookup("dqcab"),2.0,NR);	/* Q10 for Ca buffer */
 setval (lookup("dqc"),1.4,NR);		/* Q10 for channel conductance */
 setval (lookup("dqrec"),2.7,NR);	/* Q10 for photoreceptor transduction */
 setval (lookup("dqcrec"),1.8,NR);	/* Q10 for photoreceptor transduction */
					/* Baylor, Matthews & Yau, 1983) */
 setval (lookup("dnmdamg"),2e-3,NR);	/* Mg++ extracellular conc */
 setval (lookup("dmaxna"),1e-9,NR);	/* cond of Na voltage gated channel(S)*/
 setval (lookup("dnaoffsm"),0.0,NR);	/* m offset voltage for Na chan */
 setval (lookup("dnaoffsh"),0.0,NR);	/* h offset voltage for Na chan */
 setval (lookup("dnataum"),1.0,NR);	/* activation tau for Na chan */
 setval (lookup("dnatauh"),1.0,NR);	/* inactivation tau for Na chan */
 setval (lookup("dgjoff"),GJOFF,NR);	/* gap junction voltage offset */
 setval (lookup("dgjtau"),1.0,NR);	/* activation tau for gap junction */
 setval (lookup("dgjnv"),0.2,NR);	/* non-volt-sens fraction of gj cond */
 setval (lookup("dbd1"),1.0,NR);	/* voltage multiplier for BK chan alph*/
 setval (lookup("dbd2"),1.0,NR);	/* voltage multiplier for BK chan bet */
 setval (lookup("dbk1"),1e-6,NR);	/* ca multiplier for BK chan alph */
 setval (lookup("dbk2"),1e-6,NR);	/* ca multiplier for BK chan bet  */
 setval (lookup("dsd1"),0.0,NR);	/* voltage multiplier for SK chan alph*/
 setval (lookup("dsd2"),0.0,NR);	/* voltage multiplier for SK chan bet */
 setval (lookup("dsk1"),1e-7,NR);	/* ca multiplier for SK chan alph */
 setval (lookup("dsk2"),1e-7,NR);	/* ca multiplier for SK chan bet  */
 setval (lookup("dmaxk"),2e-10,NR);	/* cond of K voltage gated channel(S)*/
 setval (lookup("dnau"),12e-12,NR);	/* unitary cond of Na chan(S)=20pS@22 */
 setval (lookup("dku"),12e-12,NR);	/* unitary cond of K chan (S) */
 setval (lookup("dkau"),12e-12,NR);	/* unitary cond of KA chan (S) */
 setval (lookup("dkcasu"),8.4e-12,NR);	/* unitary cond of Kca SK ch=22 pS@35*/
 setval (lookup("dkcabu"),4.4e-11,NR);	/* unitary cond of Kca BK ch=115 pS@35*/
 setval (lookup("dkihu"),6e-12,NR);	/* unitary cond of Ih chan (S) */
 setval (lookup("dcalu"),7.6e-12,NR);	/* unit cond of Ca T chan=20 pS @35 */
 setval (lookup("dcatu"),3e-12,NR);	/* unit cond of Ca L chan=8 pS @35 */
 setval (lookup("dampau"),25e-12,NR);	/* unitary cond of AMPA chan (S) */
 setval (lookup("dcgmpu"),25e-12,NR);	/* unitary cond of cGMP chan (S) */
 setval (lookup("dkoffsn"),0.0,NR);	/* offset voltage for K chan */
 setval (lookup("dkoffsh"),0.0,NR);	/* offset voltage for K chan */
 setval (lookup("dktaum"),1.0,NR);	/* activation tau for K chan */
 setval (lookup("dktauh"),1.0,NR);	/* inactivation tau for K chan */
 setval (lookup("dsyntau"),1.0,NR);	/* time const for synaptic chans */
 setval (lookup("complam"), .1,NR);	/* fraction of lambda for compartment */
 setval (lookup("lamcrit"), .3,NR);	/* criterion for condensing comps */
 setval (lookup("timinc"), 1e-4,NR);	/* time incr for model (sec) */
 setval (lookup("stiminc"), 1e-4,NR);	/* synaptic/stim time incr (sec) */
 setval (lookup("ncomps"), 0.0,NR);	/* number of compartments */
 setval (lookup("endexp"), 50e-3,NR);	/* end of recording for model (sec) */
 setval (lookup("ploti"), 1e-3,NR);	/* plot increment (sec) */
 setval (lookup("plmax"), .04,NR);	/* max for voltage plots (volts) */
 setval (lookup("plmin"), -.08,NR);	/* min for voltage plots (volts) */
 setval (lookup("crit"), 1e-8,NR);	/* criterion for steps (volts) */
 setval (lookup("relax"), .15,NR);	/* over-relaxation constant */
 setval (lookup("relincr"), .002,NR);	/* relax increment */
 setval (lookup("synaptau"), 1.0,NR);	/* synaptic time constant multiplier */
 setval (lookup("euler"), 0.0,NR);	/* =1 -> euler mode */
 setval (lookup("implicit"), 0.0,NR);	/* =1 -> implicit mode */
 setval (lookup("stimelem"), 0.0,NR);	/* "stim" understands neural elements */
 setval (lookup("scatter"), 0.0,NR);	/* use point spread func for scatter */
 setval (lookup("debug"), 0.0,NR);	/* turn on debugging features */
 setval (lookup("debugz"), 0.0,NR);	/* turn on debugging features */
 setval (lookup("dashfl"), 0.0,NR);	/* set dashes on plot */
 setval (lookup("allowrev"), 0.0,NR);	/* allow reverse point plotting */
 setval (lookup("djnoise"), 0.0,NR);	/* > 0 -> Johnson noise for all Rm's */
 setval (lookup("plsep"), 0.0,NR);	/* set multiple separate plotting */
 setval (lookup("time"), 0.0,NR);	/* time at beginning */
  len = strlen(progname);
  stdplt = stdout;
  if (len) {
    if (strcmp(progname,"nd")==0) disp |= DISP;
    if (strcmp(progname,"ndv")==0) { 
          FILE *ftemp;
    	disp |= DISP; 
	vidmode=1; 
	if ((ftemp=popen("vid","w"))==NULL) {
	    fprintf (stderr,"Error: nc can't open stdout\n");
	}
	else stdplt = ftemp;
	pipefl = 1;
    }
    if (strcmp(progname,"ncv")==0) {
          FILE *ftemp;
	vidmode=1;
	if ((ftemp=popen("vid","w"))==NULL) {
	    fprintf (stderr,"Error: nc can't open stdout\n");
	}
	else stdplt = ftemp;
	pipefl = 1;
    }
  }

 timeptr = lookup("time");		/* get pointer to time */
 plotiptr = lookup("ploti");		/* get pointer to ploti */
 ncompptr = lookup("ncomps");		/* get pointer to ncomps */

}

/*******************************/

void iinit()

/* initialize interpreter things */

{
    int len;
    double ncvers;

  ncvers = ncversion[0]+ncversion[1]*.01+ncversion[2]*.0001;
  setval (lookup("version"), ncvers,NR);/* nc version number */
  len = strlen(progname);
  if (setvid) vidmode=1;
  else if (unsetvid) vidmode=0;
  setval (lookup("rseed"), 123456.0,NR);	/* default random seed */
  setval (lookup("prseed"), 3141592.0,NR);  /* photoreceptor random seed */

  setval (lookup("stdin"), (double)(long int)stdin,NR);   /* standard input */
  setval (lookup("stdout"),(double)(long int)stdout,NR);  /* standard output */
  setval (lookup("stderr"),(double)(long int)stderr,NR);  /* standard err out */
}

#undef NR

/*******************************/

