/* segment initchan in program nc */

/* sets up channel parameters */

#include <stdio.h>
#include "nc.h"
#include "y.tab.h"
#include "ncsub.h"
#include "ncomp.h"
#include "control.h"

static double qfgj;			/* q10 constant for gap junctions */

#define EXPSIZ 1000
#define EXPEXPSIZ 300

double exptab [EXPSIZ+1]={0};    
double expexptab [EXPEXPSIZ+1]={0};    

#define LOG3  1.09861228866

chantype *makchantype(int ctype, int cnum, int nstates, int nparm, double bt);

#ifdef __cplusplus
extern "C" {
#endif

double exp(double);
double log(double);

#ifdef __cplusplus
}
#endif

chantype *makna0(void);
chantype *makna1(void);
chantype *makna2(void);
chantype *makna3(void);

chantype *makk0(void);
chantype *makk1(void);
chantype *makk2(void);
chantype *makk3(void);
chantype *makk4(void);

chantype *makkca0(void);
chantype *makkca1(void);
chantype *makkca2(void);
chantype *makkca3(void);
chantype *makkca4(void);
chantype *makkca5(void);
chantype *makkca6(void);

chantype *makca0(void);
chantype *makca1(void);
chantype *makca2(void);
chantype *makca3(void);

chantype *makcgmp1(void);

chantype *makampa1(void);
chantype *makampa2(void);
chantype *makampa3(void);
chantype *makampa4(void);

chantype *maksyn1(void);

chantype *maknmda1(void);
chantype *maknmda2(void);

chantype *makgaba1(void);
chantype *makgaba2(void);

double qrate(chanparm *chp);
void getntresp(chan *cpnt, comp *comp1);
void isntresp(chan *cpnt, comp *comp1);
double rnt(chan *cpnt);

double ncabs(double x);

double lkexp(double val);

/*----------------------------------------*/

void makexp(void)
/* set up exponential function lookup table.  */
{
   int i;
   static int done=0;

 if (done) return;

 done = 1;
 for (i=0; i<EXPSIZ+1; i++)
   exptab[i] = exp((double)i/EXPSIZ); 
 for (i=0; i<EXPEXPSIZ+1; i++)
   expexptab[i] = exp((double)(i-EXPEXPSIZ/2)); 

/*  Test lkexp(). It's accurate to factor of 1.25e-7 */

/*  for (i=0; i<100; i++) {
      double scal;

       scal = 0.0001;
   printf ("%g %20.20g %20.20g %20.20g\n",(double)i*scal,exp((double)i*scal), 
     lkexp((double)i*scal), (exp((double)i*scal) - lkexp((double)i*scal)) /
			       exp((double)i*scal));
 } /* */
}

/*----------------------------------------*/

double lkexp(double val)

/* lookup an exp() value in the range -250 to 250. */

/* On some machines, it's faster to use this 
    interpolation scheme than use the exp() 
    routine supplied with the library.
*/

{
  int i,r;
  double rr,x;
  
  if      (val > 250.0) val = 250.0; 
  else if (val <-250.0) val = -250.0; 
  i = int(val);				/* integer for lookup in expexptab[] */
  if (val<0) i--;
  rr = (val - i) * EXPSIZ;		/* remainder for lookup in exptab[] */
  r = int(rr + 0.5);
  rr -=  r;
  x = exptab[r] * (1.0+(1.0/EXPSIZ)*rr); /* use linear interpolation */
  return (x*expexptab[i+EXPEXPSIZ/2]); 
}

/*--------------------------------------------*/

#define ITAU 2e3

static int vi = 0;

double aif(chan *cpnt)

/* alpha for integrate-and-fire channel */

{
   double v,v1,v2,retval;

   v1 = cpnt->conc[2].cest; 
   v2 = cpnt->conc[3].cest; 
   v = cpnt->comp1->v - cpnt->voffsm;
   if (v > 0 && ! v2) v1 += timinc * ITAU * v;
   if (v1 > 1.0) {
   	 v2 = 1; 
	 retval = .3;
   }
   else  retval = 0;
   cpnt->conc[2].cest = v1; 
   cpnt->conc[3].cest = v2; 
  return retval;
}

/*----------------------------------------*/

double bif(chan *cpnt)

/* beta for integrate-and-fire channel */

{
   double v1, v2, retval;

   v1 = cpnt->conc[2].cest; 
   v2 = cpnt->conc[3].cest; 
   if (v2) {
         v1 -= timinc * 1e4; 
   }
   if (v1 < 0.1) {
       v2 = 0; 
       retval = 0.3;
   }
   else  retval= 0;
   cpnt->conc[2].cest = v1; 
   cpnt->conc[3].cest = v2; 
   return retval;
}

/*--------------------------------------------*/

double qrategj (void)

/* Return rate constant multiplier for gap junctions 
   which depends on temperature. Computed at a constant time step.
   Assume normal temperature of 22 deg C (just for simplicity).
*/

{
 return stiminc * exp(LOG3 * (tempcel - 22.0) / 10.0); /* Q10 of 3 */
}

/*--------------------------------------------*/

#define GJLAMBDA 0.0013
#define GJVOFF 0.0147
#define Aa 0.077
#define Ab 0.14

double gjalpha (double v, gj *gjpnt)

/* Rate function for gap junction conductance change */

/* v is in mV, return rate in 1/sec */

/* For details, see Harris, Spray, Bennett, */
/* J.  Gen Physiol. 77:95-117, 1981.  */

{
    double vj;                          /* junction voltage */
    double alpha, n;

   if (v<0 && (gjpnt->rev)) {		/* reverse parameters */
     vj = ncabs(v);
     alpha = qfgj * gjpnt->rtaun * GJLAMBDA * 
		exp(-Aa * (vj - gjpnt->rvoff)*VTOMV/gjpnt->rvgain);
   }
   else {				/* forward parameters */
     vj = ncabs(v);
     alpha = qfgj * gjpnt->taun * GJLAMBDA * 
		exp(-Aa * (vj - gjpnt->voff)*VTOMV/gjpnt->vgain);
   }
   return alpha*MSSEC;
}

/*--------------------------------------------*/

double gjbeta (double v, gj *gjpnt)

/* Rate function for gap junction conductance change */

/* v is in mV */

/* For details, see Harris, Spray, Bennett, */
/* J.  Gen Physiol. 77:95-117, 1981.  */

{
    double vj;                          /* junction voltage */
    double beta, n;

   if (v<0 && (gjpnt->rev)) {		/* reverse parameters */
     vj = ncabs(v);
     beta  = gjpnt->rtaun * GJLAMBDA * exp( Ab * (vj - gjpnt->rvoff) *
		VTOMV/gjpnt->rvgain);
   }
   else {				/* forward parameters */
     vj = ncabs(v);
     beta  = gjpnt->taun * GJLAMBDA * exp( Ab * (vj - gjpnt->voff) *
		VTOMV/gjpnt->vgain);
   }
   beta = qfgj * beta / (1.0 + 50 * beta); 
   return beta*MSSEC;
}

/*----------------------------------------*/

chantype *makif(void)

/* Sequential-state description of 
   integrate-and-fire channel.

   Number states from 0 to n; 
   Set numstate = n;
   Set numtrans = number of transitions.
   Set cond     = conductance of state;

   For each state, set transitions:
    Set trans =   State to go to on this transition. 
    Set trate =   Function that returns basic rate for transition. 
    Set ratemul = Multiplier for rate function.
    Set rateo  =  Set to 0 for m, set to 1 for h:  sets voffset and tau.
*/

  {
     chantype *ch;
     chanstate *spnt;
     chanparm *parm;
     double n;
     int nstate, nparm;

   nstate = 2;                          /* 2 Markov states */
   nparm = 1;                           /* make 1 set of params, "n" */
   ch=makchantype(NA,4,nstate,nparm,22.0); /* make chan const, parm */
   ch->unitary = 20e-12;
   ch->vrev = vna;			/* default reversal potential */
   spnt = ch->state;
   parm = ch->parm;

   parm[0].nfval    = 0;                /* number of func vals (am, bm, etc.) */
   parm[0].nival    = 0;                /* number of implicit vals (m1, m2) */
   parm[0].dq = dqm;                    /* Q10 for m rate function */

                                /*   0 <-> 1 */
   spnt[0].numtrans   = 1;
   spnt[0].cond       = 0;
   spnt[0].trans  [0] = 1;
   spnt[0].trate  [0] = aif;
   spnt[0].ratemul[0] = 1.0;
   spnt[0].rateo  [0] = 0;	/* offsm, taum */

   spnt[1].numtrans   = 1;
   spnt[1].cond       = 1;
   spnt[1].trans  [0] = 0;
   spnt[1].trate  [0] = bif;
   spnt[1].ratemul[0] = 1.0;
   spnt[1].rateo  [0] = 1;	/* offsm, taum */

   return ch;
}

/*----------------------------------------*/
 
void initchan(void)

/* This subroutine sets up all sequential-state channel tables
    at the beginning of a "run".  To set up additional types,
    copy the "makna1()" or "makk1()" routine and modify it, then
    call it from this routine.  Make sure to set "NCHANSTYP"
    correctly.
*/

{
   static int done = 0;

 if (done) return;
 else done = 1;

 makexp();			/* set up exp() table for synapses */
 qfgj = qrategj();

 makna0();			/* set up Na tables */
 makna1();			/* set up Na sequential states */
 makna2();
 makna3();

 makif();			/* set int-and-fire states=Na type 4 */

 makk0();			/* set up Khh */
 makk1();			/* set up K 1 sequential states */
 makk2();			/* set up KA hh  sequential states */
 makk3();			/* set up KA markov */
 makk4();			/* set up K 4 sequential states */

 makkca0();			/* set up Kca sequential states */
 makkca1();			/* set up Kca sequential states */
 makkca2();			/* set up Kca sequential states */
 makkca3();			/* set up Kca sequential states */
 makkca4();			/* set up Kca sequential states */
 makkca5();			/* set up Kca sequential states */
 makkca6();			/* set up Kca sequential states */

 makca0();			/* set up Ca tables states */
 makca1();
 makca2();
 makca3();

 makampa1();
 makampa2();
 makampa3();
 makampa4();

 makcgmp1();

 maknmda1();
 maknmda2();

 makgaba1();
 makgaba2();

 maksyn1();			/* 2 state markov synapse from conduct */
}


