/* modcode.c */

/* This module contains all subroutines called by "nc.y"
   that create neural element data structures.  This
   module is compiled for use by both "nc" and "stim".
   For the "stim" version, conditional compilation statements
   remove or modify some of the subroutines so that they
   do only things associated with making and controlling
   stimuli.  Thus, both "stim" and "nc" interpret the same
   language but "stim" ignores all the "compartment" stuff and
   deals only with neural elements and stimuli.  After
   compilation, the "stim" version is "mv"ed to "stimcode.o.

     An old version of stimcode exists, however this is not
   currently used because of the difficulty of modifying it
   "in parallel" with modifications to "modcode.c".  */

#include "nc.h"
#include "y.tab.h"
#include "adef.h"
#include "ncsub.h"
#include "ncelem.h"
#include "ncomp.h"
#include "ncplot.h"
#include "stim.h"
#include "vec.h"

#ifdef __cplusplus
extern "C" {
#endif

#include <stdio.h>
#include <string.h>
#include <math.h>
#include "stdplt.h"
#undef scale

#ifdef __cplusplus
}
#endif

#define max(x, y)	(((x) < (y)) ? (y) : (x))
#define min(x, y)	(((x) < (y)) ? (x) : (y))

#define DEBUG

/* Normally stim doesn't need to know about neural elements.  
   However in a complex circuit, it is possible that the arrangement
   of photoreceptors and their stimuli need to be determined from
   information only available from some already-defined neural
   elements.  In this case, comment out the #define XSTIMM below,
   and recompile "stim".
*/

#ifdef XSTIMM		/* define XSTIMM when stim should ignore neur elems */
  #define XSTIM		/* define XSTIM for stim  */
#endif

#ifndef XSTIM
#define XMOD			/* define XMOD when not XSTIM */
#endif
		
/* variables for stim */

int recepnode = 0;                      /* receptor node from conn() */
int recepn2 = 0;
int recepn3 = 0;
int stimfflg = 0;		/* =1 -> stimulus comes from file */


/* interpreter innards: */

#define pushm(d) *stackp++ = (d)		/*  */
#define popm()  *--stackp		/* function pop still needed */

/* #define popm() pop()			/*  */

#define NSTACK 256

extern char *progname;
extern char *runfile;
extern int vidmode;
extern int debug;
extern int debugz;
extern int setdebug;
extern int setdebgz;
extern int setdisp;
extern int disp;
extern int disp_ray;
extern int stimhlp;			/* in stimsub.c */
extern int nocond;			/* no condense, in ncsub */
extern int stimelem;		/* = 1 => stim understands neural elements */

extern int stopping;
extern int returning;
extern int breaking;
extern int continuing;

extern  datum   *stackp;
extern	Inst	*pc;		/* program counter during execution */

elem *elpnt;			/* current element pointer */

int curelnum=0;			/* current element being read in */
extern node *nodepnt;		/* pointer to node list */
extern recep *recpnt;		/* pointer to receptor list */
extern recnod *reclist;		/* pointer to receptor stimulus list */
extern elem *elempnt;		/* pointer to current element */
extern int cumelem;
extern int cumnode;
extern int cumrec;

extern double xtime;		/* time variable set in "ncsub" (control.h) */
extern Symbol *timeptr;		/* pointer to "xtime" for setting time */

#ifdef XMOD		/* variables for modcode */

#define STIMCODE 0

extern plotfr plotnod[PLOTNODSIZ];  /* holds nodes to be recorded */
extern int numplots;		/* number of plots to be displayed */
extern int runyet;
			/* functions for modcode */

recstim *makvstim(double start, int nodnm1, int nodnm2, int nodnm3, int nodnm4,
			double value, char *action);
recstim *makrstim(double start, int recnm1, int recnm2, int recnm3, int recnm4,
		double inten, double wavel, char *action);
int inrect(double x, double y, double maxx, double minx, 
			double maxy, double miny, double orient);
int incirc(double x, double y, double circx, 
			double circy, double rad);

void dcomp(int n1a, int n1b, int n1c, int n1d, int elemtype, int exceptype, 
   int na, int nb, int nc, int nd, int color, double dscale, int hide, int excl);
void ncdrnod (int n1a, int n1b, int n1c, int n1d, int elemtype, int exceptype, 
   int na, int nb, int nc, int nd, int color, double dscale);
void ncdisp(int n1a, int n1b, int n1c, int n1d, int elemtype, int exceptype, 
   int na, int nb, int nc, int nd, int color, double dscale, int hide, int excl);
void ncdispc(int n1a, int n1b, int n1c, int n1d, int n2a, int n2b, int n2c, 
	int n2d, int elemtype, int exceptype, int na, int nb, int nc, int nd,
	int color, double dscale, int hide);
void ncdispn(int n1a, int n1b, int n1c, int n1d, int n2a, 
	int n2b, int n2c, int n2d, int elemtype, int exceptype, 
	int na, int nb, int nc, int nd,
	int color, double dscale, int hide, int excl);
void ncdispe (int elemnum, int color, double dscale, int hide);
void drcalib (double x, double y, double length, double size, int color);
void setrot (double xrot, double yrot, double zrot, 
	double xcent, double ycent, double zcent, 
	double rxcent, double rycent, double rzcent, double scal);
void dispstim(double stime,double dscale);
void set_icons(void);
void initray (double xrot, double yrot, double zrot,
        double xcent, double ycent, double zcent,
        double rxcent, double rycent, double rzcent, double scal);
//extern char *strncat(char *dest, const char *src, size_t n);

#else			/* variables for stimcode */

#define STIMCODE 1

plotfr plotnod[PLOTNODSIZ];  	/* holds nodes to be recorded */
int numplots;			/* number of plots to be displayed */
int runyet;

			/* functions for stimcode */
void stcomment (void);
int makblur(double rad, double sscale);
void findarrsiz(double *xmax,double *xmin,double *ymax,double *ymin);
int initc(int num, int size);
void recback(int array, double inten, double wavel);
void abslist(double ratio, double time);
void makrect(int arr, double width, double length, double xoff, double yoff, 
		double sscale, double inten, double wavel);
void recpat(int array, double xoff, double yoff, double sscale);
void stimlist(double delratio, double time);
void makspot(int arr, double dia, double xoff, double yoff, 
		double sscale, double inten, double wavel);
void maksine(int arr, double period, double xmin, double xmax, 
		double ymin, double ymax, double toff, int drift, double orient,
		double sphase, double xoff, double yoff, double sscale, 
		double inten, double contrast, double wavel);
void makgabor(int arr, double period, double xmin, double xmax, 
		double ymin, double ymax, double toff, int drift, double orient,
		double sphase, double xoff, double yoff, double sscale, 
		double inten, double contrast, double wavel,
		double xenv, double yenv);
void delc(int num);

#ifdef __cplusplus
extern "C" {
#endif

/* void mktemp(char *f);	/* */
char *mktemp(char *f);	/* */

#ifdef __cplusplus
}
#endif

#endif				/* stimcode */

extern char *rootframe;

int stimcode=STIMCODE;		/* = 1 -> says we're running "stim" */
double xrot,yrot,zrot;
double dxcent=LARGENODE,dycent=LARGENODE,dzcent=LARGENODE;
double rxcent=0,rycent=0,rzcent=0;
double dsize = 200;
static int synnum=0;		/* temporary store for synaptic "ename" */

char *stfile=0;			/* stimulus file for plotinit */

FILE *stimin=0;			/* stimulus file */
FILE *stimout=stdout;           /* stimulus output file, used only by stim */
static char stimfile[80]={0};

#ifdef __cplusplus
extern "C" {
#endif

double sqrt(double);
double sin(double);

#include "gr.h"

FILE *fopen(const char *path, const char *mode);

#ifdef __cplusplus
}
#endif

node *maknod(nodeint nodea, nodeint nodeb, nodeint nodec,
	nodeint noded);
elem *makelem(int etype, elem *epnt);
elem *tmpelem();
double setvar(char *str);
double mindist(double dist1, double dist2, double dist3);
char *prnode(int n1, int n2, int n3, int n4);
void execerror(char *s, char *t);
void ncleanup();
Symbol *getvar(double **val, short int **typ, short int **vtyp);
void checknod(elem *elpnt);
void checkelemnode(elem *elpnt);
node *findnode(nodeint node1a, nodeint node1b, nodeint node1c, 
	nodeint node1d, char *s);
void setnodlst(node *npnt, elem *cpnt);
void unsetnodlst(node *npnt, elem *cpnt);
char *prnum(char *adj, double csiz, char *fmt, double v1, double v2, double v3);
int gausnn (double mean, double stdev, double density, double ms, 
	double framex, double framey, double xcent, double ycent, 
	int numcells, float **xarr, float **yarr, 
	int filout, int textfl, int printfl);
double *darr2(Symbol *sp, int narg);
void varcopy();
int checknum(datum d1);
int checkstr(datum d1);
float *makfiltarr (int newsiz, int oldsiz, float *oarr, double val);
void plotinit(int numplots);
void findconnect(int donefl);
void initcomp(int sav_elem);
void condense();
void execute(Inst *pc);
int forexec(Inst *pc);
void maklst(conlst **head, conn *cpnt);
void initchan();
void actrun(double time, int run);
int smaxmin(double val);
char *findsym(int num);
int numdyad(dyadlst *head);
double norm_line (vec a, vec b, vec p, vec q, double *t, double *s,
			vec *lp1, vec *lp2);
void efree(void *ptr);
void delnode(node *npnt);
void delelem(elem *epnt);
elem *findelem(int num);
void copyattr(attrib *src, attrib *dest);

/*------------------------------------------*/

/* Always remember to check a new subroutine added to "modcode.c" for
   its effect on "stim". The stim subroutines should be called in exactly 
   the same way as their modcode counterparts (exception: stimulus subroutines),
   and they should always leave the stack (popm) and program counter
   (pc) correct.  However unless XSTIMM is not defined, they will
   ignore all neural elements to save time and memory.
*/ 

/*------------------------------------------*/

void eramod(void)

/* erase all of the model stuff, to allow
   more space to run something else. 
*/

{
  ncleanup();				/* erase all lists, reset pointers */
}

/*------------------------------------------*/

chattrib *chanattr(elem *elpnt, short int ctype) 

/* set up attributes for a channel,
   and link them with the parent element.
*/

{
#ifndef XSTIMM
   attrib *pnt,*apnt;
   chattrib *makchattr(void);
   cattrib *makcattr(void);
   nattrib *maknattr(void);

  switch (ctype) {

  case SYN2:
  case GLU:
  case AMPA:
  case GABA:
  case GLY:
  case NA:
  case  K:
  case  KCa:
     apnt = (attrib *)makchattr();	/* make new NA/K channel attribute */
     break;
  case NMDA:
  case CGMP:
  case CACOMP:
  case CA:
     apnt = (attrib *)makcattr();	/* make new Ca chan attribute */
     break;
  case VESNOISE:
  case CHNOISE:
     apnt = (attrib *)maknattr();	/* make new noise attribute */
     break;
  } /* switch */

  apnt->ctype = ctype;			/* set type of new attrib */
  if (!elpnt->attpnt) {			/* if element has no attributes */
     elpnt->attpnt = apnt;		/* add directly to element */
  }
  else {				/* else add to element's attr list */
     for (pnt=elpnt->attpnt; pnt->attpnt; pnt=pnt->attpnt) ;
     pnt->attpnt = apnt;
  }
  return ((chattrib *)apnt);
#else
   chattrib *makchattr(void);
   chattrib *nullattrib = makchattr();  /* make null channel attribute */

  return (nullattrib);
#endif
}

/*------------------------------------------*/

void getnod (datum *d)

/* Get up to 4 node dimensions off the stack. */

{
    int i,narg;

  narg = (long int)*pc++;
  if (narg>MAXNODIM) {
     fprintf (stderr,"getnod: more than 4 dimensions\n");
     execerror ("Bad stack", "stopping");
  }
  if (narg > 3) checknum(d[3]=popm());	/* get node number */
  else {d[3].val = NULLVAL; d[3].vtype=NUMBER;}
  if (narg > 2) checknum(d[2]=popm());
  else {d[2].val = NULLVAL; d[2].vtype=NUMBER;}
  if (narg > 1) checknum(d[1]=popm());
  else {d[1].val = NULLVAL; d[1].vtype=NUMBER;}
  if (narg > 0) checknum(d[0]=popm());
  else {d[0].val = NULLVAL; d[0].vtype=NUMBER;}
  for (i=0; i<MAXNODIM; i++) 
    d[i].val = smaxmin(d[i].val);
}


/*------------------------------------------*/

void xmod(void)

/* modify an element */

{
  datum d1;
  Symbol *param;
  double *val;
  short *typ,*vtyp;

  param  = (Symbol *)*pc++;
  switch (param->type)
   {
    case MODIFY: if (checknum(d1 = popm())) {
#ifndef XSTIMM
    		    elpnt = tmpelem();			/* Make new element; */
  		    elpnt->modif = (int)(d1.val);	/* mark it modified. */
#endif
		 }
		 break;
    case ENAME:  getvar(&val,&typ,&vtyp);	
		 if (synnum) {		/* check for synaptic ename */
		   *val = synnum;	/* save element number in var */
		    synnum = 0;
		 }
		 else *val = cumelem;	/* save element number in var */
		 if (*typ==UNDEF) *typ = VAR;
		 *vtyp = NUMBER;
#ifndef XSTIMM
  		 elpnt->saved = 1;
#endif
		 break;
   }
}

/*------------------------------------------*/

void eranode(void)

/* Delete a node and any elements that connect to it. */

{
   int found, erased;
   int nod1, nod2, nod3, nod4;
   conlst *lpnt;
   datum d[MAXNODIM];
   node *npnt;
   elem *epnt;

 getnod(d);					/* get node, check dims */

#ifndef XSTIMM
 nod1 = (int)d[0].val;
 nod2 = (int)d[1].val;
 nod3 = (int)d[2].val;
 nod4 = (int)d[3].val;
 if (!(npnt=findnode(nod1,nod2,nod3,nod4,"Eranode"))) {
   execerror ("Missing node; ","stopping...");
   return; 
 }

 found = erased = 0;
 if (lpnt=npnt->elemlst) {	/* if any elems point to node */
   for (lpnt=npnt->elemlst; lpnt; lpnt=lpnt->next) {
      epnt = (elem *)lpnt->conpnt;
      if (epnt->nodp1) {
	unsetnodlst(epnt->nodp1,epnt);      /* free the nodpnt to this elem */
	if (!epnt->nodp1->elemlst) {
		delnode(epnt->nodp1);
		if (epnt->nodp1==npnt) erased = 1;
	}
      }
      if (epnt->nodp2) {
	unsetnodlst(epnt->nodp2,epnt);      /* free the nodpnt to this elem */
	if (!epnt->nodp2->elemlst) {
		delnode(epnt->nodp2);
		if (epnt->nodp2==npnt) erased = 1;
	}
      }
      delelem(epnt);                      /* erase the element */
   }
 }
 if (!erased) delnode(npnt);
#endif
}

/*------------------------------------------*/

void erelem(void)

/* Delete an element and any nodes that connect only to it. */

{
  datum d1;
  elem *epnt,*tepnt;
  int elnum;

if (checknum(d1 = popm())) {

#ifndef XSTIMM
   elnum = int(d1.val);
   if (!(epnt=findelem(elnum))) {
        fprintf (stderr,"erelem: can't find element %d\n",elnum);
        execerror ("Missing element: ","stopping...");
        return;
    }
    else {				/* first find nodes that connect */
      checkelemnode(epnt);		/* set pointers if not done */
      if (epnt->nodp1) {
	unsetnodlst(epnt->nodp1,epnt);      /* free the nodpnt to this elem */
	if (!epnt->nodp1->elemlst) delnode(epnt->nodp1);
      }
      if (epnt->nodp2) {
	unsetnodlst(epnt->nodp2,epnt);      /* free the nodpnt to this elem */
	if (!epnt->nodp2->elemlst) delnode(epnt->nodp2);
      }
      delelem(epnt);                      /* erase the element */
    }
#endif
  }
}

/*------------------------------------------*/

void xcable(void)
{
  datum d1;
  Symbol *param;
  cable *epnt;

  param  = (Symbol *)*pc++;
  checknum(d1 = popm());
  epnt = (cable *)makelem(CABLE,elpnt);		/* make new cable */
  elpnt = (elem *)epnt;				/* for membtyp() below */

  switch (param->type)
   {
    case LENGTH: epnt->length = d1.val; 
		break;
    case DIA:    epnt->dia    = d1.val;
		 if (epnt->dia2==NULLVAL) epnt->dia2 = epnt->dia;
		break;
    case DIA2:   epnt->dia2   = d1.val;
		break;
    case CPLAM:  epnt->cplam = d1.val;
		break;
    
   }
/* printf ("cable #%d %s %6.3f\n",elpnt->elnum,param->name,d1.val);  /* */
}

/*------------------------------------------*/

void xsphere(void)
{
  datum d1;
  Symbol *param;
  sphere *epnt;

  param = (Symbol *)*pc++;
  checknum(d1 = popm());
  epnt = (sphere *)makelem(SPHERE,elpnt);	/* make new sphere */
  elpnt = (elem *)epnt;				/* for membtyp() below */

   if (param==NULL) {
	epnt->dia = d1.val;
   }
   else switch (param->type)
    {
	case DIA: epnt->dia = d1.val;
		break;
	case RADIUS: epnt->dia = d1.val * 2;
		break;
    }
/*  printf ("sphere %s %g\n",param->name,d1.val); */
}

/*------------------------------------------*/

void xcachan(void)
{
  datum d1;
  Symbol *param;
  static cattrib *apnt; 
  int npops;
  static int pump=0;

  param  = (Symbol *)*pc++;
  elpnt = makelem(CHAN,elpnt);			/* make dummy elem "chan"  */
  switch (param->type)
   {
    case CA:
    case CACOMP:
    case CGMP:
    case NMDA:
    case CAPUMP:
    case CAEXCH:
    case CABUF:
    	 	 npops=0; 
		 break;
    case TYPE:  
    case VREV:  
    case TAU:
    case TAUM:
    case TAUH:
    case TAUN:
    case TAUC:
    case TAUD:
    case OFFSET:
    case OFFSETM:
    case OFFSETH:
    case MAXCOND:
    case CAPERM:
    case DENSITY:
    case NDENSITY:
    case UNIT:
    case N:

    case CAO:
    case CAI:
    case CBOUND:
    case CASHELL:
    case KEX:
    case VMAX:
    case KM:
    case MG:
    case KD:
    case BTOT:
    case BTOTI:
		 npops=1;
		break;
   }
 
  if (npops==1) checknum(d1 = popm());

  switch (param->type)
   {
    case CACOMP:
    case NMDA:
    case CGMP:					/* link attribs to elem */
    case CA:    apnt = (cattrib *)chanattr(elpnt,param->type); 
		pump = 0;
		break;
    case CAPUMP:apnt->pump = 1; pump = param->type;
		break;
    case CAEXCH:apnt->exch = 1; pump = param->type;
		break;
    case CABUF: apnt->cabuf = 1; pump = param->type;
		break;
    case TYPE:  if (apnt) apnt->stype = (int)(d1.val);
		break;
    case VREV:  if (apnt) apnt->vrev = d1.val;
		break;
    case TAU:  if (apnt) {
		  if (apnt->taua!= NULLVAL) apnt->taua *= d1.val;
		  else 			    apnt->taua  = d1.val;
		  if (apnt->taub!= NULLVAL) apnt->taub *= d1.val;
		  else 			    apnt->taub  = d1.val;
		  if (apnt->tauc!= NULLVAL) apnt->tauc *= d1.val;
		  else 			    apnt->tauc  = d1.val;
		  if (apnt->taud!= NULLVAL) apnt->taud *= d1.val;
		  else 			    apnt->taud  = d1.val;
		}
		break;
    case TAUM:  if (apnt) {
		  if (apnt->taua!= NULLVAL) apnt->taua *= d1.val;
		  else 			    apnt->taua  = d1.val;
		}
		break;
    case TAUH:  if (apnt) {
		  if (apnt->taub!= NULLVAL) apnt->taub *= d1.val;
		  else 			    apnt->taub  = d1.val;
		}
		break;
    case TAUC:  if (apnt) {
		  if (apnt->tauc!= NULLVAL) apnt->tauc *= d1.val;
		  else 			    apnt->tauc  = d1.val;
		}
		break;
    case TAUD:  if (apnt) {
		  if (apnt->taud!= NULLVAL) apnt->taud *= d1.val;
		  else 			    apnt->taud  = d1.val;
		}
		break;
    case OFFSET: if (apnt) {
		   apnt->voffsm  = d1.val;
		   apnt->voffsh  = d1.val;
		}
		break;
    case OFFSETM: if (apnt) apnt->voffsm  = d1.val;
		break;
    case OFFSETH: if (apnt) apnt->voffsh  = d1.val;
		break;
    case MAXCOND: if (apnt) apnt->maxcond = d1.val;
		break;
    case CAPERM: if (apnt) apnt->caperm = d1.val;
		break;
    case DENSITY: if (apnt) apnt->density = d1.val;
		break;
    case NDENSITY: if (apnt) apnt->ndensity = d1.val;
		break;
    case N:    if (apnt) apnt->n = int(d1.val);
		break;
    case UNIT: if (apnt) apnt->unitary = d1.val;
		break;
    case CAO: 	  if (apnt) apnt->cao = d1.val;
		break;
    case CAI: 	  if (apnt) apnt->cai = d1.val;
		break;
    case KEX: 	  if (apnt) apnt->kex   = d1.val;
		break;
    case BTOT:	  if (apnt) apnt->btot = d1.val;
		break;
    case BTOTI:	  if (apnt) apnt->btoti = d1.val;
		break;
    case KD:	  if (apnt) apnt->bkd = d1.val;
		break;
    case VMAX: 	  if (apnt) {
		    switch (pump) {
			case CAPUMP: apnt->vmax  = d1.val; break;
			case CABUF:  apnt->bmax  = d1.val; break;
		    }
		}
		break;
    case KM: 	  if (apnt) {
		    switch (pump) {
			case CAPUMP: apnt->pkm = d1.val; break; 
		    }
		  }
		break;
    case CBOUND:  if (apnt) apnt->cabnd = d1.val;
		break;
    case CASHELL:  if (apnt) apnt->cashell = (short int)d1.val;
		break;
    case MG: if (apnt) apnt->mg = d1.val;
		break;
   }

/* fprintf (stderr,"ca chan %s %g\n",param->name,d1.val); /* */ 
}

/*------------------------------------------*/

void xchan(void)
{
  datum d1;
  Symbol *param;
  static chattrib *apnt; 
  int npops;

  param  = (Symbol *)*pc++;
  elpnt = makelem(CHAN,elpnt);		/* make dummy elem "chan"  */
  switch (param->type)
   {
    case SYN2:
    case GLU:
    case AMPA:
    case GABA:
    case GLY:
    case NMDA:
    case CGMP:
    case K:
    case KCa:
    case NA:     npops=0; 
		 break;
    case TYPE:  
    case VREV:  
    case TAU:
    case TAUM:
    case TAUH:
    case TAUN:
    case TAUC:
    case TAUD:
    case D1:
    case D2:
    case K1:
    case K2:
    case MG:
    case N:
    case UNIT:
    case THRESH:
    case OFFSET:
    case OFFSETM:
    case OFFSETH:
    case MAXCOND:
    case CAPERM:
    case NDENSITY:
    case DENSITY:npops=1;
		 break;
   }
 
  if (npops==1) checknum(d1 = popm());

  switch (param->type)
   {
    case K:
    case KCa:
    case NA:				/* link attribs to elem */
    case SYN2:  
    case GLU:  
    case AMPA:  
    case NMDA:  
    case GABA:  
    case GLY:  
    case CGMP:  apnt = (chattrib *)chanattr(elpnt,param->type); 
		break;
    case TYPE:  if (apnt) apnt->stype = (int)(d1.val);
		break;
    case VREV:  if (apnt) apnt->vrev = d1.val;
		break;
    case TAU:  if (apnt) {
		  if (apnt->taua!= NULLVAL) apnt->taua *= d1.val;
		  else 			    apnt->taua  = d1.val;
		  if (apnt->taub!= NULLVAL) apnt->taub *= d1.val;
		  else 			    apnt->taub  = d1.val;
		  if (apnt->tauc!= NULLVAL) apnt->tauc *= d1.val;
		  else 			    apnt->tauc  = d1.val;
		  if (apnt->taud!= NULLVAL) apnt->taud *= d1.val;
		  else 			    apnt->taud  = d1.val;
		}
		break;
    case TAUN:
    case TAUM:  if (apnt) {
		  if (apnt->taua!= NULLVAL) apnt->taua *= d1.val;
		  else 			    apnt->taua  = d1.val;
		}
		break;
    case TAUH:  if (apnt) {
		  if (apnt->taub!= NULLVAL) apnt->taub *= d1.val;
		  else 			    apnt->taub  = d1.val;
		}
		break;
    case TAUC:  if (apnt) {
		  if (apnt->tauc!= NULLVAL) apnt->tauc *= d1.val;
		  else 			    apnt->tauc  = d1.val;
		}
		break;
    case TAUD:  if (apnt) {
		  if (apnt->taud!= NULLVAL) apnt->taud *= d1.val;
		  else 			    apnt->taud  = d1.val;
		}
		break;
    case THRESH: 
    case OFFSET: if (apnt) {
		  apnt->voffsm  = d1.val;
		  apnt->voffsh  = d1.val;
		}
		break;
    case OFFSETM: if (apnt) apnt->voffsm  = d1.val;
		break;
    case OFFSETH: if (apnt) apnt->voffsh  = d1.val;
		break;
    case MAXCOND: if (apnt) apnt->maxcond = d1.val;
		break;
    case CAPERM: if (apnt) apnt->caperm = d1.val;
		break;
    case DENSITY: if (apnt) apnt->density = d1.val;
		break;
    case NDENSITY: if (apnt) apnt->ndensity = d1.val;
		break;
    case N: if (apnt) apnt->n = int(d1.val);
		break;
    case UNIT: if (apnt)apnt->unitary = d1.val;
		break;
    case D1: if (apnt) apnt->d1 = d1.val;
		break;
    case D2: if (apnt) apnt->d2 = d1.val;
		break;
    case K1: if (apnt) apnt->k1 = d1.val;
		break;
    case K2: if (apnt) apnt->k2 = d1.val;
		break;
   }

/* fprintf (stderr,"chan %s %g\n",param->name,d1.val); /* */ 
}

/*------------------------------------------*/

void initrgen(void)

{
  datum d1;
  Symbol *param;
  static int ngen=0;
  static int rsd=0;
  void initrand(int ngen, int rsd);

  param  = (Symbol *)*pc++;
  if (param==0) {
     checknum(d1 = popm()); 
     ngen = (int)d1.val;
  }
  else switch (param->type)
    {
     case RSD:  checknum(d1 = popm()); 
		rsd = (int)d1.val;
		break;
     case INITRAND: 
		initrand(ngen,rsd);
		ngen = rsd = 0;
		break; 
    }
}  

/*------------------------------------------*/

void noise(void)
{
  datum d1;
  Symbol *param;
  static nattrib *apnt=0; 
  int npops;
  double chnoise, vesnoise;

  param  = (Symbol *)*pc++;
  switch (param->type)
   {
    case VESNOISE:
    case CHNOISE:
    case N:	 
    case RSD:	
    case COV:	
    case REFR:	
    case UNIT:	
    case TAUD:	
    case VSIZE:	  npops = 1; 
		 break;
   }

  if (npops==1) checknum(d1 = popm());

#ifndef XSTIMM
  switch (param->type)
   {	 					/* link attributes to elem */
    case VESNOISE:				 /* quantal vesicle noise */
		 vesnoise = d1.val;
		 if (vesnoise)
		   apnt = (nattrib *)chanattr(elpnt,param->type); 
		 else apnt = (nattrib *)NULL;
		 break;

    case CHNOISE:				 /* quantal channel noise */
		 chnoise = d1.val;
		 if (chnoise)
		   apnt = (nattrib *)chanattr(elpnt,param->type);
		 else apnt = (nattrib *)NULL;
		 break;

    case N:	 if (apnt) apnt->n = (int)d1.val;
		 break;

    case COV:	 if (apnt) apnt->cov = d1.val;
		 break;

    case REFR:	 if (apnt) apnt->refr = d1.val;
		 break;

    case RSD:	 if (apnt) apnt->rseed = (int)d1.val;
		 break;

    case UNIT:	 if (apnt) apnt->unitary = d1.val;
		 break;

    case TAUM:	 
    case TAUD:	 if (apnt) apnt->taud = d1.val;
		 break;

    case VSIZE:	 if (apnt) {
		    if (apnt->ctype==VESNOISE)
			 apnt->size = d1.val;
		    else 
		     fprintf (stderr,"Snoise: vsize is illegal with chnoise.\n");
		 }
		 break;

   }
#endif
}

/*------------------------------------------*/

void xsynapse(void)
{
  datum d1;
  Symbol *param;
  int i,npops,nfilts;
  static int xfilt=0,respfl=0;
  synapse *epnt;
  static mattrib *apnt;
#define TTIMEC 10
  double ttimec[TTIMEC];

  param  = (Symbol *)*pc++;
  epnt = (synapse *)makelem(SYNAPSE,elpnt);	/* make new synapse */
  if (elpnt->ctype==ELEMENT) apnt = (mattrib *)NULL; /* reset chan attr */
  elpnt = (elem *)epnt;

  switch (param->type)
   {
    case OPEN:  
    case CLOSE: 
    case RESP: 
    case SYN2: 
    case MESGOUT: 
    case GLU: 
    case AMPA: 
    case NMDA: 
    case CNQX: 
    case GABA: 
    case BIC: 
    case PTX: 
    case GLY: 
    case STRY: 
    case CAMP: 
    case CGMP: 
		npops=0;
		break;
    case VREV:  
    case THRESH:
    case NFILT1:		/* old syntax included for compatibility */
    case NFILT1H:
    case NFILT2:
    case NFILT3: 
    case HGAIN: 
    case TFALL2:
    case TFALL3:
    case CGAIN: 
    case MAXCOND:
    case N:
    case UNIT:
    case TAUD:
    case TRCONC:
    case KD:	
    case EXPON:
    case LINEAR:
    case DYAD:	
    case TYPE:  npops = 1;
		break;
    case TIMEC1:
    case TIMEC1H:
    case TIMEC2:
    case TIMEC3: npops=0;
		nfilts = (long int)*pc++; /* # of tau's actually entered */
		if (nfilts >= TTIMEC) nfilts = TTIMEC - 1;
		for (i=0; i<TTIMEC; i++) {
		   ttimec[i] = 0;
		}
		for (i=nfilts-1; i>=0; i--) {
		   checknum(d1 = popm());
		   ttimec[i] = d1.val;
                }
		break;
   }

  if (npops==1) checknum(d1 = popm());

  switch (param->type) {
    case RESP:   synnum = cumelem;
		break;
    case N: 				/* make channel attrib */
    case UNIT: 
    case TAUD:  if (!apnt) apnt=(mattrib*)chanattr(elpnt,SYN2);
		break;
     default:   break;
  }
  switch (param->type)
   {
    case OPEN:  epnt->ntact = param->type; 
		break;
    case CLOSE: epnt->ntact = param->type; 
		break;
    case VREV:  epnt->vrev  = d1.val;
		break;
    case THRESH: epnt->thresh= d1.val;
		break;
    case HGAIN: epnt->filt1hg= d1.val;
		break;
    case NFILT1:  epnt->nfilt1 = (short int) d1.val; xfilt = 1; break;
    case NFILT1H: epnt->nfilt1h= (short int) d1.val; xfilt = 1; break;
    case NFILT2:  epnt->nfilt2 = (short int) d1.val; xfilt = 1; break;
    case NFILT3:  epnt->nfilt3 = (short int) d1.val; xfilt = 1; break;

    case TIMEC1:  if (!xfilt) epnt->nfilt1 = nfilts;
		  epnt->timec1=makfiltarr(epnt->nfilt1,0,epnt->timec1,0);
		  if (xfilt) {		/* if nfilts was given separately */
		     for (i=0; i<epnt->nfilt1; i++) 
                       epnt->timec1[i]= ttimec[0];
		  }
		  else {		/* if nfilts from num of tau's */
		     for (i=0; i<nfilts; i++) 
                       epnt->timec1[i]= ttimec[i];
		  }
		  xfilt = 0;
		break;
    case TIMEC1H:  if (!xfilt) epnt->nfilt1h = nfilts;
		  epnt->timec1h=makfiltarr(epnt->nfilt1h,0,epnt->timec1h,0);
		   if (xfilt) {		/* if nfilts was given separately */
		     for (i=0; i<epnt->nfilt1h; i++) 
                       epnt->timec1h[i]= ttimec[0];
		  }
		  else {		/* if nfilts from num of tau's */
		     for (i=0; i<nfilts; i++) 
                       epnt->timec1h[i]= ttimec[i];
		  }
		  xfilt = 0;
		break;
    case TIMEC2:  if (!xfilt) epnt->nfilt2 = nfilts;
		  epnt->timec2=makfiltarr(epnt->nfilt2,0,epnt->timec2,0);
		  if (xfilt) {		/* if nfilts was given separately */
		     for (i=0; i<epnt->nfilt2; i++) 
                       epnt->timec2[i]= ttimec[0];
		  }
		  else {		/* if nfilts from num of tau's */
		     for (i=0; i<nfilts; i++) 
                       epnt->timec2[i]= ttimec[i];
		  }
		  xfilt = 0;
		break;
    case TFALL2: epnt->tfall2 = d1.val;
		break;
    case TIMEC3:  if (!xfilt) epnt->nfilt3 = nfilts;
		  epnt->timec3=makfiltarr(epnt->nfilt3,0,epnt->timec3,0);
	          if (xfilt) {	/* if nfilts was given separately */
		     for (i=0; i<epnt->nfilt3; i++) 
                       epnt->timec3[i]= ttimec[0];
		  }
		  else {	/* if nfilts was given by num of tau's */
		     for (i=0; i<nfilts; i++) 
                       epnt->timec3[i]= ttimec[i];
		  }
		  xfilt = 0;
		break;
    case TFALL3: epnt->tfall3 = d1.val;
		break;
    case CGAIN: epnt->cgain   = d1.val;
		break;
    case MAXCOND: epnt->maxcond = d1.val;
		break;
    case TRCONC: epnt->trconc = d1.val;
		break;
    case KD:	epnt->nkd = d1.val;
		break;
    case HCOF:	epnt->npow = d1.val;
		break;
    case LINEAR:if (epnt->curve == NULLVAL) {
		 epnt->curve = LINEAR;
    		 epnt->ngain = d1.val;
		}
		break;
    case EXPON:	if (epnt->curve == NULLVAL) {
		 epnt->curve = EXPON;
    		 epnt->ngain = d1.val;
		}
		break;
    case DYAD:	if (epnt->curve == NULLVAL) {
		 epnt->curve = DYAD;
    		 epnt->ngain = d1.val;
		}
		break;
    
    case N:  	apnt->n = int(d1.val);	/* channel attrib made above */
		break;
    case UNIT:  apnt->unitary = d1.val;
		break;
    case TAUD:  apnt->taud = d1.val;
		break;
    case GLU: 
    case AMPA: 
    case NMDA: 
    case CNQX: 
    case GABA: 
    case BIC: 
    case PTX: 
    case GLY: 
    case STRY: 
    case CAMP: 
    case CGMP:  if (epnt->mesng1==NULLVAL) epnt->mesng1 = param->type;
		else epnt->mesng2 = param->type;
		break;

    default:	break;
   }
/*fprintf(stderr,"synapse %s %g curv %g\n",
		param->name,d1.val,elpnt->curve); /* */ 
}

/*------------------------------------------*/

void xgj(void)
{
  datum d1,d[MAXNODIM];
  Symbol *param;
  gapjunc *epnt;
  static int revflag=0;
 
  param  = (Symbol *)*pc++;
  epnt = (gapjunc *)makelem(GJ,elpnt);	/* make new gap junction */
  elpnt = (elem *)epnt;			/* for membtyp() below */
  if (param==NULL) {			/* conductance */
	checknum(d1=popm());
	epnt->gmax = d1.val;
  }
  else {
   switch (param->type) {
    case GJ:
    case OPEN:
    case CLOSE:
    case MESGIN:
    case REV:
		break;
    case CAMP:
    case CGMP:	getnod(d);		/* get node, check dims */
	       break;
    case AREA:
    case DIA:
    case VGAIN:
    case OFFSET:
    case TAUN:
    case GMAX:
    case GNV: checknum(d1 = popm());
	      break;
   }

   switch (param->type) {
    case AREA: epnt->area  = d1.val;
		break;
    case DIA:  epnt->area  = MPI * d1.val * d1.val * 0.25;
		break;
    case OFFSET:if (revflag)  epnt->rvoff = d1.val;
    		else epnt->voff = d1.val;
		break;
    case VGAIN: if (revflag) epnt->rvgain = d1.val;
		else epnt->vgain  = d1.val;
		break;
    case TAUN: if (revflag) epnt->rtaun= d1.val;
    		else epnt->taun= d1.val;
		break;
    case REV:  revflag = 1;
		break;
    case GMAX: epnt->gmax= d1.val;
		break;
    case GNV: epnt->gnv= d1.val;
		break;
    case CAMP: 
    case CGMP:	epnt->nodeca = (int)d[0].val;       /* set node number */ 
		epnt->nodecd = (int)d[1].val;
		epnt->nodecd = (int)d[2].val;
		epnt->nodecd = (int)d[3].val;
		epnt->modtyp = param->type; 
		break;
    case OPEN: 
		epnt->sign = 1;
		break;
    case CLOSE:
		epnt->sign = 0;
		break;
    case GJ:
		revflag = 0;
		break;
   }
 }
}

/*------------------------------------------*/

void rload(void)
{
  datum d1;
  loadelem *epnt;

  checknum(d1 = popm());
  epnt = (loadelem *)makelem(LOAD,elpnt);	/* make new load */
  elpnt = (elem *)epnt;				/* for membtyp() below */
  epnt->z = d1.val;
}

void rcap(void)
{
  datum d1;
  capac *epnt;

  checknum(d1 = popm());
  epnt = (capac *)makelem(CAP,elpnt);		/* make new cap */
  epnt->c = d1.val;
}

void xgcap(void)
{
  datum d1;
  capac *epnt;
 
  checknum(d1 = popm());
  epnt = (capac *)makelem(GNDCAP,elpnt);	/* make new gnd cap */
  elpnt = (elem *)epnt;				/* for membtyp() below */
  epnt->c = d1.val;
}

void xresistor(void)
{
  datum d1;
  resistor *epnt;

  checknum(d1 = popm());
  epnt = (resistor *)makelem(RESISTOR,elpnt);	/* make new resistor */
  epnt->z = d1.val;
}

void rbatt(void)
{
  datum d1;
  batt *epnt;

  checknum(d1 = popm());
  epnt = (batt *)makelem(BATT,elpnt);		/* make new batt */
  epnt->v = d1.val;
}

void xgbatt(void)
{
  datum d1;
  batt *epnt;

  checknum(d1 = popm());
  epnt = (batt *)makelem(GNDBATT,elpnt);	/* make new gndbatt */
  epnt->v = d1.val;
}

void membtyp(void)
{
  datum d1;
  Symbol *param;
  int ctype;

  param  = (Symbol *)*pc++;
  checknum(d1 = popm());
  ctype = elpnt->ctype;
  switch (param->type)
   {
    case RM: switch (ctype) {
		case CABLE:  ((cable *)elpnt)->Rm = d1.val; break;
		case SPHERE: ((sphere *)elpnt)->Rm = d1.val; break;
	     } break;
    case RI: switch (ctype) {
		case CABLE:  ((cable *)elpnt)->Ri = d1.val; break;
	     } break;
    case RG: switch (ctype) {		/* for gj only */
		case GJ:  ((gapjunc *)elpnt)->specres = d1.val; break;
	     } break;
    case CM: switch (ctype) {
		case CABLE:  ((cable *)elpnt)->Cm = d1.val; break;
		case SPHERE: ((sphere *)elpnt)->Cm = d1.val; break;
	     } break;
    case JNOISE: elpnt->jnoise = d1.val; break;
    case RSD:    elpnt->rsd = int(d1.val); break;

    case VREST: switch (ctype) {
		case CABLE:  ((cable *)elpnt)->vrest = d1.val; break;
		case SPHERE: ((sphere *)elpnt)->vrest = d1.val; break;
		case GNDCAP:   ((capac *)elpnt)->vrest = d1.val; break;
		case LOAD:   ((loadelem *)elpnt)->vrest = d1.val; break;
	     } break;
    case VREV: switch (ctype) {
		case CABLE:  ((cable *)elpnt)->vrev = d1.val; break;
		case SPHERE: ((sphere *)elpnt)->vrev = d1.val; break;
		case LOAD: ((loadelem *)elpnt)->vrev = d1.val; break;
	     } break;
    default: break;
   }
}

void xvbuf(void)
{
  vbuf *epnt;

  epnt = (vbuf *)makelem(BUF,elpnt);	/* make new gndbatt */
  elpnt = (elem *)epnt;			/* for membtyp() below */
  epnt->delay = 0;			/* no delay */
}

void xvbufd(void)
{
   datum d1;
   vbuf *epnt;

  checknum(d1 = popm());
  ((vbuf *)elpnt)->delay = d1.val;	/* delay in msec */
}

/*------------------------------------------*/

int smaxmin (double val)

{ 
   nodeint ival;
   static nodeint minnode=   (nodeint)(1L << (BITS(nodeint)-1));
   static nodeint maxnode= (nodeint)(~(1L << (BITS(nodeint)-1)));

  
  val = limit(val, maxnode, minnode);
  ival = (nodeint) val;

  return (ival);
}


/*------------------------------------------*/

void conn1(void)
{
  datum d[MAXNODIM];

  getnod(d);			/* get node, check dims */
  elpnt = tmpelem();		/* make new element */
  elpnt->node1a = (int)d[0].val;	/* set node number */ 
  elpnt->node1b = (int)d[1].val;
  elpnt->node1c = (int)d[2].val;
  elpnt->node1d = (int)d[3].val;
  elpnt->node2a = NULLVAL;
  elpnt->node2b = NULLVAL;
  elpnt->node2c = NULLVAL;
  elpnt->node2d = NULLVAL;
  checknod(elpnt);		/* make node associated with element */
}

/*------------------------------------------*/

void conn1l(void)
{
  datum d1,d2,d3;
  int narg;
  node *npnt;

  narg = (long int)*pc++;		/* get loc off stack first */
  if (narg > 2) d3 = popm();
  else d3.val = LARGENODE;
  if (narg > 1) d2 = popm();
  else d2.val = LARGENODE;
  checknum(d1 = popm());
  conn1();

#ifndef XSTIMM
  checknod(elpnt);		/* make node associated with element */
  if (! (npnt=elpnt->nodp1)) {
	fprintf (stderr,"conn1L: can't find node %s\n",
	    prnode(elpnt->node1a,elpnt->node1b,elpnt->node1c,elpnt->node1d));
	return;
  }
  npnt->xloc = d1.val;
  npnt->yloc = d2.val;
  npnt->zloc = d3.val;
#endif
}

/*------------------------------------------*/

double dist3d(node *n1, node *n2)
{
   double dist,xdist,ydist,zdist;

  if (!n1 || !n2) return 0.0;
  if (n1->xloc >= LARGNOD) {
   fprintf(stderr,"\ndist3d: location of node 1: %s has not been defined.\n",
		 prnode(n1->nodenm1,n1->nodenm2,n1->nodenm3,n1->nodenm4));
   execerror ("Missing location: ","stopping...");
  }  
  if (n2->xloc >= LARGNOD) {
   fprintf(stderr,"\ndist3d: location of node 2: %s has not been defined.\n",
		 prnode(n2->nodenm1,n2->nodenm2,n2->nodenm3,n1->nodenm4));
   execerror ("Missing location: ","stopping...");
  }  
  if (n1->xloc >= LARGNOD || n2->xloc >= LARGNOD)
    xdist = 0; 
  else
    xdist = n1->xloc - n2->xloc;
  if (n1->yloc >= LARGNOD || n2->yloc >= LARGNOD)
    ydist = 0; 
  else
    ydist = n1->yloc - n2->yloc;
  if (n1->zloc >= LARGNOD || n2->zloc >= LARGNOD)
    zdist = 0; 
  else
    zdist = n1->zloc - n2->zloc;
  dist = sqrt (xdist*xdist + ydist*ydist + zdist*zdist);
/* fprintf (stderr,"%g %g %g %g %g %.20g\n",
	n1->xloc,n1->yloc,n1->zloc,n2->xloc,n2->yloc,n2->zloc); /* */
  return (dist);
}


/*------------------------------------------*/

double dist2d(node *n1, node *n2)
{
   double dist,xdist,ydist;

  if (!n1 || !n2) return 0.0;
  if (n1->xloc >= LARGNOD) {
   fprintf(stderr,"\ndist2d: location of node %s has not been defined.\n",
		 prnode(n1->nodenm1,n1->nodenm2,n1->nodenm3,n1->nodenm4));
   execerror ("Missing location: ","stopping...");
  }  
  if (n2->xloc >= LARGNOD) {
   fprintf(stderr,"\ndist2d: location of node %s has not been defined.\n",
		 prnode(n2->nodenm1,n2->nodenm2,n2->nodenm3,n1->nodenm4));
   execerror ("Missing location: ","stopping...");
  }  
  if (n1->xloc >= LARGNOD || n2->xloc >= LARGNOD)
    xdist = 0; 
  else
    xdist = n1->xloc - n2->xloc;
  if (n1->yloc >= LARGNOD || n2->yloc >= LARGNOD)
    ydist = 0; 
  else
    ydist = n1->yloc - n2->yloc;
  dist = sqrt (xdist*xdist + ydist*ydist);
/* fprintf (stderr,"%g %g %g %g %g %.20g\n",
	n1->xloc,n1->yloc,n1->zloc,n2->xloc,n2->yloc,n2->zloc); /* */
  return (dist);
}

/*------------------------------------------*/

void conn1m(void)

/* "at node: elem offset=frac put node" */

/* make a new node along a cable element */

{
  datum d[MAXNODIM],d5,d6;
  int elnum,found;
  nodeint nod1a,nod1b,nod1c,nod1d;
  node *npnt,*npnt2,*npnt3;
  elem *epnt;
  attrib *apnt,*napnt;
  conlst *lpnt,*olpnt;
  double xdist,ydist,zdist,distfrac,len,ndia;

  getnod(d);			/* get node, check dims */
  nod1a=(int)d[0].val;
  nod1b=(int)d[1].val;
  nod1c=(int)d[2].val;
  nod1d=(int)d[3].val;

  checknum(d5 = popm());	/* get fractional distance */
  checknum(d6 = popm());	/* get cable element # rel to parent */

  conn1();		/* this gets parent node number off stack, */
			/*  makes new temp. element, and connects parent node */
			/*  at one end but does not set type of element, */
			/*  nor its loc */

#ifndef XSTIMM
  if (!(npnt=findnode(elpnt->node1a,elpnt->node1b,
			elpnt->node1c,elpnt->node1d,"Atmake"))) {
     fprintf (stderr,"conn1m: can't find node %s\n",
			 prnode(elpnt->node1a,elpnt->node1b,
				elpnt->node1c,elpnt->node1d));
    execerror ("Missing parent node: ","stopping...");
  }
				
  elnum = (int)(d6.val); 		/* find previously made element */
  if (!(epnt=findelem(elnum))) {
      fprintf (stderr,"Edist: can't find element %d\n",elnum);
      return;
  }
  else checkelemnode(epnt);

  for (found=0,lpnt=npnt->elemlst; lpnt && !found; lpnt=lpnt->next) {
    if (!(epnt=(elem *)lpnt->conpnt)) continue;
    if (epnt->elnum==elnum) found=1;
  }

  if (!found) {
     fprintf (stderr,"At node: can't find elem %d connected to node %s\n",
		elnum,prnode(elpnt->node1a,elpnt->node1b,
			     elpnt->node1c,elpnt->node1d));
     execerror ("Missing element: ","stopping...");
   }
				/* find the other end of element */

/* fprintf (stderr,"conn1m elem %d elnum %d  n1 %d n2 %d\n",
		epnt->elnum, elnum,epnt->nodp1,epnt->nodp2); /* */

  if ((npnt2=epnt->nodp1)==npnt) npnt2 = epnt->nodp2;
  if (!npnt2) {
      fprintf (stderr,"Atmake: can't find second node for elem %d\n", elnum);
      execerror("Missing node: ","stopping..."); 
  }
				/* stop if element is not a cable */
  if (epnt->ctype != CABLE) {
      fprintf (stderr,"At node: element %d is not a cable\n", elnum);
      execerror("Can't make node:","stopping..."); 
  } 

  elpnt = (cable *)makelem(CABLE,elpnt);         /* copy temp elem into cable */

			/* Next, find or make the new node,  */
			/* connect it properly to the new cable, */
			/*  and reconnect old cable to the new node. */ 
			/* Use old node if exists: */
if (!(npnt3=findnode(nod1a,nod1b,nod1c,nod1d,(char *)NULL))){ 
    setnodlst((npnt3=maknod(nod1a,nod1b,nod1c,nod1d)),elpnt);/* new node, pntr*/ 
  }
  elpnt->nodp2 = npnt3;		/* set pointer from new cable to new node */
  elpnt->node2a = nod1a;
  elpnt->node2b = nod1b;
  elpnt->node2c = nod1c;
  elpnt->node2d = nod1d;
  elpnt->nodp1 = npnt;		/* set pointer from new cable to old node */
  unsetnodlst(npnt,epnt);	/* del pointer from parent node to old cable */
  setnodlst(npnt,elpnt);	/* set pointer from parent node to new cable */
  setnodlst(npnt3,epnt);	/* set pointer from new node to old cable */
  
  if (epnt->nodp1==npnt) {		/* set node pointer for old cable */
	epnt->nodp1 = npnt3;		/*  to point to "new" node (npnt3) */
	epnt->node1a = nod1a;
	epnt->node1b = nod1b;
	epnt->node1c = nod1c;
	epnt->node1d = nod1d;
  }
  else {
	epnt->nodp2 = npnt3;
	epnt->node2a = nod1a;
	epnt->node2b = nod1b;
	epnt->node2c = nod1c;
	epnt->node2d = nod1d;
  }

  distfrac = d5.val;
  if (distfrac > 1.0) distfrac = 1.0;
  if (distfrac < 0.0) distfrac = 0.0;

  if ((len=((cable*)epnt)->length)==NULLVAL)
     len = dist3d(npnt,npnt2);
  ((cable *)elpnt)->length = len * distfrac;	  /*set length of new cable */
  ((cable *)epnt)->length  = len * (1 - distfrac); /*set new length of old cabl*/

						/* find new dia */
  ndia = ((cable *)epnt)->dia * (1 - distfrac) + ((cable *)epnt)->dia2*distfrac;

  ((cable *)elpnt)->dia  = ((cable *)epnt)->dia;    /* set new dia first */
  ((cable *)epnt)->dia = ndia;	    		    /* set old dia */
  ((cable *)elpnt)->dia2   = ndia;                  /* set new dia2 */
  ((cable *)elpnt)->cplam = ((cable *)epnt)->cplam; /* make same cplam */
  ((cable *)elpnt)->Rm    = ((cable *)epnt)->Rm;
  ((cable *)elpnt)->Ri    = ((cable *)epnt)->Ri;
  ((cable *)elpnt)->Cm    = ((cable *)epnt)->Cm;
  ((cable *)elpnt)->vrev  = ((cable *)epnt)->vrev;
  ((cable *)elpnt)->vrest = ((cable *)epnt)->vrest;

 				/* copy attributes from old cable to new */ 
for (apnt=epnt->attpnt; apnt; apnt=apnt->attpnt) {
   napnt = chanattr(elpnt,apnt->ctype);		/* alloc new attribute */
   copyattr(apnt,napnt);
} 
  				/* Finally, calculate location of new node */
  xdist = npnt2->xloc - npnt->xloc;
  ydist = npnt2->yloc - npnt->yloc;
  zdist = npnt2->zloc - npnt->zloc;

  npnt3->xloc = npnt->xloc + xdist * distfrac;
  npnt3->yloc = npnt->yloc + ydist * distfrac;
  npnt3->zloc = npnt->zloc + zdist * distfrac;

#endif     /* XSTIMM */
}

/*------------------------------------------*/

void conn2s(void)
{
  datum d[MAXNODIM];

  getnod(d);				/* get node, check dims */
  elpnt->node1a = (int)d[0].val;
  elpnt->node1b = (int)d[1].val;
  elpnt->node1c = (int)d[2].val;
  elpnt->node1d = (int)d[3].val;
  checknod(elpnt);		/* make nodes associated with element */
}

/*------------------------------------------*/

void conn2sl(void)
{
  datum d1,d2,d3;
  int narg;
  node *npnt;

  narg = (long int)*pc++;	/* get loc off stack first */
  if (narg > 2) d3 = popm();
  else d3.val = LARGENODE;
  if (narg > 1) d2 = popm();
  else d2.val = LARGENODE;
  checknum(d1 = popm());
  conn2s();

#ifndef XSTIMM
  checknod(elpnt);		/* make nodes associated with element */
  if (! (npnt=elpnt->nodp1)) {
	fprintf (stderr,"conn2s: can't find node %s\n",
	   prnode(elpnt->node1a,elpnt->node1b,elpnt->node1c,elpnt->node1d));
	return;
  }
  npnt->xloc = d1.val;
  npnt->yloc = d2.val;
  npnt->zloc = d3.val;
#endif
}

/*------------------------------------------*/

void conn2d(void)
{
  datum d[MAXNODIM];

  getnod(d);				/* get node, check dims */
  elpnt = tmpelem();			/* make new element */
  elpnt->node2a = (int)d[0].val;
  elpnt->node2b = (int)d[1].val;
  elpnt->node2c = (int)d[2].val;
  elpnt->node2d = (int)d[3].val;
}

/*------------------------------------------*/

void conn2dl(void)
{
  datum d1,d2,d3;
  int narg;
  node *npnt;

  narg = (long int)*pc++;
  if (narg > 2) d3 = popm();
  else d3.val = LARGENODE;
  if (narg > 1) d2 = popm();
  else d2.val = LARGENODE;
  checknum(d1 = popm());
  conn2d();
#ifndef XSTIMM
  checknod(elpnt);		/* make nodes associated with element */
  if (! (npnt=elpnt->nodp2)) {
	fprintf (stderr,"conn2dl: can't find node %s\n",
	    prnode(elpnt->node2a,elpnt->node2b,elpnt->node2c,elpnt->node2d));
	return;
  }
  npnt->xloc = d1.val;
  npnt->yloc = d2.val;
  npnt->zloc = d3.val;
#endif
}

/*------------------------------------------*/

void gettype(void)

/* Return numerical type for element. */

{
   datum d;
   Symbol *field;
 
  field = (Symbol *)*pc++;
  if (field) d.val= field->type;
  d.vtype = NUMBER;
  d.type = VAR;
  pushm (d);
}

/*------------------------------------------*/

void efield(void)

/* push elem field on stack */

{
    datum d1,d2;
    Symbol *field;
    elem *epnt;
    int elnum;

 field = (Symbol *)*pc++;
 checknum(d1 = popm());
 d2.val = 0;
 d2.type = VAR;
#ifndef XSTIMM
 elnum = (int)(d1.val);
 if (!(epnt=findelem(elnum))) {
      fprintf (stderr,"Elemfield: can't find elem %d\n",elnum);
      execerror ("Missing element; ","stopping...");
      return;
 }
 else {
  switch (field->type) {
   case TYPE:
	d2.val = epnt->ctype;
	break;
   case LENGTH:
	if (epnt->ctype != CABLE) {
	  d2.val = 0.0;
	  break;
	}
	if (((cable*)epnt)->length != NULLVAL) {
	  d2.val = ((cable*)epnt)->length;
	}
	else {			/* find length from location */
           conlst *lpnt;
	   node *npnt;
	   sphere *sepnt;
	   cable *cepnt;
           double len, mindia1, mindia2;
           int found;

         checknod(epnt);

         found = 0;
         if (npnt=epnt->nodp1) {           /* check for spheres at node 1 */
           mindia1 = 1e10;
           for (lpnt=npnt->elemlst; lpnt; lpnt=lpnt->next) {
              if (sepnt=(sphere *)lpnt->conpnt) {
                if (sepnt->ctype==SPHERE) {
                  found = 1;
                  if (mindia1 > sepnt->dia) mindia1 = sepnt->dia;
                }
              }
           }  /* for (lpnt;;) */
           if (!found) mindia1 = 0.0;
         }  /* nodp1 */

         if (npnt=epnt->nodp2) {           /* check for spheres at node 2 */
           mindia2 = 1e10;
           for (lpnt=npnt->elemlst; lpnt; lpnt=lpnt->next) {
              if (sepnt=(sphere *)lpnt->conpnt) {
                if (sepnt->ctype==SPHERE) {
                  found = 1;
                  if (mindia2 > sepnt->dia) mindia2 = sepnt->dia;
                }
              }
           }  /* for (lpnt;;) */
           if (!found) mindia2 = 0.0;
         }  /* nodp2 */

           cepnt=(cable*)epnt;
           if ((len=cepnt->length)==NULLVAL) {           /* if length not set */
             if (!epnt->nodp1 && !epnt->nodp2) len = 10.0;/* default length */
             else len = dist3d(epnt->nodp1,epnt->nodp2);   /* calc length */
             if (found) {
                 len -= (mindia1 + mindia2) * 0.5;
                 if (len < 0.0) len = 1e-3;
             }
           }
	  d2.val = len;
	}
	break;
   case N3DIST:
	checknod(epnt);
	d2.val = dist3d(epnt->nodp1,epnt->nodp2);
	break;
   case N2DIST:
	checknod(epnt);
	d2.val = dist2d(epnt->nodp1,epnt->nodp2);
	break;
   case DIA:
	d2.val = ((cable *)epnt)->dia;
	break;
   case DIA2:
	d2.val = ((cable *)epnt)->dia2;
	break;
   case RM:
	d2.val = ((cable *)epnt)->Rm;
	break;
   case RI:
	d2.val = ((cable *)epnt)->Ri;
	break;
   case CPLAM:
	d2.val = ((cable *)epnt)->cplam;	/* only for cable */
	break;
   case NODE1A:
	d2.val = epnt->node1a;
	break;
   case NODE1B:
	d2.val = epnt->node1b;
	break;
   case NODE1C:
	d2.val = epnt->node1c;
	break;
   case NODE1D:
	d2.val = epnt->node1d;
	break;
   case NODE2A:
	d2.val = epnt->node2a;
	break;
   case NODE2B:
	d2.val = epnt->node2b;
	break;
   case NODE2C:
	d2.val = epnt->node2c;
	break;
   case NODE2D:
	d2.val = epnt->node2d;
	break;
   case SCURVE:
	if (epnt->ctype == SYNAPSE) {
	   synap *spnt;
          if (spnt=(synap *)epnt->lptr)    /* get ptr to synapse from elem */
	    d2.val = spnt->curve;
	  else
	    d2.val = ((synapse *)epnt)->curve;
	 }
	break;
   case SDYAD:				/* return elem number of original */
	if (epnt->ctype == SYNAPSE)
	  if (((synapse *)epnt)->curve==DYAD) 
	    d2.val = ((synapse *)epnt)->ngain; /* elem # of dyad (orig. synap)*/
	  if (d2.val < 0) d2.val = 0;
	break;
   case DYAD:				/* return elem # of first dyad */
	if (epnt->ctype == SYNAPSE) {
	   synap *spnt;
	 if (spnt=(synap *)epnt->lptr) {   /* get ptr to synapse from elem */
	  /* if (spnt->dyad) 
	     d2.val = spnt->dyad->sdyad; */
		/* need to scan all synapse elements, find one that */
		/*  has lptr same as sdyad above, then find elem number */
	  }
	 }
	break;
   case NUMDYAD:			/* return number of dyads */
			/* Note: this only works well after "step" or "run" */
			/*  so that low-lev synap structures are set up. */
			/*  It is very inefficient searching through elems. */

	if (epnt->ctype == SYNAPSE) {
	   synap *spnt;
	 if (spnt=(synap *)epnt->lptr)    /* get ptr to synapse from elem */
	   d2.val = numdyad(spnt->dyad);
	 else {				  /* else check all elements */
	    int n;
	    elem *espnt;

	   for (n=0,espnt=elempnt; espnt; espnt=espnt->next) { 
	     if (espnt->ctype == SYNAPSE && ((synapse *)espnt)->curve==DYAD)
		if (((synapse*)espnt)->ngain==elnum) n++;
	   }
	   d2.val = n;
	  }
	 }
	break;
  }
 }
#endif
 d2.vtype = NUMBER;
 d2.type = VAR;
 pushm(d2);
}

/*------------------------------------------*/

void nfield(void)

/* push node field on stack */

{
    datum d[MAXNODIM],d5,d6;
    Symbol *field;
    int i, elnum, nod1, nod2, nod3, nod4;
    node *npnt;
    conlst *lpnt=(conlst*)NULL;
    elem *epnt=(elem*)NULL;

 field = (Symbol *)*pc++;
 if (field == (Symbol *)0) d5 = popm();		/* rel elem number */
 getnod(d);					/* get node, check dims */

#ifndef XSTIMM
 nod1 = (int)d[0].val;
 nod2 = (int)d[1].val;
 nod3 = (int)d[2].val;
 nod4 = (int)d[3].val;
 if (!(npnt=findnode(nod1,nod2,nod3,nod4,"Nfield"))) {
   execerror ("Missing node; ","stopping...");
   return; 
 }
 if (field == (Symbol *)0) {			/* "node -> elemnum" */
   elnum = (int)(d5.val);
   for (lpnt=npnt->elemlst,i=1; lpnt && i<elnum; i++)	/* find the element */
        lpnt=lpnt->next;
   if (lpnt) epnt=(elem *)lpnt->conpnt;
   if (!lpnt || !epnt) {
     fprintf (stderr,"Nodefield: can't find elem %d relative to node %s\n",
				elnum,prnode(nod1,nod2,nod3,nod4));
     execerror ("Missing element; ","stopping..."); /* */
     d6.val = -1;			
   }
   else d6.val = epnt->elnum;				/* element's number */
 }
 else {
  switch (field->type) {
   case NUMCONN:
        for (lpnt=npnt->elemlst,i=0; lpnt; i++)		/* how many elems */
          lpnt=lpnt->next; 
	d6.val = i;
	break;
   case XLOC:
	d6.val = npnt->xloc;
	break;
   case YLOC:
	d6.val = npnt->yloc;
	break;
   case ZLOC:
	d6.val = npnt->zloc;
	break;
  }
 }
#endif
 d6.vtype = NUMBER;
 d6.type = VAR;
 pushm(d6);
}

/*------------------------------------------*/

double mindist (double dist1, double dist2, double dist3)

/* find minimum distance between a point and
   a line segment. If the point is off the end
   of the line segment, return the distance to
   that end.
*/

{
    double mdist,dist12,dist22,dist32,tmp1,tmp;

				/* check to see if node is off the end: */
   if (dist1 > dist2 && (dist1*dist1 - dist2*dist2) > dist3*dist3)
		 mdist = dist2;
   else
   if (dist1 < dist2 && (dist2*dist2 - dist1*dist1) > dist3*dist3)
		 mdist = dist1;

				/* formula for distance between a point
				/* and a line segment in three dimensions: */
   else {
      if (dist3==0.0) dist3 = 1e-20;
      dist12 = dist1 * dist1;
      dist22 = dist2 * dist2;
      dist32 = dist3 * dist3;
      tmp1 = (dist32 + dist12 - dist22) / (2 * dist3);
      if ((tmp=dist12-tmp1*tmp1) <= 0) mdist = 0;
      else mdist = sqrt (tmp);
   }
  return mdist;
}

/*------------------------------------------*/

void eedist(void)

/* return 3D distance between 2 elements */

{
    datum d1,d2;
    int nod1a, nod1b, nod1c, nod1d, elnum1,elnum2;
    int inside1, inside2;
    double dist,dist1,dist2,dist3,distc;
    node *np1,*np2,*np11,*np12,*np21,*np22;
    elem *epnt1,*epnt2;
    vec p11, p12, p21, p22, p1, p2, pt1, pt2;
    double t,s;

 checknum(d1 = popm());
 checknum(d2 = popm());
 elnum1 = (int)(d1.val);
 elnum2 = (int)(d2.val);
 dist = 0;

#ifndef XSTIMM

 if (!(epnt1=findelem(elnum1))) {
      fprintf (stderr,"E3dist: can't find element %d\n",elnum1);
      execerror ("Missing element: "," stopping... ");
      return;  
 }
 if (!(epnt2=findelem(elnum2))) {
      fprintf (stderr,"E3dist: can't find element %d\n",elnum2);
      execerror ("Missing element: "," stopping... ");
      return;  
 }
 np11 = epnt1->nodp1;
 np12 = epnt1->nodp2;
 np21 = epnt2->nodp1;
 np22 = epnt2->nodp2;
 if (!np11) {
       fprintf (stderr,"E3dist: can't find first node for element %d\n",elnum1);
       return;  
 }
 if (!np21) {
       fprintf (stderr,"E3dist: can't find first node for element %d\n",elnum2);
       return;  
 }

 p11.x = np11->xloc;
 p11.y = np11->yloc;
 p11.z = np11->zloc;
 if (np11->xloc >= LARGNOD) p11.x = 0;
 if (np11->yloc >= LARGNOD) p11.y = 0;
 if (np11->zloc >= LARGNOD) p11.z = 0;

 p12.x = np12->xloc;
 p12.y = np12->yloc;
 p12.z = np12->zloc;
 if (np12->xloc >= LARGNOD) p12.x = 0;
 if (np12->yloc >= LARGNOD) p12.y = 0;
 if (np12->zloc >= LARGNOD) p12.z = 0;

 p21.x = np21->xloc;
 p21.y = np21->yloc;
 p21.z = np21->zloc;
 if (np21->xloc >= LARGNOD) p21.x = 0;
 if (np21->yloc >= LARGNOD) p21.y = 0;
 if (np21->zloc >= LARGNOD) p21.z = 0;

 p22.x = np22->xloc;
 p22.y = np22->yloc;
 p22.z = np22->zloc;
 if (np22->xloc >= LARGNOD) p22.x = 0;
 if (np22->yloc >= LARGNOD) p22.y = 0;
 if (np22->zloc >= LARGNOD) p22.z = 0;

 switch (epnt1->ctype) {

  case CABLE: 
 
    if (!np12) {
      fprintf (stderr,"E2dist: can't find second node for element %d\n",elnum1);
      return;  
    }
   break;

  case SPHERE: 
  default:				/* element is not cable */
    
    p12.x = p11.x;
    p12.y = p11.y;
    p12.z = p11.z;
    break;

  }

 switch (epnt2->ctype) {

  case CABLE: 
 
    if (!np22) {
      fprintf (stderr,"E2dist: can't find second node for element %d\n",elnum2);
      return;  
    }
   break;

  case SPHERE: 
  default:				/* element is not cable */
    
    p22.x = p21.x;
    p22.y = p21.y;
    p22.z = p21.z;
    break;

  }
		/* find closest points */

 distc = norm_line (p11,vsub(p12,p11),p21,vsub(p22,p21),&t,&s,&p1,&p2);

 if (distc < 0) {		/* if line segs are parallel */
    inside1 = inside2 = 0;
    if (vdist (p11,p12) > vdist(p21,p22)) {     /* use line 1 for dist3 */
	dist3 = vlen(vsub(p12,p11));
	if ((vdist (p21,p11)+vdist(p21,p12))<(vdist (p22,p11)+vdist(p22,p12))) {
          dist1 = vlen(vsub(p21,p11));
          dist2 = vlen(vsub(p21,p12));
	}
	else {
          dist1 = vlen(vsub(p22,p11));
          dist2 = vlen(vsub(p22,p12));
	}
    }
    else {					/* use line 2 for dist3 */
        dist3 = vlen(vsub(p22,p21));
	if ((vdist (p11,p21)+vdist(p11,p22))<(vdist (p12,p21)+vdist(p12,p22))) {
          dist1 = vlen(vsub(p11,p21));
          dist2 = vlen(vsub(p11,p22));
	} else {
          dist1 = vlen(vsub(p12,p21));
          dist2 = vlen(vsub(p12,p22));
	}
    }
  /* fprintf (stderr,"dist1 %g dist2 %g dist3 %g\n",dist1,dist2,dist3); */

    dist = mindist (dist1,dist2,dist3) - ((cable*)epnt1)->dia*0.5 - 
					 ((cable*)epnt2)->dia*0.5;
 }

 else {			/* line segs not parallel */
  inside1 = (0 <= t && t <= 1);	     /* point of intersection is inside seg 1 */
  inside2 = (0 <= s && s <= 1);      /* point of intersection is inside seg 2 */

  if (inside1) {
    if (inside2) {				/* if inside both */
       dist = distc - ((cable*)epnt1)->dia*0.5 - ((cable*)epnt2)->dia*0.5;
    }
    else {					/* if inside 1 but not 2 */
     if (vdist(p2,p21) < vdist(p2,p22)) {	/* 21 is closer */
        
        dist1 = vlen(vsub(p21,p11));
        dist2 = vlen(vsub(p21,p12));
     }
     else {					/* 22 is closer */
        dist1 = vlen(vsub(p22,p11));
        dist2 = vlen(vsub(p22,p12));
     }
     dist3 = vlen(vsub(p11,p12));
     dist = mindist (dist1,dist2,dist3) - ((cable*)epnt1)->dia*0.5;

    }
  }
  else if (inside2){				/* if inside2 but not 1 */
     if (vdist(p1,p11) < vdist(p1,p12)) {	/* 11 is closer */
        
        dist1 = vlen(vsub(p11,p21));
        dist2 = vlen(vsub(p11,p22));
     }
     else {					/* 12 is closer */
        dist1 = vlen(vsub(p12,p21));
        dist2 = vlen(vsub(p12,p22));
     }
     dist3 = vlen(vsub(p21,p22));
     dist = mindist (dist1,dist2,dist3) - ((cable*)epnt2)->dia*0.5;
  }

  else {					/* outside both */
     if (vdist(p1,p11) < vdist(p1,p12)) pt1 = p11;
     else                               pt1 = p12;
     if (vdist(p2,p21) < vdist(p2,p22)) pt2 = p21;
     else                               pt2 = p22;
     if (vdist(p1,pt1) < vdist(p2,pt2)) {
        dist1 = vlen(vsub(pt2,p11));
        dist2 = vlen(vsub(pt2,p12));
        dist3 = vlen(vsub(p11,p12));
        dist = mindist (dist1,dist2,dist3) - ((cable*)epnt1)->dia*0.5;
     }
     else {
        dist1 = vlen(vsub(pt1,p21));
        dist2 = vlen(vsub(pt1,p22));
        dist3 = vlen(vsub(p21,p22));
        dist = mindist (dist1,dist2,dist3) - ((cable*)epnt2)->dia*0.5;
     }
  }
 }
#endif
  dist = max(dist,0);			/* no negative distances */
  d1.val = dist;
  d1.vtype = NUMBER;
  d1.type = VAR;
  pushm(d1);
}

/*------------------------------------------*/

void e3dist(void)

/* return 3D distance between a node and an element */

{
    datum d1,d2,d[MAXNODIM];
    int nod1a, nod1b, nod1c, nod1d, elnum;
    double dist,dist1,dist2,dist3;
    node *npnt,*np1,*np2;
    elem *epnt;

 checknum(d2 = popm());
 elnum = (int)(d2.val);
 dist = 0;
 getnod(d);					/* get node, check dims */

#ifndef XSTIMM
 nod1a = (int)d[0].val;
 nod1b = (int)d[1].val;
 nod1c = (int)d[2].val;
 nod1d = (int)d[3].val;

 if (!(npnt=findnode(nod1a,nod1b,nod1c,nod1d,"E3dist"))) return; 

 if (!(epnt=findelem(elnum))) {
      fprintf (stderr,"E3dist: can't find element %d\n",elnum);
      execerror ("Missing element: "," stopping... ");
      return;  
 }
 np1 = epnt->nodp1;
 np2 = epnt->nodp2;
 if (!np1) {
         fprintf (stderr,"E3dist: can't find node for element %d\n",elnum);
         return;  
 }

 dist1 = dist3d (npnt,np1);

 switch (epnt->ctype) {

#define max(x, y)	(((x) < (y)) ? (y) : (x))

  case CABLE: 
 
    if (!np2) {
         fprintf (stderr,"E2dist: can't find node for element %d\n",elnum);
         return;  
    }

   dist2 = dist3d (npnt,np2);
   dist3 = dist3d (np1,np2);
   dist = mindist (dist1,dist2,dist3) - ((cable*)epnt)->dia*0.5;
   dist = max(dist,0);			/* no negative distances */
   break;

  case SPHERE: 
    dist = dist1 - ((sphere*)epnt)->dia*0.5;
    dist = max(dist,0);			/* no negative distances */
    break;

  default:				/* element is not cable */
    dist = dist1;
    break;
  }

#endif
  d1.val = dist;
  d1.vtype = NUMBER;
  d1.type = VAR;
  pushm(d1);
}

/*------------------------------------------*/

void e2dist(void)

/* return 2D distance between a node and an element */

{
    datum d1,d2,d[MAXNODIM];
    int nod1a, nod1b, nod1c, nod1d, elnum;
    double dist,dist1,dist2,dist3;
    node *npnt,*np1,*np2;
    elem *epnt;

 checknum(d2 = popm());
 elnum = (int)(d2.val);
 dist = 0;
 getnod(d);					/* get node, check dims */

#ifndef XSTIMM
 nod1a = (int)d[0].val;
 nod1b = (int)d[1].val;
 nod1c = (int)d[2].val;
 nod1d = (int)d[3].val;

 if (!(npnt=findnode(nod1a,nod1b,nod1c,nod1d,"E2dist"))) return; 

 if (!(epnt=findelem(elnum))) {
      fprintf (stderr,"E2dist: can't find element %d\n",elnum);
      execerror ("Missing element: "," stopping... ");
      return;  
 }
 np1 = epnt->nodp1;
 np2 = epnt->nodp2;
 if (!np1) {
         fprintf (stderr,"E2dist: can't find node for element %d\n",elnum);
         return;  
 }

 dist1 = dist2d (npnt,np1);

 switch (epnt->ctype) {

  case CABLE: 
 
    if (!np2) {
         fprintf (stderr,"E2dist: can't find node for element %d\n",elnum);
         return;  
    }

   dist2 = dist2d (npnt,np2);
   dist3 = dist2d (np1,np2);
   dist = mindist (dist1,dist2,dist3) - ((cable*)epnt)->dia*0.5;
   dist = max(dist,0);			/* no negative distances */
   break;

  case SPHERE: 
    dist = dist1 - ((sphere*)epnt)->dia*0.5;
    dist = max(dist,0);			/* no negative distances */
    break;

  default:				/* element is not cable */
    dist = dist1;
    break;
  }

#endif
  d1.val = dist;
  d1.vtype = NUMBER;
  d1.type = VAR;
  pushm(d1);
}

/*------------------------------------------*/

void efrac(void)

/* return point in an element that is the closest
   to a node */

{
    datum d1,d2,d[MAXNODIM];
    int nod1a, nod1b, nod1c, nod1d, narg, elnum;
    double frac,dist,dist1,dist2,dist3;
    node *npnt,*np1,*np2;
    elem *epnt;

 checknum(d2 = popm());
 elnum = (int)(d2.val);
 frac = 0;
 getnod(d);					/* get node, check dims */

#ifndef XSTIMM
 nod1a = (int)d[0].val;
 nod1b = (int)d[1].val;
 nod1c = (int)d[2].val;
 nod1d = (int)d[3].val;

 if (!(npnt=findnode(nod1a,nod1b,nod1c,nod1d,"Fdist"))) return; 

 if (!(epnt=findelem(elnum))) {
      fprintf (stderr,"Fdist: can't find element %d\n",elnum);
      execerror ("Missing element: "," stopping... ");
      return;
 }

 np1 = epnt->nodp1;
 np2 = epnt->nodp2;
 if (!np1) {
         fprintf (stderr,"Fdist: can't find node for element %d\n",elnum);
         return;  
 }

 dist1 = dist3d (npnt,np1);

 if (epnt->ctype==CABLE) {
 
    if (!np2) {
         fprintf (stderr,"Edist: can't find node for element %d\n",elnum);
         return;  
    }

   dist2 = dist3d (npnt,np2);
   dist3 = dist3d (np1,np2);
   if (dist3 == 0.0) dist3 = 1e-20;
   dist = mindist (dist1,dist2,dist3);
   if      (dist == dist1) frac = 0.0;
   else if (dist == dist2) frac = 1.0;
   else frac = sqrt (dist1*dist1 - dist*dist) / dist3;
  }

  else {				/* element is not cable */
    frac = 0.0;
  }
#endif
  d1.val = frac;
  d1.vtype = NUMBER;
  d1.type = VAR;
  pushm(d1);
}

/*------------------------------------------*/

void n3dist(void)

/* return distance between two nodes */

{
    datum d1, d[MAXNODIM];
    int nod1a, nod1b, nod1c, nod1d;
    int nod2a, nod2b, nod2c, nod2d;
    node *npnt1,*npnt2;

 getnod(d);					/* get node, check dims */
 nod2a = (int)d[0].val;
 nod2b = (int)d[1].val;
 nod2c = (int)d[2].val;
 nod2d = (int)d[3].val;

 getnod(d);					/* get node, check dims */
 nod1a = (int)d[0].val;
 nod1b = (int)d[1].val;
 nod1c = (int)d[2].val;
 nod1d = (int)d[3].val;

#ifndef XSTIMM
 if (!(npnt1=findnode(nod1a,nod1b,nod1c,nod1d,"N3dist"))) return; 
 if (!(npnt2=findnode(nod2a,nod2b,nod2c,nod2d,"N3dist"))) return; 

 d1.val = dist3d (npnt1,npnt2);
#else
 d1.val = 0;
#endif
 d1.vtype = NUMBER;
 d1.type = VAR;
 pushm(d1);
}

/*------------------------------------------*/

void n2dist(void)

/* Return 2D distance between two nodes: */
/*  ignore z distance. */

{
    datum d1, d[MAXNODIM];
    int nod1a, nod1b, nod1c, nod1d;
    int nod2a, nod2b, nod2c, nod2d;
    node *npnt1,*npnt2;

 getnod(d);					/* get node, check dims */
 nod2a = (int)d[0].val;
 nod2b = (int)d[1].val;
 nod2c = (int)d[2].val;
 nod2d = (int)d[3].val;

 getnod(d);					/* get node, check dims */
 nod1a = (int)d[0].val;
 nod1b = (int)d[1].val;
 nod1c = (int)d[2].val;
 nod1d = (int)d[3].val;
 
#ifndef XSTIMM
 if (!(npnt1=findnode(nod1a,nod1b,nod1c,nod1d,"N2dist"))) return; 
 if (!(npnt2=findnode(nod2a,nod2b,nod2c,nod2d,"N2dist"))) return; 

 d1.val = dist2d (npnt1,npnt2);
#else
 d1.val = 0;
#endif
 d1.vtype = NUMBER;
 d1.type = VAR;
 pushm(d1);
}

/*------------------------------------------*/

void xgausnn(void)

/* Make random array of cells with defined regularity. */

{
  int i;
  Symbol *param,*var;
  datum d1,d2,d3;
  static int narg,numcells=0,numpts=0;
  static double mean=0,stdev=0,density=0,reg=0;
  static double xframe=0,yframe=0,xcenter=0,ycenter=0;
  static float *x=0,*y=0;
  double *dpnt;


  param = (Symbol *)*pc++;
  switch (param->type) {

    case MEAN:  checknum(d1 = popm());
		mean = d1.val;
		break;

    case STDEV: checknum(d1 = popm());
		stdev = d1.val;
		break;

    case DENSITY: checknum(d1 = popm());
		density = d1.val;
		break;

    case REG:   checknum(d1 = popm());
		reg = d1.val;
		break;

    case SIZE:  narg = (long int) *pc++;
                if (narg > 2) d3 = popm();
                else d3.val = 0;
                if (narg > 1) d2 = popm();
                else d2.val = 0;
 		checknum(d1 = popm());
		xframe = d1.val;
		yframe = d2.val;
		break;

    case CENTER: narg = (long int) *pc++;
                if (narg > 2) d3 = popm();
                else d3.val = 0;
                if (narg > 1) d2 = popm();
                else d2.val = 0;
 		checknum(d1 = popm());
		xcenter = d1.val;
		ycenter = d2.val;
		break;

    case GAUSNN:var = (Symbol *)(*pc++);          /* array to be created */
#ifndef XSTIMM

  		if (setdisp>=0) disp = setdisp;	/* "-d n" overrides "disp=n" */
 		vidmode = (int)setvar("vidmode");
    		numpts=gausnn(mean,stdev,density,reg,xframe,yframe,
			xcenter,ycenter,numcells,&x,&y,0,!(disp||vidmode),0);

     		d1.val = numpts;	/* set up nc array dimensions */
     		d1.type = VAR;
     		d1.vtype = NUMBER;
     		d2.val = 2;
     		d2.type = VAR;
     		d2.vtype = NUMBER;
		pushm(d1);
		pushm(d2);
		dpnt = darr2(var,2);	     /* allocate space for array */
		for (i=0; i<numpts; i++) {
		  *dpnt++ = x[i];	     /* make x and y values be */
		  *dpnt++ = y[i];	     /*  successive values in array */
		}
		var->vtype = NUMBER;
		mean = stdev = density = reg = 0.0;
		xframe = yframe = xcenter = ycenter = 0;
		numcells = 0;
		if (x) efree (x);	/* erase the subroutine's arrays */
		if (y) efree (y);
#else
     		d1.val = 0;	
     		d1.type = VAR;
     		d1.vtype = NUMBER;
#endif
		pushm (d1); 	/* finally, return n */
		break; 
  } /* switch */
}

/*------------------------------------------*/

void xrecept(void)

/* read in data from rod or cone statement:

		1	x position
		2	y position  (optional)
*/

{
  datum d1,d2,d3;
  int narg;
  Symbol *param;
  node *npnt;
  double xpos,ypos,zpos;
  photrec *epnt;

#ifdef XSTIM
  recnod *maksnod(),*rpnt;
#endif

  param = (Symbol *)*pc++;			/* receptor type */
  narg  = (long int)*pc++;
  if (narg > 2) d3 = popm();			/* get z loc */
  else d3.val = LARGENODE;
  if (narg > 1) d2 = popm();			/* get y loc */
  else d2.val = LARGENODE;
  if (narg > 0) checknum(d1 = popm());		/* get x loc */
  else d1.val = LARGENODE;
  
  zpos  = d3.val;				/* z val */
  ypos  = d2.val;				/* y val */
  xpos  = d1.val;				/* x val */

  epnt = (photrec *)makelem(param->type,elpnt); /* make new photoreceptor */
  elpnt = (elem *)epnt;				/* save for recparm() below */

  if (narg > 2) 
     epnt->zpos  = zpos;			/* z val */
  if (narg > 1) 
     epnt->ypos  = ypos;			/* y val */
  epnt->xpos     = xpos;			/* x val */

						/* no z val in struct recept */
  if (!epnt->modif) {				/* never modify node */
#ifndef XSTIMM
    if (! (npnt=epnt->nodp1)) {
	fprintf (stderr,"recept: can't find node %s\n",
		prnode(epnt->node1a,epnt->node1b,epnt->node1c,epnt->node1d));
	return;
    }

     if (xpos!=LARGENODE)
        npnt->xloc = xpos;			/* By default, place node at */
     if (ypos!=LARGENODE)
        npnt->yloc = ypos;			/*  same loc. as photorecep */
     if (zpos!=LARGENODE)
        npnt->zloc = zpos;
#endif

#ifdef XSTIM
  rpnt = maksnod();
  rpnt->recnm1 = elpnt->node1a;     /* node numbers are defined by conn() */
  rpnt->recnm2 = elpnt->node1b;
  rpnt->recnm3 = elpnt->node1c;
  rpnt->recnm4 = elpnt->node1d;
  if (xpos!=LARGENODE) rpnt->xpos = xpos;
  else               rpnt->xpos = 0;
  if (ypos!=LARGENODE) rpnt->ypos = ypos;
  else               rpnt->ypos = 0;
#endif
  }   /* if (!elpnt->modif) */
}

/*------------------------------------------*/

void xrecparm(void)

/* get parameters for rods and cones */

{
  datum d1;
  Symbol *param;
  int npops;
  photrec * ppnt;

  param  = (Symbol *)*pc++;
  
  switch (param->type) {
    case DIA:   
    case MAXCOND: 
    case PIGM:  
    case PATHL: 
    case ATTF:  
    case FILT:  
    case TIMEC1:
    case DARKNOISE:
    case PHOTNOISE: 
    case RSD:   npops = 1;
		break;
    case SAVE:  
    case RESTORE:
		npops = 0;
		break;
   }
  
  if (npops==1) checknum(d1 = popm());

#ifndef XSTIMM

		/* Here, elpnt is a photrec, set by recept() above */

  ppnt = (photrec *)elpnt; 
  switch (param->type) {
    case DIA:   ppnt->dia = d1.val;
		break;
    case MAXCOND: ppnt->maxcond = d1.val; 
		break;
    case PIGM:  ppnt->pigm  = d1.val; 
		break;
    case PATHL: ppnt->pathl = d1.val; 
		break;
    case ATTF:  ppnt->attf = d1.val; 
		break;
    case FILT:  ppnt->filt = (int)d1.val; 
		break;
    case TIMEC1:ppnt->timec1 = d1.val; 
		break;
    case RSD:	ppnt->prseed = (int)d1.val; 
		break;
    case SAVE:  ppnt->save = 1; 
		break;
    case RESTORE:
		ppnt->restore = 1; 
		break;
    case PHOTNOISE:
		ppnt->photnoise = (int)d1.val; 
		break;
    case DARKNOISE:
		ppnt->darknoise = d1.val; 
		break;
   }
#endif 		/* XSTIMM */
}

/*------------------------------------------------------*/

double gaussf1(double x, double xcent, double rad)

/* Return a gaussian function of x,
   centered in an array of given size. */

{
   double distsq,tempx;

   tempx = (double)(x - xcent);
   distsq = tempx*tempx;
   return (exp(-(distsq) / (rad*rad)));

}

/********************************************************/

extern double stiminc;			/* synap/stim timinc in control.h */
#define RTIMINC stiminc

#ifdef XSTIM
					/* this part for "stim", not "nc" */
void xstim()

/* read in data from stimulus statement:
*/

{
  datum d1,d2,d3,d4,d[MAXNODIM];
  Symbol *param;
  int stimarr, backarr;
  int narg, spec;
  FILE *ftemp;

static double start=0.0;
static double dur=0.1;
static double stop=0.1;
static double wavel=1.0;
static double inten=100.0;
static double backgr=0.0;
static double blurrad=0.0,tblur=0.0;
static double speriod = 20;
static double contrast = 0.5;
static double tfreq = 0.0;
static double sscale = 1.0;
static double ymins = 0.0;
static double ymaxs = 0.0;
static double xenv = 10.0;
static double yenv = 10.0;
static double orient = 0.0;
static double sphase = 0.0;
static double puffconc = 0.0;
static int drift=0;

static int stype;
/* static int rnum1,rnum2,rnum3,rnum4;  /* not used for "stim" */
static double xstm=0, ystm=0;
static double xcent=0, ycent=0;		/* center coords for stim array */
static int centerfl=0;                  /* center specified */
static double xmax,xmin,ymax,ymin;	/* borders of of photrec array */
static double xrange, yrange;		/* size of photrec array */
static int arrsize;			/* size of stimulus array */
static int arraymade=0;			/* stimulus array already allocated */
static int stimflag=0;                  /* start stimulus */
static int blurfl=0;                    /* blur array done */
static int blursize=0;			/* radius of blur array from makblur()*/
static int cumstim=0;			/* number of stimulus reports */
static int puffmsg=0;			/* CAMP or CGMP */

extern double stimdia;                     /* defined in "stimsub.c" */
extern double stimydist;                   /* defined in "stimsub.c" */

  param  = (Symbol *)*pc++;
  
  switch (param->type)
   {
    case BAR: 
    case SPOT:  checknum(d1 = popm());
                stimdia   = d1.val;
                stype = param->type; 
                break;

    case GABOR:  
    case SINE:  checknum(d1 = popm());
                speriod   = d1.val;
                stype = param->type; 
                break;
    case RECT:
		narg = (int)*pc++;
  		if (narg > 1) d2 = popm();
		else d2.val = 1.0;
 		checknum(d1 = popm());
		stimdia = d1.val;
		stimydist = d2.val;
		stype = param->type; 
		break;
    case NODE:
		getnod(d);			/* get node num, check dims */
    /*          rnum1 = d[0].val;		/* get node number */
    /*          rnum2 = d[1].val;
                rnum3 = d[2].val;
                rnum4 = d[3].val;
    */
                break;
    case TRANSDUCER: 
    case ROD: 
    case CONE:  
		getnod(d);			/* get node num, check dims */
    /*          rnum1 = d[0].val;		/* get node number */
    /*          rnum2 = d[1].val;
                rnum3 = d[2].val;
                rnum4 = d[3].val;
    */
                stype = param->type; 
                break;
    case LOC:   narg  = (int)*pc++;
                if (narg > 0) {
                  if (narg > 1) {
                    if (narg > 2) {
                      if (narg > 3) {
		          d4 = popm();
                          ymaxs = d4.val;
                      }
		      d3 = popm();
                      ymins = d3.val;
		      if (ymins > ymaxs) {
	                double tx;
			  tx = ymins;	/* swap ymin and ymax if backwards */
			  ymins = ymaxs;
			  ymins = tx; 
		      }
                    }
                    d2 = popm();                /* read in y loc */
                    ystm = d2.val;
                  }
		  else ystm = 0;
 		checknum(d1 = popm());
                xstm = d1.val;
               }
                break;
    case CENTER:narg  = (int)*pc++;
                if (narg > 0) {
                  if (narg > 1) {
                    if (narg > 2) 
                       d3 = popm();
                    d2 = popm();                /* read in y center */
                    ycent = d2.val;
                  }
		  else ycent = 0;
 		checknum(d1 = popm());
                xcent = d1.val;
		centerfl = 1;
                }
                break;
    case SSCALE: checknum(d1 = popm());
                sscale   = d1.val;
                break;
    case START: checknum(d1 = popm());
                start = d1.val;
                stimflag = 1;
                break;
    case STOP:  checknum(d1 = popm());
                stop = d1.val;
		dur = start - stop;
                stimflag = 1;
                break;
    case DUR:   checknum(d1 = popm());
                dur   = d1.val;
                break;
    case CONTRAST: checknum(d1 = popm());
                contrast   = d1.val;
                break;
    case TFREQ: checknum(d1 = popm());
                tfreq = d1.val;
                break;
    case DRIFT: checknum(d1 = popm());
                drift = int(d1.val);
                break;
    case ORIENT: checknum(d1 = popm());
                orient = d1.val;
                break;
    case SPHASE: checknum(d1 = popm());
                sphase = d1.val;
                break;
    case XENV:	checknum(d1 = popm());
                xenv = d1.val;
                break;
    case YENV:	checknum(d1 = popm());
                yenv = d1.val;
                break;
    case WAVEL: spec = (int)*pc++;
		switch (spec) {
		  case 0: 
 		    checknum(d1 = popm());
		    wavel = d1.val;
		    break;
		  case XENON:
                    wavel = 0;
                    break;
		  case SUN:
                    wavel = 1;
                    break;
		  case TUNGSTEN:
                    wavel = 2;
                    break;
		}
		break;

    case INTEN: checknum(d1 = popm());
                inten = d1.val;
                break;

    case BACKGR: checknum(d1 = popm());
                backgr = d1.val;
                backarr = 1;
                stype = param->type; 
                break;

    case VCLAMP:
    case CCLAMP: checknum(d1 = popm());
                inten = d1.val;
                stype = param->type; 
                break;

    case PUFF:  checknum(d1 = popm());
                puffconc = d1.val;
		pc++;
                break;

    case BLUR:  checknum(d1 = popm());		 /* get blur diameter */
                tblur = d1.val/2;		 /* divide dia by 2 = radius */
		if (blurfl) {
		   if (blurrad!=tblur) blurfl = 0;  /* remake blur function */
                }
		blurrad = tblur;
                break;
    case SFILE: if (*stimfile) fclose (stimout);
                *stimfile = 0;
		if (!checkstr(d1=popm())) break; 	 /* get filename */
                strcpy (stimfile,d1.str);  		/* copy filename */
                if (stimhlp) fprintf (stderr,"stimfil '%s'\n",stimfile);
                if (*stimfile) {
                        stimfflg = 1; 
                        if ((ftemp=fopen(stimfile,"w")) == NULL) {
                          fprintf (stderr,"%s: can't open file '%s'\n",
						progname,stimfile);
                          break;
                        }
                        else stimout = ftemp;
                }
                else {
                        stimfflg = 0;
                        stimout = stdout;
                }
		stcomment();			/* print comment in stim file */
                break;

    case STIM:  
        stimarr = 0;
        backarr = 1;
					/* find photoreceptor array size */

#ifdef XSTIM
  varcopy();
  if (setdebug) debug = setdebug;
  if (setdebgz) debugz = setdebgz;
#endif

     switch (stype) {
       case BAR:
       case RECT:
       case SPOT:
       case SINE:
       case GABOR:
       case BACKGR:
        if (!blurfl) {
             if (blurrad>0.0) {
	       blursize=makblur(blurrad,sscale); /* make a blurring function */
               blurfl = 1;
	       fprintf (stimout,"## %d %-6s blur     array size %d\n",
				++cumstim,findsym(stype),blursize*2); /* */
	       if (arraymade) {			/* if backgr array made */
		   delc(2);			/* delete existing arrays */
		   arraymade = 0;
	       }
	     }
        }
	if (!arraymade) {
	   findarrsiz(&xmax,&xmin,&ymax,&ymin);  /* find maxmin of recepts */
	   if (!centerfl) {
	     xcent = (xmax+xmin) * 0.5;
	     ycent = (ymax+ymin) * 0.5;
	   }
	   xrange = xmax-xmin;
	   yrange = ymax-ymin;
	   if (sscale <=0) sscale = 1.0;
	   arrsize = (int)(max(xrange,yrange) / sscale);
	   if (arrsize==0) arrsize = 1;
	   if (blurrad>0) arrsize += blursize*2;

	   fprintf (stimout,"## %d %-6s stimulus array size %d\n",
				++cumstim,findsym(stype),arrsize);	/* */

	   if (initc(2,arrsize)<=0) {		/* make stimulus to fit */
	       fprintf (stderr,"Stimulus array is too big = %d\n",arrsize);
	       fprintf (stderr,"Blur array dia %d\n",blursize*2);
	       fprintf (stderr,"Stimulus xmax %g xmin %g ymax %g ymin %g\n",
			xmax,xmin,ymax,ymin);
    	       execerror ("can't continue"," stopping...");
	   }
	   arraymade = 1;
	}
	 break;
       default:
	  break;
      } /* switch stype */

        switch (stype) {

        case 0:
        default: break;

/* Vclamp, cclamp, rod and cone are done in xstim() for "nc" below.
   They are not performed by (the program) "stim" because they don't
   require blur, and sometimes need to be done in addition to a
   predefined, blurred stimulus.
*/

        case VCLAMP:
/*              vclist(start,    rnum1,rnum2,rnum3,rnum4,inten,wavel,"vcl");
                vclist(start+dur,rnum1,rnum2,rnum3,rnum4,inten,wavel,"eff");
*/              break;

        case CCLAMP:
/*              vclist(start,    rnum1,rnum2,rnum3,rnum4,inten,wavel,"ccl");
                vclist(start+dur,rnum1,rnum2,rnum3,rnum4,inten,wavel,"iof");
*/              break;

        case TRANSDUCER:
        case ROD:
        case CONE: 				/* flash one receptor */
/*              vclist(start,    rnum1,rnum2,rnum3,rnum4, inten,wavel,"del");
                vclist(start+dur,rnum1,rnum2,rnum3,rnum4,-inten,wavel,"del"); 
*/              break;

        case PUFF: 				/* puff neurotrans/2nd msng */
                  break;

        case BACKGR:
                  recback(stimarr,0.0,wavel);   /* zero recept stim inten */
                  recback(backarr,backgr,wavel); /* set recept backgr inten */
                  abslist(0.0, start);          /* make action list */
                  break;
        case BAR:
		makrect(stimarr, stimdia, (double)CONVSIZE,
			xstm-xcent,ystm-ycent,sscale,inten,wavel);
                if (stimflag) {
			double xr;
                  stimflag = 0;
		  if (int(stimdia/sscale+0.5) & 1) xr = 0; else xr = 0.5;
                  recpat(stimarr,
			xcent+xr*sscale,ycent,sscale); /* set recept inten */
                  stimlist(1.0, start);         /* make an action list */
                  stimlist(-1.0, start+dur);
                  recback(stimarr,0.0,1.0);   /* zero recept stim inten */
                  recback(backarr,backgr,1.0);/* set recept backgr inten */
                }
                break;

        case RECT:
		makrect(stimarr, stimdia, stimydist,xstm-xcent,ystm-ycent,
							sscale,inten,wavel);
                if (stimflag) {
                  stimflag = 0;
                  recpat(stimarr,xcent,ycent,sscale);  /* set recept inten */
                  stimlist(1.0, start);         /* make an action list */
                  stimlist(-1.0, start+dur);
                  recback(stimarr,0.0,1.0);   /* zero recept stim inten */
                  recback(backarr,backgr,1.0);/* set recept backgr inten */
                }
                break;

        case SPOT:
                makspot(stimarr, stimdia,xstm-xcent,ystm-ycent,
						sscale,inten,wavel);
                if (stimflag) {
                  stimflag = 0;
                  recpat(stimarr,xcent,ycent,sscale);  /* set recept inten */
                  stimlist(1.0, start); 	/* make an action list */
                  stimlist(-1.0, start+dur);
                  recback(stimarr,0.0,1.0);     /* zero recept stim inten */
                  recback(backarr,backgr,1.0);  /* set recept backgr inten */
                }
                break;

        case SINE:
					/* xstm is xmin; ystm is xmax. */
	  if (xstm==0 && ystm==0) {
	     xstm = -1000;
	     ystm =  1000;
	  }
	  if (xstm > ystm) {
	       double tx;
	     tx = xstm;		/* swap xmin and xmax if backwards */
	     xstm = ystm;
	     ystm = tx; 
	  }
          if (ymins==ymaxs) {
             ymins = -1000.0;
             ymaxs =  1000.0;
          }
          if (speriod <= 0.0) speriod = 1;   /* spatial period from user */
          if (drift) {		/* drifting grating */
                 double timres,tperiod,tincr,stime;

	    if (tfreq==0.0) tfreq = 1.0;
            timres = .001;
            tperiod = 1.0 / tfreq;	/* temporal period */
	    tincr = tperiod * timres;
	    tincr = abs(tincr);		/* allow negative temporal frequency */
	    if (tincr < RTIMINC) tincr = RTIMINC; /* time incr = recep timinc */
            for (stime=start; stime<(start+dur-tincr); stime += tincr) {
              maksine(stimarr,speriod,xstm,ystm,ymins,ymaxs,
				-(stime-start)/tperiod,drift,orient,sphase,
				-xcent,-ycent,sscale,inten,contrast,wavel); 
              recpat(stimarr,xcent,ycent,sscale);  /* set recept inten */
              stimlist(1.0, stime); 	    /* make a delta action list */
              stimlist(-1.0, stime+tincr);
              recback(stimarr,0.0,1.0);     /* zero recept stim inten */
              recback(backarr,backgr,1.0);  /* set recept backgr inten */

            }   /* for (stime= ; ; ) */
          }   /* if (drate) */
          else if (tfreq) {		/* stationary counterphasing grating */
                 double timres,tperiod,tincr,stime;

            timres = .001;
            tperiod = 1.0 / tfreq;	/* temporal period */
	    tincr = tperiod * timres;
	    tincr = abs(tincr);		/* allow negative temporal frequency */
	    if (tincr < RTIMINC) tincr = RTIMINC; /* time incr = recep timinc */
            for (stime=start; stime<(start+dur-tincr); stime += tincr) {
              maksine(stimarr,speriod,xstm,ystm,ymins,ymaxs,
				-(stime-start)/tperiod,0,orient,sphase,
				-xcent,-ycent,sscale,inten,contrast,wavel); 
              recpat(stimarr,xcent,ycent,sscale);  /* set recept inten */
              stimlist(1.0, stime); 	    /* make a delta action list */
              stimlist(-1.0, stime+tincr);
              recback(stimarr,0.0,1.0);     /* zero recept stim inten */
              recback(backarr,backgr,1.0);  /* set recept backgr inten */

            }   /* for (stime= ; ; ) */
          }   /* if (drate) */
          else {		/* static grating */
              maksine(stimarr,speriod,xstm,ystm,ymins,ymaxs,0.0,0,orient,sphase,
				-xcent,-ycent,sscale,inten,contrast,wavel); 

              if (stimflag) {			/* if start val is given */
                 stimflag = 0;
                 recpat(stimarr,xcent,ycent,sscale);  /* set recept inten */
                 stimlist(1.0, start); 	/* make an action list */
                 stimlist(-1.0, start+dur);
                 recback(stimarr,0.0,1.0);     /* zero recept stim inten */
                 recback(backarr,backgr,1.0);  /* set recept backgr inten */
              }
          } /* else */
	  break;

        case GABOR:
					/* xstm is xmin; ystm is xmax. */
	  if (xstm==0 && ystm==0) {
	     xstm = -1000;
	     ystm =  1000;
	  }
	  if (xstm > ystm) {
	       double tx;
	     tx = xstm;		/* swap xmin and xmax if backwards */
	     xstm = ystm;
	     ystm = tx; 
	  }
          if (ymins==ymaxs) {
             ymins = -1000.0;
             ymaxs =  1000.0;
          }
          if (speriod <= 0.0) speriod = 1;   /* spatial period from user */
          if (drift) {		/* drifting grating */
                 double timres,tperiod,tincr,stime;

	    if (tfreq==0.0) tfreq = 1.0;
            timres = .001;
            tperiod = 1.0 / tfreq;	/* temporal period */
	    tincr = tperiod * timres;
	    tincr = abs(tincr);		/* allow negative temporal frequency */
	    if (tincr < RTIMINC) tincr = RTIMINC; /* time incr = recep timinc */
            for (stime=start; stime<(start+dur-tincr); stime += tincr) {
              makgabor(stimarr,speriod,xstm,ystm,ymins,ymaxs,
				-(stime-start)/tperiod,drift,orient,sphase,
				-xcent,-ycent,sscale,inten,contrast,wavel,
				xenv, yenv); 
              recpat(stimarr,xcent,ycent,sscale);  /* set recept inten */
              stimlist(1.0, stime); 	    /* make a delta action list */
              stimlist(-1.0, stime+tincr);
              recback(stimarr,0.0,1.0);     /* zero recept stim inten */
              recback(backarr,backgr,1.0);  /* set recept backgr inten */

            }   /* for (stime= ; ; ) */
          }   /* if (drate) */
          else if (tfreq) {		/* stationary counterphasing grating */
                 double timres,tperiod,tincr,stime;

            timres = .001;
            tperiod = 1.0 / tfreq;	/* temporal period */
	    tincr = tperiod * timres;
	    tincr = abs(tincr);		/* allow negative temporal frequency */
	    if (tincr < RTIMINC) tincr = RTIMINC; /* time incr = recep timinc */
            for (stime=start; stime<(start+dur-tincr); stime += tincr) {
              makgabor(stimarr,speriod,xstm,ystm,ymins,ymaxs,
				-(stime-start)/tperiod,0,orient,sphase,
				-xcent,-ycent,sscale,inten,contrast,wavel,
				xenv,yenv); 
              recpat(stimarr,xcent,ycent,sscale);  /* set recept inten */
              stimlist(1.0, stime); 	    /* make a delta action list */
              stimlist(-1.0, stime+tincr);
              recback(stimarr,0.0,1.0);     /* zero recept stim inten */
              recback(backarr,backgr,1.0);  /* set recept backgr inten */

            }   /* for (stime= ; ; ) */
          }   /* if (drate) */
          else {		/* static grating */
              makgabor(stimarr,speriod,xstm,ystm,ymins,ymaxs,0.0,0,
				orient,sphase,
				-xcent,-ycent,sscale,inten,contrast,wavel, 
				xenv,yenv); 

              if (stimflag) {			/* if start val is given */
                 stimflag = 0;
                 recpat(stimarr,xcent,ycent,sscale);  /* set recept inten */
                 stimlist(1.0, start); 	/* make an action list */
                 stimlist(-1.0, start+dur);
                 recback(stimarr,0.0,1.0);     /* zero recept stim inten */
                 recback(backarr,backgr,1.0);  /* set recept backgr inten */
              }
          } /* else */
	  break;

        }       /* switch (stype) */

	stype = 0;		/* reset stype for next stim statement */
        break;  /* case STIM */

  }     /* switch */
}

/*------------------------------------------------------*/

void findarrsiz(double *xmax,double *xmin,double *ymax,double *ymin)

/* Find max, min for the array of photoreceptors
    described by the user's program.
*/

{
  recnod *npt;
  double xma,xmi,yma,ymi;

  xma = yma = 0.0;
  xmi = ymi = 1e10;
  for (npt=reclist; npt; npt=npt->next) {
    xma  = max(xma,npt->xpos);
    xmi  = min(xmi,npt->xpos);
    yma  = max(yma,npt->ypos);
    ymi  = min(ymi,npt->ypos);
  }
  *xmax = xma;
  *xmin = xmi;
  *ymax = yma;
  *ymin = ymi;
}


#endif 			/* end of xstim for generating stimulus file */

/********************************************************/

#ifdef XMOD			/* this part for nc (modcode.o): */

void xstim(void)

/* read in data from stimulus statement:
*/

{
#define  STFILNAM 40
  datum d1,d2,d3,d4,d[MAXNODIM];
  Symbol *param;
  int narg, spec;
  double xpos, ypos;
  static double dia, ydist, halfx, halfy, radius;
  static double orientrad, sphasenorm, sino, coso;
  double xr, yr, xcent, ycent;
  static double start,dur;
  static double inten,sinten,backgr,xsine,theta,sincr;
  static double xedgemin, xedgemax, yedgemin, yedgemax;
  static int stype,rnum1,rnum2,rnum3,rnum4,filuse=0;
  static char *filename;
  static double xloc = 0;
  static double yloc = 0;
  static double ymins = 0;
  static double ymaxs = 0;
  static double xenv = 10;
  static double yenv = 10;
  static double wavel = 1.0;
  static double speriod = 20;
  static double contrast = 0.5;
  static double tfreq = 0.0;
  static double orient = 0.0;
  static double sphase = 0.0;
  static double sscale = 1.0;
  static char stimfile[STFILNAM]={0};
  static char stimfilz[STFILNAM]={0};
  static double puffconc = 0.0;
  static int puffmsg=0;			/* CAMP or CGMP */
  static int drift = 0;
  photrec *ept;
  recep *rpnt;
  FILE *ftemp;

  param  = (Symbol *)*pc++;

  switch (param->type)
   {
    case RECT:  narg = (long int)*pc++;
  		if (narg > 1) d2 = popm();
		else d2.val = 1.0;
 		checknum(d1 = popm());
		dia   = d1.val;
		ydist = d2.val;
		stype = param->type; 
		break;
    case BAR: 
    case SPOT:  checknum(d1 = popm());
		dia   = d1.val;
		stype = param->type; 
		break;
    case GABOR:  
    case SINE:  checknum(d1 = popm());
		speriod  = d1.val;
		stype = param->type; 
		break;
    case NODE:
		getnod(d);			/* get node num, check dims */
		rnum1 = (int)d[0].val;		/* get node number */
		rnum2 = (int)d[1].val;
		rnum3 = (int)d[2].val;
		rnum4 = (int)d[3].val;
		break;

    case TRANSDUCER: 
    case ROD: 
    case CONE:
		getnod(d);			/* get node num, check dims */
		rnum1 = (int)d[0].val;		/* get node number */
		rnum2 = (int)d[1].val;
		rnum3 = (int)d[2].val;
		rnum4 = (int)d[3].val;
		stype = param->type; 
		break;
    case LOC:	narg  = (long int)*pc++;
  		if (narg > 0) {
  		  if (narg > 1) {
  		    if (narg > 2) {
                      if (narg > 3) {
                          d4 = popm();
                          ymaxs = d4.val;
                      }
                      else ymaxs = 0;
                      d3 = popm();
                      ymins = d3.val;
		      if (ymins > ymaxs) {
	                double tx;
			  tx = ymins;	/* swap ymin and ymax if backwards */
			  ymins = ymaxs;
			  ymins = tx; 
		      }
                    }  
                    else ymins = 0;
     	 	    d2 = popm();		/* read in y loc */
		    yloc = d2.val;
  		  }
                else yloc = 0;
 		checknum(d1 = popm());   	/* x loc */
		xloc = d1.val;
		}
		break;
    case CENTER: narg  = (long int)*pc++;
  		if (narg > 0) {
  		  if (narg > 1) {
  		    if (narg > 2) 
     	 	       d3 = popm();
     	 	    d2 = popm();		/* read in y loc */
  		  }
 		checknum(d1 = popm());	 	/* x loc */
		}
		break;
    case START:	checknum(d1 = popm());
		start = d1.val;
		break;
    case DUR:	checknum(d1 = popm());
		dur   = d1.val;
		break;
    case CONTRAST: checknum(d1 = popm());
                contrast   = d1.val;
                break;
    case TFREQ:	checknum(d1 = popm());
		tfreq  = d1.val;
		break;
    case DRIFT:	checknum(d1 = popm());
		drift  = int(d1.val);
		break;
    case ORIENT:checknum(d1 = popm());
		orient  = d1.val;
		break;
    case SPHASE:checknum(d1 = popm());
		sphase  = d1.val;
		break;
    case XENV:	checknum(d1 = popm());
		xenv  = d1.val;
		break;
    case YENV:	checknum(d1 = popm());
		yenv  = d1.val;
		break;
    case WAVEL: spec = (long int)*pc++;	
		switch (spec) {
		  case 0:
 		    checknum(d1 = popm());
		    wavel = d1.val;
		    break;
		  case SUN:
		    wavel = 0;
		    break;
		  case XENON:
		    wavel = 1;
		  break;
		}
		break;
    case SSCALE:checknum(d1 = popm());
		sscale = d1.val;
		break;
    case INTEN:	checknum(d1 = popm());
		inten = d1.val;
		break;
    case BACKGR: checknum(d1 = popm());
		backgr = d1.val;
		stype = param->type; 
		break;
    case VCLAMP:
    case CCLAMP: checknum(d1 = popm());
		inten = d1.val;			/* get node number */
		stype = param->type; 
		break;
    case PUFF:  checknum(d1 = popm());
                puffconc = d1.val;
		stype = param->type;
  		param = (Symbol *)*pc++;
  		puffmsg = param->type;
                break;

    case BLUR:	checknum(d1 = popm());
		/* blurrad = d1.val; */
		break;
    case SFILE:	if (!checkstr(d1=popm())) break; 	 /* get filename */
		filename = d1.str;
		if (stimfflg && *stimfile) fclose(stimin); /* close previous */
                strcpy (stimfile,filename);  		/* copy filename */
		stfile = stimfile;			/* for plotinit */
		if (! *filename) { filuse = 0; break; }
		if ((ftemp=fopen(stimfile,"r"))==NULL) { /* open file */
                  strcpy(stimfilz,"zcat ");		/* check for .gz file */
                  strncat(stimfilz,stimfile,STFILNAM-5);
                  strncat(stimfilz,".gz",STFILNAM-5);
		     if ((ftemp=popen(stimfilz,"r"))==NULL) {
			fprintf (stderr,"%s: can't open file '%s'\n",
					progname,stimfile);
			filuse = 0;
			break;
		     }
		}
		stimin = ftemp;
		filuse = 1;
		break;

    case STIM:	 			/* real time stimulus without blur */

	switch (stype) {

	case VCLAMP:
		makvstim(start, rnum1,rnum2,rnum3,rnum4,inten,"evclamp");
		makvstim(start+dur,rnum1,rnum2,rnum3,rnum4,inten,"fvcoff");
		break;

	case CCLAMP:
		makvstim(start, rnum1,rnum2,rnum3,rnum4,inten,"gclamp");
		makvstim(start+dur,rnum1,rnum2,rnum3,rnum4,inten,"hccloff");
		break;

        case PUFF: 				/* puff neurotrans/2nd msng */
	   switch (puffmsg) {
		case GLU:
		 makvstim(start,    rnum1,rnum2,rnum3,rnum4, puffconc,"i");
		 makvstim(start+dur,rnum1,rnum2,rnum3,rnum4,-puffconc,"i");
		break;
		case AMPA:
		 makvstim(start,    rnum1,rnum2,rnum3,rnum4, puffconc,"j");
		 makvstim(start+dur,rnum1,rnum2,rnum3,rnum4,-puffconc,"j");
		break;
		case NMDA:
		 makvstim(start,    rnum1,rnum2,rnum3,rnum4, puffconc,"k");
		 makvstim(start+dur,rnum1,rnum2,rnum3,rnum4,-puffconc,"k");
		break;
		case CNQX:
		 makvstim(start,    rnum1,rnum2,rnum3,rnum4, puffconc,"l");
		 makvstim(start+dur,rnum1,rnum2,rnum3,rnum4,-puffconc,"l");
		break;
		case GABA:
		 makvstim(start,    rnum1,rnum2,rnum3,rnum4, puffconc,"m");
		 makvstim(start+dur,rnum1,rnum2,rnum3,rnum4,-puffconc,"m");
		break;
		case BIC:
		 makvstim(start,    rnum1,rnum2,rnum3,rnum4, puffconc,"n");
		 makvstim(start+dur,rnum1,rnum2,rnum3,rnum4,-puffconc,"n");
		break;
		case PTX:
		 makvstim(start,    rnum1,rnum2,rnum3,rnum4, puffconc,"o");
		 makvstim(start+dur,rnum1,rnum2,rnum3,rnum4,-puffconc,"o");
		break;
		case GLY:
		 makvstim(start,    rnum1,rnum2,rnum3,rnum4, puffconc,"p");
		 makvstim(start+dur,rnum1,rnum2,rnum3,rnum4,-puffconc,"p");
		break;
		case STRY:
		 makvstim(start,    rnum1,rnum2,rnum3,rnum4, puffconc,"q");
		 makvstim(start+dur,rnum1,rnum2,rnum3,rnum4,-puffconc,"q");
		break;
		case CAMP:
		 makvstim(start,    rnum1,rnum2,rnum3,rnum4, puffconc,"r");
		 makvstim(start+dur,rnum1,rnum2,rnum3,rnum4,-puffconc,"r");
		break;
		case CGMP:
		 makvstim(start,    rnum1,rnum2,rnum3,rnum4, puffconc,"s");
		 makvstim(start+dur,rnum1,rnum2,rnum3,rnum4,-puffconc,"s");
		break;
		case CA:
		 makvstim(start,    rnum1,rnum2,rnum3,rnum4, puffconc,"t");
		 makvstim(start+dur,rnum1,rnum2,rnum3,rnum4,-puffconc,"t");
		break;
	    }
           break;
	case ROD:			/* add to inten on specific receptor */
	case CONE: 
	case TRANSDUCER: 
		makrstim(start+dur,rnum1,rnum2,rnum3,rnum4,-inten,wavel,"del");
		makrstim(start,    rnum1,rnum2,rnum3,rnum4, inten,wavel,"del");
		break;	

       }

      if (filuse) stimfflg = 1;
      else {

        stimfflg = 0;

	switch (stype) {

	case VCLAMP:			/* don't make these stim twice */ 
	case CCLAMP:
	case PUFF:
	case ROD:
	case CONE: 
	case TRANSDUCER: 
		  break;

	case BACKGR: 					/* abs inten */

	       			/* look for existing photoreceptors: */
		if (runyet) {
	  	 for (rpnt=recpnt; rpnt; rpnt=(recep*)rpnt->next) {
		  if (rpnt->ctype == ROD || rpnt->ctype == CONE ||
		      rpnt->ctype == TRANSDUCER)
		    makrstim (start,rpnt->recnm1,rpnt->recnm2,rpnt->recnm3,
					rpnt->recnm4,backgr,wavel,"abs");
		  }
                 }
				/* look for new rod, code elements, */
			        /*  even if run yet: */

	  	 for (ept=(photrec*)elempnt; ept; ept=(photrec*)ept->next) {
		  if (ept->ctype == ROD || ept->ctype == CONE ||
		      ept->ctype == TRANSDUCER)
		    makrstim (start,ept->node1a,ept->node1b,ept->node1c,
					ept->node1d,backgr,wavel,"abs");
		 }
		break;


	case BAR:			/* stim for possibly all receptors */
	  halfx = dia / 2;
	  halfy = 1000;
	  if (runyet) {
	   for (rpnt=recpnt; rpnt; rpnt=(recep*)rpnt->next) {
	    if (rpnt->ctype == ROD || rpnt->ctype == CONE || 
		      rpnt->ctype == TRANSDUCER)
	    if (inrect(rpnt->xloc,rpnt->yloc, xloc+halfx, xloc-halfx,
			yloc+halfy, yloc-halfy, 0.0)) {
		makrstim(start,    rpnt->recnm1,rpnt->recnm2,rpnt->recnm3,
					rpnt->recnm4,inten,wavel,"del");
		makrstim(start+dur,rpnt->recnm1,rpnt->recnm2,rpnt->recnm3,
					rpnt->recnm4,-inten,wavel,"del");
	     }
	   }
	  }
	  else {
	   for (ept=(photrec*)elempnt; ept; ept=(photrec*)ept->next) {
	    if (ept->ctype == ROD || ept->ctype == CONE || 
		      ept->ctype == TRANSDUCER)
	    if (inrect(ept->xpos,ept->ypos,
			xloc+halfx, xloc-halfx,
			yloc+halfy, yloc-halfy, 0.0)) {
		makrstim(start,    ept->node1a,ept->node1b,ept->node1c,
					ept->node1d,inten,wavel,"del");
		makrstim(start+dur,ept->node1a,ept->node1b,ept->node1c,
					ept->node1d,-inten,wavel,"del");
	    }
	   }
	  }
	  break;

	case SINE:			/* stim for possibly all receptors */
					/* xloc is xmin; yloc is xmax. */
          if (yloc==0 & xloc==0) {		/* xmax, xmin */
             yloc = 1000.0;
             xloc =  -1000.0;
          }
	  if (xloc > yloc) {
	            double tx;
	          tx = xloc;		/* swap xmin and xmax if backwards */
		  xloc = yloc;
		  yloc = tx; 
	  	}
          if (ymins==ymaxs) {
             ymins = -1000.0;
             ymaxs =  1000.0;
          }
          orientrad = MPI / 180.0 * orient;
          sphasenorm = sphase / 360.0;
          coso = cos(orientrad);
          sino = sin(orientrad);
          xcent = (yloc + xloc) * 0.5;          /* yloc is xmax; xloc is xmin */
          ycent = (ymaxs + ymins) * 0.5; 
          if (speriod <= 0.0) speriod = 1;   /* spatial period from user */
          if (drift) {			/* drifting grating */

                 double timres,tperiod,tincr,stime;

	    if (tfreq==0.0) tfreq = 1.0;
	    timres = .001;
            tperiod = 1.0 / tfreq;	/* temporal period */
            tincr = timres * tperiod;	/* time incr = time res * t period */
	    tincr = abs(tincr);		/* allow negative temporal frequency */
	    if (tincr < RTIMINC) tincr = RTIMINC; /* time incr = recep timinc */
            for (stime=start; stime<(start+dur-tincr); stime += tincr) {
	      for (ept=(photrec*)elempnt; ept; ept=(photrec*)ept->next) 
	        if (ept->ctype == ROD || ept->ctype == CONE ||
		      ept->ctype == TRANSDUCER) {
		   xpos = ept->xpos;
		   ypos = ept->ypos;

				/* if the photorec is under grating */
	           if (inrect(xpos,ypos,yloc,xloc,ymaxs,ymins,orient)) {
		     xr = (xpos-xcent)*coso - (ypos-ycent)*sino + xcent; 
		     theta = (xr-xcent)/speriod +
				-(stime-start)*drift/tperiod + sphasenorm;
                     sinten = inten * (1 + contrast * sin(2*MPI*theta));
		     makrstim(stime,      ept->node1a,ept->node1b,ept->node1c,
					ept->node1d,sinten,wavel,"del");
		     makrstim(stime+tincr,ept->node1a,ept->node1b,ept->node1c,
					ept->node1d,-sinten,wavel,"del");
		   } /* if (inrect()) */
		   else {		/* set mean intens if outside */
                     sinten = inten;	
		     makrstim(stime,      ept->node1a,ept->node1b,ept->node1c,
					ept->node1d,sinten,wavel,"del");
		     makrstim(stime+tincr,ept->node1a,ept->node1b,ept->node1c,
					ept->node1d,-sinten,wavel,"del");
		   }
                  } /* if (ept->ctype==ROD... */
            }   /* for (stime= ; ; ) */
          }   /* if (drift) */
          else if (tfreq) {		/* stationary counterphasing grating */

                 double timres,tperiod,tincr,stime;

	    if (tfreq==0.0) tfreq = 1.0;
	    timres = .001;
            tperiod = 1.0 / tfreq;	/* temporal period */
            tincr = timres * tperiod;	/* time incr = time res * t period */
	    tincr = abs(tincr);		/* allow negative temporal frequency */
	    if (tincr < RTIMINC) tincr = RTIMINC; /* time incr = recep timinc */
            for (stime=start; stime<(start+dur-tincr); stime += tincr) {
	      for (ept=(photrec*)elempnt; ept; ept=(photrec*)ept->next) 
	        if (ept->ctype == ROD || ept->ctype == CONE ||
		      ept->ctype == TRANSDUCER) {
		    static double stheta, ttheta;

		   xpos = ept->xpos;
		   ypos = ept->ypos;

				/* if the photorec is under grating */
	           if (inrect(xpos,ypos,yloc,xloc,ymaxs,ymins,orient)) {
		     xr = (xpos-xcent)*coso - (ypos-ycent)*sino + xcent; 
		     stheta = (xr-xcent)/speriod + sphasenorm;
		     ttheta = -(stime-start)/tperiod;
                     sinten = inten * (1 + contrast * sin(2*MPI*stheta) *
						      sin(2*MPI*ttheta) );
		     makrstim(stime,      ept->node1a,ept->node1b,ept->node1c,
					ept->node1d,sinten,wavel,"del");
		     makrstim(stime+tincr,ept->node1a,ept->node1b,ept->node1c,
					ept->node1d,-sinten,wavel,"del");
		   } /* if (inrect()) */
		   else {		/* set mean intens if outside */
                     sinten = inten;	
		     makrstim(stime,      ept->node1a,ept->node1b,ept->node1c,
					ept->node1d,sinten,wavel,"del");
		     makrstim(stime+tincr,ept->node1a,ept->node1b,ept->node1c,
					ept->node1d,-sinten,wavel,"del");
		   }
                  } /* if (ept->ctype==ROD... */
            }   /* for (stime= ; ; ) */
          }   /* if (tfreq) */
          else {		/* static grating */
	    for (ept=(photrec*)elempnt; ept; ept=(photrec*)ept->next) {
	      if (ept->ctype == ROD || ept->ctype == CONE ||
		      ept->ctype == TRANSDUCER) {
		 xpos = ept->xpos;
		 ypos = ept->ypos;
	 /*        fprintf  (stderr,"%g %g %g %g %g %g %g\n",
				xpos,ypos,yloc,xloc,ymaxs,ymins,orient); /* */
	         if (inrect(xpos,ypos,yloc,xloc,ymaxs,ymins,orient)) {
		   xr = (xpos-xcent)*coso - (ypos-ycent)*sino + xcent; 
	           theta = (xr-xcent) / speriod + sphasenorm;
                   sinten = inten * (1 + contrast * sin(2*MPI*theta));
		   makrstim(start,    ept->node1a,ept->node1b,ept->node1c,
					ept->node1d,sinten,wavel,"del");
		   makrstim(start+dur,ept->node1a,ept->node1b,ept->node1c,
					ept->node1d,-sinten,wavel,"del");
	         } /* if (inrect()) */
		 else {		/* set mean intens if outside */
                     sinten = inten;	
		     makrstim(start,    ept->node1a,ept->node1b,ept->node1c,
					ept->node1d,sinten,wavel,"del");
		     makrstim(start+dur,ept->node1a,ept->node1b,ept->node1c,
					ept->node1d,-sinten,wavel,"del");
		 }
               }  /* if (ept->ctype==ROD... */
	     }   /* for (ept= ; ; ) */
          } /* else */
	  break;

	case GABOR:			/* stim for possibly all receptors */
					/* xloc is xmin; yloc is xmax. */
          if (yloc==0 & xloc==0) {		/* xmax, xmin */
             yloc = 1000.0;
             xloc =  -1000.0;
          }
	  if (xloc > yloc) {
	            double tx;
	          tx = xloc;		/* swap xmin and xmax if backwards */
		  xloc = yloc;
		  yloc = tx; 
	  	}
          if (ymins==ymaxs) {
             ymins = -1000.0;
             ymaxs =  1000.0;
          }
          orientrad = MPI / 180.0 * orient;
          sphasenorm = sphase / 360;
          coso = cos(orientrad);
          sino = sin(orientrad);
          xcent = xloc;		            /* yloc is xmax; xloc is xmin */
          ycent = yloc; 
	  xedgemax = xcent + xenv * 5;
	  xedgemin = xcent - xenv * 5;
	  yedgemax = ycent + yenv * 5;
	  yedgemin = ycent - yenv * 5;
          if (speriod <= 0.0) speriod = 1;   /* spatial period from user */
          if (drift) {			/* drifting grating */

                 double timres,tperiod,tincr,stime;

	    if (tfreq==0.0) tfreq = 1.0;
	    timres = .001;
            tperiod = 1.0 / tfreq;	/* temporal period */
            tincr = timres * tperiod;	/* time incr = time res * t period */
	    tincr = abs(tincr);		/* allow negative temporal frequency */
	    if (tincr < RTIMINC) tincr = RTIMINC; /* time incr = recep timinc */
            for (stime=start; stime<(start+dur-tincr); stime += tincr) {
	      for (ept=(photrec*)elempnt; ept; ept=(photrec*)ept->next) 
	        if (ept->ctype == ROD || ept->ctype == CONE ||
		      ept->ctype == TRANSDUCER) {
		   xpos = ept->xpos;
		   ypos = ept->ypos;

				/* if the photorec is under grating */
	           if (inrect(xpos,ypos,
			xedgemax,xedgemin,yedgemax,yedgemin,orient)) {
		     xr = (xpos-xcent)*coso - (ypos-ycent)*sino + xcent; 
		     yr = (xpos-xcent)*sino + (ypos-ycent)*coso + ycent; 
		     theta = (xr-xloc)/speriod +
				-(stime-start)*drift/tperiod + sphasenorm;
                     sinten = inten * (1 + contrast * sin(2*MPI*theta) *
			gaussf1(xr,xcent,xenv) * gaussf1(yr,ycent,yenv));
		     makrstim(stime,      ept->node1a,ept->node1b,ept->node1c,
					ept->node1d,sinten,wavel,"del");
		     makrstim(stime+tincr,ept->node1a,ept->node1b,ept->node1c,
					ept->node1d,-sinten,wavel,"del");
		   } /* if (inrect()) */
		   else {		/* set mean intens if outside */
                     sinten = inten;	
		     makrstim(stime,      ept->node1a,ept->node1b,ept->node1c,
					ept->node1d,sinten,wavel,"del");
		     makrstim(stime+tincr,ept->node1a,ept->node1b,ept->node1c,
					ept->node1d,-sinten,wavel,"del");
		   }
                } /* if (ept->ctype==ROD... */
            }   /* for (stime= ; ; ) */
          }   /* if (drift) */
          else if (tfreq) {		/* stationary counterphasing grating */

                 double timres,tperiod,tincr,stime;

	    if (tfreq==0.0) tfreq = 1.0;
	    timres = .001;
            tperiod = 1.0 / tfreq;	/* temporal period */
            tincr = timres * tperiod;	/* time incr = time res * t period */
	    tincr = abs(tincr);		/* allow negative temporal frequency */
	    if (tincr < RTIMINC) tincr = RTIMINC; /* time incr = recep timinc */
            for (stime=start; stime<(start+dur-tincr); stime += tincr) {
	      for (ept=(photrec*)elempnt; ept; ept=(photrec*)ept->next) 
	        if (ept->ctype == ROD || ept->ctype == CONE ||
		      ept->ctype == TRANSDUCER) {
		    static double stheta, ttheta;

		   xpos = ept->xpos;
		   ypos = ept->ypos;

	           if (inrect(xpos,ypos,
			xedgemax,xedgemin,yedgemax,yedgemin,orient)) {
		     xr = (xpos-xcent)*coso - (ypos-ycent)*sino + xcent; 
		     yr = (xpos-xcent)*sino + (ypos-ycent)*coso + ycent; 
		     stheta = (xr-xloc)/speriod + sphasenorm;
		     ttheta = -(stime-start)/tperiod;
                     sinten = inten * (1 + contrast * sin(2*MPI*stheta) *
						      sin(2*MPI*ttheta) *
						gaussf1(xr,xcent,xenv) *
						gaussf1(yr,ycent,yenv));
		     makrstim(stime,      ept->node1a,ept->node1b,ept->node1c,
					ept->node1d,sinten,wavel,"del");
		     makrstim(stime+tincr,ept->node1a,ept->node1b,ept->node1c,
					ept->node1d,-sinten,wavel,"del");
		   } /* if (inrect()) */
		   else {		/* set mean intens if outside */
                     sinten = inten;	
		     makrstim(stime,      ept->node1a,ept->node1b,ept->node1c,
					ept->node1d,sinten,wavel,"del");
		     makrstim(stime+tincr,ept->node1a,ept->node1b,ept->node1c,
					ept->node1d,-sinten,wavel,"del");
		   }
                } /* if (ept->ctype==ROD... */
            }   /* for (stime= ; ; ) */
          }   /* if (tfreq) */
          else {		/* static grating */
	    for (ept=(photrec*)elempnt; ept; ept=(photrec*)ept->next) {
	      if (ept->ctype == ROD || ept->ctype == CONE ||
		      ept->ctype == TRANSDUCER) {
		 xpos = ept->xpos;
		 ypos = ept->ypos;

	 /*        fprintf  (stderr,"%g %g %g %g %g %g %g\n",
				xpos,ypos,yloc,xloc,ymaxs,ymins,orient); /* */

	         if (inrect(xpos,ypos,xedgemax,xedgemin,
					yedgemax,yedgemin,orient)) {
		   xr = (xpos-xcent)*coso - (ypos-ycent)*sino + xcent; 
		   yr = (xpos-xcent)*sino + (ypos-ycent)*coso + ycent; 
	           theta = (xr-xloc) / speriod + sphasenorm;
                   sinten = inten * (1 + contrast * sin(2*MPI*theta) *
						gaussf1(xr,xcent,xenv) *
						gaussf1(yr,ycent,yenv));
		   makrstim(start,    ept->node1a,ept->node1b,ept->node1c,
					ept->node1d,sinten,wavel,"del");
		   makrstim(start+dur,ept->node1a,ept->node1b,ept->node1c,
					ept->node1d,-sinten,wavel,"del");
	         } /* if (inrect()) */
		 else {		/* set mean intens if outside */
                     sinten = inten;	
		     makrstim(start,    ept->node1a,ept->node1b,ept->node1c,
					ept->node1d,sinten,wavel,"del");
		     makrstim(start+dur,ept->node1a,ept->node1b,ept->node1c,
					ept->node1d,-sinten,wavel,"del");
		 }
               }  /* if (ept->ctype==ROD... */
	     }   /* for (ept= ; ; ) */
          } /* else */
	  break;

	case RECT:			/* stim for possibly all receptors */
	  halfx = dia / 2;
	  halfx = ydist / 2;
          if (runyet) {
	  for (rpnt=(recep*)recpnt; rpnt; rpnt=(recep*)rpnt->next) {
	    if (rpnt->ctype == ROD || rpnt->ctype == CONE || 
		      rpnt->ctype == TRANSDUCER)
	    if (inrect(rpnt->xloc,rpnt->yloc, xloc+halfx, xloc-halfx,
			yloc+halfy, yloc-halfy, orient)) {
		makrstim(start,    rpnt->recnm1,rpnt->recnm2,rpnt->recnm3,
					rpnt->recnm4,inten,wavel,"del");
		makrstim(start+dur,rpnt->recnm1,rpnt->recnm2,rpnt->recnm3,
					rpnt->recnm4,-inten,wavel,"del");
	    }
	   }
          }
	  else {
	   for (ept=(photrec*)elempnt; ept; ept=(photrec*)ept->next) {
	    if (ept->ctype == ROD || ept->ctype == CONE || 
		      ept->ctype == TRANSDUCER)
	    if (inrect(ept->xpos,ept->ypos, xloc+halfx, xloc-halfx,
			yloc+halfy, yloc-halfy, orient)) {
		makrstim(start,    ept->node1a,ept->node1b,ept->node1c,
					ept->node1d,inten,wavel,"del");
		makrstim(start+dur,ept->node1a,ept->node1b,ept->node1c,
					ept->node1d,-inten,wavel,"del");
	    }
	  }
	 }
	  break;

	case SPOT: 
	  radius = dia / 2;
	  if (runyet) {			/* look for existing photoreceptors */
	   for (rpnt=(recep*)recpnt; rpnt; rpnt=(recep*)rpnt->next) {
	    if (rpnt->ctype == ROD || rpnt->ctype == CONE ||
		      rpnt->ctype == TRANSDUCER) {
	    if (incirc(rpnt->xloc,rpnt->yloc, xloc, yloc,radius)) {
		makrstim(start,    rpnt->recnm1,rpnt->recnm2,rpnt->recnm3,
					rpnt->recnm4,inten,wavel,"del");
		makrstim(start+dur,rpnt->recnm1,rpnt->recnm2,rpnt->recnm3,
					rpnt->recnm4,-inten,wavel,"del");
	     }
	    }
	   }
	  }
	  else {
	   for (ept=(photrec*)elempnt; ept; ept=(photrec*)ept->next) {
	    if (ept->ctype == ROD || ept->ctype == CONE ||
		      ept->ctype == TRANSDUCER) {
	    if (incirc(ept->xpos,ept->ypos, xloc, yloc,radius)) {
		makrstim(start,    ept->node1a,ept->node1b,ept->node1c,
					ept->node1d,inten,wavel,"del");
		makrstim(start+dur,ept->node1a,ept->node1b,ept->node1c,
					ept->node1d,-inten,wavel,"del");
	    }
	   }
 	  }
	 }
	  break;
	 } 	/* switch (stype) */
	 start = 0.0;
	}	/* else  (!filuse) */

	stype = 0;		/* reset stype for next stim statement */

	break;	/* case STIM */


  }	/* switch  (param->type) */
}

#endif

/*----------------------------------------------------*/

void sortstimfile(void)

/* Sort the stimulus file after all stimulus events 
   have been put into it.
*/

{
#ifdef XSTIM
   static char ftemplate[] = "/usr/tmp/stimXXXXXX";
   static char stemp[30],tbuf[120];
   
  strcpy (stemp, ftemplate);
  mktemp (stemp);
  sprintf (tbuf,"sort +0 -1 +0n -1 +1n -2 +2n -3 +3n -4 +4n -5 +7 -8 %s > %s && mv %s %s",
					stimfile,stemp,stemp,stimfile);
  system (tbuf);
#endif
}

/*----------------------------------------------------*/

void incrpl(void)

/* Increment number of plots.
   Used between V[] expressions in plot statement,
    and also at end of plot statement.
*/

{
#ifndef XSTIMM
 if (++numplots < PLOTNODSIZ)
     plotnod[numplots].cnod1 = 0; 	/* zero at end */ 
#endif
}

/*----------------------------------------------------*/

void vplot(void)

/* read in data from "plot V[]"  with default max min
*/

{
  datum d1,d2,d3,d4,d5;
  int narg,type,pmod,pval,cagparm;
  Symbol *param;
  

 param  = (Symbol *)*pc++;
 narg = (long int)*pc++;

 if (narg > 3) d4 = popm();
 else d4.val = NULLVAL;
 if (narg > 2) d3 = popm();
 else d3.val = NULLVAL;
 if (narg > 1) d2 = popm();
 else d2.val = NULLVAL;
 if (narg > 0) checknum(d1 = popm());		/* can't use getnod() here */
 else d1.val = NULLVAL;				/* because of S: below */
  
#ifndef XSTIMM
 if (narg>0) {
   if ((((long int)param)<2000) ||
      ((((long int)param)>2000) && (param->type!=S))) {
       plotnod[numplots].cnod1 = (int)d1.val;
       plotnod[numplots].cnod2 = (int)d2.val;
       plotnod[numplots].cnod3 = (int)d3.val;
       plotnod[numplots].cnod4 = (int)d4.val;
     }
 }
 pval = pmod = 0;

 if ((type=(long int)param) < 2000) {

   cagparm = (long int)*pc++;
   switch (type) {
   case G:
	switch(cagparm) {

	  case 0:  pmod = NRECG0; break;	/* record conductance */
	  case 1: d5 = popm();
		   pval = int(d5.val);
		   pmod = NRECG;		/* record cai from Ca shell */
		   break;
	  case VREV:pmod = NRECGV; break;
	  case I:   pmod = NRECGI; break;
	  case M:   pmod = NRECGM; break;
	  case H:   pmod = NRECGH; break;
	  case CA:  pmod = NRECGC; break;
	}
	break;

   case CA:
	switch(cagparm) {

	  case 0:  pmod = CREC_CACONC; 
		   pval = 1; break;	/* record cai ([Ca] inside) */
	  case 1:  d5 = popm();
		   pval = int(d5.val);
		   pmod = CREC_CACONC;		/* record cai from Ca shell */
		   break;
	  case VREV:pmod = CREC_CAV; break;
	  case I:   pmod = CREC_CAIT; break;
	  case IP:  pmod = CREC_CAIP; break;
	  case IE:  pmod = CREC_CAIE; break;
	  case IPE: pmod = CREC_CAIPE; break;
	}
	break;
   case CABUF:
	switch(cagparm) {

	  case 1:  d5 = popm();
		   pval = int(d5.val);
		   pmod = CREC_CABUF;		/* record cai from Ca shell */
		   break;
	default:  break;
	}
	break;
   default: break;
   }
 }
 else 
  switch (param->type) {
   case V:
	pmod  = VREC;
	break;
   case I:
	pmod  = IREC;
	break;
   case L:
	pmod  = LREC;
	break;
   case FA0: case FA1: case FA2: case FA3: case FA4: case FA9:
   case FH0: case FH1: case FH2: case FH3: case FH4:
   case FB0: case FB1: case FB2: case FB3: case FB4:
   case FC0: case FC1: case FC2: case FC3: case FC4:
	pmod  = NRECA0 + (param->type - FA0);
	break;
   case G0: case G1: case G2: case G3:
	pmod  = NRECG0 + (param->type - G0); 
	break;
   case S:
 	plotnod[numplots].spnt  = (char *)d1.sym;
	pmod  = SREC;
	break;
   case CABLE:
	pmod  = CABLE;
 	plotnod[numplots].cnod2 = (int)(d2.val * CABLFRAC);
	break;
   case GLU:
   case AMPA:
   case NMDA:
   case CNQX:
   case GABA:
   case PTX:
   case BIC:
   case GLY:
   case STRY:
   case CGMP:
   case CAMP: pmod  = param->type - GLU + CREC_GLU;

 }
 plotnod[numplots].pval  = pval;
 plotnod[numplots].pmod  = pmod;

 if (strcmp(gfrname(),rootframe))		 /* if diff than root frame */
   strcpy(plotnod[numplots].plframe,gfrname());  /* copy current frame name */
#endif
}

/*------------------------------------------------------*/

void vplotm(void)

/* read in max, min data from "plot V() max min" statement
*/

{
  datum d1,d2;

 checknum(d2 = popm());
 checknum(d1 = popm());
 
#ifndef XSTIMM
 if (d1.val < d2.val) { double t; t = d1.val; d1.val=d2.val; d2.val=t; }
 plotnod[numplots].pymax  = d1.val;
 plotnod[numplots].pymin  = d2.val;
#endif
}

/*------------------------------------------------------*/

void xrecord(void)

/* allow user to record from nodes directly */

{
    datum d1,d2,d3,d4,d5;
    Symbol *param;
    int nod1, nod2, nod3, nod4;
    int narg, caparm, pmod, pval, type;
    double record(int cnod1, int cnod2, int cnod3, int cnod4, int pmod,int pval);
#ifdef XMOD
    double reccable(int elemnum, double fracdist);
#endif

 param  = (Symbol *)*pc++;
 narg = (long int) *pc++;

 if (narg > 3) d4 = popm();			/* can't use getnod() */
 else d4.val = NULLVAL;			/* because of d2.val below */
 if (narg > 2) d3 = popm();
 else d3.val = NULLVAL;
 if (narg > 1) d2 = popm();
 else d2.val = NULLVAL;
 checknum(d1 = popm());

 pmod = pval = 0;
 if ((type=(long int)param) < 2000) {

   caparm = (long int)*pc++;
   switch (type) {
   case G:
	switch(caparm) {

	  case 0:  pmod = NRECG; break;		/* record conductance */
	  case 1: d5 = popm();
		   pval = int(d5.val);
		   pmod = NRECG;		/* record cai from Ca shell */
		   break;
	  case VREV:pmod = NRECGV; break;
	  case I:   pmod = NRECGI; break;
	  case M:   pmod = NRECGM; break;
	  case H:   pmod = NRECGH; break;
	  case CA:  pmod = NRECGC; break;
	}
	break;

   case CA:
	switch(caparm) {

	  case 0:  pmod = CREC_CACONC; break;	/* record cao ([Ca] outside) */
	  case 1: d5 = popm();
                   pval = int(d5.val);
		   pmod = CREC_CACONC;		/* record cai from Ca shell */
		   break;
	  case VREV:pmod = CREC_CAV; break;
	  case I:   pmod = CREC_CAIT; break;
	  case IP:  pmod = CREC_CAIP; break;
	  case IE:  pmod = CREC_CAIE; break;
	  case IPE: pmod = CREC_CAIPE; break;
	}
	break;
   case CABUF:
	switch(caparm) {
	  case 1: d5 = popm();
                   pval = int(d5.val);
		   pmod = CREC_CABUF;		/* record cai from Ca shell */
		   break;
	  default: break;	
	}
	break;
   default: break;
   }
 }
 else 
 switch (param->type) {
   case V:
	pmod = VREC;
	break;
   case I:
	pmod = IREC;
	break;
   case L:
	pmod = LREC;
	break;
   case FA0: case FA1: case FA2: case FA3: case FA4: case FA9:
   case FH0: case FH1: case FH2: case FH3: case FH4:
   case FB0: case FB1: case FB2: case FB3: case FB4:
   case FC0: case FC1: case FC2: case FC3: case FC4:
   case G0: case G1: case G2: 
	pmod = NRECA0 + (param->type - FA0);
	break;
   case CABLE:
	pmod = CABLE;
	break;

   case GLU:
   case AMPA:
   case NMDA:
   case CNQX:
   case GABA:
   case PTX:
   case BIC:
   case GLY:
   case STRY:
   case CGMP:
   case CAMP:
	pmod = param->type - GLU + CREC_GLU;
	break;
  }
  if (pmod==CABLE) {
#ifdef XMOD
	double fracdist;
	int elemnum;

     fracdist = d2.val;
     elemnum = (int)d1.val;
     d1.val = reccable(elemnum,fracdist);
#else
     d1.val = 0.0;
#endif
   }
   else {
#ifndef XSTIMM
     nod1 = (int)d1.val;
     nod2 = (int)d2.val;
     nod3 = (int)d3.val;
     nod4 = (int)d4.val;
 #ifndef XSTIM
     d1.val = record (nod1,nod2,nod3,nod4,pmod,pval);
 #else
     d1.val = 0.0;
 #endif
#else
     d1.val = 0.0;
#endif
   }
   d1.vtype = NUMBER;
   d1.type = VAR;
   pushm(d1);
}

/*------------------------------------------------------*/

void dispnod(void)
{
  datum d1,d2,d3,d4,d[MAXNODIM];
  static int narg1,narg2;
  static int n1a,n1b,n1c,n1d,n2a,n2b,n2c,n2d;
  static int na,nb,nc,nd;
  Symbol *param,nulparm;
  static int disptype,dcolor = NULLVAL, excl=0, hide=0;
  static double cline=0,dscale=1.0,stime=0;
  static int elemtype=0;
  static int exceptype=0,except=0;
  static double xcalib=.95,ycalib=.05;

  param = (Symbol *)*pc++;
  if (param==0) {
  	nulparm.type = CONNECT;
	param = &nulparm;
  }
#ifdef XMOD
  varcopy();
  if (setdebug) debug = setdebug;
  if (setdebgz) debugz = setdebgz;
  if (!runyet || elempnt) {
	findconnect(1);
  }
#endif
  switch (param->type) {

  case ONLY:  excl = 1;
	      break;

  case MATCHING:
  case ELEMENT:
  case RANGE:
  case CONNECT:
 
  disptype = param->type; 
  narg2 = (long int)*pc++;
  if (narg2) {
    pc--;
    getnod(d);				/* args for node2 are first */
    					/*  because they're on top of stack */
    n2a = (int)d[0].val;
    n2b = (int)d[1].val;
    n2c = (int)d[2].val;
    n2d = (int)d[3].val;
    if (n2a<0) n2a = NULLVAL;
    if (n2b<0) n2b = NULLVAL;
    if (n2c<0) n2c = NULLVAL;
    if (n2d<0) n2d = NULLVAL;
  }
  else n2a=n2b=n2c=n2d=NULLVAL;

  narg1 = (long int)*pc++;
  if (narg1) {
    pc--;
    getnod(d);
    if (except) {
       na = (int)d[0].val;
       nb = (int)d[1].val;
       nc = (int)d[2].val;
       nd = (int)d[3].val;
       if (na<0) na = NULLVAL;
       if (nb<0) nb = NULLVAL;
       if (nc<0) nc = NULLVAL;
       if (nd<0) nd = NULLVAL;
    }
    else {
       n1a = (int)d[0].val;
       n1b = (int)d[1].val;
       n1c = (int)d[2].val;
       n1d = (int)d[3].val;
       if (n1a<0) n1a = NULLVAL;
       if (n1b<0) n1b = NULLVAL;
       if (n1c<0) n1c = NULLVAL;
       if (n1d<0) n1d = NULLVAL;
       na= nb= nc=nd= NULLVAL;		/*  clear except node */
    }
  }
  else {				/* if node not specified: */
    if (except) na= nb= nc= nd= NULLVAL; /*  clear except node */
    else n1a=n1b=n1c=n1d=NULLVAL;   	/*  clear n1 when not except */
  }

  break;

  case COMPS:
  case CABLE:
  case SPHERE:
  case SYNAPSE:
  case CHAN:
  case ROD:
  case CONE:
  case TRANSDUCER:
  case GJ:
  case LOAD:
  case RESISTOR:
  case CAP:
  case GNDCAP:
  case BATT:
  case GNDBATT:
  case BUF:
  case NODE:
    if (except) exceptype = param->type;
    else         elemtype = param->type;
    break;

  case EXCEPT:  except = 1;
	      break;

  case XROT:				/* get rotation for display */
    checknum(d1 = popm());
    xrot = -d1.val;
    break;

  case YROT:
    checknum(d1 = popm());
    yrot = d1.val;
    break;

  case ZROT:
    checknum(d1 = popm());
    zrot = d1.val;
    break;

  case SIZE:				/* get size of display window */
    checknum(d1 = popm());
    dsize = d1.val;
    break;

  case DSCALE:				/* get display scale */
    checknum(d1 = popm());
    dscale = d1.val;
    break;

  case COLOR:				/* get color */
    checknum(d1 = popm());
    dcolor = (int)d1.val;
    break;

  case CALIBLIN:			/* get calib line */
    narg1 = (long int)*pc++;
    if (narg1) {
      if (narg1 > 2) d3 = popm();		/* get possible "loc" */
      if (narg1 > 1) d2 = popm();
      checknum(d1 = popm());
      xcalib = d1.val; 
      ycalib = d2.val; 
    }
    checknum(d1 = popm());
    cline = d1.val;
    break;

  case STIM:				/* get stimulus time  */
    checknum(d1 = popm());
    stime = d1.val;
    disptype = param->type;
    break;

  case HIDE:				/* make picture with hidden lines */
    hide = 1;
    break;

  case CENTER:
    narg1 = (long int)*pc++;
    if (narg1) {
      if (narg1 > 2) d3 = popm();
      else d3.val = 0;
      if (narg1 > 1) d2 = popm();
      else d2.val = 0;
      checknum(d1 = popm());
    }
    dxcent = -d1.val;
    dycent = -d2.val;
    dzcent = -d3.val;
    break;

  case RMOVE:
    narg1 = (long int)*pc++;
    if (narg1) {
      if (narg1 > 2) d3 = popm();
      else d3.val = 0;
      if (narg1 > 1) d2 = popm();
      else d2.val = 0;
      checknum(d1 = popm());
    }
    rxcent = d1.val;
    rycent = d2.val;
    rzcent = d3.val;
    break;

  case WINDOW:
      checknum(d4 = popm());
      checknum(d3 = popm());
      checknum(d2 = popm());
      checknum(d1 = popm());
    break;
 
  case DISPLAY:
	if (!disptype) return;

#ifdef XMOD

   setrot(xrot,yrot,zrot,dxcent,dycent,dzcent,rxcent,rycent,rzcent,dsize); 

   if (disp_ray) {		/* move camera, not objects */
     initray(xrot,yrot,zrot,dxcent,dycent,dzcent,
			rxcent,rycent,rzcent,dsize); /* initialize ray-tracer */
   }

   set_icons();				/* set user-defined neural elem icons */

   if (disp & DSTIM) 
    if (disptype == STIM)
     if (stime){			/* display the stimulus */
	dispstim(stime,dscale);
        stime=0;
      }

  if (elemtype==COMPS && (disp & (DCOMP|DCONN))) {
      if (hide) hidstart();
      initcomp(1);         /* make compartments, don't erase them */
      if (debug & 256) nocond = 1;
      if (!nocond) condense();
      switch(disptype) {
       case MATCHING:
       case CONNECT: dcomp (n1a,n1b,n1c,n1d,elemtype,exceptype,
					na,nb,nc,nd,dcolor,dscale,hide,excl);

/*	 else  dcomp2 (n1a,n1b,n1c,n1d,n2a,n2b,n2c,n2d,elemtype,exceptype,
					na,nb,nc,nd,dcolor,dscale,hide); */
		 break;

      }  /* switch */
    }  /* elemtype==COMPS) */

   if ((elemtype==NODE) && (disp & DNODE)) {
        ncdrnod (n1a,n1b,n1c,n1d,elemtype,exceptype,na,nb,nc,nd,dcolor,dscale);
   }

   if (disp & DISP) {		/* if "disp" variable or "-d 1" has been set */
				/* first, translate, rotate, and scale: */
    if (hide) hidstart();

    if (elemtype!=NODE)		/* elemtype is CABLE, SPHERE, SYNAPSE, etc. */
    switch (disptype) {

     case MATCHING:
     case CONNECT: if (narg2==0) ncdisp (n1a,n1b,n1c,n1d,elemtype,exceptype,
					na,nb,nc,nd,dcolor,dscale,hide,excl);
		 else ncdispc (n1a,n1b,n1c,n1d,n2a,n2b,n2c,n2d,elemtype,
				exceptype,na,nb,nc,nd,dcolor,dscale,hide);
		 break;
     case RANGE:    ncdispn (n1a,n1b,n1c,n1d,n2a,n2b,n2c,n2d,elemtype,
				exceptype,na,nb,nc,nd,dcolor,dscale,hide,excl);
		 break;
     case ELEMENT: ncdispe (n1a,dcolor,dscale,hide);
		 break; 
    }
    if (disp) { 
      if (cline) drcalib (xcalib,ycalib,cline,dsize,dcolor);
      cline=0;
    }
   }		/* if (disp & DISP) */

    disptype = 0;
    elemtype = 0;
    except = exceptype = 0;
    rxcent=rycent=rzcent=0;
    na=nb=nc=nd=NULLVAL;
    dscale = 1.0;
    dcolor = NULLVAL;
    excl = 0;
    if (hide) hidstop();

#endif		/* XMOD */

  break;     /* case DISPLAY */
 } 	   /* switch (param) */
}


/*------------------------------------------------------*/

#define NUMDIM MAXNODIM			/* number of node dimensions */

        Symbol *eparam;

void foreacode(void)			/* foreach statement */

/* execute a set of statements for one of three conditions (options):

    1) For every element, or each element of a certain type (e.g. "cable")

    2) For every element, or each element of a certain type (e.g. "cable")
         that matches a node specification.  

    3) for each node that matches a node specification.
*/
         
{
	datum d;
	Inst *savepc = pc;
	int i,narg,match,efilt;
	int arg[NUMDIM];
	int val[NUMDIM];
        double *varp[NUMDIM];
	double *elemvar;
	short *typ,*vtyp;
        node *npnt,*tnpnt;
	elem *epnt,*tepnt;

    eparam = (Symbol *)*pc++;		/* get element type */ 
    efilt = 0;
/* if (eparam) fprintf (stderr,"foreacode efilt name '%s'\n",eparam->name);
*/
    if (eparam) switch (eparam->type) {

     case ELEMENT:
			efilt = 1;
			break;
     case CABLE:
     case SPHERE:
     case SYNAPSE:
     case CHAN:
     case ROD:
     case CONE:
     case TRANSDUCER:
     case GJ:
     case LOAD:
     case RESISTOR:
     case CAP:
     case GNDCAP:
     case BATT:
     case GNDBATT:
     case BUF:
			efilt = 2;
			break;	

     default:		efilt = 0;
			break;	

    }

    if (efilt) {
       getvar(&elemvar,&typ,&vtyp);  /* var pointer for element */
       if (*typ==UNDEF) *typ = VAR;
       *vtyp = NUMBER;
    }
    
    narg = (long int)*pc++;		/* number of snode args to expect */

   if (narg) {
    if (narg>NUMDIM) narg = NUMDIM;
    for (i=0; i<NUMDIM; i++) {
       varp[i] = (double *)NULL;
       val[i]  = NULLVAL;
       arg[i]  = (long int)*pc++;
    }
#ifndef XSTIMM
    execute (savepc+8);			/* evaluate node descriptors */
#endif

   for (i=narg-1; i>=0; i--) {
         if (arg[i]) {
	    getvar(&varp[i],&typ,&vtyp);/*var pointer for node dim */
            if (*typ==UNDEF) *typ = VAR;
            *vtyp = NUMBER;
	 }
	 else {
           checknum(d = popm()); 	/*else get value of node dim */
	   val[i] = (int)d.val;
         }
   }
  }   /* if (narg) */


#ifndef XSTIMM
 if (efilt) {
   for (epnt=elempnt; epnt; epnt=tepnt) {	/* search all elems */
     tepnt = epnt->next;		/* pre-increment in case deleted */
     if (efilt==2) if (epnt->ctype!=eparam->type) continue;

     npnt = (node *)NULL;
     if (narg) {
       npnt = epnt->nodp1;		/* test first node */
       if (npnt) 
        for (match=1,i=0; i<narg; i++) {
 	 if (!arg[i]) 			/* if dimension was given a value */
	  switch (i) {
	   case 0: if ((val[i]>=0) &&(val[i]!=npnt->nodenm1)) match = 0; break; 
	   case 1: if ((val[i]>=0) &&(val[i]!=npnt->nodenm2)) match = 0; break; 
	   case 2: if ((val[i]>=0) &&(val[i]!=npnt->nodenm3)) match = 0; break; 
	   case 3: if ((val[i]>=0) &&(val[i]!=npnt->nodenm4)) match = 0; break; 
	  }	
        }
      if (!match) {
        npnt = epnt->nodp2;		/* test second node */
        if (npnt) 
        for (match=1,i=0; i<narg; i++) {
	 if (!arg[i]) 			/* if dimension was given a value */
	   switch (i) {
	    case 0: if ((val[i]>=0) &&(val[i]!=npnt->nodenm1)) match = 0; break; 
	    case 1: if ((val[i]>=0) &&(val[i]!=npnt->nodenm2)) match = 0; break; 
	    case 2: if ((val[i]>=0) &&(val[i]!=npnt->nodenm3)) match = 0; break; 
	    case 3: if ((val[i]>=0) &&(val[i]!=npnt->nodenm4)) match = 0; break; 
	   }	
        }
        if (!match) continue;
       }  /* if (!match) */

       if (match) {
       *elemvar = epnt->elnum;		/* set the element number */
       if (npnt)
        for (i=0; i<narg; i++) {
	 if (arg[i]) {			/* If dimension was given blank var, */
	   switch (i) {			/*   set variable for this iteration */
	   case 0: *varp[i] = npnt->nodenm1; break; 
	   case 1: *varp[i] = npnt->nodenm2; break; 
	   case 2: *varp[i] = npnt->nodenm3; break; 
	   case 3: *varp[i] = npnt->nodenm4; break; 
	   }	
         }
       }		/* run loop code if elem and node match requested */

       if (!forexec(savepc+(int)*(savepc+6))) break;	

      }    /* if (match) */
     }    /* if (narg) */

     else {		/* run loop code if only elem match requested */

        *elemvar = epnt->elnum;		/* set the element number */
        if (!forexec(savepc+(int)*(savepc+6))) break;	
     }

   }  /* for (epnt;;) */
 }  /* if (efilt) */



 else 		/* no element, just node to match */
   for (npnt=nodepnt; npnt; npnt=tnpnt) {	/* search all nodes */
      tnpnt = npnt->next;
      for (match=1,i=0; i<narg; i++) {
	if (!arg[i]) 			/* if dimension was given a value */
	  switch (i) {
	   case 0: if ((val[i]>=0) &&(val[i]!=npnt->nodenm1)) match = 0; break; 
	   case 1: if ((val[i]>=0) &&(val[i]!=npnt->nodenm2)) match = 0; break; 
	   case 2: if ((val[i]>=0) &&(val[i]!=npnt->nodenm3)) match = 0; break; 
	   case 3: if ((val[i]>=0) &&(val[i]!=npnt->nodenm4)) match = 0; break; 
	  }	
     }
     if (!match) continue;

     for (i=0; i<narg; i++) {
	if (arg[i])			/* If dimension was given blank var, */
	  switch (i) {			/*   set variable for this iteration */
	  case 0: *varp[i] = npnt->nodenm1; break; 
	  case 1: *varp[i] = npnt->nodenm2; break; 
	  case 2: *varp[i] = npnt->nodenm3; break; 
	  case 3: *varp[i] = npnt->nodenm4; break; 
	  }	
     }

    if (!forexec(savepc+(int)*(savepc+6))) break;	

    }  /* for (npnt;;) */

#endif
  if (!returning) {
	pc = savepc+(int)*(savepc+7);	/* next stmt */
  }
}

/*------------------------------------------------------*/

int forexec(Inst *body)
{
  execute(body);		/* body of element, node loop */
  if (stopping) {
	if (returning)
		return(0);
	stopping = 0;
	if (breaking) {
		breaking = 0;
		return (0);
	}
		continuing = 0;
   }
   return (1);
}

/*------------------------------------------------------*/

static double elim_xmin = -LARGENODE;	/* Window limit for elements */
static double elim_xmax =  LARGENODE;	/*  set by elimit() */
static double elim_ymin = -LARGENODE;
static double elim_ymax =  LARGENODE;
static double elim_zmin = -LARGENODE;
static double elim_zmax =  LARGENODE;

int eclip (node *npt1, node *npt2)

/* Establish a window beyond which elements are clipped.
   If both nodes (one in the case of single-node elements)
   are outside the window, return 1, otherwise return 0.
*/

{
   int c1,c2;

   if (!npt1 && !npt2) return 1;  		/* pointers not found */
   c1 = 1;
   if (npt1) {
      c1 = 0;
      if      (npt1->xloc < elim_xmin) { c1 = 1; }
      else if (npt1->xloc > elim_xmax) { c1 = 1; }
      if      (npt1->yloc < elim_ymin) { c1 = 1; }
      else if (npt1->yloc > elim_ymax) { c1 = 1; }
      if      (npt1->zloc < elim_zmin) { c1 = 1; }
      else if (npt1->zloc > elim_zmax) { c1 = 1; }
   }
   c2 = 1;
   if (npt2) {
      c2 = 0;
      if      (npt2->xloc < elim_xmin) { c2 = 1; }
      else if (npt2->xloc > elim_xmax) { c2 = 1; }
      if      (npt2->yloc < elim_ymin) { c2 = 1; }
      else if (npt2->yloc > elim_ymax) { c2 = 1; }
      if      (npt2->zloc < elim_zmin) { c2 = 1; }
      else if (npt2->zloc > elim_zmax) { c2 = 1; }
   }
  return (c1&&c2);
}

/*------------------------------------------------------*/

void elimit (void)

/* Set "soft" window limits on element location.
   An element entirely outside window is deleted. 
*/

{
	datum d1,d2;
  	elem *epnt;
  	int elnum;
	Symbol *param;

  param = (Symbol *)*pc++;		/* get element type */ 
  switch (param->type) {
  	case X: 
  	case Y:
  	case Z:
		checknum(d2 = popm());
		checknum(d1 = popm());
		break;
  	case ELEMENT: 
		checknum(d1 = popm());
		break;
  } 
  switch (param->type) {
 	case X: 
	  	if (d1.val < d2.val) 
			{ double t; t = d1.val; d1.val=d2.val; d2.val=t; }
		elim_xmax = d1.val;
		elim_xmin = d2.val;
		break;
  	case Y:
	  	if (d1.val < d2.val) 
			{ double t; t = d1.val; d1.val=d2.val; d2.val=t; }
		elim_ymax = d1.val;
		elim_ymin = d2.val;
		break;
  	case Z:
	  	if (d1.val < d2.val) 
			{ double t; t = d1.val; d1.val=d2.val; d2.val=t; }
		elim_zmax = d1.val;
		elim_zmin = d2.val;
		break;
  	case ELEMENT:
#ifndef XSTIMM

   	elnum = int(d1.val);
   	if (!(epnt=findelem(elnum))) {
          fprintf (stderr,"elimit: can't find element %d\n",elnum);
          execerror ("Missing element: ","stopping...");
    	}
    	else {				  /* first find nodes that connect */
          findconnect(1);		  /* set pointers if not done */
	  if (eclip (epnt->nodp1,epnt->nodp2)) {

 			/* first free the nodes' pointers to this elem */
	    if (epnt->nodp1) {
		unsetnodlst(epnt->nodp1,epnt);
		if (!epnt->nodp1->elemlst) delnode(epnt->nodp1);
	    }
	    if (epnt->nodp2) {
		unsetnodlst(epnt->nodp2,epnt);
		if (!epnt->nodp2->elemlst) delnode(epnt->nodp2);
	    }
	    delelem(epnt);                     /* erase the element */
	  }
	}
#endif
        break; 	/* case ELEMENT */
  } 
}

/*------------------------------------------------------*/

void findconnect (int donefl)

/* Find and/or make all nodes that connect to all elements.
   Do only once, except do always when donefl = 0.
*/

{
   elem *epnt;
   static int done = 0;

#ifndef XSTIMM
 if (elempnt==NULL) done = 0;			/* reset at beginning */
 if (!done || !donefl)
   for (epnt=elempnt; epnt; epnt=epnt->next) { /* check nodes for element*/
     checkelemnode(epnt);
   }
 done = 1;
#endif

/*
if (debug & 1)
 for (npnt=nodepnt; npnt; npnt=npnt->next)
  fprintf (stderr,"node %d %d conn %d\n",npnt->nodenm1,npnt->nodenm2, 
		(npnt->elemlst ? npnt->elemlst->num : NULLVAL));

 for (epnt=elempnt; epnt; epnt=epnt->next)
  fprintf (stderr,"element %d node1 %d %d %d %d node2 %d %d %d\n",
		epnt->elnum,epnt->node1a,epnt->node1b,epnt->node1c,
		epnt->node1d,epnt->node2a,epnt->node2b);

	/* */
}

/*------------------------------------------------------*/

void checknod (elem *epnt)

/* Find the nodes that an element connects to;
   set up the element's node pointers.
   If node doesn't exist, create a new node.
   Don't set node's element pointer because element
   has not been fully defined yet.
*/

{
   node  *npnt,*newnod;
   int found;

#ifndef XSTIMM
    if (!epnt) return;
    found = 0;
    if (epnt->node1a != NULLVAL) {
      if ((npnt=
	findnode(epnt->node1a,epnt->node1b,epnt->node1c,epnt->node1d,
			(char *)NULL))) {
	   epnt->nodp1 = npnt;			/* set node pointer */
/*	   setnodlst(npnt,epnt); 		/* set elem pointer */
	   found = 1;
      }
      if (! found) {				/* if not, then make it */
 	    newnod=maknod(epnt->node1a,epnt->node1b,epnt->node1c,epnt->node1d);
/* 	    setnodlst(newnod=maknod(
	      epnt->node1a,epnt->node1b,epnt->node1c,epnt->node1d),epnt); /* */
	    epnt->nodp1 = newnod;		/* set node pointer */
      }
    }	/* if (epnt->node1a != NULLVAL) */

    found = 0;
    if (epnt->node2a == NULLVAL) return;	/* ignore if no second node */
      if ((npnt=
	findnode(epnt->node2a,epnt->node2b,epnt->node2c,epnt->node2d,
			(char *)NULL))) {
	   epnt->nodp2 = npnt;			/* set node pointer */
/*	   setnodlst(npnt,epnt);	 	/* set elem pointer */
	   found = 1;
      }
    if (! found) {				/* if not, then make it */
 	    newnod=maknod(epnt->node2a,epnt->node2b,epnt->node2c,epnt->node2d);
/* 	    setnodlst(newnod=maknod(
	       epnt->node2a,epnt->node2b,epnt->node2c,epnt->node2d),epnt); /* */
	    epnt->nodp2 = newnod;		/* set node pointer */
    }

/* fprintf (stderr,"checknod el %d nod1 %d nod2 %d\n",
			epnt->elnum,epnt->node1a,epnt->node2a); /* */

/* fprintf (stderr,"checknod elem %d  n1 %d n2 %d\n",
		epnt->elnum, epnt->nodp1,epnt->nodp2); /* */

#endif		/* XSTIMM */
}

/*------------------------------------------------------*/

void checkelemnode (elem *epnt)

/* 
   Find the nodes that connect to an element;
   for each node, set up the node's element pointer list,
   and set the element's 2 node pointers.
   If node doesn't exist, create a new node.
*/

{
   node  *npnt,*newnod;
   int found;

#ifndef XSTIMM
    if (!epnt) return;
    found = 0;
    if (epnt->node1a != NULLVAL) {
      if ((npnt=
	findnode(epnt->node1a,epnt->node1b,epnt->node1c,epnt->node1d,
			(char *)NULL))) {
	   epnt->nodp1 = npnt;			/* set node pointer */
	   setnodlst(npnt,epnt); 		/* set elem pointer */
	   found = 1;
      }
      if (! found) {				/* if not, then make it */
 	    setnodlst(newnod=maknod(
	     epnt->node1a,epnt->node1b,epnt->node1c,epnt->node1d),epnt);
	    epnt->nodp1 = newnod;		/* set node pointer */
      }
    }	/* if (epnt->node1a != NULLVAL) */

    found = 0;
    if (epnt->node2a == NULLVAL) return;	/* ignore if no second node */
      if ((npnt=
	findnode(epnt->node2a,epnt->node2b,epnt->node2c,epnt->node2d,
			(char*)NULL))) {
	   epnt->nodp2 = npnt;			/* set node pointer */
	   setnodlst(npnt,epnt);	 	/* set elem pointer */
	   found = 1;
      }
    if (! found) {				/* if not, then make it */
 	    setnodlst(newnod=maknod(
	      epnt->node2a,epnt->node2b,epnt->node2c,epnt->node2d),epnt);
	    epnt->nodp2 = newnod;		/* set node pointer */
    }

/* fprintf (stderr,"checknod el %d nod1 %d nod2 %d\n",
			epnt->elnum,epnt->node1a,epnt->node2a); /* */

/* fprintf (stderr,"checknod elem %d  n1 %d n2 %d\n",
		epnt->elnum, epnt->nodp1,epnt->nodp2); /* */

#endif		/* XSTIMM */
}

/*------------------------------------------------------*/

void setnodlst(node *npnt, elem *epnt)
              
/* Add an element to a node's element list.  */

{
   int found;
   conlst *lpnt;

#ifndef XSTIMM
  if (!npnt) {
	fprintf (stderr,"setnodlst: invalid node pointer\n");
        return;
  }
  found = 0;
  for (lpnt=npnt->elemlst; lpnt; lpnt=lpnt->next) {
    if (lpnt->conpnt == (conn*)epnt) found = 1;
  }

  if (! found) 
    maklst(&npnt->elemlst,(conn *)epnt);
#endif
}

/*------------------------------------------------------*/

void unsetnodlst(node *npnt, elem *epnt)
              
/* Delete an element from a node's element list */

{
  conlst *lpnt, *olpnt, *tlpnt;

#ifndef XSTIMM
  if (!npnt) {
	fprintf (stderr,"unsetnodlst: invalid node pointer\n");
        return;
  }
  olpnt = (conlst *)NULL;
  for (lpnt=npnt->elemlst; lpnt;)  {
     if (lpnt->conpnt== (conn *)epnt) {		/* delete this one */
        if (olpnt) olpnt->next = lpnt->next;
        else     npnt->elemlst = lpnt->next;
        tlpnt = lpnt;
        lpnt = lpnt->next;
        efree (tlpnt);
     }
     else {
        olpnt = lpnt;
        lpnt = lpnt->next;			/* skip to next */
     }
  }
#endif
}

/*------------------------------------------------*/
    
void modrun(void)
{
  datum d1;
  Symbol *mtyp;

if (debug & 1) fprintf (stderr,"modrun start\n");

  mtyp = (Symbol *)*pc++;
  varcopy();

  if (setdebug) debug  = setdebug;	/* "-y n" overrides "debug=n" */
  if (setdebgz) debugz = setdebgz;	/* "-z n" overrides "debugz=n" */

  switch (mtyp->type)           /* do pops */
    {
     case RUN:  break;
     case STEP:  checknum(d1=popm());
                break;
    }

#ifdef XMOD
  if (!runyet) {
        initchan();			/* set up sequential-state chans */
  }
  if (!runyet || elempnt) {
	findconnect(0);
  }
  switch (mtyp->type)
    {

     case RUN:  actrun(0.0,1);
		runyet = 0;
		break;

     case STEP: actrun(d1.val,0);
		break;
    
    }
#else
  switch (mtyp->type)           /* do pops */
    {
     case RUN:  break;
     case STEP:  
    		xtime = d1.val + xtime;  /* if XSTIM, set the time, at least */
    		timeptr->val = xtime;
                break;
    }
#endif
}

/*------------------------------------------*/

char *prnode (int n1, int n2, int n3, int n4) {

   static char nbuf[40] = {0};
   static char undefd[] = "undefined";
   int test;

   nbuf[0] = 0;
   test = 0;
   if (n1 != NULLVAL) test |= 1;
   if (n2 != NULLVAL) test |= 2;
   if (n3 != NULLVAL) test |= 4;
   if (n4 != NULLVAL) test |= 8;

   switch (test) {

     default:
     case 0: 	sprintf (nbuf,"%s",undefd);
		break;
     case 1: 	sprintf (nbuf,"[%d]",n1);
		break;
     case 3: 	sprintf (nbuf,"[%d][%d]",n1,n2);
		break;
     case 7: 	sprintf (nbuf,"[%d][%d][%d]",n1,n2,n3);
		break;
     case 15: 	sprintf (nbuf,"[%d][%d][%d][%d]",n1,n2,n3,n4);
		break;
   }

   return (nbuf);
}

/*------------------------------------------*/

