/* nc.h */

struct Symbol;
struct array;

struct datum {				/* interpreter stack type */
	short type;			/* UNDEF,CONST,VAR,ARRAY, etc. */
	short vtype;			/* NUMBER, LITCHAR, STRING, etc. */
	union {
		double val; 		/* numeric value on stack */
		int   argnum;		/* arg number index for stack var */
		array *arrp;		/* pointer to array */
		char  *str; 		/* string value on stack */ 
		Symbol *sym;
	};
};

typedef void (*Inst)();
typedef datum (*Fpa)(...);
#define STOPC	((Inst) 0) 

struct 	Symbol { 			/* symbol table entry */
	char *name;
	short type;			/* UNDEF,CONST,VAR,ARRAY, etc. */
	short vtype;			/* NUMBER, LITCHAR, STRING, etc. */
	union {
		double	val;		/* VAR */
		array  *arrp;		/* pointer to array */
		Fpa 	ptr;		/* BLTIN */
		Inst 	*defn;		/* FUNCTION, PROCEDURE */
		int	argnum;		/* ARG number for formal def */
		char	*str;		/* STRING */
	};
	Symbol	*next;			/* pointer to next Symbol */
};

struct 	array { 			/* array type, alloc on heap */
		double	*arr;		/* pointer to values */
		int 	ndim;		/* number of dimensions */
		int 	dim[1];		/* size of first dimension */
};
 
Symbol *install(char *, int, double), *lookup(char *), 
	*lookupnt(char *, int), *lookupnta(char *, int, int starg);

/* #define LARGENUM 1e38 */		/* undef for float */
#define LARGENUM 1e307			/* undef for double */

#define LARGENODE 1e38			/* undef for node vals: float */
#define LARGNOD (LARGENODE * 0.999)

#define PCOND 1                         /* print compartments (conductances) */
#define PCOMP 2                         /* print compartments (as spheres) */

#define DISP  1                         /* allow "display" statement to run */
#define DCOMP 2                         /* display compartments */
#define DCONN 4                         /* display connections */
#define DNODE 8                         /* display nodes */
#define DSTIM 16                        /* display stimulus */

int yylex(void);            /* interp */
void yyerror(char *s);      /* report compile-time error */

extern "C" {
	int system(const char *str);
}

void expop();
datum pop(void);
void push(datum d);
void popsav(void);
void pushsav(void);
void stackmove(void);
void evalvar(void), notinit(void), add(void), sub(void);
void mul(void), xdiv(void), mod(void);
void negate(void), power(void);
void preinc(void), predec(void), postinc(void), postdec(void);
void popone(void);
void addeq(void), subeq(void), muleq(void), diveq(void);
void pfprintf(void), pprintf(void), psprintf(void);
void pscanf(void), pfscanf(void), psscanf(void);
void pfopen(void), pfclose(void), pfgets(void);
void crlf(void);
void darray(void), initarr(void), erasarr(void), erelem(void), eranode(void); 
void xcable(void), xgj(void), xsynapse(void), xrecept(void); 
void xrecparm(void);
void rload(void), xresistor(void), xsphere(void), xvbuf(void);
void xvbufd(void);
void conn1(void), conn1l(void), conn2s(void), conn2sl(void);
void conn2d(void), conn2dl(void);
void xnode(), relative(), modrun(void), xstim(void), membtyp(void);
void vplot(void), vplotm(void);
void rbatt(void), xgbatt(void), xgcap(void), rcap(void), xrecord(void);
void grph(void), gplot(void); 
void txtf(void), xfread(void), xfwrite(void), xchan(void), xcachan(void);
void noise(void), xmod(void), eedist(void);
void e3dist(void), e2dist(void), efrac(void), n3dist(void), n2dist(void);
void xgausnn(void), incrpl(void), elimit(void), initrgen(void);
void dispnod(void), dispndn(), disprot(), efield(void), nfield(void);
void conn1m(void), dofft(void), gettype(void);
void defnonly(char *s);
void define(Symbol *sp, int narg, Inst *p);
void erasarg(int starg);

extern Inst *progp, *progbase, prog[], *code(Inst f);
void assign(void), bltin(void), varpush(void), constpush(void);
void print(void), local(void), locend(void), dlocarray(void);
void prexpr(void), prstr(), bitand(void), bitor(void), varread(void);
void gt(void), lt(void), eq(void), ge(void), le(void), ne(void);
void xand(void), orx(void), xnot(void);
void ifcode(void), whilecode(void), call(void), arg(), argassign(), xexit();
void funcret(void), procret(void);
void forcode(void),breakcode(void),contcode(void);
void edit(), eramod(void), pushfil(void), foreacode(void), xdebugf(void);
