/* ncelem.h */
/* neural element structures for "nc" */

struct elem;
struct attrib;
struct chattrib;
struct cattrib;
struct node;
struct conlst;
struct comp;
struct conn;

struct elem {

	short int ctype;		/* element type used to establish id */
	short int saved;		/* =1 -> save element */
	int elnum;			/* the element number */
	elem *next;			/* pointer to next element */
	elem *last;			/* pointer to last element */
	elem *hnext;			/* pointer to next hashed element */
	nodeint node1a;			/* nodes at ends of branch */
	nodeint node1b;			/* second dim */
	nodeint node1c;			/* third dim */
	nodeint node1d;			/* fourth dim */
	nodeint node2a;
	nodeint node2b;
	nodeint node2c;
	nodeint node2d;
	node *nodp1;			/* pointer to node 1 */
	node *nodp2;			/* pointer to node 2 */
	attrib *attpnt;			/* pointer to attributes */
	float jnoise;			/* Johnson noise for Rm */
	int   rsd;			/* noise seed */
	int modif;			/* nonzero -> modification of element */
	elem *lptr;			/* pntr to low level struc for modif */
	};

struct sphere: elem {
	float dia;			/* diameter of sphere */
	float Rm;			/* Rm for sphere membrane  */
	float Cm;			/* Cm for sphere membrane  */
	double vrev; 			/* membrane reversal potential    */
	float vrest; 			/* membrane initial resting pot   */
	};

struct cable: elem {
	float dia;			/* diameter of cable first end */
	float Rm;			/* Rm for branch */
	float Cm;			/* Cm for branch */
	double vrev; 			/* membrane reversal potential    */
	float vrest; 			/* membrane initial resting pot   */
	float Ri;			/* Ri for branch */
	float dia2;			/* diameter of cable second end */
	float length;		
	float cplam;			/* space constant for compartments */
};

struct synapse: elem {

	float thresh;			/* impedance looking into branch  */
	float *timec1; 			/* time const for syn filter */
	float *timec1h; 		/* time const for syn filter */
	float *timec2;			/* time const for syn filter */
	float *timec3;			/* time const for syn filter */
	short int nfilt1; 		/* num of low pass filters */
	short int nfilt1h; 		/* num of high pass filters  */
	short int nfilt2; 		/* num of low pass filters */
	short int nfilt3; 		/* num of low pass filters */
	float filt1hg; 			/* gain for high pass filter */
	float tfall2; 			/* falling tau for synapse */
	float tfall3; 			/* falling tau for synapse */
	float maxcond; 			/* max conductance for synapse */
	float trconc;			/* transmitter calib factor */
	double vrev; 			/* membrane reversal potential    */
	float nkd; 			/* Kd for synapse */
	float npow; 			/* Binding Hill coeff for synapse */
	float ngain; 			/* value of xfer func */
	float ckd; 			/* Kd for synapse */
	float cgain; 			/* cycG gain for synapse */
	short int ntact;		/* neurotrans action: open, close */
	short int curve;		/* transfer curve type: lin, expon */
	short int mesng1;		/* second messenger comp #1 */
	short int mesng2;		/* second messenger comp #2 */
	};

struct photrec: elem {
	float xpos;
	float ypos;
	float zpos;
	float dia;
	float maxcond;
	float pigm;
	float pathl;
	float attf;
	float timec1;
	float darknoise;
	float qc;
	float qcond;
	unsigned int prseed;
	short int photnoise;
	short int filt;
	char  save;
	char  restore;
	};

struct gapjunc: elem {
	float area;
	float specres;
	float vgain			/* forward parameters */;
	float voff;
	float taun;
	float rvgain;			/* reverse parameters */
	float rvoff;
	float rtaun;
	float gmax;
	float gnv;			/* non-volt sens conductance */
	short int modtyp;		/* type of modulation cyca/g */
	short int sign;			/* modulated open or close by cyca */
	nodeint nodeca;			/* node that controls modulation */
	nodeint nodecb;
	nodeint nodecc;
	nodeint nodecd;
	};

struct resistor: elem {
	float z;			/* resistance */
	};

struct loadelem: elem {
	float z;			/* resistance to gnd */
	double vrev; 			/* membrane reversal potential    */
	float vrest; 			/* membrane reversal potential    */
	};

struct capac: elem {
	float c;			/* capacitance */
	float vrest;
	};

struct batt: elem {
	float v;			/* voltage */
	};

struct vbuf: elem {
	float delay;			/* delay in msec */
	};

struct attrib {				/* part of an element */
					/* base attribute */
	short int ctype;		/* type of attrib */
	short int stype;		/* channel sub-type */
	attrib *attpnt;			/* pointer to attributes */
	elem *lptr;			/* pntr to low level struc for modif */
	};

struct mattrib: attrib {		/* part of an element */

	double vrev;
	double voffsm;
	double voffsh;
	float maxcond;
	float density;
	float ndensity;
	float unitary;
	float caperm;
	float taua;
	float taub;
	float tauc;
	float taud;
	float n;
	};

struct chattrib: mattrib {		/* part of an element */
					/* channel attribute */
	float d1;			/* chtyp for K2 chan */
	float d2;
	float k1;
	float k2;
	};

struct cattrib: mattrib {		/* part of an element */
					/* calcium attribute */
	short int exch;
	short int pump;
	short int cabuf;
	float cao;
	float cai;
	float kex;
	float ekm;
	float vmax;
	float pkm;
	float bkd;
	float bmax;
	float btot;
	float btoti;
	float cabnd;
	float mg;			/* magnesium conc. */
	short int cashell;
	};

struct nattrib: attrib {		/* part of an element */
					/* noise attribute */
	float n;
	float size;			/* vesicle size */
	float cov;			/* stdev / mean */
	float refr;			/* refractory period */
	float unitary;			/* size of unitary conductance */
	float taud;			/* time constant for 2-state chan */
	unsigned int rseed;
	};

struct node {
	short int ctype;		/* type of node = 1 */
	nodeint nodenm1;		/* node dimension 1 */
	nodeint nodenm2;		/* node dimension 2 */
	nodeint nodenm3;		/* node dimension 3 */
	nodeint nodenm4;		/* node dimension 4 */
	float xloc;			/* location of node */
	float yloc;
	float zloc;
	conlst *elemlst;		/* element list for connections */
	comp *comptr;			/* compartment which represents node */
	int  compnum;			/* number of comp representing node */
	node *hnext;			/* pointer to next node in hash table */
	node *next;			/* pointer to next node for foreach */
	};

struct conlst {				/* list of connections */
	conlst *next;			/* next in list */
	conn *conpnt;			/* pointer to conn */
	};

#define NLABELS 20
#define NULLVAL (-32768)
#define NULND (-1)

