/* Module ncmak in program nc */

/* Makes cable branches, nodes, compartments,
   connections, and synapses. 
*/


#include <stdio.h>
#include "adef.h"
#include "nc.h"
#include "y.tab.h"
#include "ncsub.h"
#include "ncomp.h"
#include "ncelem.h"
#include "control.h"

#define DEBUG 		/* */

#ifdef XSTIMM         	/* define XSTIMM when stim should ignore neur elems */
 #define XSTIM         	/* define XSTIM when stim should ignore */
#endif

#ifndef XSTIM
#define XMOD                    /* define XMOD when not XSTIM */
#endif

/* #define MALLOPT		/* */

#ifdef MALLOPT
#include <malloc.h>
#endif

#define MPI 3.14159265358979323846264

#include "nclist.h"

node *nodepnt=0;
node *nodend=0;
elem *elempnt=0;
elem *elemend=0;

extern double iono[];
extern double ioni[];
extern int    ionz[];

extern float pigmlen[];	/* path length through o.s. (in ncstim) */

#ifdef __cplusplus
extern "C" {
#endif

  #ifdef CPML
  #include <cpml.h>
  #else
  #include <math.h>
  #endif

#ifdef __cplusplus
}
#endif

double ncabs(double x);
char *emalloc(unsigned int n);
double calctau(double tau, double timestep);
double akcacalc(double v, double ca, double tau, double d1, double k1);
double bkcacalc(double v, double ca, double tau, double d2, double k2);
void execerror (char *s, char *t);
void freelst (conlst *clst);
void maklst(conlst **head, conn *cpnt);
void addlst(conlst **head, conlst *h2);
void dellst(conlst **head, conn *cpnt);
void makdyad(dyadlst **head, synap *cpnt);
chan *makchan(attrib *apnt, comp *comp1, comp *comp2, int saved);
void coneset(int n);
void saverec(recep *rpnt);
void restorec(recep *rpnt);
double calcchaninf(double v, chanparm *chp);
double dochani(chan *chpnt, double crit);
char *rinitstate (unsigned int seed, char *state, int n);
void delcomp  (comp *cpnt);
void delchan  (chan *chpnt);
void delsynap (synap *spnt);
void delrecep (recep *rpnt);
void delelem  (elem *epnt);
void einstall (elem *newelem);
elem *findelem (int num);
void unsetnodlst (node *nodp, elem *epnt);
node *findnode(nodeint node1a, nodeint node1b, 
		nodeint node1c, nodeint node1d, char *s);
void setreceptim(recep *rpnt, double timec);
chantype *getchantype(int ctype, int cnum);
double setbind(synap *spnt, double chligand);
double setcond(synap *spnt, double bound);

double gjalpha (double v, gj *gjpnt);
double gjbeta  (double v, gj *gjpnt);
double gamdev(int a);
char *rsetstate (char *state);
double qcond (chantype *ch);
double nernstv(double cratio);
void docacompi (cacomp *capnt);
float *makfiltarr (int newsiz, int oldsiz, float *oarr, double val);
lpfilt *maksynfilt (int stype, int nfilt, double tfall, double val,
                float *timec, double timestep);
char *findsym(int num);
char *prnode (int n1, int n2, int n3, int n4);
double qcavoff (chanparm *chp);
void restorstate(void);
void efree (void *ptr);

#ifndef XSTIM
void ninithash(void);
#endif
#ifndef XSTIMM
void einithash(void);
#endif

/*------------------------------------------------*/

void initmk(void)

/* reset all lists */

{
#ifdef MALLOPT
  static int called=0;
if (!called) {
   called = 1;
   mallopt(M_MXFAST,sizeof(comp)+1);	/* allocate comps, nodes, etc. faster */
   mallopt(M_NLBLKS,100);		/* set size of holding blocks  */
   mallopt(M_GRAIN,8);			/* set grain size  */
}
#endif

cumelem=0;				/* cumulative lengths of lists */
cumattr=0;
cumnattr=0;
cumcattr=0;
cumnode=0;
cumcomp=0;
cumconn=0;
cumchan=0;
cumsynap=0;
cumload=0;
cumrec=0;
cumrecst=0;
cumcacomp=0;
cumcycacomp=0;
cumcycgcomp=0;
cumclst=0;
cumdlst=0;
cumrand=0;
cumlpfilt=0;
cumntcomp=0;
cumgj=0;
delelmt=0;


elempnt=0;				/* pointers to lists */
elemend=0;
nodepnt=0;
nodend=0;
compnt=0;
compend=0;
connpnt=0;
connend=0;
synpnt=0;
synend=0;
loadpnt=0;
loadend=0;
chanpnt=0;
chanend=0;
recpnt=0;
recend=0;
recspnt=0;
recsend=0;

#ifdef XSTIM
 reccum = 0;                     /* reset number of receptors */
 reclist = 0;
 reclend = 0;
#endif
}

/*------------------------------------------------*/

void ncleanup(void)

/* erase all lists */

{
   attrib *apnt,*tapnt;
   elem *epnt,*tepnt;
   comp *cpnt,*tcpnt;
   conn *chpnt,*tchpnt;
   conn *cnpnt,*tcnpnt;
   node *npnt, *tnpnt;
   synap *spnt,*tspnt;
   load  *lpnt,*tlpnt;
   recep *rpnt,*trpnt;
   recstim *rspnt, *trspnt;
#ifdef XSTIM
   recnod *npt, *tnpt;
#endif

#ifndef XSTIMM
 for (epnt=elempnt; epnt; ) {		/* elements */
    tepnt = epnt;
    epnt = epnt->next;
    delelem(tepnt);			/* delete the element */
 }
 einithash();				/* erase the element hash table */
#endif

 for (npnt=nodepnt; npnt; ) {		/* nodes */
    freelst (npnt->elemlst);		/* free the node's element list */
    tnpnt = npnt;
    npnt = npnt->next;
    efree (tnpnt);
}
#ifndef XSTIM
 ninithash();				/* erase the node hash table */
#endif

 for (cpnt=compnt; cpnt; ) {		/* compartments */
    tcpnt = cpnt;
    cpnt = cpnt->next;
    delcomp (tcpnt);
}

 for (cnpnt=connpnt; cnpnt; ) {		/* connections */
    tcnpnt = cnpnt;
    cnpnt = cnpnt->next;
    efree (tcnpnt);
}

 for (lpnt=loadpnt; lpnt; ) {		/* loads */
    tlpnt = lpnt;
    lpnt = lpnt->next;
    efree (tlpnt);
}

 for (spnt=synpnt; spnt; ) {		/* synapses */
    tspnt=spnt;
    spnt= (synap*)spnt->next;
    delsynap (tspnt);
}

 for (chpnt=chanpnt; chpnt; ) {		/* channels */
    tchpnt=(chan*)chpnt;
    chpnt= chpnt->next;
    delchan((chan*)tchpnt);
}

for (rpnt=recpnt; rpnt; ) {		/* photoreceptors */
    trpnt=rpnt;
    rpnt=(recep*)rpnt->next;
    delrecep(trpnt);
}

for (rspnt=recspnt; rspnt; ) {		/* photoreceptor stimuli */
    trspnt = rspnt;
    rspnt = (recstim *)rspnt->next;
    efree (trspnt);
}

#ifdef XSTIM
for (npt=reclist; npt; ) {
    tnpt = npt;
    npt = npt->next;
    efree (tnpt);
}
#endif

initmk();				/* reset all lists and counts */
}

/*------------------------------------------------*/

void freelst (conlst *pnt)

/* Free a conlst (list of connections), part of either
   a node, compartment, or cacomp, but not its connection (conpnt).
*/

{
   conlst *lpnt,*tlpnt;

   for (lpnt=pnt; lpnt; ) {
        tlpnt = lpnt;
        lpnt = lpnt->next;
	efree (tlpnt);
    }
}

/*------------------------------------------------*/

void freendp (node *nodp, elem *epnt)
             
/* Reverse of "setnodlst()" in modcode.c.
   Subtracts a "conlst" from a node's element list,
    but maintains the list by patching pointers.
   Frees the list pointer from a node to an element.
   Used when the element is to be freed. 
*/

{
  unsetnodlst (nodp,epnt);
}

/*------------------------------------------------*/

void copyattr(attrib *src, attrib *dest)

/* Copy params but not pointers from one attrib to another. */

{
   chattrib *chsrc,*chdest;
   cattrib *casrc,*cadest;
   nattrib *nsrc,*ndest;

   if (!src || !dest) {
      fprintf (stderr,"copyattr: Missing pointer.\n");
      return;
   }

   dest->ctype   = src->ctype;
   dest->stype   = src->stype;
   dest->lptr    = src->lptr;

   switch (src->ctype) {

      case AMPA:
      case GABA:
      case SYN2:
      case NA:
      case K:
      case KCa:
	  chsrc = (chattrib *)src;
	  chdest = (chattrib *)dest;
	  chdest->vrev    = chsrc->vrev;
	  chdest->voffsm  = chsrc->voffsm;
	  chdest->voffsh  = chsrc->voffsh;
	  chdest->maxcond = chsrc->maxcond;
	  chdest->density = chsrc->density;
	  chdest->ndensity = chsrc->ndensity;
	  chdest->unitary = chsrc->unitary;
	  chdest->caperm  = chsrc->caperm;
	  chdest->taua    = chsrc->taua;
	  chdest->taub    = chsrc->taub;
	  chdest->tauc    = chsrc->tauc;
	  chdest->taud    = chsrc->taud;
	  chdest->d1      = chsrc->d1;
	  chdest->d2      = chsrc->d2;
	  chdest->k1      = chsrc->k1;
	  chdest->k2      = chsrc->k2;
	  chdest->n       = chsrc->n;
          break;

      case NMDA:
      case CGMP:
      case CA:
      case CACOMP:
	  casrc = (cattrib *)src;
	  cadest = (cattrib *)dest;
	  cadest->vrev    = casrc->vrev;
	  cadest->voffsm  = casrc->voffsm;
	  cadest->voffsh  = casrc->voffsh;
	  cadest->maxcond = casrc->maxcond;
	  cadest->density = casrc->density;
	  cadest->ndensity = casrc->ndensity;
	  cadest->unitary = casrc->unitary;
	  cadest->caperm  = casrc->caperm;
	  cadest->taua    = casrc->taua;
	  cadest->taub    = casrc->taub;
	  cadest->tauc    = casrc->tauc;
	  cadest->taud    = casrc->taud;
	  cadest->n      = casrc->n;

	  cadest->exch   = casrc->exch;
	  cadest->pump   = casrc->pump;
	  cadest->cabuf  = casrc->cabuf;
	  cadest->cao    = casrc->cao;
	  cadest->cai    = casrc->cai;
	  cadest->kex    = casrc->kex;
	  cadest->ekm    = casrc->ekm;
	  cadest->vmax   = casrc->vmax;
	  cadest->pkm    = casrc->pkm;
	  cadest->bkd    = casrc->bkd;
	  cadest->bmax   = casrc->bmax;
	  cadest->btot   = casrc->btot;
	  cadest->btoti  = casrc->btoti;
	  cadest->cabnd  = casrc->cabnd;
	  cadest->mg     = casrc->mg;
	  cadest->cashell= casrc->cashell;
          break;

   case VESNOISE:
   case CHNOISE:
	  nsrc  = (nattrib *)src;
	  ndest = (nattrib *)dest;
	  ndest->n     = nsrc->n;   
	  ndest->size  = nsrc->size;   
	  ndest->cov   = nsrc->cov;   
	  ndest->refr  = nsrc->refr;   
	  ndest->unitary = nsrc->unitary;   
	  ndest->taud  = nsrc->taud;   
	  ndest->rseed = nsrc->rseed;   
          break;
   }
}

/*------------------------------------------------*/

cattrib *makcattr(void)

/* make a new neural element to contain calcium attributes.
   Return a pointer to the new element. */

{
    cattrib *apnt;

#ifdef DEBUG 
  if (debug & 1 && debugz & 16)  fprintf (stderr,"makcattrib %d\n",cumelem);
#endif

  if ((apnt=(cattrib *)emalloc(sizeof(cattrib))) == (cattrib *)NULL) {
     fprintf (stderr,"no space left for cattrib %d\n",cumelem);
     return ((cattrib *)NULL);  
  }
  apnt->stype   = NULLVAL;
  apnt->vrev    = NULLVAL;
  apnt->voffsm  = NULLVAL;
  apnt->voffsh  = NULLVAL;
  apnt->taua    = NULLVAL;
  apnt->taub    = NULLVAL;
  apnt->tauc    = NULLVAL;
  apnt->taud    = NULLVAL;
  apnt->maxcond = NULLVAL;
  apnt->density = NULLVAL;
  apnt->ndensity= NULLVAL;
  apnt->n       = NULLVAL;
  apnt->unitary = NULLVAL;

  apnt->cao     = NULLVAL;
  apnt->cai     = NULLVAL;
  apnt->kex     = NULLVAL;
  apnt->vmax    = NULLVAL;
  apnt->pkm     = NULLVAL;
  apnt->ekm     = NULLVAL;
  apnt->btot    = NULLVAL;
  apnt->btoti   = NULLVAL;
  apnt->bkd     = NULLVAL;
  apnt->bmax    = NULLVAL;
  apnt->cabnd   = NULLVAL;
  apnt->caperm  = NULLVAL;
  apnt->cashell = NULLVAL;
  apnt->mg      = NULLVAL;
  apnt->pump    = 0;
  apnt->cabuf   = 0;
  apnt->exch    = 0;
  apnt->attpnt  = (attrib *)NULL;
  cumcattr++;
  return (apnt); 
}

/*------------------------------------------------*/

chattrib *makchattr(void)

/* make a new neural element to contain attributes.
   Return a pointer to the new element. */

{
    chattrib *apnt;

#ifdef DEBUG 
  if (debug & 1 && debugz & 16)  fprintf (stderr,"makattrib %d\n",cumelem);
#endif

  if ((apnt=(chattrib *)emalloc(sizeof(chattrib))) == (chattrib *)NULL) {
     fprintf (stderr,"no space left for attrib %d\n",cumelem);
     return ((chattrib *)NULL);  
  }
  apnt->ctype   = NULLVAL;
  apnt->stype   = NULLVAL;
  apnt->lptr    = (elem *)NULL;
  apnt->vrev    = NULLVAL;
  apnt->voffsm  = NULLVAL;
  apnt->voffsh  = NULLVAL;
  apnt->maxcond = NULLVAL;
  apnt->density = NULLVAL;
  apnt->ndensity= NULLVAL;
  apnt->unitary	= NULLVAL;
  apnt->caperm  = NULLVAL;
  apnt->taua    = NULLVAL;
  apnt->taub    = NULLVAL;
  apnt->tauc    = NULLVAL;
  apnt->taud    = NULLVAL;
  apnt->n	= NULLVAL;
  apnt->d1 	= NULLVAL;
  apnt->d2 	= NULLVAL;
  apnt->k1 	= NULLVAL;
  apnt->k2 	= NULLVAL;
  apnt->attpnt  = (attrib *)NULL;
  cumattr++;
  return (apnt); 
}

/*------------------------------------------------*/

nattrib *maknattr(void)

/* make a new neural element to contain noise attributes.
   Return a pointer to the new element. */

{
    nattrib *apnt;

#ifdef DEBUG 
  if (debug & 1 && debugz & 16)  fprintf (stderr,"maknattrib %d\n",cumelem);
#endif

  if ((apnt=(nattrib *)emalloc(sizeof(nattrib))) == (nattrib *)NULL) {
     fprintf (stderr,"no space left for nattrib %d\n",cumelem);
     return ((nattrib *)NULL);  
  }
  apnt->ctype   = NULLVAL;
  apnt->stype   = NULLVAL;
  apnt->lptr    = (elem *)NULL;
  apnt->n       = NULLVAL;
  apnt->size    = NULLVAL;
  apnt->unitary = NULLVAL;
  apnt->taud    = NULLVAL;
  apnt->rseed   = 0;
  apnt->cov     = NULLVAL;
  apnt->refr    = NULLVAL;
  apnt->attpnt  = (attrib *)NULL;
  cumnattr++;
  return (apnt); 
}

/*------------------------------------*/

load *makload(loadelem *epnt, comp *comp1)
               
/* make a new resistive load and link it to the load list. */

{
    load *lpnt;
    double rp, r;
 
    if ((lpnt=(load *)emalloc(sizeof(load))) == (load*)NULL) {
      fprintf (stderr,"no space left for load %d\n",cumload+1);
      return ((load *)NULL);  
    }
    lpnt->next = (load *)NULL;
    if (!loadpnt) loadpnt = lpnt;  	/* save head if first load */
    if (loadend)
      loadend->next = lpnt;
    loadend = lpnt;

    loadend->comp1  = comp1; 		 /* compartment we're connected to */
    maklst (&comp1->clst,(conn*)loadend); /* link presynaptic cell conn */
    loadend->ctype  = epnt->ctype;	 /* set load type */
    if (loadend->ctype == ELEMENT)
		 loadend->ctype = LOAD;  /* default load type */
 					 /* use default values if necess */
   
    if ((r=epnt->z)==NULLVAL)   r = 1e6; /* resistance of load */
    if ((rp=epnt->vrev)==NULLVAL) rp = 0; /* reversal potential */
    if (r==0) r = 1.0;
    loadend->conduct = 1.0 / r;
    loadend->vrev = rp;                 /* reversal potential */
    cumload++; 				/* increment total */
    return (loadend); 
}

/*------------------------------------*/

void modload(loadelem *ldpnt)
               
/* modify an existing load */

{
    elem *epnt;
    load *lpnt;
    double z;

     if (!(epnt=findelem(ldpnt->modif))) {
      fprintf (stderr,"modload: can't find element %d\n",ldpnt->modif);
      return;
     }

    lpnt = (load *)epnt->lptr;	   /* get pointer to synapse from elem */
    if (lpnt==(load *)NULL) {
       fprintf (stderr,"modload: can't find load for elem %d\n",ldpnt->modif);
       return;  
    }
    if (lpnt->ctype != LOAD) {
       fprintf (stderr,"modload: element %d is not a load\n",ldpnt->modif);
       return;  
    }

    if (ldpnt->z != NULLVAL) {
       z = ldpnt->z;
       if (z==0.0) z = 1e8;
       lpnt->conduct = 1.0/z;  				/* conductance */
    }
    if (ldpnt->vrev!=NULLVAL)  lpnt->vrev = ldpnt->vrev; /* reversal pot */
}

/*------------------------------------*/

char *makrand (int rndsiz, char *mesg, int num)

{
      char *stpnt;

 if ((stpnt=(char *)emalloc(rndsiz)) == (char *)NULL) {
   fprintf (stderr,"upk_chan: No space left for %s rand state %d.\n", mesg,num);
 }
 cumrand++;
 return stpnt;
}

/*------------------------------------*/

void linksyn (conn *cpnt)

/* Add a connection to the list of synapses. */

{
 if (!synpnt) synpnt = (synap *)cpnt;  	/* save head if first synap */
 if (synend)
   synend->next = (synap *)cpnt;
 synend = (synap *)cpnt;
}

/*------------------------------------*/

ntcomp *findntcomp (comp *cpnt, int type)

/* Find a nt concentration given its type. */

{
     int found;
     ntcomp *npnt;

   if (!cpnt) {
	fprintf (stderr,"findntcomp: can't find compartment\n");
	return (ntcomp*)NULL;
   }
   for (found=0,npnt=cpnt->ntpnt; npnt; npnt=npnt->ntpnt) { /* search list */
      if (npnt->ctype==type) {
	 found = 1;
	 break;
      }
   }
   if (!found) return (ntcomp*)NULL;
   else        return npnt;
}

/*------------------------------------*/

ntcomp *makntcomp (comp *cpnt, int type, double val, int n)

/* Make nt concentration for a compartment. */

{
     int found;
     ntcomp *npnt,*ntp;

   if (!cpnt) return (ntcomp*)NULL;
   if (!(npnt=findntcomp(cpnt,type))) {
      if ((npnt=(ntcomp *)emalloc(sizeof(ntcomp))) == (ntcomp*)NULL) {
        fprintf (stderr,"makntcomp: no space left for ntcomp\n");
        return ((ntcomp *)NULL);  
      }
      npnt->ntpnt = cpnt->ntpnt;
      cpnt->ntpnt = npnt;
      npnt->ctype = type;
   }
   npnt->val = val;
   npnt->n += n;
   cumntcomp++;
   return (npnt);
}

/*------------------------------------*/

void delntcomp (ntcomp *npnt)

/* Delete a list of ntcomps. */

{
   ntcomp *npt,*tpt;

  if (!npnt) return;

  for (npt=npnt; npt; npt=tpt) {	/* erase list */
    tpt = npt->ntpnt;
    efree (npt);
  }
}

/*------------------------------------*/

int setnt (comp *cpnt, int type, double val)

/* Add a concentration of nt to a compartment */

{
     int found;
     ntcomp *npnt;

  if (!(npnt=findntcomp(cpnt,type))) {
	npnt = makntcomp (cpnt, type, val, 1);
  }
  else if (npnt) {
    npnt->val += val;
    return 1;
  }
  else return 0;
}

/*------------------------------------*/

double getnt (comp *cpnt, int type)

/* Return neurotransmitter concentration for a compartment. */
/*  Idea is that nt can be puffed onto a node and the    */
/*  compartment keeps track of each such substance. */

/* If a nt comp has not already been made, make a new one. */

{
     int found;
     ntcomp *npnt;

   if (!(npnt=findntcomp(cpnt,type))) return 0;
   else return (npnt->val / npnt->n);
}

/*------------------------------------*/

int isnt (comp *cpnt, int type)

/* Return neurotransmitter concentration for a compartment. */
/*  Idea is that nt can be puffed onto a node and the    */
/*  compartment keeps track of each such substance. */

/* If a nt comp has not already been made, make a new one. */

{
     int found;
     ntcomp *npnt;

   if (!(npnt=findntcomp(cpnt,type))) return 0;
   else return 1;
}


/*------------------------------------*/

double ghkvi (chantype *chtyp, double vrev)

/* calculate denominator of GHK potential for a channel type */
/*  but leave out Ca terms as these will be added later */

/*  Use const field eqn from Lewis 1979, J.Physiol 286:417.*/

{
    int i,j, ctype, ionv, ionv2;
    double perm, ci, df, nf, cavoff;

   cavoff = qcavoff(chtyp->parm) * dcaspvrev;
   df = exp ( FR * (vrev-cavoff) / ktemp); /* multiplier for Ca perm */
   nf = 1.0 / ( 1.0 + df);
   for (ci=i=0; i<NPERM; i++) {
       if (i==PCA) continue;	      /* don't add Ca terms here */
       perm = chtyp->perm[i];
       ionv = ionz[i];
       ionv2 = ionv*ionv;             /* perm multiplier of 4 for Ca chans */
       switch(ionv) {
	 case -1:					/* Cl, etc */
          	ci += perm * iono[i]; break;
	 case 1:					/* Na, K, etc */
          	ci += perm * ioni[i] * ionv; break;
	 case 2:					/* Mg++ etc. */
          	ci += perm * ioni[i] * ionv2 * nf * df; break;
	}
     }
   return (ci);
}
/*------------------------------------*/

double ghkvo (chantype *chtyp, double vrev)

/* calculate numerator of GHK potential for a channel type */
/*  but leave out Ca terms as these will be added later */

/*  Use const field eqn from Lewis 1979, J.Physiol 286:417.*/

{
    int i, ionv, ionv2;
    double perm, co, df, nf, cavoff;

   cavoff = qcavoff(chtyp->parm) * dcaspvrev;
   df = exp ( FR * (vrev-cavoff) / ktemp); /* multiplier for Ca perm */
   nf = 1.0 / ( 1.0 + df);
   for (co=i=0; i<NPERM; i++) {
       if (i==PCA) continue;	      /* don't add Ca terms here */
       perm = chtyp->perm[i];
       ionv = ionz[i];
       ionv2 = ionv*ionv;             /* perm multiplier of 4 for Ca chans */
       switch(ionv) {
	 case -1:					/* Cl, etc */
          	co += perm * ioni[i]; break;
	 case 1:					/* Na, K, etc */
          	co += perm * iono[i]; break;
	 case 2:
          	co += perm * iono[i] * ionv2 * nf; break; /* allow Mg++, etc */
	}
     }
   return (co);
}

/*------------------------------------*/

double ghkv (chantype *chtyp, double cao, double cai)

/* Calculate GHK potential for a channel type. */

/* Na, K, and Cl concentrations are in ioni[] and iono[]. */
/* Ca concentration is given in cao, cai and is assumed to vary. */
/* The terms "df" and "nf" allow ions with valence of 2 to be */
/*  combined with Na, K, and Cl ions. */

/*  See derivation of const field eqn in Lewis 1979, J.Physiol 286:417. */

/* "cavoff" is surface potential, emprically fit to data in Lewis (1979) */ 
/*  It is derived from a gating shift of 18 mV/factor of 10 change in cao */
/*  Then it is multiplied by "dcasurf" (.18) to give surface potential here */

{
    int i, j, ctype, ion, ionv, ionv2;
    double perm, ci, co, df, nf;
    double ionox, ionix, cavoff, err, vrev;

   ctype = chtyp->ctype;
   vrev = chtyp->vrev;               /* start by assuming vrev = 0 */
   cavoff = qcavoff(chtyp->parm) * dcaspvrev;
   for (err=1.0,j=0; j<20 && ncabs(err) > 1e-4; j++) {
     df = exp ( FR * (vrev-cavoff) / ktemp); /* multiplier for Ca perm */
     nf = 1.0 / ( 1.0 + df);
     ci = ghkvi(chtyp,vrev);
     co = ghkvo(chtyp,vrev);
     ion = PCA;
     perm = chtyp->perm[ion];
     ionv = ionz[ion];
     ionv2 = ionv*ionv;               /* perm multiplier of 4 for Ca chans */
     switch(ionv) {
	 case 2:
		if (ion==PCA) {
		  ionox = cao;
		  ionix = cai;
	  	}
	  	else {
	 	  ionox = iono[ion];
		  ionix = ioni[ion];
	  	}
          	co += perm * ionox * ionv2 * nf;
          	ci += perm * ionix * ionv2 * nf * df;
	  break;
   }
   err = vrev;
   vrev = nernstv (co/ci);           /* find reversal potential */
   vrev += cavoff;
   err -= vrev;
   switch (ctype) {	/* empirically derived relaxation multipliers */
     case CA:   vrev += err * .45; break;
     case NMDA:
     case AMPA: 
     case CGMP: vrev += err * .06; break;
     case NA:   vrev += err * .0015; break;
     case GABA: 
     case K: 
     case KCa:    break;
   };
   /* fprintf (stderr,"%d %s vrev %g err * %g\n",
		j, findsym(chtyp->ctype),vrev,err); /* */
   }
   return (vrev);
}

/*------------------------------------*/

unsigned short crc (unsigned short crc, unsigned char val)

/* Given a remainder up to now, return the new CRC after one
   char is added. */

{
    int i;
    unsigned int ans;

    ans = crc ^ val << 8;
    for (i=0; i<8; i++) {
       if (ans & 0x8000)
	   ans = (ans <<= 1) ^ 4129;
       else
	   ans <<= 1; 
    }
    return ans;
}

/*------------------------------------*/

int noduniq(nodeint nodea, nodeint nodeb, 
		nodeint nodec, nodeint noded, int seed)

/* Make unique int from 4 integer (short int) values (e.g. node * number). */
/* Output val needs to be 4 byte integer for rsrandom(). */

{
    unsigned int x;
    unsigned short crcval;

   x = crcval = 0;
   crcval = crc (crcval,  noded & 0x00ff);
   crcval = crc (crcval, (noded & 0xff00) >> 8);
   crcval = crc (crcval,  nodec & 0x00ff);
   crcval = crc (crcval, (nodec & 0xff00) >> 8);
   x = crcval;
   x <<= 16;
   crcval = 0;
   crcval = crc (crcval,  nodeb & 0x00ff);
   crcval = crc (crcval, (nodeb & 0xff00) >> 8);
   crcval = crc (crcval,  nodea & 0x00ff);
   crcval = crc (crcval, (nodea & 0xff00) >> 8);
   x += crcval;

   if (seed < 0) seed = rseed;
   x ^= seed;			/* random modifier for phot noise */

   return (x & 0x7fffffff);
}

/*------------------------------------*/

synap *maksynap(synapse *epnt, comp *comp1, comp *comp2)
               
/* Make a new synapse and link it to the synapse list. */

{
    int sf1,sf1h,sf2,sf3,nt,i,stop;
    unsigned int nrseed;
    synap *spnt;
    lpfilt *fpnt;
    attrib *apnt;
    chattrib *ntpnt;
    nattrib *napnt;
    cacomp *capnt;
    double st, sn, sg, nkd, npow, sc, rp, mx, trc;
    double sf1hg, ft1, ft2, tf2, ft3, tf3;
    synapse *oepnt;
    ntchan *makntchan(attrib *apnt, comp *comp1, int ctype, int saved);
    void makexp(void);
    chattrib *chsyn=(chattrib*)NULL;
 
    if ((spnt=(synap *)emalloc(sizeof(synap))) == (synap *)NULL) {
      fprintf (stderr,"no space left for synapse %d\n",cumsynap+1);
      return ((synap *)NULL);  
    }
    spnt->next = (synap *)NULL;
    linksyn (spnt);			/* add to list of synapses */

    cumsynap++; 			/* increment total */
    spnt->num = cumsynap;

    spnt->comp1  = comp1; 		/* presynaptic cell */
    spnt->comp2  = comp2; 		/* postsynaptic cell */
    spnt->num1   = comp1->num;	/* comp # for presynap cell */
    spnt->num2   = comp2->num;	/* comp # for postsynap cell */
    maklst (&comp1->clst,(conn*)spnt);/* link presynaptic cell conn */
    maklst (&comp2->clst,(conn*)spnt);/* link postsynaptic cell conn */
    spnt->ctype  = epnt->ctype;	/* set synapse type */
    if (spnt->ctype == ELEMENT)
		 spnt->ctype = SYNAPSE;  /* default synapse type */
    					/* use default values if necess */
    if ((sn=epnt->ngain)==NULLVAL) sn = dsn; 	/* neurotrans gain */
    if ((sc=epnt->curve)==NULLVAL) sc = dsc; 	/* synap curve: lin or expon */
    if ((sg=epnt->cgain)==NULLVAL)   sg = dsg; 	/* default cycG gain */
    if ((nt=epnt->ntact)==NULLVAL)   nt = OPEN; /* neurotransmitter action */
    if ((nkd=epnt->nkd)==NULLVAL)    nkd = dskd; /* half-max saturation point */
    if ((npow=epnt->npow)==NULLVAL) npow = dshc; /* hill coeff at postsyn rec */
    if ((st=epnt->thresh)==NULLVAL)  st = dst;     /* synaptic threshold */

    if ((sf1=(int)(epnt->nfilt1))==NULLVAL)sf1=1;  /* number of filters */
    if (!epnt->timec1) epnt->timec1=makfiltarr(sf1, 0, (float*)NULL, 1.0);
    if (epnt->timec1[0]==0) epnt->timec1[0]= dfta; /* tau of filters (msec) */

    if ((sf1h=(int)(epnt->nfilt1h))==NULLVAL)sf1h=0; /* number of h filters */
    if ((sf1hg=(epnt->filt1hg))==NULLVAL)sf1hg=0;    /* gain of h filter */
    if (epnt->timec1h)
      if (epnt->timec1h[0]==0) epnt->timec1h[0]= dftah; /* high pass tau (ms) */

    if ((sf2=(int)(epnt->nfilt2))==NULLVAL)sf2=1;/* number of filters */
    if (!epnt->timec2) epnt->timec2=makfiltarr(sf2, 0, (float*)NULL, 1.0);
    if (epnt->timec2[0]==0) epnt->timec2[0]=dftb; /* tau of filters (msec) */
    if ((tf2=epnt->tfall2)==NULLVAL) tf2 = 0.0;	 /* falling tau of filt (msec)*/

    if ((sf3=(int)(epnt->nfilt3))==NULLVAL)sf3=0;/* number of filters */
    if ((tf3=epnt->tfall3)==NULLVAL) tf3 = 0.0;	 /* falling tau of filt (msec)*/
    if ((trc=epnt->trconc)==NULLVAL) trc = dstr;
    if ((rp=epnt->vrev)==NULLVAL) {	/* reversal potential */
       rp = ghkv (getchantype(AMPA,0),dcao,dcai);
       spnt->setvrev = 0;
    }
    else spnt->setvrev = 1;

    spnt->trconc = trc;
    spnt->thresh = st;
    spnt->nkd    = nkd;
    spnt->npow   = npow;
    spnt->ckd    = dgkd;
    spnt->vrev = rp;			/* reversal potential */
    spnt->maxcond = NULLVAL;		/* unset to allow makchan to set */
    if ((mx=epnt->maxcond)!=NULLVAL) spnt->maxcond = mx;
    spnt->oldc   = 0.0;
				/* allocate filter space if filters specified */

    if (epnt->attpnt) {			/* if synapse has chan attributes */
      for (stop=0,apnt=epnt->attpnt; apnt && !stop; apnt=apnt->attpnt) {
           ntchan *chpnt;

       switch (apnt->ctype) {

     /* The idea here is to make a synaptic channel if a
	specific postsynaptic channel type has been defined,
	or if N, taud, or unit has been defined, or if channel 
	noise has been defined. If channel noise has been defined, 
	but a channel type (and N, taud, and unit) have not been 
	defined, then make a new chattrib and link it before the
	chnoise nattrib, then use the normal method for defining a
	postsynaptic channel with noise (channel chattrib comes first,
	then nattrib comes after it in the attrib list). Always stop
	after the first channel definition, look at noise attribs 
	further below.
      */

						/* default:        */ 
         case CHNOISE:  chsyn = makchattr();	/* make dummy chan */
			chsyn->ctype = SYN2; 
			chsyn->attpnt = apnt;
			chsyn->taud = ((nattrib*)apnt)->taud;
			apnt = chsyn;
         case AMPA: 
         case CGMP: 
         case NMDA: 
         case GABA: 
         case SYN2: 
           chpnt = (ntchan*)makchan(apnt,spnt->comp2,(comp*)spnt,epnt->saved); 

           spnt->resp = chpnt;			/* Give synapse chan props */
						/*  to replace filt3. */

          ntpnt = (chattrib *)apnt;
	  if (chpnt) {
	     if (epnt->trconc==NULLVAL) spnt->trconc = chpnt->chtyp->trconc;
             spnt->maxcond = chpnt->maxcond;
             if (ntpnt->vrev == NULLVAL) {
	        chpnt->vrev = rp;
             }
             else spnt->vrev = ntpnt->vrev;
	  }
	 if (chsyn) { efree(chsyn); }
	 stop = 1;
	 break;

	default: break;

       }     /* switch (ctype) */

                
      }  /* for (apnt= ; ; ) */
    }

    if (spnt->maxcond == NULLVAL) spnt->maxcond=dmaxsyn *
					 qcond(getchantype(SYN2,0));

    spnt->vstate = (char *)NULL;
    spnt->vsites = 0;
    spnt->cov = 1.0;
    spnt->vrefr = 0;
    spnt->vtint = 1.0;		/* max vesicle interval */
    spnt->vtime = 0;
    spnt->vflag = 0;
    spnt->mesng1 = 0;
    spnt->mesng2 = 0;

    for (apnt=epnt->attpnt; apnt; apnt=apnt->attpnt) {/* set up noise params */
       napnt = (nattrib *)apnt;
       switch (napnt->ctype) {

 	  case CHNOISE:
	  break;

 	  case VESNOISE:
	  if (napnt->n==NULLVAL)     	/* use defaults if not defined */	
	      spnt->vsites = (int)(dsvn);	
	  else
	      spnt->vsites = (int)(napnt->n);	

	  if (napnt->size==NULLVAL)
	      spnt->vsize = dvsz;	
	  else
	      spnt->vsize = napnt->size;	

	  if (napnt->refr==NULLVAL)
	       spnt->vrefr = 0;
	  else if (stiminc) spnt->vrefr = int(napnt->refr/stiminc+0.5);
	  if (spnt->vrefr < 0) spnt->vrefr = 0;
	  if (spnt->vrefr > MAXVINT) spnt->vrefr = MAXVINT;

	  if (napnt->cov==NULLVAL)
	       spnt->cov = 1.0;
	  else spnt->cov = napnt->cov;
	  if (spnt->cov > 1) spnt->cov = 1.0;

	  if ((nrseed=napnt->rseed) <= 0)
	    nrseed = noduniq(27091, spnt->num, 12491, spnt->num,rseed^4323);
          spnt->vstate = makrand(RNDSIZ,"vesicle",cumsynap);
	  if (spnt->vstate) rinitstate (nrseed,spnt->vstate,RNDSIZ);
	  if (spnt->vstate) rsetstate (spnt->vstate);
	  spnt->vtint = gamdev(1);
	  restorstate();
 
	  break;

         }  /* switch */
      }  /* for (apnt;;) */

      spnt->ntact = nt;
      spnt->ngain = sn;
      spnt->cgain = sg;
      spnt->curve = (int)(sc);
      if (spnt->curve==DYAD) {		/* if this synapse is 2nd in dyad */
        if (!(oepnt=(synapse*)findelem((int)sn))) {
           fprintf (stderr,"maksyn: can't find element %d\n",(int)sn);
           return ((synap*)NULL);
        }
        else {	/* making a dyad synapse, connect back to original */
          if (oepnt->lptr) { 			  /* if 1st synap exists */ 
           makdyad((dyadlst **)&((synap *)oepnt->lptr)->dyad,spnt);  
           makdyad((dyadlst **)&spnt->dyad,(synap *)oepnt->lptr);
          }
        }
      }
      else {	/* making a normal synapse, can connect dyad to it later */
         spnt->dyad = 0;
         oepnt = epnt;
      }
		/* make cycA/G comps, connect to postsynaptic comp2 */

    if (epnt->mesng1!=NULLVAL) {
       spnt->mesng1 = epnt->mesng1;
       makntcomp (comp2,spnt->mesng1,0,1); 
    }
    if (epnt->mesng2!=NULLVAL) {
       spnt->mesng2 = epnt->mesng2;
       makntcomp (comp2,spnt->mesng2,0,1); 
    }


    if (tf2) tf2 = 1.0 - calctau(tf2,stiminc/MSEC);
    if (tf3) tf3 = 1.0 - calctau(tf3,stiminc/MSEC);
    
    spnt->filt1  = maksynfilt (LP,sf1, 0, 0,oepnt->timec1,stiminc/MSEC);
    spnt->filt1h = maksynfilt (HP,sf1h,sf1hg,0,oepnt->timec1h,stiminc/MSEC);
    spnt->filt2  = maksynfilt (LP,sf2, tf2,  0, epnt->timec2,stiminc/MSEC);
    spnt->filt3  = maksynfilt (LP,sf3, tf3,  0, epnt->timec3,stiminc/MSEC);

/* fprintf (stderr,"sf1 %d sf1h %d ft1 %g sf2 %d ft2 %g sf3 %d ft3 %g\n",
				sf1,sf1h,ft1,sf2,ft2,sf3,tf3); /* */

/* fprintf (stderr,"nt %d se %g si %g sc %g st %g nkd %g mx %g rp %g\n",
				nt,se,si,sc,st,nkd,mx,rp); /* */

    return (spnt); 
}

/*------------------------------------*/

void setfiltval (lpfilt *fpnt, double val)

/* set a temporal filter for a synapse */

{
     int i;

  if (!fpnt) return;
  switch (fpnt->ctype) {
    case LPFILT:
        for (i=0; i<fpnt->nfilt; i++) {
           fpnt->lfilt[i] = val;	 /* init low-pass filt array */
        }
	break;

    case MFILT:  break;

  }
}

/*------------------------------------*/

void setfilttau (lpfilt *fpnt, int nfilt, float *timec, double tfall)

/* set a temporal filter for a synapse */

{
     int i;
     double timecn;

  if (!fpnt) return;
  switch (fpnt->ctype) {
    case LPFILT:
      if (timec) {
        if (nfilt > fpnt->nfilt) {
          fpnt->ftau = makfiltarr(nfilt, fpnt->nfilt,fpnt->ftau,0.0);
        }
        fpnt->nfilt = nfilt;
        for (i=0; i<nfilt; i++) {
           fpnt->ftau[i] = calctau(timec[i],MSEC/stiminc); /* set time const */
        }
      }
      switch (fpnt->stype) {

           case LP: if (tfall!=NULLVAL) 
			fpnt->tfall = 1.0-calctau(tfall,MSEC/stiminc);
		    break;

           case HP: if (tfall != NULLVAL) fpnt->tfall = tfall;   
		    break;
      }
     break;

    case MFILT:  break;

  }
}

/*------------------------------------*/

void modfiltval (lpfilt *fpnt, int nfilt, double val)

/* change the number of filters in a synaptic temporal filter */

{
     int i;
     float *lfilt;

  if (!fpnt) return;
  switch (fpnt->ctype) {
    case LPFILT:
	if (nfilt != NULLVAL) {
	  lfilt = fpnt->lfilt;
	  if (nfilt > fpnt->nfilt) {
             fpnt->lfilt = makfiltarr(nfilt, fpnt->nfilt,lfilt,val);
	  }
    	  fpnt->nfilt = nfilt;
	}
	break;

    case MFILT:  break;

  }
}

/*------------------------------------*/

void setsynv(synap *spnt, double v)
              
/* Set a synapse's internal activation from 
   its presynaptic voltage.
   If "synapse" is a voltage buffer, then
   set "destination" voltage equal to the
   "source" voltage.
 */

{
   int i;
   double cond,sa,st,tr;
   dbuf *dpnt;
   void maknois(chan *chpnt);
#ifndef XSTIM
    double getcurv(synap *spnt, double val);
#endif

switch (spnt->ctype) {

case SYNAPSE:
 st = spnt->thresh;			/* synaptic threshold */
 sa = v - st;				/* synaptic activation in mv */

 setfiltval(spnt->filt1,sa);
 setfiltval(spnt->filt1h,sa);

#ifdef XSTIM
    tr = 0;        			/* find resting transmitter rel */
    cond = 0;        			/* resting conductance */
#else
    if (spnt->curve==DYAD) {
         synap *tspnt;

       if (spnt->dyad) {
         tspnt = spnt->dyad->sdyad;
         if (tspnt) tr = getcurv (tspnt,v - tspnt->thresh);
       }
    }
    else tr = getcurv(spnt,sa);        	/* find resting transmitter rel */
			
    setfiltval(spnt->filt2,tr);		 /* initialize filter 2 */

    if (spnt->resp) {
      switch (spnt->resp->ctype){       /* initialize postsynaptic chan */
      
         case AMPA: 			/* set states & conductance */
         case NMDA: 
         case GABA: 
	   cond = dochani((chan*)spnt->resp,1e-8);  
	   maknois((chan*)spnt->resp);	/* set chans in states */
	   break;
         case CGMP: 
         case SYN2:
           cond = setbind(spnt, tr);	/* calc binding */
           if (spnt->filt3) setfiltval(spnt->filt3,cond); /* init filter 3 */
           cond = setcond(spnt, cond);	/* calc static conductance */
	   spnt->conduct = cond;
	   cond = dochani((chan*)spnt->resp,1e-8);  
	   maknois((chan*)spnt->resp);	/* set chans in states */
	   ((chan*)spnt->resp)->conduct = cond * spnt->maxcond;
	   break;

      }   /* switch (spnt->ctype) */
    }
    else {				/* standard postsynaptic apparatus */
       cond = setbind(spnt, tr);	/* calc binding */
       setfiltval(spnt->filt3,cond);	 /* initialize filter 3 */
       cond = setcond(spnt, cond);	/* calc static conductance */
    }
#endif

    if (spnt->mesng1){
        spnt->oldc = 0;
        setnt(spnt->comp2,spnt->mesng1, (cond-spnt->oldc)*spnt->trconc);
        spnt->oldc = cond;
    }
    if (spnt->mesng2){
        spnt->oldc = 0;
        setnt(spnt->comp2,spnt->mesng2, (cond-spnt->oldc)*spnt->trconc);
        spnt->oldc = cond;
    }
    break;

  case BUF:

    dpnt = (dbuf *)spnt;
    if (dpnt->delay) {
       short int val;
       double vf;

      vf = v / DELCONV; 
      if (vf > 0) 
         val = (int)(vf + 0.5); 		/* input voltage */
      else
         val = (int)(vf - 0.5); 		/* input voltage */
      if (dpnt->delbuf)
        for (i=0; i<dpnt->delay; i++) 
           dpnt->delbuf[i] = val;
      dpnt->delpnt = dpnt->delbuf;
    }
   else {
     dpnt->comp2->v = v; 
   }
   break;
  }  /* end switch */
}

/*------------------------------------*/

synap *modsynap(synapse *sepnt)
               
/* modify an old synapse */

{
    int i,sfi,tfi;
    double dval;
    elem *epnt;
    synap *spnt;
    attrib *apnt;
    nattrib *napnt;
    lpfilt *fpnt;
#ifndef XSTIM
    double getcurv(synap *spnt, double val);
#endif

    if (!(epnt=findelem(sepnt->modif))) {
      fprintf (stderr,"modsyn: can't find element %d\n",sepnt->modif);
      return ((synap*)NULL);
    }

    spnt = (synap *)epnt->lptr;	/* get pointer to synapse from elem */
    if (spnt==(synap*)NULL) return ((synap *)NULL); 
    if (spnt->ctype != SYNAPSE) {
       fprintf (stderr,"modsyn: element %d is not a synapse\n",sepnt->modif);
       return ((synap*)NULL);  
    }

#ifndef XSTIM
   if (sepnt->ngain!=NULLVAL)spnt->ngain= sepnt->ngain; /* neurotrans gain */
   if (sepnt->curve!=NULLVAL) spnt->curve= sepnt->curve; /* synap curv:lin,exp*/
   if (sepnt->cgain!=NULLVAL) spnt->cgain = sepnt->cgain;/* cycG gain */
   if (sepnt->ntact!=NULLVAL) spnt->ntact = sepnt->ntact;/* neurotrans action*/
   if (sepnt->nkd!=NULLVAL)    spnt->nkd = sepnt->nkd;   /* half-max sat point*/
   if (sepnt->npow!=NULLVAL)   spnt->npow = sepnt->npow; /* Hill coeff */
   if (sepnt->ckd!=NULLVAL)    spnt->ckd = sepnt->ckd;   /* half-max sat point*/
   if (sepnt->maxcond!=NULLVAL)spnt->maxcond = sepnt->maxcond; /* max conduct*/
   if (sepnt->thresh!=NULLVAL)spnt->thresh = sepnt->thresh; /* synaptic thresh*/
   if (sepnt->vrev!=NULLVAL)  spnt->vrev = sepnt->vrev;  /* reversal potential*/
   if (sepnt->trconc!=NULLVAL) spnt->trconc = sepnt->trconc;
   if (sepnt->mesng1!=NULLVAL) spnt->mesng1 = sepnt->mesng1;
   if (sepnt->mesng2!=NULLVAL) spnt->mesng2 = sepnt->mesng2;


   setfilttau(spnt->filt1, sepnt->nfilt1, sepnt->timec1,NULLVAL);
   setfilttau(spnt->filt1h,sepnt->nfilt1h,sepnt->timec1h,sepnt->filt1hg);
   setfilttau(spnt->filt2, sepnt->nfilt2, sepnt->timec2,sepnt->tfall2);
   setfilttau(spnt->filt3, sepnt->nfilt3, sepnt->timec3,sepnt->tfall3);

	/* If we're changing number of filters,
        /*  we initialize them to the value of filter before them. */
        /* But if number of filters is 0, set them from comp */

   dval = spnt->comp1->v - spnt->thresh;
   modfiltval(spnt->filt1, sepnt->nfilt1, dval);
   modfiltval(spnt->filt1h, sepnt->nfilt1h, dval);

   dval = getcurv(spnt, dval);
   modfiltval(spnt->filt2, sepnt->nfilt2, dval);

   if (sfi=(sepnt->nfilt3!=NULLVAL)) {	/* modify nfilt in third filter */
        double tr,bnd,cond;
        double k,kk,ll,pp;

      tr = dval;
      pp = spnt->npow;
      k = spnt->nkd;
      if (floor(pp) == pp) {         /*  integer power */
      	kk = k;
      	ll = tr;
              for (i=int(pp); i>1; i--) {
         	   kk *= k;
        	   ll *= tr;
              }
      }
      else {				/* non-integer power */
       	kk = pow(k,pp);
       	ll = pow(tr,pp);
      }
      bnd = ll / (ll + kk);		/* fraction bound */

      switch (spnt->ntact) {		/* find resting conductance */
      	case OPEN:
 		cond = bnd;
		break;
	case CLOSE:
		cond = (1 - bnd * spnt->cgain);
		break;
      }	
      modfiltval(spnt->filt3, sepnt->nfilt3, cond);
   }

    for (apnt=sepnt->attpnt; apnt; apnt=apnt->attpnt) {/* set up noise params */
      napnt = (nattrib *)apnt;
      switch (napnt->ctype) {

 	case CHNOISE:
	break;

 	case VESNOISE:
	if (napnt->n!=NULLVAL)
	    spnt->vsites = (int)(napnt->n);	
	if (napnt->size!=NULLVAL)
	    spnt->vsize = napnt->size;	
        if (napnt->rseed > 0) rinitstate (napnt->rseed,spnt->vstate,RNDSIZ);
	break;

        }  /* switch */
      }  /* for (apnt;;) */

#endif
    return (spnt); 
}

/*------------------------------------*/

void delsynap (synap *spnt)

{
  synap *tspnt;
  dyadlst *lpnt, *nlpnt;
  void deldyad(dyadlst **head);

  tspnt = spnt;
  if (spnt->vstate) efree (spnt->vstate);
  if (spnt->filt1)  efree (spnt->filt1);
  if (spnt->filt2)  efree (spnt->filt2);
  if (spnt->filt3)  efree (spnt->filt3);
  if (spnt->dyad) {
   for (lpnt=spnt->dyad; lpnt; lpnt=nlpnt) {
     nlpnt = lpnt->next;
     if (lpnt->sdyad) {
        deldyad(&lpnt->sdyad->dyad);   /* delete pointer in other synapse */
     }
     efree (lpnt);
   }
  }
  spnt = (synap*)spnt->next;
  efree (tspnt);
  cumsynap--;
}

/*------------------------------------*/
recpar rectypes[NUMREC];

void initrec(void)

/* initialize receptor transduction cascade constants */

/* Calibration procedure:

 Read:

    Tranchina, D. (1998) J. Gen. Physiol. 111: 3-6.

    Nikonov, S., Engheta, N., and Pugh, E.N. Jr. (1998) 
	J.  Gen. Physiol. 111: 7-37.

    Calvert, P.D., Ho, T.W., LeFebvre, Y.M, and Arshavsky, V.Y (1998)   
	J.  Gen. Physiol. 111: 39-51.


    1) adjust delay times for rhod, rhod2, rstar. Time constant
for rstar should be 0.5 msec @ 37 deg (Nikonov et al, 1998).

    2) adjust rise and fall of gpr (G), it is fast here, and its
time course (duration) sets rise time of pde.

    3) adjust rise and fall of pde (E). Its rise detemines rise
time of response. Its fall determines fall time constant of light response 
(Nikonov et al, 1998).

    4) adjust starting cycg, cond, ca, cax to give flat trace
with no response (tcomp21c).

    5) adjust sfactor (speed factor) to give peak at 200 msec.

    6) adjust cax rise and fall to give response proper asymmetry.

    7) adjust starting cycg, cond, ca, cax to give flat trace
with no response (tcomp21c).

    8) readjust sfactor (speed factor) to give peak at 200 msec.

    9) Adjust dmaxrod to give 34 pA max response.

    10) Adjust lgain to give single photon response of 0.7 pA. 
*/

{
    recpar *cpt;
    double rrate, qrate;
    double cabuf, cycgbuf, loopgain;
    int pigm;
    double sfactor;


    cpt = &rectypes[0];		/* set up rod cascade constants */

    cpt->ctype = ROD;
    cpt->recslow = 1;		/* rod is slow by this factor */

    cpt->qc = dqrec;		/* Q10 factor for conductance shape */
    cpt->qcond = dqcrec;	/* Q10 factor for conductance cond */
				/* not currently used */

    cpt->qt = 37.0;		/* base temperature for channel rate & cond */
    cpt->oldtempcel = tempcel;
    qrate = exp(log(cpt->qc) * (tempcel - cpt->qt) / 10.0);

    sfactor = 1.0;		/* speedup of cascade & intensity */
    rrate = stiminc * qrate /sfactor;    /* stiminc = timinc for syn/stimuli */

    pigm = 0;
    cpt->pigm = pigm;
    cpt->pathl = pigmlen[pigm];	/* path length through o.s. */

    cabuf    = 1.0;
    cycgbuf  = 1.0;
    loopgain = 1.0;

    cpt->pdebase = .04;
    cpt->kdgcyc  = .010;
    cpt->kdrhodk = .002;

    cpt->rhod  = 0.0;
    cpt->rhodk = 0.0;
    cpt->rhod2 = 0.0;
    cpt->rstar = 0.0;
    cpt->gpr   = 0.0;
    cpt->pde   = cpt->pdebase;

    cpt->cycg  = 1.05605; 	/* values taken from tcomp21b */
    cpt->cond  = .994444; 
    cpt->totcond = 20.0;
    cpt->ca    =  0.364626; 
    cpt->cax   =  0.152104; 

    cpt->lgain    = rrate * 2.5;	/* adjust to give R* resp of 0.7 pA */
    cpt->rgain1   = calctau (0.0004,rrate);
    cpt->rgain2   = calctau (0.0004,rrate); /* 3 rhod steps give .5 ms tau */
    cpt->rgain3   = rrate * 800; 
    cpt->rkgain   = rrate * 5;	 /* adjust this for amount of adaptation */

					     /*  for R* activation */
    cpt->ggain    = rrate * 8.0;
    cpt->gcygain  = rrate * 11.2 / cycgbuf * loopgain;
    cpt->pdegain  = rrate * 80.0 / cycgbuf;
    cpt->condgain = rrate * 448 * loopgain;
    cpt->gca      = rrate * 270.0 / cabuf * loopgain;
    cpt->cxgain   = rrate * 128.0 * loopgain;

    cpt->decrstar = 1 - (rrate * 20);
    cpt->pdedec   = 1 - (rrate * 16);	/* controls tail of cond */
    cpt->deccond  = 1 - (rrate * 480);
    cpt->capump   =      rrate * 1000 / cabuf;
    cpt->deccax   = 1 - (rrate * 320);

					/* voltage activation of dark curr */
					/* assume linear, but offset */
    cpt->vrev  =  .72;			/* 0.1 activation / -80 mv */

    for (pigm=1; pigm<NUMREC; pigm++) {
      coneset(pigm);
    }
}

/*------------------------------------*/

void coneset(int pigm)
{
    recpar *cpt;
    double rrate, qrate, sfactor;
    double cabuf, cycgbuf, loopgain;

    cpt = &rectypes[pigm];		/* set up cone cascade constants */

    cpt->ctype = CONE;
    cpt->recslow = 1.0;			/* cone is slow by this factor */

    cpt->qc = dqrec;		/* Q10 factor for conductance shape */
    cpt->qcond = dqcrec;	/* Q10 factor for conductance cond */
				/* not currently used */

    cpt->qt = 37.0;		/* base temperature for channel rate & cond */
    cpt->oldtempcel = tempcel;
    qrate = exp(log(cpt->qc) * (tempcel - cpt->qt) / 10.0);

    sfactor = .5;		/* speedup of cascade & intensity */
    rrate = stiminc * qrate /sfactor;    /* stiminc = timinc for syn/stimuli */

    cpt->pigm = pigm;
    cpt->pathl = pigmlen[pigm];	/* path length through o.s. */
 
    cabuf    = 1.0;
    cycgbuf  = 1.0;
    loopgain = 1.0;

    cpt->pdebase = .04;
    cpt->kdgcyc  = .020;
    cpt->kdrhodk = .002;

    cpt->rhod  = 0.0;
    cpt->rhodk = 0.0;
    cpt->rhod2 = 0.0;
    cpt->rstar = 0.0;
    cpt->gpr   = 0.0;
    cpt->pde   = cpt->pdebase;

    cpt->cycg  = 1.03805; 	/* values taken from tcomp21b */
    cpt->cond  = .902444; 
    cpt->totcond = 20.0;
    cpt->ca    =  0.315626; 
    cpt->cax   =  0.184104; 

    cpt->lgain    = rrate * .25;	/* adjust to give R* resp ~ 0.02 pA */
    cpt->rgain1   = calctau (0.0004,rrate);
    cpt->rgain2   = calctau (0.0004,rrate); /* 3 rhod steps give .5 ms tau */
    cpt->rgain3   = rrate * 800; 
    cpt->rkgain   = rrate * 50;	     /* adjust this for amount of adaptation */

					     /*  for R* activation */
    cpt->ggain    = rrate * 100.0;
    cpt->gcygain  = rrate * 11.2 / cycgbuf * loopgain;
    cpt->pdegain  = rrate * 100.0 / cycgbuf;
    cpt->condgain = rrate * 448 * loopgain;
    cpt->gca      = rrate * 270.0 / cabuf * loopgain;
    cpt->cxgain   = rrate * 5.0 * loopgain;

    cpt->decrstar = 1 - (rrate * 20);
    cpt->pdedec   = 1 - (rrate * 1000);	/* controls tail of cond */
    cpt->deccond  = 1 - (rrate * 480);
    cpt->capump   =      rrate * 1000 / cabuf;
    cpt->deccax   = 1 - (rrate * 10);
					/* voltage activation of dark curr */
					/* assume linear, but offset */
    cpt->vrev  =  -0.008;		/* gated reversal pot */
 }

/*------------------------------------*/

recep *makr(photrec *epnt, int ctype)

/* make a dummy receptor and link it to the list. */

{
    recep *rpnt;
 
  if ((rpnt=(recep *)emalloc(sizeof(recep))) == (recep*)NULL) {
     fprintf (stderr,"no space left for receptor %d\n",cumrec+1);
      return ((recep*)NULL);  
  }
  rpnt->next = (recep*)NULL;
  if (!recpnt) recpnt = rpnt;	  	/* save head if first recep */
  if (recend)
    recend->next = rpnt;
  recend = rpnt;

  rpnt->ctype = ctype;			/* set channel to dummy type */
  rpnt->xloc  = epnt->xpos;		/* x loc of receptor */
  rpnt->yloc  = epnt->ypos;		/* y loc of receptor */
  rpnt->recnm1 = epnt->node1a;		/* receptor number from node */
  rpnt->recnm2 = epnt->node1b;
  rpnt->recnm3 = epnt->node1c;
  rpnt->recnm4 = epnt->node1d;
  cumrec++;	 			/* increment total receptors */
  return (rpnt); 
}

/*---------------------------------------------------*/

recep *makrecep(photrec *epnt, comp *cpnt)
               
/* make a new photoreceptor and link it to the list. */

{
    int pigm,filt,pnois, tspeed;
    unsigned int nrseed;
    float dnois;
    recep *rpnt;
    recpar *cpt;
    double maxcond, dia, pathl, attf;
    double recslow, timec, timefactor;
 
  if ((rpnt=(recep *)emalloc(sizeof(recep))) == (recep*)NULL) {
     fprintf (stderr,"no space left for receptor %d\n",cumrec+1);
      return ((recep*)NULL);  
  }
  rpnt->next = (recep*)NULL;
  if (!recpnt) recpnt = rpnt;  	/* save head if first recep */
  if (recend)
    recend->next = rpnt;
  recend = rpnt;

  cumrec++;	 			/* increment total receptors */
  rpnt->comp1 = cpnt; 			/* no presynaptic cell */
  maklst(&cpnt->clst,(conn*)rpnt);	/* compartment assoc with recep */
  rpnt->ctype = epnt->ctype;		/* set channel type: ROD or CONE */
  rpnt->xloc  = epnt->xpos;		/* x loc of receptor */
  rpnt->yloc  = epnt->ypos;		/* y loc of receptor */

  if (epnt->ctype == ROD) {

    if ((maxcond=epnt->maxcond)==NULLVAL) maxcond=dmaxrod;/* max conductance*/
    if ((dia=epnt->dia)==NULLVAL) dia = 1.674; /* 2.2 um2 area */
    epnt->dia = dia;
    rpnt->area = dia * dia * MPI / 4;	/* photon collecting area of rod */
    rpnt->maxcond = maxcond;		/* maximum conductance of channel */
    rpnt->chtyp  = &rectypes[0];	/* rod constants */
    pigm = 0;				/* rod pigment type */
    if ((attf=epnt->attf)==NULLVAL) attf = .9;   /* misc attenuation factor */
   }

 else {				/* cone */

    if ((maxcond=epnt->maxcond)==NULLVAL) maxcond=dmaxcon; /* max conductance */
    if ((dia=epnt->dia)==NULLVAL) dia = 1.674; /* 2.2 um2 area */
    epnt->dia = dia;
    rpnt->area = dia * dia * MPI / 4;	/* photon collecting area of cone */
    rpnt->maxcond = maxcond;		/* maximum conductance of channel */
    if ((pigm=(int)(epnt->pigm))==NULLVAL) pigm = 1; /* cone default pigment */
    rpnt->chtyp  = &rectypes[pigm];	/* cone constants */
    if ((attf=epnt->attf)==NULLVAL) attf = .9;   /* misc attenuation factor */
   }

  cpt = rpnt->chtyp;			/* pointer to receptor constants */
  restorec(rpnt);			/* set receptor state starting vals */
  if ((pathl=epnt->pathl)==NULLVAL) pathl=cpt->pathl; /*default path length*/
  rpnt->pathl = pathl;
  if ((filt=epnt->filt)==NULLVAL) filt = 0; 	/* filter over pigment */
  rpnt->filt = filt;
  rpnt->iflux = 0.0;
  if ((timec=epnt->timec1)==NULLVAL) timec = 1.0; /* time speed factor */
  if (timec > 10.0) timec = 10.0;
  if (timec <= 0.0) timec = 0.001;
  recslow = cpt->recslow;
  if (timec > recslow) {
    tspeed = 1;				/* must have at least 1 iteration */
    timefactor = recslow / timec;	/* but can slow it down a lot */
  } 
  else {
    tspeed = (int) (recslow / timec);		/* number of time steps */
    timefactor = recslow / (tspeed * timec);
  }
  rpnt->tspeed = tspeed; 	/* number of time steps to run photrec */
  rpnt->intenadj = 1.0 / (tspeed * timefactor); /* norm intensity */
  
  if (timefactor != 1.0) {
    saverec(rpnt);		/* if timec is nonstandard, save chtyp */
    setreceptim(rpnt,timefactor);
  }

  if ((dnois=epnt->darknoise)==NULLVAL) dnois = 0;	/* dark noise */
  if (dnois) {
     rpnt->dnois = dnois;
     rpnt->dstate = (char*)NULL;
     if ((nrseed=epnt->prseed) == 0) 	/* use nodenum if rseed not set */
	nrseed = noduniq(epnt->node1a, epnt->node1b,
			 epnt->node1c, epnt->node1d, prseed^1111);
     if (nrseed > 0) {			/* use default random if neg */
        rpnt->dstate  = makrand (RNDSIZ,"dark",cumrec);
        if (rpnt->dstate) rinitstate (nrseed,rpnt->dstate,RNDSIZ);
     }
  }
  if ((pnois=epnt->photnoise)==NULLVAL) pnois = 0;	/* photon noise */
  if (pnois) {
     rpnt->pnois = pnois;
     rpnt->pstate = (char*)NULL;
     if ((nrseed=epnt->prseed) >= 0) {		/* use default random if neg */
#define IMASK 0x7fffffff
        if (nrseed==0) /* use nodenum if rseed not set */
	  nrseed = noduniq(epnt->node1a, epnt->node1b,
			   epnt->node1c, epnt->node1d, prseed^2222);
        rpnt->pstate = makrand (RNDSIZ,"phot",cumrec);
        if(rpnt->pstate)
            rinitstate(((nrseed&IMASK)+1098667)&IMASK,rpnt->pstate,RNDSIZ);
#undef IMASK
      }
   
  }
  rpnt->attf = attf;
  rpnt->recnm1 = epnt->node1a;		/* receptor number from node */
  rpnt->recnm2 = epnt->node1b;
  rpnt->recnm3 = epnt->node1c;
  rpnt->recnm4 = epnt->node1d;
  return (rpnt); 
}

/*------------------------------------*/

void saverec(recep *rpnt)
                

/* Save a photoreceptor's kinetic states for a later restore */

{
    recpar *rp,*cpt;
    char *emalloc(unsigned int n);

  cpt = rpnt->chtyp;			/* pointer to receptor constants */
  if (cpt== &rectypes[cpt->pigm]) { /* if orig estimates, make space */

    if ((rp=(recpar *)emalloc(sizeof(recpar))) == (recpar *)NULL) {
       fprintf (stderr,"no space left for receptor save\n");
       return;  
    }
    *rp = *cpt;			/* copy the old constants */
  }
  else rp = cpt;

  rp->rhod  = rpnt->rhod;                /* set the new values */
  rp->rhod2 = rpnt->rhod2;
  rp->rstar = rpnt->rstar;
  rp->gpr  = rpnt->gpr;
  rp->pde  = rpnt->pde;
  rp->cycg = rpnt->cycg;
  rp->cond = rpnt->cond;
  rp->ca   = rpnt->ca;
  rp->cax  = rpnt->cax;
  rpnt->chtyp = rp;			/* pointer to receptor constants */
}

/*------------------------------------*/

void restorec(recep *rpnt)
                 

/* restore photoreceptor kinetic states */

{
    recpar *cpt;

  cpt = rpnt->chtyp;			/* pointer to receptor constants */
  rpnt->rhod  = cpt->rhod;              /* initialize receptor constants */
  rpnt->rhod2 = cpt->rhod2;
  rpnt->rstar = cpt->rstar;
  rpnt->gpr  = cpt->gpr;
  rpnt->pde  = cpt->pde;
  rpnt->cycg = cpt->cycg;
  rpnt->cond = cpt->cond;
  rpnt->ca   = cpt->ca;
  rpnt->cax  = cpt->cax;
}

/*------------------------------------*/

void setreceptim(recep *rpnt, double timec)
                 

/* set time constant of photoreceptor kinetic states */

{
    recpar *cpt;

  cpt = rpnt->chtyp;			/* pointer to receptor constants */
  if (cpt== &rectypes[cpt->pigm]) { /* if orig estimates, make space */
     fprintf (stderr,"Error, can't change original receptor constants\n");
     return;
  }

   cpt->lgain    *= timec;
   cpt->rgain1   *= timec;
   cpt->rgain2   *= timec;
   cpt->rgain3   *= timec;
   cpt->rkgain   *= timec;
   cpt->ggain    *= timec; 
   cpt->gcygain  *= timec;
   cpt->pdegain  *= timec;
   cpt->condgain *= timec;
   cpt->gca      *= timec;
   cpt->cxgain   *= timec;

   cpt->decrstar = 1.0 - ((1.0 - cpt->decrstar) * timec);
   cpt->pdedec   = 1.0 - ((1.0 - cpt->pdedec) * timec);
   cpt->deccond  = 1.0 - ((1.0 - cpt->deccond) * timec);
   cpt->capump  *=    timec; 
   /* cpt->capump =  1.0 - ((1.0 - cpt->capump) * timec);   ??  new */
   cpt->deccax   = 1.0 - ((1.0 - cpt->deccax) * timec);
}

/*------------------------------------*/

recep *modrecep(photrec *pepnt)
               
/* modify a photoreceptor */

{
    recep *rpnt;
    recpar *cpt;
    double maxcond, dia, attf, pathl;
    int filt, pnois, tspeed;
    float dnois;
    elem *epnt;
    double recslow, timec, timefactor;

    if (!(epnt=findelem(pepnt->modif))) {
      fprintf (stderr,"modrec: can't find element %d\n",pepnt->modif);
      return ((recep*)NULL);
    }
 
   rpnt = (recep *)epnt->lptr;	/* get pointer to recept from elem */
   if (rpnt==(recep*)NULL) return ((recep*)NULL); 
   if (rpnt->ctype != ROD && rpnt->ctype != CONE) {
    fprintf (stderr,"modrec: element %d is not a photoreceptor\n",pepnt->modif);
       return ((recep*)NULL);  
   }
					/* don't want to change type or loc: */

/*  rpnt->ctype = pepnt->ctype;		/* set channel type: ROD or CONE */
/*  rpnt->xloc  = pepnt->xpos;		/* x loc of receptor */
/*  rpnt->yloc  = pepnt->ypos;		/* y loc of receptor */

  if ((maxcond=pepnt->maxcond)!=NULLVAL)	/* maximum conductance */
     rpnt->maxcond = maxcond;	/* maximum conductance of channel */
  if ((dia=pepnt->dia)!=NULLVAL) 	
     rpnt->area = dia * dia * MPI / 4; /* photon collecting area of rod */
  if ((attf=pepnt->attf)!=NULLVAL)   	/* misc attenuation factor */
     rpnt->attf = attf;
  if ((pathl=pepnt->pathl)!=NULLVAL) 	/* path length */
      rpnt->pathl = pathl;
  if ((filt=pepnt->filt)!=NULLVAL)	/* filter over pigment */
      rpnt->filt = filt;
  if ((dnois=pepnt->darknoise)!=NULLVAL)/* photon noise */
      rpnt->dnois = dnois;
/*  if ((pnois=pepnt->photnoise)!=NULLVAL)/* photon noise */
/*      rpnt->pnois = pnois;   */       /* never modify photnoise */

  if ((timec=pepnt->timec1)!=NULLVAL) {  /* time speed factor */
    if (timec > 10.0) timec = 10.0;
    if (timec <= 0.0) timec = 0.001;
    cpt = rpnt->chtyp;
    recslow = cpt->recslow;
    if (timec > recslow) {
      tspeed = 1;			/* must have at least 1 iteration */
      timefactor = recslow / timec;	/* but can slow it down a lot */
    } 
    else {
      tspeed = (int) (recslow / timec);		/* number of time steps */
      timefactor = recslow / (tspeed * timec);
    }
    rpnt->tspeed = tspeed; 	/* number of time steps to run photrec */
    rpnt->intenadj = 1.0 / (tspeed * timefactor); /* norm intensity */
  
    if (timefactor != 1.0) {
      saverec(rpnt);		/* if timec is nonstandard, save chtyp */
      setreceptim(rpnt,timefactor);
    }
  }

  if (pepnt->save) {			/* save equilibrium kinetic states */
     saverec(rpnt);
  }
  else 
    if (pepnt->restore) {		/* restore kinetic states */
     restorec(rpnt);
  }

  return (rpnt);
}

/*------------------------------------*/

void delrecep (recep *rpnt)

{
     recep *trpnt;
     recpar *cpt;

  trpnt = rpnt;

  cpt = rpnt->chtyp;			/* pointer to receptor constants */
  if (cpt!= &rectypes[cpt->pigm]) {     /* if saved, delete new chtyp */
    efree (cpt);
  }
  if (rpnt->dstate) efree (rpnt->dstate);
  if (rpnt->pstate) efree (rpnt->pstate);
  rpnt = (recep *)rpnt->next;
  efree (trpnt);
  cumrec--;
}

/*------------------------------------*/

conn *makconn(comp *comp1, comp *comp2, double s, int type)

/* make a new connection betw compartments and link
    it to both compartments' conn lists. */
/* connect comp1 to comp2 with conductance s; */

{
    conn *cpnt;

#ifdef DEBUG
 if (debug & 1 && debugz & 16)
   fprintf (stderr,"makconn: comp %d comp %d\n",comp1->num, comp2->num);
#endif

    if ((cpnt=(conn *)emalloc(sizeof(conn))) == (conn*)NULL) {
      fprintf (stderr,"no space left for conn %d\n",cumconn+1);
      return ((conn*)NULL);  
    }
					/* add gj or op amp to synapse list */
					/*  so it can be made time-varying */ 
/*    spnt->next = (conn*)NULL;
    if (!synpnt) synpnt = cpnt;  	/* save head if first synap */
/*    if (synend)
      synend->next = cpnt;
    synend = cpnt;
*/
    cpnt->next = (conn*)NULL;			/* zero list pointer */
    if (!connpnt) connpnt = cpnt;	/* increment connection list */
    else if (connend) connend->next = cpnt;
    cpnt->last = connend;		/* pointer to last connection */
    connend = cpnt;

    maklst(&comp1->clst, cpnt);
    maklst(&comp2->clst, cpnt);
    cpnt->comp1 = comp1; 		/* set pointers to comps */
    cpnt->comp2 = comp2; 
    cpnt->num1 = comp1->num;		/* set comp numbers */
    cpnt->num2 = comp2->num;		/*  for parallel implementation */
    cpnt->conduct = s;			/* conductance of connection */
    cpnt->ctype = type;
    cumconn++; 				/* increment total */
    return (cpnt); 
}

/*------------------------------------*/

conn *modconn(elem *cepnt, double s)
               
/* modify a connection betw compartments */

{
    conn *cpnt;
    elem *epnt;

#ifdef DEBUG
 if (debug & 1 && debugz & 16)
   fprintf (stderr,"modconn: elem %d\n",cepnt->elnum);
#endif

  if (!(epnt=findelem(cepnt->modif))) {
      fprintf (stderr,"modconn: can't find element %d\n",cepnt->modif);
      execerror ("Missing element: "," stopping... ");
      return ((conn*)NULL);
  }

  cpnt = (conn *)epnt->lptr;	/* get pointer to recept from elem */
  if (cpnt==(conn*)NULL) return ((conn*)NULL); 
  /* if (cpnt->ctype != GJ && cpnt->ctype != RESISTOR) {
     fprintf (stderr,"modconn: element %d is not a gj %d\n",cepnt->modif);
     return ((conn*)NULL);  
  } */

    cpnt->conduct = s;			/* new conductance of connection */
    return (cpnt); 
}

/*------------------------------------*/

double getmesng(gj *gjpnt, int mesng)

/* get cA/GMP concentration from remote node (compartment)
   if one is defined, or from neighboring compartments to
   element (normally a gap junction). */

{
   double val;
   node *npnt;

  if (!gjpnt) return 0;
  val = 0;
  if (npnt=gjpnt->nodec) {	/* modulation is from defined node */
        val = getnt(npnt->comptr,mesng);
  } /* if nodec */

  else {                /* modulation is from neighboring compartments */
    val =  getnt(gjpnt->comp1,mesng);
    val = (getnt(gjpnt->comp2,mesng) + val) / 2.0;
  }
  return val;
}

/*------------------------------------*/

gj *makgj(gapjunc *gepnt, comp *comp1, comp *comp2, int type)

/* Make a new gj betw compartments and link
    it to both compartments' conn lists. */
/* Connect comp1 to comp2 with conductance s; */

{
    gj *gjpnt;
    node *npnt;
    double  gmax, gnv, gjrate, n, tau, vg, vj, voff;
    double alpha, beta, specres, area, cyc;

#ifdef DEBUG
 if (debug & 1 && debugz & 16)
   fprintf (stderr,"makgj: comp %d comp %d\n",comp1->num, comp2->num);
#endif

    if (type != GJ) {
      fprintf (stderr,"makgj: incorrect type %d\n",type);
      return ((gj*)NULL);
    }
    if ((gjpnt=(gj *)emalloc(sizeof(gj))) == (gj*)NULL) {
      fprintf (stderr,"no space left for gj %d\n",cumconn+1);
      return ((gj*)NULL);  
    }
    gjpnt->next = (gj*)NULL;		/* zero list pointer */
    linksyn (gjpnt);			/* add to list of synapses */

    maklst(&comp1->clst, gjpnt);	/* set the list pointers */
    maklst(&comp2->clst, gjpnt);
    gjpnt->comp1 = comp1; 		/* set pointers to comps */
    gjpnt->comp2 = comp2; 
    gjpnt->num1 = comp1->num;		/* set comp numbers */
    gjpnt->num2 = comp2->num;		/*  for parallel implementation */
    gjpnt->ctype = type;
    cumgj++; 				/* increment total */

			/* Set gj conductance. */
			/* The idea is to use what info is given, */
			/* letting "gmax" override "area", */
			/* and "specres" override "drg". */
 
    if ((gmax=gepnt->gmax)==NULLVAL) { 
      if ((specres=gepnt->specres) == NULLVAL) specres = drg;
      if (specres <= 0.0) {
	 fprintf (stderr,"makgj: specres is zero, using default 5e6\n"); 
	 specres = 5e6;
      } 
      if ((area=gepnt->area) == NULLVAL) area = 0.0004;  /* um2 */
      if (area <= 0.0) {
	 fprintf (stderr,"makgj: area is zero, using default 0.04\n"); 
	 area = 0.0004;
      } 
      gmax = area / specres;          /* conductance = gj area / spec res */
    }
    if (gmax < 0.0) gmax = 100e-12;  /* absolute default */

    if ((vg=gepnt->vgain)==NULLVAL) vg = 1.0;
    gjpnt->vgain = vg; 			/* voltage gain */
    if ((voff=gepnt->voff)==NULLVAL) voff = dgjoff;
    gjpnt->voff = voff; 		/* voltage offset */
    if ((tau=gepnt->taun)==NULLVAL) tau = 1.0;
    gjpnt->taun = tau;			/* extra rate factor */

	/* reverse parameters */

    if ((vg=gepnt->rvgain)!=NULLVAL) gjpnt->rev=1;
    else vg = gjpnt->vgain;
    if ((voff=gepnt->rvoff)!=NULLVAL) gjpnt->rev=1;
    else voff = gjpnt->voff;
    if ((tau=gepnt->rtaun)!=NULLVAL) gjpnt->rev=1;
    else tau = gjpnt->taun;  
    gjpnt->rvgain = vg; 	
    gjpnt->rvoff = voff; 	
    gjpnt->rtaun = tau;			

    if ((gnv=gepnt->gnv)==NULLVAL) gnv = dgjnv;
    if (gnv < 0) gnv = 0;
    if (gnv > 1) gnv = 1;
    gjpnt->maxcond = gmax; 		/* max conductance, never varies */
    gjpnt->gnv = gnv;			/* conductance not modulated */


   cyc = 0;
   gjpnt->modtyp=gepnt->modtyp;
   if (gjpnt->modtyp != 0) {   		/* if cycA/G modulation */
     gjpnt->sign = gepnt->sign;		/* sign of modulation */
     if (gepnt->nodeca != NULLVAL) {
       if (!(npnt=findnode(gepnt->nodeca,gepnt->nodecb,
			   gepnt->nodecc,gepnt->nodecd,"makgj"))){
		execerror ("makgj: can't find mod node for gj","Stopping...");
	}
	gjpnt->nodec=npnt;	/* save node pointer for future ref */
     }
     cyc = getmesng (gjpnt, gepnt->modtyp);
     if (gjpnt->sign) cyc = 1.0 - cyc;
   }
					/* calculate equilib conductance */
    vj = gjpnt->comp1->v - gjpnt->comp2->v;
    alpha = gjalpha(vj, gjpnt);
    beta =  gjbeta (vj, gjpnt);
    n = alpha / (alpha + beta);
    gjpnt->n = n;
    gjpnt->conduct = ( gnv + n * (1 - gnv) ) * gmax * (1.0-cyc);
    return (gjpnt); 
}

/*------------------------------------*/

gj *modgj(gapjunc *gepnt)
               
/* Modify a gap junction. */
/* Currently only maxcond can be modified. */

{
    gj *gjpnt;
    elem *epnt;
    gapjunc *gopnt;
    node *npnt;
    double gmax, n, vg, vj, tau, alpha, beta;
    double area, specres, gnv, cyc, gjrate;
    int recalc;

#ifdef DEBUG
 if (debug & 1 && debugz & 16)
   fprintf (stderr,"modconn: elem %d\n",gepnt->elnum);
#endif

  if (!(epnt=findelem(gepnt->modif))) {
      fprintf (stderr,"modgj: can't find element %d\n",gepnt->modif);
      execerror ("Missing element: "," stopping... ");
      return ((gj*)NULL);
  }

  gjpnt = (gj *)epnt->lptr;	/* get pointer to recept from elem */
  if (gjpnt==NULL) return ((gj*)NULL); 
  /* if (gjpnt->ctype != GJ) {
     fprintf (stderr,"modgj: element %d is not a gj %d\n",gepnt->modif);
     return ((gj*)NULL);  
  } */


    if ((gmax=gepnt->gmax)!=NULLVAL) {
	recalc = 1;
    }
    else {
      gopnt = (gapjunc *)epnt;	/* pointer to orig gap junction spec */

      recalc = 0;		/* first, find if new numbers */
      if ((specres = gepnt->specres) != NULLVAL) recalc = 1;
      if ((area=gepnt->area) != NULLVAL) recalc = 1;
      if (recalc) {		/* if so, find correct vals */
        if ((specres = gepnt->specres) == NULLVAL) {
          if ((specres = gopnt->specres) == NULLVAL) {  /* use orig */
            specres = drg;
          }
	}
        if (specres <= 0.0) {		/* check specres for legal value */
	   execerror ("modgj: specific res is zero.","stopping...\n");
	   return ((gj*)NULL);
        } 
        if ((area=gepnt->area) == NULLVAL) {		/* um2 */
           if ((area=gopnt->area) == NULLVAL) {
		area = 0.0004;		/* small default value */
	   }
	}
        if (area <= 0.0) {		/* check area for legal value */
	   execerror ("modgj: area is zero.","stopping..."); 
	   return ((gj*)NULL);
        } 
      }  /* if recalc */
      gmax = area / specres; 
    }
    if (gmax < 0.0) gmax = 100e-12;	/* absolute default */
    if (recalc) gjpnt->maxcond = gmax;
    else        gmax = gjpnt->maxcond;

    if (gepnt->vgain!=NULLVAL) gjpnt->vgain  = gepnt->vgain; 
    if (gepnt->voff!=NULLVAL) gjpnt->voff    = gepnt->voff; 
    if (gepnt->taun !=NULLVAL) gjpnt->taun   = gepnt->taun; 
    if (gepnt->rvgain!=NULLVAL) gjpnt->rvgain = gepnt->rvgain; 
    if (gepnt->rvoff!=NULLVAL) gjpnt->rvoff   = gepnt->rvoff; 
    if (gepnt->rtaun !=NULLVAL) gjpnt->rtaun  = gepnt->rtaun; 
    if (gepnt->gnv  !=NULLVAL) gjpnt->gnv   = gepnt->gnv;
    if (gepnt->modtyp) gjpnt->modtyp = gepnt->modtyp;   

    if ((tau=gepnt->taun)==NULLVAL) gjrate = 1.0;
    else {
         if (tau < 0) tau = 1;
         if (tau == 0) gjrate = 0;
         else gjrate = dgjtau / tau;
    }
    gjpnt->taun = gjrate;		/* extra rate factor */

    if ((tau=gepnt->rtaun)==NULLVAL) gjrate = 1.0;
    else {
         if (tau < 0) tau = 1;
         if (tau == 0) gjrate = 0;
         else gjrate = dgjtau / tau;
    }
    gjpnt->rtaun = gjrate;		/* extra rate factor */

    cyc = 0;
    if (gjpnt->modtyp != 0) {   /* if cycA/G modulation */
      if (gepnt->nodeca != NULLVAL) {
        if (!(npnt=findnode(gepnt->nodeca,gepnt->nodecb,
			    gepnt->nodecc,gepnt->nodecd,"makgj"))){
 		execerror ("modgj: can't find mod node for gj","Stopping...");
 	}
	gjpnt->nodec=npnt;	/* save node pointer for future ref */
      }
      cyc = getmesng (gjpnt, gepnt->modtyp);
      if (gjpnt->sign) cyc = 1.0 - cyc;
    }
					/* calculate equilib conductance */
    vj = gjpnt->comp1->v - gjpnt->comp2->v;
    alpha = gjalpha(vj, gjpnt);
    beta =  gjbeta (vj, gjpnt);
    n = alpha / (alpha + beta);
    gjpnt->n = n;
    gnv = gjpnt->gnv;			/* conductance not modulated */
    gjpnt->conduct = ( gnv + n * (1 - gnv) ) * gmax * (1.0-cyc);
    return (gjpnt); 
}

/*------------------------------------*/

dbuf *makdelbuf(comp *comp1, comp *comp2, double delay, int type)
                       
/* make a delayed buffer connection betw compartments and link
    it to both compartments' conn lists. */
/* connect comp1 to comp2 with conductance s; */

{
    dbuf *cpnt;
    int i,idelay,v;
    short int *dpnt;
    double vf;

#ifdef DEBUG
 if (debug & 1 && debugz & 16)
   fprintf (stderr,"makdelbuf: comp %d comp %d\n",comp1->num, comp2->num);
#endif

    if ((cpnt=(dbuf *)emalloc(sizeof(dbuf))) == (dbuf*)NULL) {
      fprintf (stderr,"no space left for conn %d\n",cumconn+1);
      return ((dbuf*)NULL);  
    }
#define MAXDELAY 10000
					/* delay in msec, */
   idelay = (int)(delay * 10);		/*  time steps of 100 usec */
   if (idelay < 0) {
      idelay = 0;
      fprintf (stderr,"Error: negative delay in buffer, setting to zero.\n");
   }   
   else if (idelay > 10000) {
      fprintf (stderr,"Warning: possible error; excessive delay %d in buffer\n", 
				idelay);
      if (idelay > MAXDELAY) idelay = MAXDELAY;
   }

   if (idelay)  {
     if ((dpnt=(short int *)emalloc(idelay * sizeof(short int))) == 
					(short int*)NULL) {
       fprintf (stderr,"no space left for delay buffer %d\n",cumconn+1);
       return ((dbuf*)NULL);  
     }
   }
   else dpnt = (short int*)NULL;
					/* add delay buffer to synapse list */
					/*  so it can be made time-varying */ 
    cpnt->next = (dbuf*)NULL;
    if (!synpnt) synpnt = (synap *)cpnt; /* save head if first synap */
    if (synend)
      synend->next = (synap *)cpnt;
    synend = (synap *)cpnt;

    maklst(&comp1->clst, (conn*)cpnt);
    maklst(&comp2->clst, (conn*)cpnt);
    cpnt->comp1 = comp1; 
    cpnt->comp2 = comp2; 
    cpnt->conduct = 0;			/* conductance of connection */
    cpnt->delay = idelay;		/* length of delay buffer (time) */
    cpnt->ctype = type;
    cpnt->delbuf = dpnt;		/* pointer to circular buffer */
    cpnt->delpnt = dpnt;		/* input/output pointer */

    vf = comp1->v / DELCONV;		/* convert voltage */
    if (vf > 0) 
         v = (int)(vf+0.5); 		/* input voltage */
    else
         v = (int)(vf-0.5); 		/* input voltage */
    if (v>MAXSHORT) v = MAXSHORT;	/* no overflow of short int */
    else if (v<MINSHORT) v = MINSHORT;
    if (dpnt)
      for (i=0; i<idelay; i++) 
        *dpnt++ = v;			/* set buffer to constant voltage */
    cumconn++; 				/* increment total */
    return (cpnt); 
}

/*------------------------------------*/

chan *alloc_chan(short ctype, short stype)

/* Allocate space for channels according to their type */

{
   int chansiz,hh,ns;
   chan *chpnt;
   chantype *chtyp;
   stconc *conc;
   static int num=0;

  if (stype < 0) stype = 0;
  if (stype > NCHANCONST) stype = NCHANCONST;

  chpnt = (chan *)NULL;

  chtyp = getchantype(ctype,stype);
  ns = chtyp->numstate;
  hh = chtyp->hh;
  switch (ctype) {
    case NA:
 	 if (hh) chansiz = sizeof(hhchan);
         else    chansiz = sizeof(chan);
       break;	/* case NA */
 
    case K:
       switch (stype) {

	case 0:					/* HH */
	case 2:					/* KA */
	 chansiz = sizeof(hhchan);
         break;

	case 1:					/* Markov */
	case 3:					/* Markov */
	case 4:					/* Markov */
         chansiz = sizeof(chan);
         break;

	default:
 	  if (hh) chansiz = sizeof(hhchan);
          else    chansiz = sizeof(chan);
         break;

       }  /* switch (stype) */
       break;

    case KCa:
       switch (stype) {

	case 6:					/* bKCa, Markov */
	case 5:					/* sKCa, Markov */
	case 4:					/* sKCa, Markov */
	case 3:					/* bKCa, Markov */
	case 2:					/* bKCa */
	case 1:					/* sKCa, Markov */
	case 0:					/* sKCa */
       default:
         chansiz = sizeof(kcachan);
         break;

       }  /* switch (stype) */
       break;

    case CA:
       switch (stype) {

	case 0:
	   chansiz = sizeof(hhchan);	/* simple H-H */
	  break;
        default:			/* channel defined by sequen states */
 	  if (hh) chansiz = sizeof(hhchan);
          else    chansiz = sizeof(chan);
	  break;
        }  /* switch (stype) */
       break;

    case AMPA:
    case CGMP:
    case NMDA:
    case GABA:
    case SYN2:
         chansiz = sizeof(ntchan);
        break;

    }  /* switch (ctype) */

  if ((chpnt=(chan *)emalloc(chansiz)) == (chan*)NULL) {
      fprintf (stderr,"no space left for chan %d\n",cumchan+1);
      return ((chan*)NULL);  
  }

  chpnt->numstate = ns;
  if (ns) {
     if ((conc=(stconc *)emalloc(ns*sizeof(stconc))) == (stconc*)NULL) {
      fprintf (stderr,"no space left for %d stconc in chan %d\n",ns,cumchan+1);
      return ((chan*)NULL);  
    } 
   chpnt->conc = conc; 
  }
  chpnt->num = cumchan;
  return chpnt;
}

/*------------------------------------*/

void linkchan (chan *chpnt)

/* link a channel into chan list */

{
  chpnt->next = (chan*)NULL;
  if (!chanpnt) {
    chpnt->last = (chan*)NULL;
    chanpnt = chpnt;  	/* save head if first chan */
  }
  if (chanend) {
    chanend->next = chpnt;
    chpnt->last = chanend;
  }
  chanend = (chan*)chpnt;
}

/*------------------------------------*/

int chaniss (int ctype, int stype)

/* return a 1 if channel is sschan */

{
    int typ;
    chantype *chtyp;

  chtyp = getchantype(ctype,stype);
  if (chtyp->numstate==0) typ = 0;
  else                    typ = 1;
  return typ;
}

/*------------------------------------*/

void initsschan(chan *chpnt, comp *comp1, comp *comp2, chantype *chtyp,
		double maxcond, double vrev, 
		double arate, double brate, double crate, double drate)

/* initialize a sequential-state channel's states */

{
    int i,j, numtr, nstate;
    chanstate *stpnt;
    char *trans;
    char *optrans;

  chpnt->comp1 = comp1;			/* set loc of voltage sens. */
  chpnt->comp2 = comp2;
  nstate = chpnt->numstate;  
  if (nstate > 0) {
    for (i=0; i<nstate; i++) {		/* set initial concs */
       chpnt->conc[i].cval = 0.0;
       chpnt->conc[i].cest = 0.0;
       chpnt->conc[i].nchan = 0;
    }
    chpnt->conc[0].cest = 1.0;
  }
  chpnt->maxcond  = maxcond;
  chpnt->vrev = vrev;
  chpnt->arate = arate;
  chpnt->brate = brate;
  chpnt->crate = crate;
  chpnt->drate = drate;
  dochani(chpnt,1e-8);
}

/*------------------------------------*/

void setsschan(chan *chpnt, comp *comp1, comp *comp2, chantype *chtyp,
		double maxcond, double vrev, 
		double arate, double brate, double crate, double drate)

/* initialize a sequential-state channel's states */

{
    int i,j, numtr, nstate;
    chanstate *stpnt;
    char *trans;
    char *optrans;

  chpnt->comp1 = comp1;			/* set loc of voltage sens. */
  chpnt->comp2 = comp2;
  nstate = chpnt->numstate;  
  chpnt->maxcond  = maxcond;
  chpnt->vrev = vrev;
  chpnt->arate = arate;
  chpnt->brate = brate;
  chpnt->crate = crate;
  chpnt->drate = drate;
  dochani(chpnt,1e-8);
}

/*------------------------------------*/

void setnchan(chan *chpnt)

/* Set the nchan (number of channels) in a chan to be integer. */
/* Use after all chans in a compartment have been condensed.   */

/* The nchan param of a channel is floating point to allow */
/*  several channels to be added together during the condensation */
/*  process.  This procedure enforces an integral number and */
/*  modifies "maxcond" so its value is consistent with "nchan". */

{
     double unitary, nchan;

   if (!chpnt) return;
   switch (chpnt->ctype) {

         case CA:
         case  K:
        case KCa:
         case NA:
       case GABA:
       case SYN2:
       case NMDA:
       case CGMP: break;

         case GJ:
        case CAP:
        case ROD:
       case CONE:
       case TRANSDUCER:
       case BATT:
    case SYNAPSE:
   case AXIALRES:
   case RESISTOR:
       case LOAD:
         default: return;    /* ignore non-channel connections */
		  break;
   }
   if (chpnt->nchan > 0) {
      unitary = chpnt->maxcond / chpnt->nchan;
      nchan = int(chpnt->nchan + 0.5);
      chpnt->nchan = nchan;  
      chpnt->maxcond = unitary * nchan;  
   }
}

/*------------------------------------*/

void maknois(chan *chpnt)

/* Calculate the population of an sschan's states when it is first defined.  */

{
        int i,nstate, nchano, totchan;
        chanstate *stpnt;
        stconc *conc;

   if (chpnt->nchan > 0) {		/* set up noisy states */
      nstate = chpnt->numstate;
      conc = chpnt->conc;
      stpnt = chpnt->chtyp->state;
      nchano = totchan = 0;
      for (i=0; i<nstate; i++,stpnt++,conc++) {            /* Total */
         conc->nchan = int(conc->cest * chpnt->nchan + 0.5);
         totchan += conc->nchan;
         nchano += int(conc->nchan*stpnt->cond+0.5);
      }
      chpnt->conc[0].nchan += int(chpnt->nchan-totchan); /* Make correct tot */
      if (nchano > chpnt->nchan) nchano = int(chpnt->nchan);
      chpnt->cno = nchano;
      /* conduct = ((double)nchano)/chpnt->nchan; */
   }
}

/*------------------------------------*/

void sethhinf (chan *chpnt, comp *comp1)

/* set HH m and h to equilibrium values */

{
   chanparm *chp;

  chp = &chpnt->chtyp->parm[0];
  ((hhchan *)chpnt)->m  = calcchaninf(comp1->v-chpnt->voffsm,chp);
  if (chpnt->chtyp->numparm > 1) {
    chp++;
    ((hhchan *)chpnt)->h = calcchaninf(comp1->v-chpnt->voffsh,chp);
  }
}

/*------------------------------------*/

chan *makchan(attrib *apnt, comp *comp1, comp *comp2, int saved)
                 
/* make a new channel at a compartment and link
   it to the compartment's conn list. */

{
    int i,ns,stype,stop,ctype;
    chan *chpnt;
    chan *addchan(comp *pnt, chan *chpnt);
    ntchan *ntpnt;
    chanparm *chp;
    chantype *chtyp;
    attrib *npnt;
    mattrib *chapnt;
    chattrib *chapntk;
    double chn,chunitary;
    double voffm,voffh,vrev,maxcond;
    double arate,brate,crate,drate;
    double dtaua,dtaub,dtauc,dtaud,taua,taub,tauc,taud;
    double doffsm, doffsh, dvrev, caperm;
    double alph,bet,k1,k2,d1,d2,ca,mg;

#ifdef DEBUG
 if (debug & 1 && debugz & 16)
   fprintf (stderr,"makchan: type %d %d\n",apnt->ctype,apnt->stype);
#endif
    			/* first, figure out how much space to allocate */

  ctype = apnt->ctype;
  if (apnt->stype == NULLVAL) stype = 0;	/* channel sub-type */
  else			      stype = apnt->stype;
  chtyp = getchantype(ctype,stype);

			/* next, figure out a value for maxcond */
			/* this may affect unitary cond or N */

  chapnt = (chattrib *)apnt;
  chunitary = chapnt->unitary;
  chn = chapnt->n;

  if ((maxcond=chapnt->maxcond)==NULLVAL) {  /* if maxcond not defined */

   if (comp2 && ((synap*)comp2)->maxcond != NULLVAL)
       maxcond = ((synap*)comp2)->maxcond;   /* define maxcond from synapse */
   else {

		/* first check if maxcond can be defined from chan N */

     if (chn!=NULLVAL) {
	   double n, unitary;

	n = chn;
        if (chunitary==NULLVAL) {   
           if ((unitary=chtyp->unitary)<=0) unitary = dscu;
	   else unitary *= qcond(chtyp); 
	}
	else unitary = chunitary;
        if (unitary <= 0) n = 0;
        maxcond = n * unitary;	/* maxcond computed from N before chnoise */
     }
    else {
		/* next check if maxcond can be defined from chnoise N */

    for (stop=0,npnt=apnt->attpnt; npnt && !stop; npnt=npnt->attpnt) { 
        nattrib *napnt;

      napnt = (nattrib *)npnt;
      switch (npnt->ctype) {
	  double n, unitary;

        case CHNOISE:
        if ((n=napnt->n)==NULLVAL) n = chn;	/* use prev.  defined N */
        if (n!=NULLVAL) {
           if ((unitary=napnt->unitary)==NULLVAL) {   
	      if (chunitary!=NULLVAL) unitary = chunitary; 
	      else {		/* use default value, chan type's first */
	        if ((unitary=chtyp->unitary)<=0) unitary = dscu;
	        else unitary *= qcond(chtyp); 
	      }
              if (unitary <= 0) n = 0;
	   }
           maxcond = n * unitary;
        }
	else chn = n;			/* save NULLVAL as evidence for below */
        break;

	case VESNOISE: break;

        default: stop=1; break;		/* stop on other channels */
      }
     }   /* for (npnt;;) */
   }  /* else try to define maxcond from chnoise */

   if (chn==NULLVAL) {        /* if maxcond still not defined, use default */
           conlst *nlpnt;
           node *nodpnt;

      switch (ctype) {
      case AMPA:
      case CGMP:
      case GABA:
      case SYN2:
      case NMDA: maxcond = dmaxsyn; break;
      case CA:   maxcond = dmaxca; break;
      case NA:   maxcond = dmaxna; break;
      case KCa:  maxcond = dmaxk; break;
      case K:    maxcond = dmaxk; break;
      } 
    if (nlpnt=comp1->nodlst) nodpnt=(node*)nlpnt->conpnt;
    
 printf ("# makchan: maxcond, N for 'chan %s type %d' at node %s not defined,\n",
		findsym(ctype),stype,prnode(nodpnt->nodenm1,nodpnt->nodenm2,
					    nodpnt->nodenm3,nodpnt->nodenm4));
   printf ("#   using default %g\n",maxcond);
    }
   }  /* ! (comp2 && ((synap*)comp2)->maxcond != NULLVAL) */

  }   /* if (maxcond==NULLVAL) */

  chpnt = alloc_chan(ctype,stype);
  linkchan(chpnt);
  cumchan++;

  chpnt->ctype = ctype;
  chpnt->stype = stype;
  chpnt->saved = saved;	/* save for addchan() below */

  dtaud = 0.2;		/* set default noise flicker rate */
  doffsm = 0.0; doffsh = 0.0;
  dvrev = chtyp->vrev;

  switch (ctype) {		/* find default params */
      case GABA:
		 if (dvrev==NULLVAL) dvrev = vcl;
      case SYN2: 
      case AMPA:
      case CGMP:
      case NMDA: dtaua = dtaub = dtauc = dsyntau;
		 if (dvrev==NULLVAL) dvrev = .66 * vna + .33 * vk;
		 break;
      case CA:   dtaua=dcataum; dtaub=dcatauh; dtauc= 1.0;
		 doffsm = dcaoffs; 
		 if (dvrev==NULLVAL) dvrev = vna;	/* set in makca() */
		 break;
      case NA:   dtaua=dnataum; dtaub=dnatauh; dtauc= 1.0; 
		 dtaud = dnataud;
		 doffsm = dnaoffsm; doffsh = dnaoffsh;
		 if (dvrev==NULLVAL) dvrev = vna;
		 break;

      case K:    doffsm = dkoffsn; doffsh = dkoffsh;
		 if (dvrev==NULLVAL) dvrev = vk;

	switch (stype) {
	 default:
	 case 0:
	 case 2:
		 dtaua=dktaum; dtaub=dktauh; dtauc= 1.0; 
		 dtaud = dktaud; 
		 break;
	 case 1:
	 case 3:
		dtaua=dktaum; dtaub=dktauh; dtauc= 1.0;
		dtaud = dktaud; 
		break;
	}
	break;

      case KCa:    doffsm = dkoffsn; doffsh = dkoffsh;

	switch (stype) {
	  case 0:			/* hh type */
	  case 1:			/* markov */
	  case 2:
	  case 3:
	  case 4:
	  case 5:
	  case 6:
         default:
		dtaua=dkcataum; dtaub=dkcatauh; dtauc= 1.0; 
		dtaud = 1.0;
		break;
	}
	break;
  } 

  if ((taua=chapnt->taua) == NULLVAL) taua = dtaua;
  else if (taua <=0)                  taua = 1e-6;
  arate = 1.0 / taua;
  if ((taub=chapnt->taub) == NULLVAL) taub = dtaub;
  else if (taub <=0)                  taub = 1e-6;
  brate = 1.0 / taub;
  if ((tauc=chapnt->tauc) == NULLVAL) tauc = dtauc;
  else if (tauc <=0)                  tauc = 1e-6;
  crate = 1.0 / tauc;
  if ((taud=chapnt->taud) == NULLVAL) taud = dtaud;
  else if (taud <=0)                  taud = 1e-6;
  drate = 1.0 / taud;


  if ((voffm=chapnt->voffsm) == NULLVAL) voffm = doffsm;
  if ((voffh=chapnt->voffsh) == NULLVAL) voffh = doffsh;

  if ((caperm=chapnt->caperm)== NULLVAL) caperm = chtyp->perm[PCA];
  if (caperm < 0) caperm = 0;
  if (caperm > 1) caperm = 1;
  chpnt->caperm = caperm;
  
  if ((vrev=chapnt->vrev) == NULLVAL) {
        double tp;

    tp = chtyp->perm[PNA] + chtyp->perm[PK] + chtyp->perm[PCL] + caperm;
    if (tp > 0) vrev = ghkv(chtyp,dcao,dcai);
    else        vrev = dvrev;
    chpnt->setvrev = 0;
  }
  else chpnt->setvrev = 1;		/* vrev set by user */ 

  chpnt->nao     = dnao;
  chpnt->ko	 = dko;
  chpnt->naratio = dnai/dnao;
  chpnt->kratio  = dki/dko;

  chpnt->chtyp  = chtyp;
  switch (ctype) {

    case NMDA:
       ntpnt = (ntchan*)chpnt;
       if ((mg=((cattrib*)chapnt)->mg) == NULLVAL) mg = dnmdamg; 
       ntpnt->mg = mg;

                /* continue on to below */
    case AMPA:
    case CGMP:
    case GABA:
    case SYN2:
       ntpnt = (ntchan*)chpnt;
       ntpnt->voffsm = voffm; 
       ntpnt->voffsh = voffh; 
       switch (stype) {
         default:
	   case 1:
             initsschan((chan *)ntpnt,comp1,comp2,chpnt->chtyp,maxcond,vrev,
			arate,brate,crate,drate);
	   break;
         }
        break;

    case NA:
       chpnt->voffsm = voffm; 
       chpnt->voffsh = voffh; 
       if (chpnt->chtyp->hh) sethhinf(chpnt,comp1);
       initsschan(chpnt,comp1,comp2,chpnt->chtyp,maxcond,vrev,
		arate,brate,crate,drate);
       break;

    case K:
       chpnt->voffsm = voffm; 
       chpnt->voffsh = voffh; 
       if (chpnt->chtyp->hh) sethhinf(chpnt,comp1);
       initsschan(chpnt,comp1,comp2,chpnt->chtyp,maxcond,vrev,
		arate,brate,crate,drate);
       break;

    case KCa:
       chpnt->voffsm = voffm; 
       chpnt->voffsh = voffh; 
       if (chpnt->chtyp->hh) sethhinf(chpnt,comp1);

       chapntk = (chattrib*)chapnt;
       switch (stype) {			/* select default rate constants */
            case 0:
            case 1:
            case 4:
            case 5:
             if ((d1=chapntk->d1)==NULLVAL) d1 = dsd1; 
             if ((d2=chapntk->d2)==NULLVAL) d2 = dsd2; 
             if ((k1=chapntk->k1)==NULLVAL) k1 = dsk1; 
             if ((k2=chapntk->k2)==NULLVAL) k2 = dsk2; 
	     break;
            case 2:
            case 3:
            case 6:
             if ((d1=chapntk->d1)==NULLVAL) d1 = dbd1; 
             if ((d2=chapntk->d2)==NULLVAL) d2 = dbd2; 
             if ((k1=chapntk->k1)==NULLVAL) k1 = dbk1; 
             if ((k2=chapntk->k2)==NULLVAL) k2 = dbk2; 
	     break;
       }
       switch (stype) {			/* set default rate constants */
            case 0:
            case 1:
            case 2:
            case 3:
            case 4:
            case 5:
           default: 

	  ((kcachan *)chpnt)->d1 = d1;
	  ((kcachan *)chpnt)->d2 = d2;
	  ((kcachan *)chpnt)->k1 = k1;
 	  ((kcachan *)chpnt)->k2 = k2;
            break;

       }
       switch (stype) {		/* special things that some K chans need */
	case 0:						/* Kca SK, HH */
	case 2:						/* Kca BK, HH */
	  if (comp1->capnt) {				/* found calcium comp*/
	     ca = comp1->capnt->cais[0];
	     ((kcachan *)chpnt)->initfl=1;		/* initialized OK */
	  }
	  else {		/* calcium not set up yet, must do later */
	     ca = dcai;
	     ((kcachan *)chpnt)->initfl=0;	/* not properly initialized */
	  }
	  alph = akcacalc((comp1->v-chpnt->voffsm),ca,arate,d1,k1);
	  bet  = bkcacalc((comp1->v-chpnt->voffsm),ca,arate,d2,k2);
	  ((kcachan *)chpnt)->m = alph / (alph + bet);
          ((kcachan *)chpnt)->h = 1;
	  break;
					/* same as 2: above */
					/*  but sequential state */

	case 1:				/* SKca, Markov */
	case 3:				/* BKCa, Markov */
	case 4:				/* SKCa, Markov, Hirshberg et al 1998 */
	case 5:				/* SKCa, Sah and Clements, 1999 */
	case 6:				/* BKCa, Markov, Horrigan  et al 1999 */
	default: 
               
	  if (comp1->capnt) {				/* found calcium comp*/
	     ((kcachan *)chpnt)->initfl=1;		/* initialized OK */
	  }
	  else {		/* calcium not set up yet, must do later */
	     ((kcachan *)chpnt)->initfl=0;	/* not properly initialized */
	  }
	  break;

       } /* switch (stype) */
       initsschan(chpnt,comp1,comp2,chpnt->chtyp,maxcond,vrev,
		arate,brate,crate,drate);
       break;

    case CA:
       chpnt->voffsm = voffm; 
       chpnt->voffsh = voffh; 
       if (chpnt->chtyp->hh) sethhinf(chpnt,comp1);
       initsschan(chpnt,comp1,comp2,chpnt->chtyp,maxcond,vrev,
		arate,brate,crate,drate);
      break;

  }
  chpnt->nchan = 0;			/* set noise params */
  for (stop=0,npnt=apnt->attpnt; npnt && !stop; npnt=npnt->attpnt) { 
      nattrib *napnt;

      napnt = (nattrib *)npnt;
      switch (npnt->ctype) {
	   double nchan, unitary;
	   int nrseed;

 	case CHNOISE:			/* If "chnoise" has been set */

	if ((unitary=napnt->unitary)==NULLVAL) {	/* find unitary */
	   if (chunitary!=NULLVAL) unitary = chunitary; 
	   else {		/* compute unit from N, or default value */
	     nchan = napnt->n;		
	     if (nchan==NULLVAL) nchan=chn;  /* Check for prev defined N */
	     if ((nchan==0)||(nchan==NULLVAL)) {/* N undef, use default unit */
	       if ((unitary=chtyp->unitary)<=0) unitary = dscu;
	       else unitary *= qcond(chtyp);
	     }
	     else unitary = chpnt->maxcond / nchan;
	   }
        }
	if (unitary < 0) unitary = dscu;
	if (unitary < 0) unitary = 0;
	
	nchan = napnt->n;		/*  both N, unit could be diff value */
	if (nchan==NULLVAL) nchan=chn;  /* Check for previously defined N */
	if (nchan==NULLVAL) { 	/* if N still not defined, use default unitary*/
	   if (unitary <= 0)   nchan = 0;
	   else nchan = chpnt->maxcond / unitary;	
	}
	chpnt->nchan = nchan;			/* floating point number */

	if ((taud=napnt->taud)!=NULLVAL) {
		if (taud <= 0) taud = 1e-6;
		chpnt->drate = 1.0 / taud;
	} 
        if (chaniss(ctype,stype)) maknois(chpnt);   /* set state populations */

	nrseed = napnt->rseed;
	if (nrseed <= 0)      
	    nrseed = noduniq(25471, chpnt->num, 15149, chpnt->num,rseed^3131);
        chpnt->cstate = makrand(RNDSIZ,"chan",cumchan);
	if (chpnt->cstate) rinitstate (nrseed,chpnt->cstate,RNDSIZ);

	break;

	case VESNOISE: break;

	default: stop=1; break;		/* stop on other channels */
      }
  }
					/* add chan to comp's connections */
  chpnt = addchan(comp1, (chan *)chpnt);
					/*  but possibly erase new channel   */
					/*  and return pointer to old one.   */
#ifdef DEBUG
 if (debug & 1 && debugz & 16)
   fprintf (stderr,"makchan end\n");
#endif
    return ((chan*)chpnt); 
}

/*------------------------------------*/

chan *modchan(elem *cepnt, attrib *apnt, double lcap)
                 
/* Modify a channel. */

{
    int i,ns,stype,stop;
    elem *epnt;
    comp *comp1;
    chan *chpnt;
    chantype *chtyp;
    chanparm *chp;
    attrib *npnt;
    chattrib *chapnt;
    double voffm,voffh,vrev,maxcond,area,m,h,caperm;
    double arate,brate,crate,drate;
    double taua,taub,tauc,taud;
    static stconc *conc;
    double alph,bet,k1,k2,d1,d2,ca;
    double chn, chunitary;
  
#ifdef DEBUG
 if (debug & 1 && debugz & 16)
   fprintf (stderr,"modchan: type %d %d\n",apnt->ctype,apnt->stype);
#endif

  if (!(epnt=findelem(cepnt->modif))) {
      fprintf (stderr,"modchan: can't find element %d\n",cepnt->modif);
      execerror ("Missing element: "," stopping... ");
      return ((chan *)NULL);
  }

  chpnt = (chan *)epnt->lptr;	/* get pointer to chan from elem */
  if (chpnt==((chan*)NULL)) return ((chan *)NULL); 
  comp1 = chpnt->comp1;

  chtyp = getchantype(chpnt->ctype,chpnt->stype);

  if (lcap<=0) lcap = 1e-6;
  area = comp1->cap / lcap;

  if (apnt->stype == NULLVAL) stype = 0;	/* channel sub-type */
  else			      stype = apnt->stype;
 
  chapnt = (chattrib *)apnt;
  chunitary = chapnt->unitary;
  chn = chapnt->n;
  if (chapnt->density != NULLVAL) chapnt->maxcond=chapnt->density*area;
  if (chapnt->ndensity != NULLVAL) chapnt->n=int(chapnt->ndensity*area*UM2CM2);
  if (chapnt->maxcond != NULLVAL) maxcond = chapnt->maxcond;
  else {   /* must calc new maxcond or use old val */

     if (chn!=NULLVAL) {		/* N set in chan attrib */
           double n, unitary;

        n = chn;
        if (chunitary==NULLVAL) {   
           if ((unitary=chtyp->unitary)<=0) unitary = dscu;
           else unitary *= qcond(chtyp); 
        }
        else unitary = chunitary;
        if (unitary <= 0) n = 0;
        maxcond = n * unitary;  /* maxcond computed from N before chnoise */
     }
   else {
     for (stop=0,npnt=apnt->attpnt; npnt && !stop; npnt=npnt->attpnt) {
        nattrib *napnt;
        double n,unitary;

      napnt = (nattrib *)npnt;
      switch (npnt->ctype) {

        case CHNOISE:	
        if ((n=napnt->n)!=NULLVAL) {		/* N set in noise attrib */
           if ((unitary=napnt->unitary)==NULLVAL) {  /* unitary not defined */
	        if (n <= 0) unitary = 0;
		else unitary = chpnt->maxcond / n + 0.5;
           }
	   maxcond = n * unitary;
        }
        else if ((unitary=napnt->unitary)!=NULLVAL) {/* unitary def but not N */
		if (unitary <= 0) n = 0; 
		else n = chpnt->maxcond / unitary + 0.5;
	        maxcond = n * unitary;
	}
	else maxcond = chpnt->maxcond;		/* else use original maxcond */
        break;
        default: stop=1; break;
      }
    }
   }
  }
  if ((caperm=chapnt->caperm)!= NULLVAL) caperm = chpnt->caperm;
  if (caperm < 0) caperm = 0;
  if (caperm > 1) caperm = 1;
  chpnt->caperm = caperm;
  if (chapnt->vrev != NULLVAL) vrev = chapnt->vrev;
  else {
        double tp;

    tp = chtyp->perm[PNA] + chtyp->perm[PK] + caperm;
    if (tp > 0)
	if (comp1->capnt)
	    vrev = ghkv(chtyp,comp1->capnt->cao,comp1->capnt->cais[0]);
 	else	
	    vrev = ghkv(chtyp,dcao,dcai);
    else    vrev = chpnt->vrev;
  }

  if (chapnt->voffsm != NULLVAL) chpnt->voffsm = chapnt->voffsm;
  if (chapnt->voffsh != NULLVAL) chpnt->voffsh = chapnt->voffsh; 

  if (chapnt->taua != NULLVAL) taua = chapnt->taua;
  else taua = 1.0/chpnt->arate;
  if (taua <=0) taua = 1e-6;
  arate = 1.0/taua;

  if (chapnt->taub != NULLVAL) taub = chapnt->taub;
  else taub = 1.0/chpnt->brate;
  if (taub <=0) taub = 1e-6;
  brate = 1.0/taub;

  if (chapnt->tauc != NULLVAL) tauc = chapnt->tauc;
  else tauc = 1.0/chpnt->crate;
  if (tauc <=0) tauc = 1e-6;
  crate = 1.0/tauc;

  if (chapnt->taud != NULLVAL) taud = chapnt->taud;
  else taud = 1.0/chpnt->drate;
  if (taud <=0) taud = 1e-6;
  drate = 1.0/taud;

  chpnt->maxcond  = maxcond;
  chpnt->vrev = vrev;
  chpnt->arate = arate;
  chpnt->brate = brate;
  chpnt->crate = crate;
  chpnt->drate = drate;


  switch (chpnt->ctype) { 	/* can't change type of chan */

    case NMDA:
       if (((cattrib*)chapnt)->mg != NULLVAL) 
		((ntchan*)chpnt)->mg = ((cattrib*)chapnt)->mg;

                /* continue on to below */
    case AMPA:
    case CGMP:
    case GABA:
    case SYN2:
       switch (stype) {
         default:
	   case 1:
	  dochani(chpnt,1e-8);
         }
        break;

    case NA:
       if (chpnt->chtyp->hh) sethhinf(chpnt,comp1);
       switch (stype) {
        case 0:			/* set up equilibrium rate constants */
	   m = ((hhchan *)chpnt)->m;
	   h = ((hhchan *)chpnt)->h;
       	   ((hhchan *)chpnt)->conduct = m*m*m*h;
	  break;

         default: break;

       }
       dochani(chpnt,1e-8);
       break;

    case K:
       if (chpnt->chtyp->hh) sethhinf(chpnt,comp1);
       switch (stype) {

        default: break;

        case 0:				/* set up equilibrium rate constants */
	   m = ((hhchan *)chpnt)->m;
       	   ((hhchan *)chpnt)->conduct = (m*m)*(m*m);
	  break;

        case 2:				/* type A deactivating potassium */
	   m = ((hhchan *)chpnt)->m;
	   h = ((hhchan *)chpnt)->h;
       	   ((hhchan *)chpnt)->conduct = m*m*m*h;
	  break;

       } /* switch (stype) */
       dochani(chpnt,1e-8);
       break;

    case KCa:
       if (chpnt->chtyp->hh) sethhinf(chpnt,comp1);
         switch (stype) {
           default:
            case 0:
            case 1:
            case 4:
            case 5:
             if ((d1=chapnt->d1)==NULLVAL) d1 = dsd1; 
             if ((d2=chapnt->d2)==NULLVAL) d2 = dsd2; 
             if ((k1=chapnt->k1)==NULLVAL) k1 = dsk1; 
             if ((k2=chapnt->k2)==NULLVAL) k2 = dsk2; 
             break;
            case 2:
            case 3:
            case 6:
             if ((d1=chapnt->d1)==NULLVAL) d1 = dbd1; 
             if ((d2=chapnt->d2)==NULLVAL) d2 = dbd2; 
             if ((k1=chapnt->k1)==NULLVAL) k1 = dbk1; 
             if ((k2=chapnt->k2)==NULLVAL) k2 = dbk2; 
             break;
         }
         switch (stype) {
           default:
            case 0:
            case 1:
            case 2:
            case 3:
            case 4:
            case 5:
            case 6:
	  ((kcachan *)chpnt)->d1 = d1;
	  ((kcachan *)chpnt)->d2 = d2;
	  ((kcachan *)chpnt)->k1 = k1;
 	  ((kcachan *)chpnt)->k2 = k2;
            break;

        }
        switch (stype) {

        default: break;
	case 0:				/* Ca sensitive K channel */
	case 2:				
	  arate = 1/taua;
	  arate = 1/taua;


	  if (comp1->capnt) {				/* found calcium comp*/
	     ca = comp1->capnt->cais[0];
	  }
	  else {		/* calcium not set up yet, must do later */
	     ca = dcai;
	  }
	  alph = akcacalc((comp1->v-chpnt->voffsm),ca,arate,d1,k1);
	  bet  = bkcacalc((comp1->v-chpnt->voffsm),ca,arate,d2,k2);
	  m = alph / (alph + bet);
	  ((kcachan *)chpnt)->m = m;
          ((kcachan *)chpnt)->h = 1;
 	  ((kcachan *)chpnt)->conduct = m;
	  break;

	case 1:				/* Ca sensitive K channel */
	case 3:				
	case 4:
	case 5:
	case 6: break;

       } /* switch (stype) */
       dochani(chpnt,1e-8);
       break;

    case CA:
       if (chpnt->chtyp->hh) sethhinf(chpnt,comp1);
       switch (stype) {
        case 0:			/* set up equilibrium rate constants */
	   m = ((hhchan *)chpnt)->m;
	   h = ((hhchan *)chpnt)->h;
       	   ((hhchan *)chpnt)->conduct = m*m*m;
	  break;

        case 2:			/* set up equilibrium rate constants */
	   m = ((hhchan *)chpnt)->m;
	   h = ((hhchan *)chpnt)->h;
       	   ((hhchan *)chpnt)->conduct = m*m*h;
	  break;

	default: break;

	} /* switch (stype) */
       dochani(chpnt,1e-8);
       break;
  }
  for (npnt=apnt->attpnt; npnt; npnt=npnt->attpnt) { /* set noise params */
      nattrib *napnt;
      double unitary,nchan;

     napnt = (nattrib *)npnt;
     switch (npnt->ctype) {

 	case CHNOISE:

	nchan = chpnt->nchan;
	if (napnt->n != NULLVAL) nchan = (int)(napnt->n);	

	else {		/* try to calc nchan from unitary cond */

	   if (napnt->unitary != NULLVAL) unitary = napnt->unitary;
	   else {
	     if (nchan == 0) unitary = 0;
	     else unitary = chpnt->maxcond / nchan;
	   }
	   if (unitary <= 0) nchan = 0;
	   else nchan = chpnt->maxcond/unitary + 0.5;	
	}
        chpnt->nchan = int(nchan);

	if (napnt->rseed > 0) {
	   if (chpnt->cstate) rinitstate (napnt->rseed,chpnt->cstate,RNDSIZ);
        }
	break;
     }
  }
#ifdef DEBUG
 if (debug & 1 && debugz & 16)
   fprintf (stderr,"modchan end\n");
#endif
  return ((chan*)chpnt); 
}

/*------------------------------------*/

chan *addchan(comp *pnt, chan *chpnt)
               
/* Add a channel to a compartment's connections. */
/* Channels will be condensed together later after */
/*  all compartments have been condensed. */

{
   register conlst *lpnt;
   conlst *clpnt;
   chan *cpnt;
   int ctype, stype;

  ctype = chpnt->ctype;
  stype = chpnt->stype;

  if (!pnt || !chpnt) return ((chan*)NULL);

   maklst(&pnt->clst, (conn*)chpnt);	/* add to comp's connections */
   chpnt->comp1 = pnt; 
   return chpnt;		/* return pointer to new channel */
}

/*------------------------------------*/

void condense_comp_chan(comp *pnt)
               
/* Check channels in a compartment's connections. */
/* If a channel of same type already exists,     */
/*   compress it with the existing one.
/* Can't add conductances when either old or new chan has */
/*   been saved, because they might be modified after creation */
{
   int n;
   conlst *cclpnt;
   register chan *chpnt;
   register conlst *lpnt;
   conlst *olpnt,*tlpnt;
   conlst *calpnt;
   chan *cpnt;
   cacomp *capnt;
   kcachan *k1pnt,*k2pnt;
   kcachan *k1pnts,*k2pnts;
   register int found, ctype, stype;

  n = 0;
  if (!pnt) return;
  for (olpnt=(conlst*)NULL,cclpnt=pnt->clst; cclpnt; ) {
    n++;
    if (!(chpnt=(chan*)cclpnt->conpnt)) {
	fprintf(stderr,"condense_comp_chan: missing connection...\n");
        break;
    }
    ctype = chpnt->ctype;
    stype = chpnt->stype;

    switch (ctype) {		/* if not a membrane channel, go to next */
	case  CA:
	case  NA:
	case KCa:   break;	/* keep checking this channel further */
	case   K:   break;	/* keep checking this channel further */
				/*  in for() loop below */
      case CGMP:
      case GABA:
      case AMPA:
      case NMDA:
      case SYN2:
	default:   olpnt = cclpnt;
		   cclpnt= cclpnt->next;
		   continue;	/* don't condense synapses, etc. */
		   break;
    }

   for (found=0,lpnt=cclpnt->next; lpnt && !found; lpnt=lpnt->next) {

    if (!(cpnt=(chan*)lpnt->conpnt)) {
	fprintf(stderr,"condense_comp_chan: missing connection...\n");
 	break;
    }
    if (cpnt==chpnt) continue;		/* skip if same chan */

        if (cpnt->ctype==ctype) { 
	 if (cpnt->stype==stype) { 
#ifdef DEBUG
	       if (debug & 1 && debugz & 32)
  fprintf (stderr,"condense_comp_chan: comp %d chan type %s %d matches\n",
		pnt->num, findsym(ctype), stype);
#endif
	   if (cpnt->nchan && chpnt->nchan) {	/* check unitary conductance */
	          double unit1, unit2, uratio;
	       unit1 = cpnt->maxcond / cpnt->nchan;
	       unit2 = chpnt->maxcond / chpnt->nchan;
	       uratio = unit1/unit2;
	       if (uratio < .8 || uratio > 1.2) {
#ifdef DEBUG
	       if (debug & 1 && debugz & 32)
   fprintf (stderr,"condense_comp_chan: maxcond %g %g unitary %g %g not same\n",
			cpnt->maxcond,chpnt->maxcond,unit1,unit2);
#endif
		continue; 			/* not same u cond */
	       }
	   }
	  if (((ctype==CA) || (cpnt->vrev==chpnt->vrev)) &&
	    (cpnt->voffsm==chpnt->voffsm) &&
	    (cpnt->voffsh==chpnt->voffsh) &&
	    ((!cpnt->nchan) == (!chpnt->nchan))&& /* both same noise status ? */
	    (cpnt->arate==chpnt->arate) &&
	    (cpnt->brate==chpnt->brate) &&
	    (cpnt->crate==chpnt->crate) &&
	    (cpnt->drate==chpnt->drate) &&
	    (!cpnt->saved) &&
	    (!chpnt->saved) )
          found = 1;
#ifdef DEBUG
           else if (debug & 1 && debugz & 32)
              fprintf (stderr,"condense_comp_chan: other parms not same\n");
#endif
	 }  /* if (stype) */
	} /* if (ctype) */

     switch (chpnt->ctype) {

     case K:
       if (found) {

	   switch (chpnt->stype) {

         case 0:
         case 1:
         case 3:
         case 4:
         case 5:
	 default: break;

        }  /* switch (chpnt->stype) */
      }
	break;

     case KCa:
       if (found) {
	 switch (chpnt->stype) {

         case 0:
         case 2:
	      k1pnt = (kcachan*)cpnt;
	      k2pnt = (kcachan*)chpnt;
     	      if ((k1pnt->d1!=k2pnt->d1) ||
     	          (k1pnt->d2!=k2pnt->d2) ||
     	          (k1pnt->k1!=k2pnt->k1) ||
     	          (k1pnt->k2!=k2pnt->k2))
	      found = 0;
              break;
         case 1:
         case 3:
         case 5:
	      k1pnts = (kcachan*)cpnt;
	      k2pnts = (kcachan*)chpnt;
     	      if ((k1pnts->d1!=k2pnts->d1) ||
     	          (k1pnts->d2!=k2pnts->d2) ||
     	          (k1pnts->k1!=k2pnts->k1) ||
     	          (k1pnts->k2!=k2pnts->k2))
	      found = 0;
              break;

         case 4:
         case 6:  break;
	 default: break;

        }  /* switch (chpnt->stype) */
      }
	break;

     case NA:
	   switch (chpnt->stype) {

		case 0:
		case 1:
		case 2: 
		case 3: break;

		default:
		case 4:
		case 5: found = 0; break;
	   }
	   break;
		
     case CA: 
	break;		/* cacomp ptr to chan is deleted in delchan() */
		
     case AMPA:		/* do nothing because we want these to be separate */
     case CGMP:
     case GABA:
     case NMDA:
     case SYN2:
     default:
	    found = 0;
            break;

     }  /* switch (chpnt->ctype) */

     if (found) break;

   }   /* for (found=0,lpnt;;) */

   if (found) {
	cpnt->maxcond += chpnt->maxcond;  /* add the conductance */
	cpnt->nchan += chpnt->nchan;      /* add number of channels */
	if (chaniss(cpnt->ctype,cpnt->stype)) maknois((chan *)cpnt);

		/* patch pointers in list */
	
	if (chpnt->last) chpnt->last->next = chpnt->next;
	if (chpnt->next) chpnt->next->last = chpnt->last;
	if (chanpnt==chpnt) chanpnt=(chan*)chpnt->next;
	if (chanend==chpnt) chanend=(chan*)chpnt->last;
	delchan (chpnt);	/* no need for new (or moved) channel conn */

	tlpnt = cclpnt;		/* erase this one */
	if (olpnt) {
	   olpnt->next = cclpnt->next;
	}
	else {
	    pnt->clst = cclpnt->next;
	}
	cclpnt=cclpnt->next;
	efree (tlpnt);		/* erase conlst pointer to chan */

   }  /* if (found) */ 

   else {
       olpnt = cclpnt;
       cclpnt = cclpnt->next;
   }
  } /* for (cclpnt;;) */

      /* Finally, set nchan to an integral number */

  for (lpnt=pnt->clst; lpnt; lpnt=lpnt->next ) {
    if (!(chpnt=(chan*)lpnt->conpnt)) {
	fprintf(stderr,"set_comp_nchan: missing connection...\n");
        break;
    }
    setnchan(chpnt);
  } /* for (cclpnt;;) */
}

/*------------------------------------*/

void delchan(chan *chpnt)

/* Delete a channel and its substructures */

{
  if (!chpnt) return;
  if (chpnt->cstate) efree (chpnt->cstate);	/* free the random noise gen */ 
  if (chpnt->conc) efree (chpnt->conc);		/* free the state concs */
  efree (chpnt);
  cumchan--;
}

/*------------------------------------*/

cacomp *makcacomp(elem *epnt, cattrib *apnt, comp *comp1, double area)

{
      int i, cashell, nshell;
      double cai, cao, vrev, *cshp, *b;
      double dia, r, dr, dca, core, cabnd;
      double pkm, vmax, ekm, kex;
      chantype *chtyp;
      cacomp *capnt;
      conlst *tmp;

    if ((capnt=(cacomp *)emalloc(sizeof(cacomp))) == (cacomp*)NULL) {
      fprintf (stderr,"makcacomp: no space left for ca comp %d\n",cumcacomp+1);
      return ((cacomp*)NULL);  
    }
    cumcacomp++;
    capnt->next = (cacomp*)NULL;
    capnt->comp1 = comp1;		/* set ca comp pntr to voltage comp */
    comp1->capnt = capnt;		/* set voltage comp pntr to ca comp */

    cao=apnt->cao;
    cai=apnt->cai;
    vrev=apnt->vrev;

    chtyp = getchantype(CA,0);		/* assume Ca type 0 for cai calc */

						/* If calcium not determined */
    if (cao==NULLVAL || cai==NULLVAL) {		/*  must hunt for info */
      if (vrev!=NULLVAL) {			/* Find ca conc from vrev */
        if (cai==NULLVAL) {
		double df, nf, cavoff;
	   if (cao == NULLVAL) cao = dcao;

		/* The idea here is to apply the GHK voltage equation */
		/*  backwards to calculate a value for cai. */

	   cavoff = qcavoff(chtyp->parm) * dcaspvrev;
           df = exp ( FR * (vrev-cavoff) / ktemp); /* multiplier for Ca perm */
	   nf = 1.0 / ( 1.0 + df);
           cai = ((4.0*cao*nf  + ghkvo(chtyp,vrev)) /
		exp((vrev-cavoff)*FR/ktemp) - ghkvi(chtyp,vrev)) / (4.0*nf*df);
           /* cai = cao / exp(vrev*f2rt) - dpki; */
	   if (cai < 10e-9) cai = 10e-9;
        }
        else if (cao==NULLVAL) {
          cao = dcao;				/* always use default cao */
        }	
		/* Note: when both vrev and cai are set by user */
		/*   then vrev and cai/cao will be inconsistent */
		/*  If so, then Ca current for [Ca] will be derived from cai */
		/*   and total Ca chan current will be derived from vrev. */
		/*  This represents the case when [K]i affects total Ca chan */
		/* current and thus the channel's reversal potential. */
      }
      else {  /* if (vrev==NULLVAL) */	      /* vrev unknown, use defaults */
        if (cao==NULLVAL) {
          cao = dcao;
        }
        if (cai==NULLVAL) {
          cai = dcai;
        }
      }
    } 
    if (cai<=0.0) cai = 10e-9;		/* make cai nonzero */
    capnt->cao  = cao;
    capnt->cai  = cai;

    if ((nshell=apnt->cashell)==NULLVAL) nshell=dcashell; /* num of shells */
    if (nshell <= 1) {
	nshell = 1;	/* min of 1 shell */
	fprintf 
       (stderr,"# makcacomp: num of ca shells is zero or neg, setting to %d\n",
			nshell);
    }

    /* Try to get best est. for radius and area. */
    /*  area comes in as cm2 (from cap measurement) */
    /* Calib in dm (decimeters) because molarity is moles/liter = m/1000 cm3 */

					/* diameter in microns */
    if   ((epnt->ctype==SPHERE) && ((dia=((sphere *)epnt)->dia)!=NULLVAL)) {
         r = dia * 0.5 * DMUM;		/* radius in dm (decimeters) */
         area = (4.0 * MPI * r*r);	/* shell surface (dm2) */
    }
    else if ((epnt->ctype==CABLE) && area) { /* if cable with non-zero area */
          area *= 1e-2;
          r = sqrt (area / (4.0*MPI)); /* radius in dm (decimeters) */
    }
    else {				/* find radius from area (capacitance) */
       if (area) {			/* in this case area is cm2 */
          area *= 1e-2;
          r = sqrt (area / (4.0*MPI)); /* radius in dm (decimeters) */
       }
       else {
          r = CACOMPRAD * DMUM;		/* (calib in dm) make it 5 um radius */
          area = (4.0 * MPI * r*r);	/* shell surface (dm2) */
       }
    }
    dr = CASHD*DMUM;			/* thickness of shell in dm (1e-6) */
    if (r < (dr * (nshell+5))) {	/* if sphere is too small */
	  int nnshell;
       nnshell = int(r / dr + 0.5);
       if (nnshell > 15) nshell = nnshell - 5;
       else              nshell = nnshell - int(nnshell/3.0 + 0.5);
       if (nshell<=1) nshell = 1;
    } 
    dca = ddca * 1e-2;			/* Diffusion const for Ca in dm2/sec*/

    cashell = nshell + 1;		/* core is extra "shell" */
    capnt->cashell = cashell;		/* assign pointer to ca shells */

					/* allocate space for shells & core */

    if ((cshp=(double *)emalloc(cashell*sizeof(double))) == (double*)NULL) {
      fprintf(stderr,"makcacomp: no space left for %d calcium shells\n",nshell);
      return ((cacomp*)NULL);  
    }
    capnt->cais = cshp;			/* assign pointer to ca shells */

    for (i=0; i<cashell; i++) {
       capnt->cais[i]  = cai;		/* set initial Ca concentration */
    }

    if (apnt->cabuf) {
	 double bkd, cabf, cabt, cabti, *cab, carate;

      if ((cabt=apnt->btot) ==NULLVAL) cabt = dcabt; /* Tot cabuf conc */
      if ((cabti=apnt->btoti)==NULLVAL) cabti = dcabti; /* Tot cabuf conc */
      capnt->cabt = cabt;
      capnt->cabti = cabti;
						/* set rates */
      if ((cabf=apnt->bmax)==NULLVAL) {
         carate = exp(log(dqcab)*(tempcel-BASETCA)/10.0);
	 cabf = dcabf;				/* Forw rate for cabuf */
	 if (cabf==0) cabf=1;
      }
      else carate = 1.0;
      if ((bkd=apnt->bkd) ==NULLVAL) bkd = dcabr/cabf; 	/* Kd for cabuf */

		/* if vmax rate not set, use default values with temp coeff */

      capnt->cabf = cabf * timinc * carate;
      if (bkd==0) bkd = 1;
      capnt->cabr = bkd * cabf * timinc * carate;

      if ((cab=(double *)emalloc(cashell*sizeof(double))) == (double*)NULL) {
    fprintf(stderr,"makcacomp: no space left for %d calcium buffer\n",nshell);
        return ((cacomp*)NULL);  
      }
      capnt->cab = cab;			/* assign pointer to ca shells */
      if (capnt->cab) {
        for (i=0; i<cashell; i++) {
          if (i==0) capnt->cab[i]  = cabti;		/* Ca buffer conc */
          else capnt->cab[i]  = cabt;			/* Ca buffer conc */
        }
      }
      cabnd = 1.0;
    }
    else {
	capnt->cabt = 0;
	capnt->cabf = 0;
	capnt->cabr = 0;
        capnt->cab = (double *)NULL;	/* assign pointer to ca shells */
        if ((cabnd=apnt->cabnd)==NULLVAL) cabnd=dcabnd; /* ratio bnd to free */
        cabnd = 1.0 / (cabnd+1);	   /* multiplier is one more */
    } 


    switch (epnt->ctype) {

    case CA:
    case SPHERE:
    default:			 /* shell factors for diffusion eqn. */

	capnt->casf0 = timinc*F2/(4*MPI*r*r*dr);  	/* caflux */
	capnt->casfn = dca * timinc/(dr*dr)*cabnd;	/* norm shells */
	core = r - nshell * dr;
	capnt->casfc = dca * timinc *3.0/(dr*core)*cabnd; /* D*t*SA/(dr*vol) */
							  /*  for core shell */
	break;

    case CABLE:			/* must fix this for cylindrical shells */

	capnt->casf0 = timinc*F2/(4*MPI*r*r*dr);	/* caflux (t/M/sec/A/V*/
	capnt->casfn = dca * timinc/(dr*dr)*cabnd; 	/* norm shells */
	core = r - nshell * dr;
	capnt->casfc = dca * timinc *3.0/(dr*core)*cabnd; /* D*t*SA/(dr*vol) */
							  /*  for core shell */
	break;
    }
    if (!implicit) { 			/* Crank-Nicholson factor */
	capnt->casfn *= 0.5;  
	capnt->casfc *= 0.5;   
    }

    if (apnt->cabuf) docacompi(capnt);

    if (apnt->pump) {
      if ((pkm=apnt->pkm)==NULLVAL) pkm = dcapkm;    /* Km for ca pump */
      if ((vmax=apnt->vmax)==NULLVAL) {
		vmax = dcavmax; /* Vmax for ca pump */
      }
      vmax *= exp(log(dqcavmax) * (tempcel - BASETCA)/10.0);
      capnt->pkm   = pkm;
      capnt->vmax = vmax * area;
    }
    else capnt->vmax = 0;

    if (apnt->exch) {		/* precalculate values for Na-Ca exchanger  */
				/* From de Schutter & Smolen 1998  */
				/* after DiFrancesco & Noble 1985  */
				/* and Gabbiani,Midtgaard and  Knopfel 1994 */

      if ((kex=apnt->kex)==NULLVAL) kex = dcakex; /* Exch rate for ca exch */
      capnt->kexi  = kex  * area *     dnao*dnao*dnao*1e12;
      capnt->kexo  = kex  * area * cao*dnai*dnai*dnai*1e12;
      capnt->ego = EG*FR/ktemp;
      capnt->egi = (EG-1.0)*FR/ktemp;
    }
   else {
      capnt->kexi = 0;
      capnt->kexo = 0;
   } 
   vrev = r2ft * log(cao/cai);   /* Nernst equation for Ca */
   capnt->vrev = vrev;
				/* surface potential due to Ca */
   capnt->cavoff = dcaspvrev* qcavoff(chtyp->parm);

  return capnt;
}

/*------------------------------------*/

chan *makca(elem *epnt, cattrib *apnt, comp *comp1, double area, int saved)
    	           
/* Make a new calcium channel, and if it doesn't have one,
   make a new calcium compartment, too.  
   Area calibrated in cm2.
*/

{
    chan *chpnt,*addchan(comp *pnt, chan *chpnt);
    cacomp *capnt;
    int i,chansiz,ns,cashell,nshell;
    double cao, cai, vrev;
    double maxcond;

#ifdef DEBUG
 if (debug & 1 && debugz & 16)
   fprintf (stderr,"makca: comp %d\n",comp1->num);
#endif
		/* don't make chan if maxcond 0 */ 

  if ((maxcond=apnt->maxcond)==NULLVAL) maxcond = dmaxca;

  chpnt = makchan(apnt,comp1,(comp*)NULL,saved);       /* make channel */

  if (!(capnt=comp1->capnt)) { 
    if ((capnt=makcacomp(epnt,apnt,comp1,area))==NULL) { /* make new ca comp */
      fprintf (stderr,"makca: no space for ca comp %d\n",comp1->num);
      return ((chan*)NULL);
    }
  }

  if (!chpnt->setvrev) {
     cao = capnt->cao;
     cai = capnt->cais[0];
     vrev = ghkv(chpnt->chtyp,cao,cai);   /* GHK V equation, Cai + Ki */
     chpnt->vrev = vrev;
  }

			/*  end of ca comp stuff */
#ifdef DEBUG
 if (debug & 1 && debugz & 16)
   fprintf (stderr,"makca end.\n");
#endif

  return chpnt;
}

/*------------------------------------*/

int findrecep(int num1, int num2, int num3, int num4, recep **rpnt, char *str)
{
   recep *pnt;

  for (pnt=recpnt; pnt; pnt=(recep *)pnt->next) {
    if ((pnt->recnm1 == num1) && (pnt->recnm2 == num2) && 
	(pnt->recnm3==num3) && (pnt->recnm4==num4)) {
	*rpnt = pnt;
	return 1;
    }
  }
  if (str) {
     if (num4 != NULLVAL)
  	 fprintf(stderr,"\n%s: can't find recep %d %d %d %d\n",
			str,num1,num2,num3,num4);
     else if (num3 != NULLVAL)
  	 fprintf(stderr,"\n%s: can't find recep %d %d %d\n",str,num1,num2,num3);
     else if (num2 != NULLVAL)
  	 fprintf(stderr,"\n%s: can't find recep %d %d\n",str,num1,num2);
     else 
  	 fprintf(stderr,"\n%s: can't find recep %d\n",str,num1);
   }
  return 0; 
}

/*------------------------------------*/

void maklst(conlst **head, conn *cpnt)

/* add an entry to a connection list. */

{
    conlst *lpnt,*tmp,*oldtmp;
    int found;

  if (!head) { 
     fprintf (stderr,"maklst: incorrect list pointer\n");
     return;  
  }
  found = 0;
  for (oldtmp=tmp= *head; tmp; oldtmp=tmp,tmp=tmp->next){ /* find end of list */
     if (tmp->conpnt==cpnt) {
        found = 1;			/* check to see if already there */
        break;
     }
  }
  if (!found) {
 
    if ((lpnt=(conlst *)emalloc(sizeof(conlst))) == NULL) {
       fprintf (stderr,"maklst: no space left for conlist\n");
       return;  
    }
    if (oldtmp) oldtmp->next = lpnt;
    else *head = lpnt;
 
    lpnt->next = (conlst *)NULL;
    lpnt->conpnt = cpnt;		/* set pointer to connection */
    cumclst++;
  }
}

/*------------------------------------*/

void addlst(conlst **head, conlst *h2)

/* add one connection list (h2) to another (head). */

{
    conlst *tmp;

  if (!head) { 
     fprintf (stderr,"addlst: incorrect list pointer\n");
     return;  
  }
  for (tmp=h2; tmp; tmp=tmp->next){ /* find end of list */
      maklst (head, tmp->conpnt);
  }
}

/*------------------------------------*/

void dellst(conlst **head, conn *cpnt)

/* delete an entry from a connection list. */

{
    conlst *tmp,*oldtmp;
    int found;

  if (!head) { 
     fprintf (stderr,"lst: incorrect list pointer\n");
     return;  
  }
  if (! *head) return;			/* empty list: nothing to do */
  else if (! cpnt) return;
  else {
    for (found=0,oldtmp=tmp= *head; tmp; oldtmp=tmp,tmp=tmp->next) { 
       if (tmp->conpnt==cpnt) {
          found = 1;
          break;
       }
    }
    if (found) {
      oldtmp->next = tmp->next;
      if (*head==tmp) *head = (conlst*)NULL;
      efree (tmp);
      cumclst--;
    }
  }    /* else */
}

/*------------------------------------*/


void makdyad(dyadlst **head, synap *spnt)

/* add an entry to a synapse's dyad list */

{
    dyadlst *lpnt,*tmp,*oldtmp;

  if (!head) { 
     fprintf (stderr,"makdyad: incorrect list pointer\n");
     return;  
  }
  if ((lpnt=(dyadlst *)emalloc(sizeof(dyadlst))) == (dyadlst*)NULL) {
     fprintf (stderr,"dyadlst: no space left for dyadlist\n");
     return;  
  }
  if (! *head) *head = lpnt;	/* save head if first one */
  else {
    for (oldtmp=tmp= *head; tmp; tmp=tmp->next)  /* find end of list */
       oldtmp = tmp;			   /*  and get pointer to last */
    if (oldtmp) oldtmp->next = lpnt;
  }
 
  lpnt->sdyad = spnt;		/* set pointer to connection */
  lpnt->sdyadnum = spnt->num;
  lpnt->next = (dyadlst*)NULL;
  cumdlst++;
}

/*------------------------------------*/

int numdyad(dyadlst *head)

/* Return the number of dyads in a dyad list. */

{
    dyadlst *lpnt,*nlpnt;
    int ndyad;

  if (!head) { 
     fprintf (stderr,"numdyad: incorrect list pointer\n");
     ndyad=0;  
  }
  else {
    for (ndyad=0,lpnt=head; lpnt; lpnt=lpnt->next,ndyad++) { }
  } 
  return ndyad;
}

/*------------------------------------*/

void deldyad(dyadlst **head)

/* delete a dyad list (but not the synapses it points to.) */

{
    dyadlst *lpnt,*nlpnt;

  if (!head) { 
     fprintf (stderr,"deldyad: incorrect list pointer\n");
     return;  
  }
  if (! *head) return;			/* empty list: nothing to do */
  else {
    for (lpnt=*head; lpnt; lpnt=nlpnt) { 
      nlpnt = lpnt->next;
      efree (lpnt);
      cumdlst--;
    }
   *head = (dyadlst*)NULL;
  } 
}

/*------------------------------------*/
