/* Header file ncomp.h for program "nc". */
/* Contains low-level data structures for runtime */

#define NUMSTATE 12                     /* max number of states in channel */
#define NUMTRANS 6                      /* max transitions to other sts */
#define NCHANCONST 40                   /* max number of channel types */
#define RNDSIZ	32			/* size of random number state table */
#define HMICSEC	0.0001			/* 100 microseconds in seconds */
#define MSEC	0.001			/* 1 msec in seconds */
#define HMICMS	0.1			/* 100 microseconds in msec */
#define MVOLT	1000.0			/* V to mV */
#define MSSEC	1000.0			/* msec to sec */
#define BASETC 6.3			/* temperature for calc HH rates */
#define BASETSYN 22.0			/* temperature for calc synap rates */
#define BASETCA  22.0			/* temp for calc Ca pump/buf rates */
#define CHANRATE 1000.0			/* default chan rate = 1 msec tau */
#define NNTR 20				/* number of neurotransmitters */
#define NUMFILT 10			/* number of filters in lpfilt */

struct conn; 
struct dbuf;
struct chan;
struct hhchan; 
struct kcachan;
struct stconc;
struct sschan;
struct chanstate;
struct chantype;
struct lpfilt;
struct synap;
struct load;
struct recep;
struct recpar;
struct recstim;
struct cacomp;
struct conlst;
struct dyadlst;
struct node;
struct ntcomp;

struct id {
	short int ctype;		/* type of object */
	short int stype;		/*  subtype */
	};

struct comp: id {			/* structure for compartment */
        short int miscfl;               /* misc flags: VEXT IEXT VBAT CA ... */
        int num;			/* comp number */
        double v;                       /* voltage */
        double oldv;                    /* voltage at previous time step */
        double vest;                    /* volt estimate */
        double implf;                   /* implicit factor */
        double nodc;                    /* node current not varying w/vest */
        double tcond;                   /* total static comp conductance */ 
        double tcondn;                  /* total cond not varying w/vest */ 
        double vrev;                    /* membrane reversal potential */
        float tcondm;                   /* total cond through membrane */
        float verr;                     /* abs value of voltage error */
        float k;                        /* calculation constant */
        float rm;                       /* membrane conductance in comp */
        float cap;                      /* capacitance in comp */
        float jnoise;                   /* membrane conductance in comp */
        float extv;                     /* voltage clamp voltage */
        float extvi;                    /* voltage clamp current */
        float exti;                     /* current clamp value */
        short int g;               	/* verr greater than zero */
        short int t;               	/* total verr calculated */
        short int virt;               	/* virt compartment */
	float relax;			/* relax for this comp only */
	char *jstate;			/* state array for Johnson noise */
	cacomp *capnt;			/* pointer to calcium compartment */
	ntcomp *ntpnt;			/* list of neurotr compartments */
        conlst *clst;           	/* list of conns to other comps */
        conlst *nodlst;         	/* (possible) nodes for this comp */
        comp *hnext;             	/* pointer to next comp for hash */
        comp *next;             	/* pointer to next compartment */
        comp *last;             	/* pointer to last compartment */
        };

struct conn: id {                   	/* structure for comp connection */
	int num;			/* number of conn (only for synap) */
        conn *next;             	/* pointer to next connection (gj) */
        conn *last;             	/* pointer to last connection (gj) */
        double conduct;                 /* conductance of connection */
        comp *comp1;                    /* first compartment connection */
        comp *comp2;                    /* second compartment connection */
	int num1;			/* comp # of first comp. */
	int num2;			/* comp # of second comp. */
        };

struct gj: conn {                   	/* structure for gap junction */
        float maxcond;                  /* maximum conductance */
        float vgain;               	/* mV/e-fold change (V sens) */
        float voff;               	/* volt. offset for expont. change */
        float taun;			/* activation tau */
        float rvgain;               	/* rev. mV/e-fold change (V sens) */
        float rvoff;               	/* reverse offset for expon. change */
        float rtaun;			/* activation tau */
        float n;			/* conductance fraction */
        float gnv;			/* ratio of min/max conductance */
	short int rev;			/* reverse parms set */
	short int modtyp;		/* CycA compartment */
	short int sign;			/* sign of modulation: open or close */
	node *nodec;			/* node to check for cycg modulation */
        };

struct ntcomp: id {			/* neurotrans compartment */
	short int n;			/* number of inputs */
	float val;			/* concentration */
	float tau;			/* time constant of decay */
	ntcomp *ntpnt;			/* pointer to next ntcomp */
	};

struct dbuf: conn {                   	/* structure for delayed buffer */
        float delay;               	/* size of delay buffer */
	short *delbuf;			/* circular buffer pointer */
	short *delpnt;			/* input/output buffer pointer */
        };

struct chan: conn {            		/* basis stuct for membrane channel */
	float maxcond;			/* maximum total channel conductance */
        float voffsm;                   /* activation offset voltage */
        float voffsh;                   /* inactivation offset voltage */
        double vrev;                    /* reversal potential */
        float gvrev;                    /* indiv slope vrev (if setvrev) */
	chantype *chtyp;		/* channel constants */
        char *cstate;           	/* channel random state */
        float caperm;                   /* rel Ca permeability */
        float nao;                 	/* nao for channel */
        float naratio;                 	/* nai/nao for channel */
        float ko;                 	/* ko for channel */
        float kratio;                 	/* ki/ko for channel */
        float nchan;              	/* total number of channels */
        int cno;              		/* number of channels actually open */
        float arate;                    /* forward rate (1/tau) */
        float brate;                    /* reverse rate */
        float crate;                    /* deactivation rate */
        float drate;                    /* noise flicker rate */
	short int setvrev;		/* = 1 -> vrev set by user */
	short int saved;		/* = 1 -> has saved elem */
	short int numstate;             /* number of states used */
	short int k;             	/* state alternator */
	stconc *conc;			/* state concentr (variable size arr) */
        };

struct hhchan: chan {                 	/* structure for membrane channel */
        double m;                       /* m,n activation, range 0 - 1 */
        double h;                  	/* h inactivation  range 0 - 1 */
        };

struct kcachan: hhchan {             	/* structure for KCa membrane channel */
        float d1;                  	/* voltage multiplier for alpha */
        float d2;                  	/* voltage multiplier for beta */
        float k1;                  	/* Ca multiplier for alpha */
        float k2;                  	/* Ca multiplier for beta */
        short initfl;                  	/* 1=> m initialized, 0=> m not init */
        };

struct ntchan: chan {                	/* struct for sequen-state synap chan */
	float mg;			/* magnesium conc. */
	float gly;			/* glycine conc. */
        };

struct dstconc {                 	/* structure for state concentration */
	double dcon;			/* change in state concentr estimate */
	double icon;			/* implicit change estimate */
	int   dchan;			/* change in channels in this state */
        };

struct stconc {                 	/* structure for state concentration */
	double cval;			/* state concentr */
	double cest;			/* state concentr */
	int   nchan;			/* number of channels in this state */
        };

struct chanstate {              	/* structure for chan state constants*/
        float cond;                     /* normalized conductance of state */
	short int numtrans;		/* number of transitions from state */
	char     trans[NUMTRANS];	/* four possible state transitions */
	double (*trate[NUMTRANS])
		  (chan *cpnt);		/* function returning rate for trans */
	float  ratemul[NUMTRANS];	/* rate multiplier for transition */
	char   rateo[NUMTRANS];		/* offset voltage for transition */
	};				/* rateo=0->voffsm; rateo=1->voffsh;*/

struct chanparm {                	/* structure for channel rate params */
	short int pn;			/* parm number m=0, h=1, etc. */
	short int nfval;		/* number of alpha, beta, etc. vals */
	short int nival;		/* number of implicit values */
	double *fval;			/* alpha, beta, etc. vals for integ. */
	double *ival;			/* implicit values for integration */
        float *chanftab;		/* rate func tables */
        float *chanitab;		/* implicit tables if required */
        double (*chancalc)(double,int); /* default rate function */
        char *funcname;			/* user-defined rate function */
        double qrate;			/* Q10 rate calculated from dq, qt */
        double dq;			/* Q10 factor for rate calc */
        double qt;			/* base temperature for rate funcs */
        double dqca;			/* Q10 for cao offset on chan voffset */
        double qcao;			/* base cao for chan offset: 1.15 mM */
        double qcavoff;			/* voltage offset calc from dqca,qcao */
        double voff;			/* voltage offset to modify rate funcs*/
        double oldvf;			/* Variables that need to be checked */
        double oldvi;
        double oldmr;
        double oldtimincf;		/*  to see if table needs to be */
        double oldtiminci;
        double oldtempcel;		/*  regenerated. */
        double oldcao;
	};

struct chantype: id {                	/* master struct for chan constants */
        short int numstate;             /* number of states used */
	short int numparm;		/* number of params, set to 2 for HH */
	short int hh;			/* = 1 -> HH channel, otherwise SS */
        double qc;			/* Q10 factor for conductance */
        double qt;			/* base temp for conductance */
        double qcond;			/* Q10 multiplier for conductance */
	double vrev;			/* default reversal potential */
	double *perm;			/* rel permeability to ions */
	float unitary;			/* default unitary conductance */
	float trconc;			/* default tr conc multiplier */
	chanstate *state;		/* state definitions */
	chanparm *parm;			/* rate params */ 
	float *respamp;			/* response amplitudes */
	short int *nvval;		/* number of nonlinear df tables */
	float *chanvtab;		/* nonlinear driving force */
	};


struct cacomp: id {			/* structure for calcium comp/chan */
        int num;                	/* compartment number */
	cacomp *next;			/* pointer to next ca comp/chan */
        comp *comp1;                    /* voltage compartment     */
        int vcnum;                      /* voltage compartment num */
	double casf0;			/* shell fact for caflux */
	double casfn;			/* shell factor = 1/(dr*dr) */
	double casfc;			/* shell factor for core */
	double vrev;			/* calcium reversal potential */
	double *cais;			/* shells for ca diffusion */
	double *cab;			/* buffer conc at each Ca shell */
	float cao;			/* calcium concentration outside */
	float cai;			/* starting calcium conc inside */
	float nao;			/* Na concentration outside */
	float nai;			/* Na conc inside */
	float ko;			/* K concentration outside */
	float ki;			/* K conc inside */
	float cavoff;			/* voltage offset due to cao */
	float cabti;			/* Ca buffer in first shell */
	float cabt;			/* Ca buffer in shell */
	float cabf;			/* forward rate to Ca.B /M/sec */
	float cabr;			/* reverse rate to Ca.B /sec */
	float ica;			/* current through calcium chan */
	float vmax;			/* calcium pump Vmax */
	float pkm;			/* calcium pump half-max */
	float kexi;			/* Na-calcium exchange rate const */
	float kexo;			/* Na-calcium exchange rate const */
	short int cashell;		/* number of ca shells */
	float ego;			/* Voltage barrier location * F/R */
	float egi;			/* 1-Voltage barrier location * F/R */
	float ipump;			/* total current from pump */
	float iexch;			/* total current from exch */
	float slope;			/* "linearized" exchanger conductance */
	float eica;			/* exhanger y zero-crossing */
	};

struct synap: conn {                  	/* structure for synaptic connection */

        double oldc;                    /* old conduct,active */
        double vrev;                    /* reversal potential */
        float thresh;                   /* offset (threshold) for lookup tab */
        float nkd;                      /* nt dissociation constant (half max)*/
	float npow;			/* Hill coeff for nt binding */
        float ngain;                    /* = 1/b (mvolts for efold increase) */
        float ckd;                      /* cG dissociation constant (half max)*/
        float cgain;                    /* gain (multiplier) after sat. */
        float maxcond;                  /* maximum conductance (num of chans) */
        float vsize;                    /* vesicle size (in terms of trel) */
        float cov;             		/* Coeff of variation multiplier */
        char *vstate;           	/* vesicle random state */
        float vtime;              	/* ves. release time, comp to vrefr */
        float vtint;              	/* ves. threshold interval for reset */
	float trconc;			/* max neurotr conc at postsyn recept */
        short int setvrev;             	/* =1 -> vrev set by user */
        short int vrefr;              	/* refractory period for ves. release */
        short int vflag;              	/* ves. release flag, after refr done */
        short int vsites;              	/* number of ves noise release sites */
        short int ntact;               	/* open or closed */
        short int mesng1;              	/* output to cyca/g in postsyn cell*/
        short int mesng2;              	/* output to cyca/g in postsyn cell*/
        short int curve;               	/* linear or exponential */
        ntchan *resp;           	/* postsynaptic channel response */
        dyadlst *dyad;            	/* pointer to dyad (correl. synapse) */
	lpfilt *filt1;			/* low pass filter before tr release */
	lpfilt *filt1h;			/* high pass filter before tr release */
	lpfilt *filt2;			/* low pass filter after tr release */
	lpfilt *filt3;			/* low pass filter after saturation */
        };

struct randstate {			/* random state indexed by number */
	int ngen;			/* number of generator */
	randstate *next;		/* pointer to next state */
	char rstate[RNDSIZ];		/* random state */
	};

struct lpfilt: id {			/* low pass filter for synapse */
        short int nfilt;                /* number of filters hi or lo-pass */
        float *lfilt;			/* array of filters to delay psp */
        float *ftau;			/* tau's for filters */
        float tfall;                    /* falling phase time constant */
	};

struct load: id {              		/* structure for synaptic connection */
        load *next;	            	/* pointer to next load */
        float conduct;                  /* conductance of channel */
        comp *comp1;                    /* first compartment connection */
        int num1;			/* number of first compartment */
        double vrev;                    /* reversal potential */
	};

struct recep: load {                  	/* structure for receptor */
        recpar *chtyp;          	/* receptor constants */
        float xloc;                     /* location of receptor */
        float yloc;
	float pathl;			/* path length through pigment */
	float attf;			/* attenuation factor */
	float tspeed;			/* time speed factor */
	float intenadj;			/* intensity adjust factor */
	float dnois;			/* dark contin. noise (frac of phot)*/
	short int filt;			/* filter number (0=none,1=mac pigm)*/
	short int pnois;		/* poisson noise from photon flux */
	char *dstate;			/* state array for dark noise */
	char *pstate;			/* state array for photon noise */
        float area;                     /* photon collecting area of recept */
        double flux;                    /* average photon flux */
        float iflux;                    /* instantaneous photon flux */
        float rhod;                     /* rhodopsin */
        float rhodk;                    /* rhodopsin kinase */
        float rhod2;                    /* MR I */
        float rstar;                    /* MR II (R*)  */
        float gpr;                      /* G prot */
        float pde;                      /* phosphodiesterase */
        float gcyc;                     /* G cyclase enzyme activation level */
        float cycg;                     /* cyclic G level */
        float cond;                     /* dark conductance normalized 0 to 1 */
        float ca;                       /* calcium level */
        float cax;                      /* delayed calcium level */
        float maxcond;                  /* multiplier to get true conductance */
        nodeint recnm1;			/* receptor node num1 */ 
        nodeint recnm2;			/* receptor number 2 */ 
        nodeint recnm3;			/* receptor number 3 */ 
        nodeint recnm4;			/* receptor number 4 */ 
        };

struct recpar: id {                 	/* structure for receptor constants */
	short int pigm;			/* pigment type (0 - 10) */
	float pathl;			/* default pigment path length */
        float rhod;                     /* rhodopsin */
        float rhodk;                    /* rhodopsin kinase */
        float rhod2;                    /* MR I */
        float rstar;                    /* MR II (R*) */
        float gpr;                      /* G prot */
        float pde;                      /* phosphodiesterase */
        float gcyc;                     /* G cyclase enzyme activation level */
        float cycg;                     /* cyclic G level */
        float cond;                     /* dark conductance normalized 0 to 1 */
        float totcond;                  /* absolute conductance (20 x dark) */
        float ca;                       /* calcium level */
        float cax;                      /* delayed calcium level */
	float qc;			/* Q10 value for transduction shape */
	float qcond;			/* Q10 value for transduction cond */
	float qt;			/* base temp for transduction Q10 */
	float oldtempcel;		/* old temp for transduction Q10 */
        float lgain;                 /* rhodopsin gain */
        float rkgain;                   /* rhodopsin kinase gain */
        float rgain1;                   /* rhod gain */
        float rgain2;                   /* rhod2 gain */
        float rgain3;                   /* rstar gain */
        float ggain;                    /* G protein gain */
        float pdebase;                  /* dark level of PDE */
        float pdegain;
        float gcygain;
        float condgain;
        float capump;
        float gca;
        float cxgain;
        float kdgcyc;
        float kdrhodk;
        float decrstar;			/* decay constants */
        float pdedec;
        float deccond;
        float deccax;
	float recslow;			/* slowness factor */
        double vrev;                    /* gated channel reversal potential */
        };

struct recstim: id {                	/* structure for receptor stimulus */
        recstim *next;         		/* pointer to next rec stim */
        recstim *rnext;        		/* pointer to next stim in bin table */
        short int mode;                 /* absolute or delta */
        nodeint recnm1;                 /* receptor number num 1*/
        nodeint recnm2;                 /* receptor number 2 */
        nodeint recnm3;                 /* receptor number 3 */
        nodeint recnm4;                 /* receptor number 4 */
        double time;                    /* time of stimulus */
        float val;                      /* stimulus intensity */
        float wavel;                    /* stimulus wavelength */
        };

struct dyadlst {                        /* list of dyad-connected synapses */
        dyadlst *next;                  /* next in list */
        synap *sdyad;                   /* pointer to synapse */
        int sdyadnum;                   /* synapse element number */
        };

