/* Test of spike generator */ 
/* single compartement */
/* based on Andrew Ishida's kinetics and conductances */

/* scompa.n: Script to show spike adaptation. */
/* scompb.n: scompa.n, tuned to 37 deg */
/* scompc.n: Derived from scompa.n @ 22 deg C subst "ndensity" for "density"*/
/* scompd.n: scompc.n, tuned to 37 deg */
/* scompi.n: Andrew Ishida's model */

timinc   = 1e-4;       /* basic time step */
ploti    = 1e-4;       /* plot increment */
tempcel  = 22;         /* temperature */
crit     = 1e-10;      /* voltage accuracy criterion */ 

/* set relative permeabilities for minor ions to zero */
dpcana=0;
dpnaca=0;
dpcak=0;
dpkca=0;
dpnak=0;
dpkna=0;

dqm = 3;		/* Q10 for Na activation */

dcavoff=0;		/* calcium generated voltage offset */
calcnernst=0;

vna =  0.093; 
vk  = -0.098; 
vcl = -0.065;

grest = -.063;		/* starting resting potential */

drm=12000;              /* makes .26 nS for 10 um dia soma */
			/* look with "nc -p 1 file.n" */
na_mult =  1.0;
ca_mult =  1;
k0_mult =  1;
k2_mult =  1; // kca
k3_mult =  1; // ka

plot_freq = 1;

ntemp = 22;		/* temperature where rate funcs were defined */

qfactm = exp(log(dqm) * (ntemp-6.3)/10);           /* Q10 factor for 22 deg */
qfacth = exp(log(dqh) * (ntemp-6.3)/10);
qfactn = exp(log(dqn) * (ntemp-6.3)/10); 

MSSEC = 1000;

x = setvar();

/*---------------------------------------------*/

func qcond (cond) {

/* convert from "density" S/cm2 to "ndensity" N/um2 at 22 deg C */

  return (1e8 * cond * exp(log(1.4)*(22-6.3)/10));
};

/*---------------------------------------------*/

func calcna0m (v, func)

/* Calculate Na rate functions, given voltage in mv,
   and return rates in 1/sec.

   The "func" parameter defines:

    1   alpha m
    2   beta  m
*/

{
   local val,x,y,r;

  if (func==1) {                                 /* alpha m */

    r = 1.77;                                    /* 2.3 / 1.3 */
    y = -0.13 * (v+50.); 
    x = exp (y*r) - 1.;
    if (abs(x) > 1e-5)                        /* singularity */
       val = y / x
    else
       val = 1/r;
  }

  else if (func==2) {                           /* beta m */
    val =  3 * exp ((v+61) / -17.); 
  };
  return val * MSSEC / qfactm;
};

/*---------------------------------------------*/

func calcna0h (v, func)

/* Calculate Na rate functions, given voltage in mv,
   and return rates in 1/sec.

   The "func" parameter defines:

    1   alpha h
    2   beta  h
*/

{
   local val,x,y;

  if (func==1) {                                       /* alpha h */
    val =  0.2 * exp (-0.05 * (v+75));
  }
  else if (func==2) {                                  /* beta h */
    y = -0.12 * (v+35.);
    val = 2.1 / (1.0 + exp (y));
  };
  return val * MSSEC / qfacth;
};

/*---------------------------------------------*/

func calck0 (v,  func)

/* Calculate K rate functions, given voltage in mv,
   and return rates in 1/sec.

   The "func" parameter defines:

    1   alpha n
    2   beta  n
*/

{
   local val,x,y,r;

  if (func==1) {                                /* alpha n */

    r = 0.068; 
    y = -0.1 * (v+37); 
    x = exp (y) - 1.;
    if (abs(x) > 1e-5)                        /* singularity */
       val = y * r / x
    else
       val = r;
  }

  else if (func==2) {                            /* beta n */
    val =  0.1 * exp ((v+50) / -80); 
  };
  return val * MSSEC / qfactn;
};

/*---------------------------------------------*/

/* make model here */

soma = 1;
at soma sphere dia 10 vrest grest;

at soma chan 
   Na type 0                       /* Na HH channel */
	maxcond 94e-9 
//	chnoise=1 
       ename naxx;


at soma chan 
       K type 0                      /* Kdr channel */
	maxcond 29.8e-9
//	chnoise=1
	ename khh;  


// at soma chan 
//        KCa type 0                     /* Kca channel */ 
// 	ndensity k2_mult*10e-3/qcond(dkcasu) 
// 	k1=2e-4 k2=5e-4 d1=0 d2=0 
// //	chnoise=1 
// 	ename kca;
//
// at soma chan            /* KA channel */
//        K type 2 
// 	ndensity k3_mult*35e-3/qcond(dkau) 
// //	chnoise=1
// 	ename ka; 
// 
// at soma chan  
//       Ca type 0                    /* L-type Ca channel */ 
//	taum 1 offset .00 
//	ndensity ca_mult*3e-3/qcond(dcalu)
//	capump vmax 1e-4 km=.5e-6 //caexch kex .5e-9 
//	cabuf vmax 1e8 kd .5e-6 btot 3e-6 btoti 10e-6
//	cshell 10
//       ename cachan;


  /* current clamp, then record spike train ( and [Ca] ) */

  waittime = 0.1; // Let the system relax before stimulus.
  stimdur  = 0.4;
  afterstim = 0.00;
  nstim=5;
  ndstim=0;
  endexp=waittime+nstim*stimdur+ndstim*stimdur+afterstim;
// endexp = .1;
  stimampl = 2e-12;

for (i=1; i<=nstim; i++) {
    stim node soma cclamp i*stimampl start waittime+(i-1.0)*stimdur dur stimdur; 
};
for (i=1; i<=ndstim; i++) {
    stim node soma cclamp nstim*stimampl-i*stimampl start waittime+(nstim+i-1.0)*stimdur dur stimdur; 
};

proc onplot () {                /* this procedure runs at plot time */

  if (plot_freq) onplot2();
};

if (plot_freq) {

/* Process intracellular voltage to find spike times, */
/* inter-spike intervals, and instantaneous spike */
/* frequencies. */

spikrate = 0;
spikint = 0;
spiktim = 0;
spikthresh = -.03;
spikyet = 0;

vh = 0;
oldvh = 0;
oldspiktim = 0;

func spikplot (nplot, xval, yval)
{
   if (yval > 50) retval = 7
   else if (yval > 20)  retval = 5
   else                 retval = 6;
   if (yval < 10) retval = -1;
   return (retval);
};

proc onplot2 ()

/* calculate instantaneous frequency from inter-spike interval */

{
  vh = (V[soma] > spikthresh);

  if (vh && !oldvh) {           /* spike here */
     spiktim = time;
     if (spikyet) {
        spikint = spiktim - oldspiktim + 1e-12;
        spikrate = 1 / spikint;
     };
     spikyet = 1;
     oldspiktim = spiktim;
  }
  else spikrate = 0;
  oldvh = vh;
};

};

 plot V[soma] pen 4 max .05 min -.1;
//  plot I[soma] pen 5 max 1e-9 min -1e-9;
//  plot G(I) naxx ; /*I_na*/
//  plot G(I) khh; /* I_k_dr */
//  plot G kca max 10e-9 min 0; /* I Kca */
//  plot Ca(I) [soma]; /* ica*/
//  plot Ca(1) [soma] max 3e-6 min 0; /*[ca]*/
//  plot cabuf(1) [soma] max 30e-6 min 0 pen 6; /*[ca]*/
//  plot G(I) ka max 1e-9 min -1e-9; /* IKA*/
// plot V[hillock] pen 3;
// plot V[axon1] pen 3;
// plot G khh pen 5 max 100e-9 min 0;
// plot G ka pen 2 max 1e-9 min 0;
// plot Ca(vrev) [soma] max .1 min 0; 
// plot Ca(5)  [soma] pen 5 max 3e-6 min 0;
//  plot Ca(99) [soma] pen 5 max 3e-6 min 0;
// plot  G kca pen 6 max 200e-12 min 0;

if (plot_freq) plot S spikrate max 200 min 0 Char 'o' size .01 vpen spikplot;

run;

